dnl acinclude.m4

# serial 1

dnl Checks for `tclsh' and set output variable
dnl `TCLSH' to the path of the Tcl interpreter.
AC_DEFUN(AC_PROG_TCLSH, [
	AC_PATH_PROG(TCLSH,tclsh)
	])

dnl Checks for `wish' and set output variable
dnl `TKSH' to the path of the Tk interpreter.
AC_DEFUN(AC_PROG_TKSH, [
	AC_PATH_PROG(TKSH,wish)
	])

dnl Checks which version of `tclsh' is used and
dnl puts that in the variable TCLSH_VERSION
AC_DEFUN(AC_TCLSH_VERSION, [
dnl AC_MSG_CHECKING(Tcl version number)
cat > conftest.tcl <<\EOF
changequote(<<, >>)dnl
puts $tcl_version
exit
changequote([, ])dnl
EOF
TCLSH_VERSION=`$TCLSH conftest.tcl`
rm conftest.tcl
TCLSH_VERSION_MAJOR=`expr $TCLSH_VERSION : '\(.*\)\..*' \| $TCLSH_VERSION`
TCLSH_VERSION_MINOR=`expr $TCLSH_VERSION : '.*\.\(.*\)' \| 0`
dnl AC_MSG_RESULT([${TCLSH_VERSION} found])
])

dnl Checks which version of `wish' is used and
dnl puts that in the variable TKSH_VERSION
AC_DEFUN(AC_TKSH_VERSION, [
dnl AC_MSG_CHECKING(Tk version number)
cat > conftest.tcl <<\EOF
changequote(<<, >>)dnl
puts $tk_version
exit
changequote([, ])dnl
EOF
TKSH_VERSION=`$TKSH conftest.tcl`
rm conftest.tcl
TKSH_VERSION_MAJOR=`expr $TKSH_VERSION : '\(.*\)\..*' \| $TKSH_VERSION`
TKSH_VERSION_MINOR=`expr $TKSH_VERSION : '.*\.\(.*\)' \| 0`
dnl AC_MSG_RESULT([${TKSH_VERSION} found])
])

dnl Requires a certain Tcl version
AC_DEFUN(AC_REQUIRE_TCLSH_VERSION, [
AC_PROG_TCLSH
AC_MSG_CHECKING(for Tcl version $1.$2)
AC_TCLSH_VERSION
if test "$TCLSH_VERSION_MAJOR" -ge $1; then
	if test "$TCLSH_VERSION_MINOR" -ge $2; then
		AC_MSG_RESULT([$TCLSH_VERSION found, sufficient])
	else
		AC_MSG_ERROR([only $TCLSH_VERSION found, too early])
	fi
else
	AC_MSG_ERROR([only $TCLSH_VERSION found, too early])
fi
])

dnl Requires a certain Tk version
AC_DEFUN(AC_REQUIRE_TKSH_VERSION, [
AC_PROG_TKSH
AC_MSG_CHECKING(for Tk version $1.$2)
AC_TKSH_VERSION
if test "$TKSH_VERSION_MAJOR" -ge $1; then
	if test "$TKSH_VERSION_MINOR" -ge $2; then
		AC_MSG_RESULT([$TKSH_VERSION found, sufficient])
	else
		AC_MSG_ERROR([only $TKSH_VERSION found, too early])
	fi
else
	AC_MSG_ERROR([only $TKSH_VERSION found, too early])
fi
])

dnl Find the tclConfig.sh and set some things defined therein
AC_DEFUN(AC_TCLCONFIG_SH, [
	AC_MSG_CHECKING(for tclConfig.sh)
	TCLCONFIG_SH=`expr $TCLSH : '\(.*\)/bin' \| ""`/lib/tclConfig.sh
dnl	should use a AC_CHECK_FILE() here but that macro doesn't seem to exist
	if test -r $TCLCONFIG_SH ; then
		AC_MSG_RESULT([$TCLCONFIG_SH])
		. $TCLCONFIG_SH
dnl		Strip off bare `-L's
		TCL_LIB=`echo ${TCL_LIB_SPEC} | sed 's/-L //g'`
		AC_SUBST(TCL_LIBS)
		AC_SUBST(TCL_LIB)
		TCL_INCLUDES=-I$TCL_PREFIX/include
		AC_SUBST(TCL_INCLUDES)
	else
		AC_MSG_ERROR([not found, best guess was $TCLCONFIG_SH])
	fi
])

dnl Find the tkConfig.sh and set some things defined therein
AC_DEFUN(AC_TKCONFIG_SH, [
	AC_MSG_CHECKING(for tkConfig.sh)
	TKCONFIG_SH=`expr $TKSH : '\(.*\)/bin' \| ""`/lib/tkConfig.sh
dnl	should use a AC_CHECK_FILE() here but that macro doesn't seem to exist
	if test -r $TKCONFIG_SH ; then
		AC_MSG_RESULT([$TKCONFIG_SH])
		. $TKCONFIG_SH
dnl		Strip off bare `-L's
		TK_LIB=`echo ${TK_LIB_SPEC} | sed 's/-L //g'`
		AC_SUBST(TK_LIBS)
		AC_SUBST(TK_LIB)
		TK_INCLUDES=-I$TK_PREFIX/include
		AC_SUBST(TK_INCLUDES)
	else
		AC_MSG_ERROR([not found, best guess was $TKCONFIG_SH])
	fi
])


dnl Find the tclxConfig.sh and set some things defined therein
AC_DEFUN(AC_TCLXCONFIG_SH, [
	AC_MSG_CHECKING(for tclxConfig.sh)
	TCLXCONFIG_SH=`expr $TCLSH : '\(.*\)/bin' \| ""`/lib/tclxConfig.sh
dnl	should use a AC_CHECK_FILE() here but that macro doesn't seem to exist
	if test -r $TCLXCONFIG_SH ; then
		AC_MSG_RESULT([$TCLXCONFIG_SH])
		. $TCLXCONFIG_SH
dnl		Strip off bare `-L's
		TCLX_LIB=`echo ${TCLX_LIB_SPEC} | sed 's/-L //g'`
		AC_SUBST(TCLX_LIBS)
		AC_SUBST(TCLX_LIB)
dnl Should set this but it isn't defined in the tclxConfig.sh
dnl		TCLX_INCLUDES=-I$TCLX_PREFIX/include
dnl		AC_SUBST(TCLX_INCLUDES)
	else
		AC_MSG_ERROR([not found, best guess was $TCLXCONFIG_SH])
	fi
])

dnl Find the tkxConfig.sh and set some things defined therein
AC_DEFUN(AC_TKXCONFIG_SH, [
	AC_MSG_CHECKING(for tkxConfig.sh)
	TKXCONFIG_SH=`expr $TKSH : '\(.*\)/bin' \| ""`/lib/tkxConfig.sh
dnl	should use a AC_CHECK_FILE() here but that macro doesn't seem to exist
	if test -r $TKXCONFIG_SH ; then
		AC_MSG_RESULT([$TKXCONFIG_SH])
		. $TKXCONFIG_SH
dnl		Strip off bare `-L's
		TKX_LIB=`echo ${TKX_LIB_SPEC} | sed 's/-L //g'`
		AC_SUBST(TKX_LIBS)
		AC_SUBST(TKX_LIB)
dnl Should set this but it isn't defined in the tclxConfig.sh
dnl		TKX_INCLUDES=-I$TKX_PREFIX/include
dnl		AC_SUBST(TKX_INCLUDES)
	else
		AC_MSG_ERROR([not found, best guess was $TKXCONFIG_SH])
	fi
])

dnl set the bin directory based on arch type
AC_DEFUN(AC_SET_BINDIR, [
	lardbindir=${bindir}/${host}
	AC_SUBST(lardbindir)
	AC_MSG_RESULT([lardbindir set to ${lardbindir}])
	])

dnl set the lib directory based on arch type
AC_DEFUN(AC_SET_LIBDIR, [
	intsize=`expr 8 \* $ac_cv_sizeof_int`
changequote(<<, >>)dnl
	if [ ${ac_cv_c_bigendian} = yes ] ; then
changequote([, ])dnl
		lardlibdir=${libdir}/B`expr 8 \* ${ac_cv_sizeof_int}`
	else
		lardlibdir=${libdir}/L`expr 8 \* ${ac_cv_sizeof_int}`
	fi
	AC_SUBST(lardlibdir)
	AC_MSG_RESULT([lardlibdir set to ${lardlibdir}])
	])

dnl set where any scripts go
AC_DEFUN(AC_SET_SCRIPTDIR, [
	lardcommondir=${bindir}/common
	AC_SUBST(lardcommondir)
	AC_MSG_RESULT([lardcommondir set to ${lardcommondir}])
	])

dnl set where any elisp files go
AC_DEFUN(AC_SET_ELISPDIR, [
	lardelispdir=${libdir}/elisp
	AC_SUBST(lardelispdir)
	AC_MSG_RESULT([lardelispdir set to ${lardelispdir}])
	])

