/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: lexer.l.head,v 1.3 2001/06/18 14:36:48 bardslea Exp $


	The source for the lexical analyser for Lard assembly language

	11 Jun 1999, AMULET Group
*/


%{
#include <config.h>
#include <string.h>

#include <error.h>

#include "symtab.h"
#include "parser.h"
#include "main.h"
#include "../bytecode/bytecode.h"

#define MAX_INPUT_LINE_LENGTH 65536
char infn[256];

#undef YY_INPUT
#define YY_INPUT(buf,result,max_size)  	       	  \
  {						  \
    char * fg_res;				  \
						  \
    fg_res = fgets(input_buffer,                  \
        max_size<MAX_INPUT_LINE_LENGTH?max_size:MAX_INPUT_LINE_LENGTH,yyin); \
    strcpy(buf,input_buffer);			  \
    if (fg_res==NULL)				  \
      result = YY_NULL;				  \
    else					  \
      result = strlen(input_buffer);		  \
  }

%}


NUMBER	\-?[0-9]+
WORD	[a-zA-Z0-9_\!\%\^\&\*\-\=\+\;\:\@\~\<\>\.\/\?\$\[\]\{\}\|\,]+

%x string
%x chr

%%

^#" "[0-9]+" "\".*\".*$	{ sscanf(yytext,"# %d \"%[^\"]",&line_num,infn); }

{NUMBER}	{ sscanf(yytext,"%d",&yylval.number); return (NUM); }

                /* String and character literals */

<INITIAL>\"     { BEGIN(string);
                  return (DQUOTE); }

<string>\"      { BEGIN(INITIAL);
                  return (DQUOTE); }
                  
<INITIAL>\'     { BEGIN(chr);
                  return (SQUOTE); }

<chr>\'         { BEGIN(INITIAL);
                  return (SQUOTE); }
                  
<string,chr>\\[0-7]{1,3} { yylval.character = strtol(&yytext[1],NULL,8);
                             return (CHAR_I); }
<string,chr>\\n       { yylval.character = '\n'; return (CHAR_I); }
<string,chr>\\t       { yylval.character = '\t'; return (CHAR_I); }
<string,chr>\\r       { yylval.character = '\r'; return (CHAR_I); }
<string,chr>\\b       { yylval.character = '\b'; return (CHAR_I); }
<string,chr>\\f       { yylval.character = '\f'; return (CHAR_I); }
<string,chr>\\.       { yylval.character = yytext[1]; return (CHAR_I); }
<string,chr>\\\n      { line_num++; yylval.character = '\n'; return (CHAR_I); }

<string,chr>\n        { yyerror(
                          "Unterminated string/character at end of line"); }

<string><<EOF>> |
<chr><<EOF>>           { yyerror(
                          "Unterminated string/character at end of file"); }

<string,chr>[^\\\n]   { yylval.character = yytext[0]; return (CHAR_I); }

\#.*$

EQN     { return(KWEQN); }
EQS     { return(KWEQS); }
ALLOC	{ return(KWALLOC); }
FREE	{ return(KWFREE); }
PUSH	{ return(KWPUSH); }
POP	{ return(KWPOP); }
POPFORGET	{ return(KWPOPFORGET); }
LOAD	{ return(KWLOAD); }
STORE	{ return(KWSTORE); }
SETN	{ return(KWSETN); }
SETSYM	{ return(KWSETSYM); }
SETSYM_D	{ return(KWSETSYM_D); }
SETC	{ return(KWSETC); }
SETR	{ return(KWSETR); }
ADD	{ return(KWADD); }
SUB	{ return(KWSUB); }
RSB	{ return(KWRSB); }
MUL	{ return(KWMUL); }
DIV	{ return(KWDIV); }
MOD	{ return(KWMOD); }
NOT	{ return(KWNOT); }
AND	{ return(KWAND); }
OR	{ return(KWOR); }
XOR	{ return(KWXOR); }
SHL	{ return(KWSHL); }
SHR	{ return(KWSHR); }
INC	{ return(KWINC); }
DEC	{ return(KWDEC); }
INC4	{ return(KWINC4); }
DEC4	{ return(KWDEC4); }
CMPEQ	{ return(KWCMPEQ); }
CMPNEQ	{ return(KWCMPNEQ); }
CMPLT	{ return(KWCMPLT); }
CMPLTEQ	{ return(KWCMPLTEQ); }
CMPGT	{ return(KWCMPGT); }
CMPGTEQ	{ return(KWCMPGTEQ); }
SETNNZ	{ return(KWSETNNZ); }
SETSYMNZ	{ return(KWSETSYMNZ); }
NOP	{ return(KWNOP); }
BUILTIN	{ return(KWBUILTIN); }
END	{ return(KWEND); }
FORK	{ return(KWFORK); }
DIE	{ return(KWDIE); }
YIELD	{ return(KWYIELD); }
SETPRI	{ return(KWSETPRI); }
SETMYPRI	{ return(KWSETMYPRI); }
RUNCHECK	{ return(KWRUNCHECK); }
LINE	{ return(KWLINE); }
FILEREF	{ return(KWFILEREF); }
STEPPOINT	{ return(KWSTEPPOINT); }
GETTIME	{ return(KWGETTIME); }
"stdin"	{ yylval.number=BI_stdin; return(BUILTIN); }
"stdout"	{ yylval.number=BI_stdout; return(BUILTIN); }
"stderr"	{ yylval.number=BI_stderr; return(BUILTIN); }
"openin"	{ yylval.number=BI_openin; return(BUILTIN); }
"openout"	{ yylval.number=BI_openout; return(BUILTIN); }
"openinout"	{ yylval.number=BI_openinout; return(BUILTIN); }
"close"	{ yylval.number=BI_close; return(BUILTIN); }
"fprintint"	{ yylval.number=BI_fprintint; return(BUILTIN); }
"fprintstr"	{ yylval.number=BI_fprintstr; return(BUILTIN); }
"fprintchar"	{ yylval.number=BI_fprintchar; return(BUILTIN); }
"freadint"	{ yylval.number=BI_freadint; return(BUILTIN); }
"freadstr"	{ yylval.number=BI_freadstr; return(BUILTIN); }
"freadchar"	{ yylval.number=BI_freadchar; return(BUILTIN); }
"teof"	{ yylval.number=BI_teof; return(BUILTIN); }
"beof"	{ yylval.number=BI_beof; return(BUILTIN); }
"setdefinpbase"	{ yylval.number=BI_setdefinpbase; return(BUILTIN); }
"fputc"	{ yylval.number=BI_fputc; return(BUILTIN); }
"fgetc"	{ yylval.number=BI_fgetc; return(BUILTIN); }
"fwrite"	{ yylval.number=BI_fwrite; return(BUILTIN); }
"fread"	{ yylval.number=BI_fread; return(BUILTIN); }
"fseek"	{ yylval.number=BI_fseek; return(BUILTIN); }
"ftell"	{ yylval.number=BI_ftell; return(BUILTIN); }
"flen"	{ yylval.number=BI_flen; return(BUILTIN); }
"ftrunc"	{ yylval.number=BI_ftrunc; return(BUILTIN); }
"rand"	{ yylval.number=BI_rand; return(BUILTIN); }
"getversion"	{ yylval.number=BI_getversion; return(BUILTIN); }
"regtime"	{ yylval.number=BI_regtime; return(BUILTIN); }
"nexttime"	{ yylval.number=BI_nexttime; return(BUILTIN); }
"promote_waits"	{ yylval.number=BI_promote_waits; return(BUILTIN); }
"export_var"	{ yylval.number=BI_export_var; return(BUILTIN); }
"update_export_var"	{ yylval.number=BI_update_export_var; return(BUILTIN); }
"tcl_cmd"	{ yylval.number=BI_tcl_cmd; return(BUILTIN); }
"tcl_cmd_noerr"	{ yylval.number=BI_tcl_cmd_noerr; return(BUILTIN); }
"tcl_cmd_int"	{ yylval.number=BI_tcl_cmd_int; return(BUILTIN); }
"tcl_cmd_str"	{ yylval.number=BI_tcl_cmd_str; return(BUILTIN); }
"fatal"	{ yylval.number=BI_fatal; return(BUILTIN); }
"bitsdiff"	{ yylval.number=BI_bitsdiff; return(BUILTIN); }
"int2str"	{ yylval.number=BI_int2str; return(BUILTIN); }
"int2hexstr"	{ yylval.number=BI_int2hexstr; return(BUILTIN); }
"create_sparray"	{ yylval.number=BI_create_sparray; return(BUILTIN); }
"destroy_sparray"	{ yylval.number=BI_destroy_sparray; return(BUILTIN); }
"index_sparray"	{ yylval.number=BI_index_sparray; return(BUILTIN); }
"mpintop"	{ yylval.number=BI_mpintop; return(BUILTIN); }
"formatdata"	{ yylval.number=BI_formatdata; return(BUILTIN); }
"parsedata"	{ yylval.number=BI_parsedata; return(BUILTIN); }
"stringop"	{ yylval.number=BI_stringop; return(BUILTIN); }
"i2s"	{ yylval.number=BI_i2s; return(BUILTIN); }
"warning"	{ yylval.number=BI_warning; return(BUILTIN); }
"lardpos"	{ yylval.number=BI_lardpos; return(BUILTIN); }
"fsetunbuffered"	{ yylval.number=BI_fsetunbuffered; return(BUILTIN); }
"cosimulation_cmd"	{ yylval.number=BI_cosimulation_cmd; return(BUILTIN); }
"PC"	{ yylval.number=REG_PC; return(REG); }
"T1"	{ yylval.number=REG_T1; return(REG); }
"T2"	{ yylval.number=REG_T2; return(REG); }
"T3"	{ yylval.number=REG_T3; return(REG); }
"T4"	{ yylval.number=REG_T4; return(REG); }
"T5"	{ yylval.number=REG_T5; return(REG); }
"T6"	{ yylval.number=REG_T6; return(REG); }
"T7"	{ yylval.number=REG_T7; return(REG); }
"T8"	{ yylval.number=REG_T8; return(REG); }
"CXP"	{ yylval.number=REG_CXP; return(REG); }
"FPNUM"	{ yylval.number=REG_FPNUM; return(REG); }
"TID"	{ yylval.number=REG_TID; return(REG); }
"TIME"	{ yylval.number=REG_TIME; return(REG); }
"PS"	{ yylval.number=STK_PS; return(STK); }
"RAS"	{ yylval.number=STK_RAS; return(STK); }
"CXS"	{ yylval.number=STK_CXS; return(STK); }
"PTIME"	{ yylval.number=STK_PTIME; return(STK); }

^EXTERN\ {WORD}        { if (pass==1)
	        	   putsym(yytext+7,addr,TRUE);
	        	 return (LABELDEF); }

^{WORD}		{ if (pass==1)
		    putsym(yytext,addr,FALSE);
		  return (LABELDEF); }

{WORD}		{ if (pass==2)
		    yylval.symbol=getsym(yytext)-addr;
		  return (SYM); }

<<EOF>>         { return (END_OF_FILE); }

[ \t]+

\n              { line_num++; }



.		{ fprintf(stderr,"Illegal character %c (%d) in input (line %d)\n",
			  yytext[0],(int)yytext[0],line_num);
		  exit(1);
		}

%%

