/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: main.c,v 1.3 2001/06/18 14:36:48 bardslea Exp $


	The entry point for the Lard assembler

	11 Jun 1999, AMULET Group
*/

#include <config.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <alloc.h>
#include <error.h>

#include "assem.h"
#include "../bytecode/bytecode.h"
#include "symtab.h"
#include "reloc.h"

FILE * outf = NULL;
int addr;
int pass;

char * codesect;
int codesectsize;
int realcodesectsize;

extern int yydebug;

bool saveintsyms;

char * infilename="";
char line_input_buffer[65536];
char * input_buffer;
int line_num;

void parse_arguments(int argc, char *argv[])
{
  int a;

  yyin=NULL;
  saveintsyms=FALSE;
  
  for(a=1;a<argc;a++)
    {
      if (strcmp(argv[a],"-?")==0 || strcmp(argv[a],"-help")==0
          || strcmp(argv[a],"--help")==0)
      {
        fatal("ICODE assembler\n"
              "Usage: %s [-syms] -o outfile infile\n",argv[0]);
      }
      else if (strcmp(argv[a],"-syms")==0)
      {
        saveintsyms=TRUE;
      }
      else if (strcmp(argv[a],"-o")==0)
      {
        if (outf)
        {
          fatal("Can't specify more than one output file\n");
        }
        outf=fopen(argv[a+1],"w");
        if (!outf)
        {
          fatal("Can't open output file %s\n",argv[a+1]);
        }
        a++;
      }
      else
      {
        if (yyin)
        {
          fatal("Can't specify more than one input file\n");
        }
        infilename=argv[a];
        yyin=fopen(argv[a],"r");
        if (!yyin)
        {
          fatal("Can't open input file %s\n",argv[a]);
        }
      }
    }

  if (!yyin)
  {
    fatal("No input file specified\n");
  }
  if (!outf)
  {
    fatal("No output file specified\n");
  }
}


void make_sectheader(int secttype, int start, int end, int p1, int p2)
{
  struct section_desc sd;

  sd.sect_type=secttype;
  sd.start=start;
  sd.end=end;
  sd.private1=p1;
  sd.private2=p2;
  fwrite(&sd,sizeof(struct section_desc),1,outf);
}


int main(int argc, char * argv[])
{
  int e, f;

  input_buffer = line_input_buffer;

  codesect=palloc(0);
  codesectsize=0;
  realcodesectsize=0;
  relocsect=palloc(0);
  relocsectsize=0;
  
  addr=0;
  parse_arguments(argc,argv);
  pass=1;
  yyparse();
  pass=2;
  fseek(yyin,0,SEEK_SET);
  line_num=0;
  addr=0;
  yyparse();

  makeextsymsect();
  if (saveintsyms)
    makeintsymsect();
  
  e=(saveintsyms?5:4)*sizeof(struct section_desc);
  make_sectheader(HEADER_SECTTYPE,0,e,0,0);
  f=e+codesectsize;
  make_sectheader(CODE_SECTTYPE,e,f,0,0);
  e=f+relocsectsize;
  make_sectheader(RELOC_SECTTYPE,f,e,0,0);
  f=e+externsymsectsize;
  make_sectheader(EXTERNSYM_SECTTYPE,e,f,0,0);
  if (saveintsyms)
  {
    e=f+internsymsectsize;
    make_sectheader(INTERNSYM_SECTTYPE,f,e,0,0);
  }
  
  fwrite(codesect,codesectsize,1,outf);
  fwrite(relocsect,relocsectsize,1,outf);
  fwrite(externsymsect,externsymsectsize,1,outf);
  if (saveintsyms)
    fwrite(internsymsect,internsymsectsize,1,outf);
  
  return(0);
}

