/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: parser.y.pt1,v 1.3 2001/06/18 14:36:48 bardslea Exp $


	First part of the Lard assembly language parser 

	11 Jun 1999, AMULET Group
*/

%{

/* C DECLARATIONS */

#include <config.h>
#include <stdlib.h>
#include <string.h>

#include <alloc.h>
#include <apasprintf.h>
#include <error.h>

#include "symtab.h"
#include "main.h"
#include "../bytecode/bytecode.h"

#define YYERROR_VERBOSE

/* NB. make this a 2^n number */
#define CODESECT_BLOCK_SIZE 1024

extern int yylex (void);

void enlarge_codesect ( int newsize )
{
  if (newsize>realcodesectsize)
  {
    realcodesectsize += (newsize + (CODESECT_BLOCK_SIZE - 1)) & ~(CODESECT_BLOCK_SIZE - 1);
    codesect=prealloc(codesect,realcodesectsize);
  }
}

void emit ( int c )
{
  if (pass==2)
  {
    enlarge_codesect(codesectsize+4);
    *(int *)(codesect+codesectsize)=c;
    codesectsize+=4;
  }
  addr++;
}

void emitstr ( char * s )
{
  int slen;
  int i;

  slen=((strlen(s)+4)&~3);

  if (pass==2)
  {
    enlarge_codesect(codesectsize+slen);
    strcpy(codesect+codesectsize,s);
    codesectsize=codesectsize+slen;
    for(i=-4;((*(codesect+codesectsize+i))!='\0') && (i<0);i++);
    for(;i<0;i++) {(*(codesect+codesectsize+i))='\0';}
  }
  addr+=(slen/4);
}


%}


/* BISON DECLARATIONS */

/* The possible types of the semantic value of a token */

%union 
{
  int number;             /* integers */
  int symbol;             /* symbols */
  char * string;          /* strings */
  char character;         /* characters */
}

/* Types for the rules */

/* The tokens.  First those that have a semantic value : */

%token <number>    NUM BUILTIN REG STK SYM
%token <character> CHAR_I

/* Type for the rules */

%type <character>  CHAR
%type <string>     STR string

/* Symbols */

%token LABELDEF DQUOTE SQUOTE END_OF_FILE

/* keywords */

%token
KWEQN
KWEQS

KWALLOC
KWFREE
KWPUSH
KWPOP
KWPOPFORGET
KWLOAD
KWSTORE
KWSETN
KWSETSYM
KWSETSYM_D
KWSETC
KWSETR
KWADD
KWSUB
KWRSB
KWMUL
KWDIV
KWMOD
KWNOT
KWAND
KWOR
KWXOR
KWSHL
KWSHR
KWINC
KWDEC
KWINC4
KWDEC4
KWCMPEQ
KWCMPNEQ
KWCMPLT
KWCMPLTEQ
KWCMPGT
KWCMPGTEQ
KWSETNNZ
KWSETSYMNZ
KWNOP
KWBUILTIN
KWEND
KWFORK
KWDIE
KWYIELD
KWSETPRI
KWSETMYPRI
KWRUNCHECK
KWLINE
KWFILEREF
KWSTEPPOINT
KWGETTIME

/*
 * $Log: parser.y.pt3,v $
 * Revision 1.1.1.1  1999/07/15 18:47:31  bardslea
 * Imported LARD at 2.0.12
 *
 * Revision 1.1  1996/08/07 19:56:22  phil
 * version 1
 *
 */


%%


/* GRAMMAR RULES */

input_e:	input END_OF_FILE	{ YYACCEPT; }

input:	  /* nothing */
	| input cmd
	;

STR:	  DQUOTE string DQUOTE			{ $$=$2; }
	;

string:           /* nothing */                 { char * p=palloc(1);
                                                  *p='\0'; $$=p; }
                | string CHAR_I                 { char * p=$1;
                                                  apasprintf(&p,"%c",$2);
                                                  $$=p; }
                ;

CHAR:	  SQUOTE CHAR_I SQUOTE			{ $$=$2; }
	;

cmd:	  LABELDEF		{ }
	| KWEQN		NUM	{ emit($2); }
	| KWEQS		STR	{ emitstr($2); }




	| KWALLOC	REG REG	{ emit(BC_ALLOC|($2<<8)|($3<<16)); }
	| KWFREE	REG	{ emit(BC_FREE|($2<<8)); }
	| KWPUSH	STK REG	{ emit(BC_PUSH|($2<<8)|($3<<16)); }
	| KWPOP	STK REG	{ emit(BC_POP|($2<<8)|($3<<16)); }
	| KWPOPFORGET	STK REG	{ emit(BC_POPFORGET|($2<<8)|($3<<16)); }
	| KWLOAD	REG REG	{ emit(BC_LOAD|($2<<8)|($3<<16)); }
	| KWSTORE	REG REG	{ emit(BC_STORE|($2<<8)|($3<<16)); }
	| KWSETN	REG NUM	{ emit(BC_SETN|($2<<8)); emit($3); }
	| KWSETSYM	REG SYM	{ emit(BC_SETSYM|($2<<8)); emit($3); }
	| KWSETSYM_D	REG SYM	{ emit(BC_SETSYM_D|($2<<8)); emit($3); }
	| KWSETC	REG CHAR	{ emit(BC_SETC|($2<<8)); emit($3); }
	| KWSETR	REG REG	{ emit(BC_SETR|($2<<8)|($3<<16)); }
	| KWADD	REG REG	{ emit(BC_ADD|($2<<8)|($3<<16)); }
	| KWSUB	REG REG	{ emit(BC_SUB|($2<<8)|($3<<16)); }
	| KWRSB	REG REG	{ emit(BC_RSB|($2<<8)|($3<<16)); }
	| KWMUL	REG REG	{ emit(BC_MUL|($2<<8)|($3<<16)); }
	| KWDIV	REG REG	{ emit(BC_DIV|($2<<8)|($3<<16)); }
	| KWMOD	REG REG	{ emit(BC_MOD|($2<<8)|($3<<16)); }
	| KWNOT	REG	{ emit(BC_NOT|($2<<8)); }
	| KWAND	REG REG	{ emit(BC_AND|($2<<8)|($3<<16)); }
	| KWOR	REG REG	{ emit(BC_OR|($2<<8)|($3<<16)); }
	| KWXOR	REG REG	{ emit(BC_XOR|($2<<8)|($3<<16)); }
	| KWSHL	REG REG	{ emit(BC_SHL|($2<<8)|($3<<16)); }
	| KWSHR	REG REG	{ emit(BC_SHR|($2<<8)|($3<<16)); }
	| KWINC	REG	{ emit(BC_INC|($2<<8)); }
	| KWDEC	REG	{ emit(BC_DEC|($2<<8)); }
	| KWINC4	REG	{ emit(BC_INC4|($2<<8)); }
	| KWDEC4	REG	{ emit(BC_DEC4|($2<<8)); }
	| KWCMPEQ	REG REG REG	{ emit(BC_CMPEQ|($2<<8)|($3<<16)|($4<<24)); }
	| KWCMPNEQ	REG REG REG	{ emit(BC_CMPNEQ|($2<<8)|($3<<16)|($4<<24)); }
	| KWCMPLT	REG REG REG	{ emit(BC_CMPLT|($2<<8)|($3<<16)|($4<<24)); }
	| KWCMPLTEQ	REG REG REG	{ emit(BC_CMPLTEQ|($2<<8)|($3<<16)|($4<<24)); }
	| KWCMPGT	REG REG REG	{ emit(BC_CMPGT|($2<<8)|($3<<16)|($4<<24)); }
	| KWCMPGTEQ	REG REG REG	{ emit(BC_CMPGTEQ|($2<<8)|($3<<16)|($4<<24)); }
	| KWSETNNZ	REG REG NUM	{ emit(BC_SETNNZ|($2<<8)|($3<<16)); emit($4); }
	| KWSETSYMNZ	REG REG SYM	{ emit(BC_SETSYMNZ|($2<<8)|($3<<16)); emit($4); }
	| KWNOP		{ emit(BC_NOP); }
	| KWBUILTIN	BUILTIN	{ emit(BC_BUILTIN|($2<<8)); }
	| KWEND		{ emit(BC_END); }
	| KWFORK	REG	{ emit(BC_FORK|($2<<8)); }
	| KWDIE		{ emit(BC_DIE); }
	| KWYIELD		{ emit(BC_YIELD); }
	| KWSETPRI	REG REG	{ emit(BC_SETPRI|($2<<8)|($3<<16)); }
	| KWSETMYPRI	REG	{ emit(BC_SETMYPRI|($2<<8)); }
	| KWRUNCHECK	REG REG	{ emit(BC_RUNCHECK|($2<<8)|($3<<16)); }
	| KWLINE	SYM NUM	{ emit(BC_LINE); emit($2); emit($3); }
	| KWFILEREF	SYM STR	{ emit(BC_FILEREF); emit($2); emitstr($3); }
	| KWSTEPPOINT		{ emit(BC_STEPPOINT); }
	| KWGETTIME	REG	{ emit(BC_GETTIME|($2<<8)); }

/*
 * $Log: parser.y.pt5,v $
 * Revision 1.1.1.1  1999/07/15 18:47:31  bardslea
 * Imported LARD at 2.0.12
 *
 * Revision 1.1  1996/08/07 19:56:22  phil
 * version 1
 *
 */


	;

%%





