/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: symtab.c,v 1.1.1.1 1999/07/15 18:47:30 bardslea Exp $


	The Lard assembler symbol table

	11 Jun 1999, AMULET Group
*/
/*
 * $Log: symtab.c,v $
 * Revision 1.1.1.1  1999/07/15 18:47:30  bardslea
 * Imported LARD at 2.0.12
 *
 * Revision 1.1  1996/08/07 19:56:22  phil
 *
 */

#include <string.h>
#include <stddef.h>
#include <stdlib.h>

#include "symtab.h"
#include "main.h"
#include "error.h"
#include "reloc.h"
#include "alloc.h"

char * internsymsect;
int internsymsectsize;

char * externsymsect;
int externsymsectsize;

struct symrec * sym_table = NULL;

void putsym (char *sym_name, int addr, bool external)
{
  struct symrec * * ptr;
  int cmp;
  
  ptr=&sym_table;
  while (TRUE)
  {
    if (!(*ptr))
    {
      (*ptr)=palloc(sizeof(struct symrec));
      (*ptr)->name=palloc(strlen(sym_name)+1);
      strcpy((*ptr)->name,sym_name);
      (*ptr)->addr=addr;
      (*ptr)->external=external;
      (*ptr)->left=NULL;
      (*ptr)->right=NULL;
      return;
    }
    cmp=strcmp((*ptr)->name,sym_name);
    if (cmp==0)
    {
      fatal("symbol %s has multiple definitions\n",sym_name);
    } else if (cmp<0)
    {
      ptr=&((*ptr)->right);
    } else
    {
      ptr=&((*ptr)->left);
    }
  }
}



int getsym (char *sym_name)
{
  struct symrec *ptr;
  int cmp;
  
  for(ptr=sym_table;ptr;)
  {
    cmp=strcmp(ptr->name,sym_name);
    if (cmp==0)
    {
      return(ptr->addr);
    } else if (cmp<0)
    {
      ptr=ptr->right;
    } else
    {
      ptr=ptr->left;
    }
  }

  addreloc(addr+1,sym_name);
  return(0);
}


void makesymsect_i(struct symrec * p, char ** symsectptr,
                   int * symsectsizeptr, bool external)
{
  int slen;
  int i;

  if (p)
  {
    if (p->external==external)
    {
      slen=(strlen(p->name)+4)&~3;
      (*symsectptr)=prealloc((*symsectptr),
                            (*symsectsizeptr)+4+slen);
      *((int *)((*symsectptr)+(*symsectsizeptr)))=(p->addr);
      strcpy((*symsectptr)+(*symsectsizeptr)+4,p->name);
      (*symsectsizeptr)=(*symsectsizeptr)+4+slen;
      for(i=-4;((*((*symsectptr)+(*symsectsizeptr)+i))!='\0') && (i<0);i++);
      for(;i<0;i++) {(*((*symsectptr)+(*symsectsizeptr)+i))='\0';}
    }
    makesymsect_i(p->left,symsectptr,symsectsizeptr,external);
    makesymsect_i(p->right,symsectptr,symsectsizeptr,external);
  }
}


void makesymsect(char ** symsectptr, int * symsectsizeptr, bool external)
{
  (*symsectptr)=palloc(0);
  (*symsectsizeptr)=0;
  makesymsect_i(sym_table,symsectptr,symsectsizeptr,external);
}


void makeintsymsect(void)
{
  makesymsect(&internsymsect,&internsymsectsize,FALSE);
}


void makeextsymsect(void)
{
  makesymsect(&externsymsect,&externsymsectsize,TRUE);
}

