/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: codegen.c,v 1.1.1.1 1999/07/15 18:47:31 bardslea Exp $


	Icode code generation

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>

#include <alloc.h>
#include <apasprintf.h>
#include <error.h>

#include "debug.h"
#include "parser_structs.h"
#include "decls.h"
#include "scopes.h"
#include "main.h"
#include "codegen.h"
#include "dump.h"
#include "output.h"
#include "escapify.h"
#include "dcode.h"

void codegen_expr ( struct expr * expr, struct expr * decls,
                    char * * caller_code, bool external );
void init_codegen ( void );
void codegen_paramdecls ( struct list * decls, char * * code, bool plmp );
void codegen_decl ( struct expr * decl, char * * code, bool external );
void codegen_param_decl ( struct expr * decl, char * * code, bool plmp );
void codegen_type_decl ( struct expr * decl, bool external );
void codegen_type_decl_builtin ( struct expr * proto, char * * code );
void codegen_type_decl_alias ( struct expr * decl, char * * code );
void codegen_val_decl ( struct expr * decl );
void codegen_val_decl_const ( struct expr * decl,
			      struct expr * proto, struct expr * defn );
void codegen_val_decl_enum ( struct expr * decl );
void codegen_val_decl_convfunc ( struct expr * decl, struct expr * proto );
void codegen_var_decl ( struct expr * decl, char * * code, bool external );
void codegen_expr_decl ( struct expr * decl, bool external );
int count_paramdecls ( struct list * args );
int count_impdecls ( struct expr * expr );
int count_impdecls_args ( struct list * args );
int count_decls ( struct expr * expr );
int count_decl ( struct expr * decl );
void codegen_expr_decl_builtin ( struct expr * decl, struct expr * proto );
void codegen_type_param_decl ( struct expr * param, char * * code, bool plmp );
void codegen_val_param_decl ( struct expr * param, char * * code, bool plmp );
void codegen_var_param_decl ( struct expr * param, char * * code, bool plmp );
void codegen_expr_param_decl ( struct expr * param, char * * code, bool plmp );
void codegen_fcall ( struct expr * expr, struct expr * decls, char * * code,
                     bool external );
void codegen_lib ( struct expr * expr, struct expr * decls, char * * code );
void codegen_arrow ( struct expr * expr, char * * code );
void codegen_type ( struct expr * expr, struct expr * decls, char * * code );
void codegen_val ( struct expr * expr, char * * code );
void codegen_var ( struct expr * expr, char * * code );
void codegen_exprcall ( struct expr * expr, struct expr * decls,
                        char * * caller_code, bool external );
void codegen_exprparamcall ( struct expr * expr, char * * code );
void codegen_exprcall_params ( struct list * aargs, struct list * fargs,
                               char * * code );
void codegen_impparams ( struct expr * actualc, struct expr * formalc,
                         char * * code );
void codegen_impparams_args ( struct list * actuals, struct list * formals,
                              char * * code );
void codegen_numlit ( struct expr * expr, char * * code );
void codegen_strlit ( struct expr * expr, char * * code );
void codegen_charlit ( struct expr * expr, char * * code );
void codegen_lib_decl ( struct expr * decl, char * * code );
void codegen_impparam_decls ( struct expr * expr, char * * code, bool plmp );
void codegen_impparam_decls_args ( struct list * args, char * * code,
                                   bool plmp );
void codegen_paramdecls_free ( struct list * decls, char * * code );
void codegen_param_decl_free ( struct expr * decl, char * * code );
void codegen_impparam_decls_free ( struct expr * expr, char * * code );
void codegen_impparam_decls_args_free ( struct list * args, char * * code );
void codegen_decls ( struct expr * expr, char * * code, bool external );
void codegen_decls_free ( struct expr * expr, char * * code );
void codegen_decl_free ( struct expr * expr, char * * code );
void codegen_var_decl_free ( struct expr * decl, char * * code );
void dcode_decl ( struct expr * decl );
void dcode_finish ( struct expr * expr );



struct symrec * class_ident;
struct expr * class_decl;     
struct expr * type_class;
struct expr * val_class;
struct expr * var_class;
struct expr * expr_class;
struct expr * lib_class;
struct expr * varargs_class;
struct expr * overloaded_class;
struct expr * readdecl;
struct expr * islibdecl;
struct expr * evaldecl;
struct expr * mkexprdecl;

int currentnestlevel=0;
int top_localnum=0;
int * localnump=&top_localnum;
int infilenum=0;

bool add_plump;
bool add_prof;

bool gen_dcode;
FILE * dcode_file;
int dcode_term_nulls;

int strnum=0;

char * ename ( char x, struct expr * expr )
{
  char * p = NULL;
  
  apasprintf(&p,"%c_%s_%d_%d",x,expr->u.fc.ident->name,
             expr->line,expr->idcode);
  return(p);
}

char * dname ( char x, struct expr * decl )
{
  char * p = NULL;
  
  apasprintf(&p,"%c_%s_%d_%d",x,get_proto(decl)->u.fc.ident->name,
             decl->line,decl->idcode);
  return(p);
}

char * bname ( char x, struct expr * expr )
{
  char * p = NULL;
  
  if (expr->decl->overloaded)
  {
      apasprintf(&p,"%c_%s_%d",x,expr->u.fc.ident->name,
                 expr->decl->overloaded);
  } else
  {
    apasprintf(&p,"%c_%s",x,expr->u.fc.ident->name);
  }
  return(p);
}

char * bdname ( char x, struct expr * decl )
{
  char * p = NULL;

  if (decl->overloaded)
  {
    apasprintf(&p,"%c_%s_%d",x,get_proto(decl)->u.fc.ident->name,
               decl->overloaded);
  } else
  {
    apasprintf(&p,"%c_%s",x,get_proto(decl)->u.fc.ident->name);
  }
  return(p);
}


void init_codegen ( void )
{
  class_ident=putsym("class");
  class_decl=NULL;
  type_class=NULL;
  val_class=NULL;
  var_class=NULL;
  expr_class=NULL;
  lib_class=NULL;
  varargs_class=NULL;
  overloaded_class=NULL;
}



void codegen_expr ( struct expr * expr, struct expr * decls, char * * code,
                    bool external )
{
  if (expr->sort==fcall_es && expr->u.fc.ident==dot_ident)
    {
      if (decls)
        codegen_expr(arg(2,expr),decls,code,external);
      else
        codegen_expr(arg(2,expr),expr,code,external);
      return;
    }

  switch(expr->sort)
    {
    case fcall_es:
      codegen_fcall(expr,decls,code,external);
      break;
    case num_es:
      codegen_numlit(expr,code);
      break;
    case str_es:
      codegen_strlit(expr,code);
      break;
    case char_es:
      codegen_charlit(expr,code);
      break;
    }
}

	  
void codegen_decl ( struct expr * decl, char * * code, bool external )
{
  struct expr * proto;
  struct expr * class;
  struct expr * defn;
  
  if (decl->u.fc.ident==use_ident)
  {
    if (gen_dcode && external)
    {
      dcode_decl(decl);
    }
    return;
  }
  
  proto=get_proto(decl);
  class=get_class(decl);
  defn=get_defn(decl);
  
  if (proto->sort==fcall_es && proto->u.fc.ident==class_ident)
  {
    if (!compstdlib)
      fatal_error(decl->line,"Got declaration of class while not "
                  "compiling standard library");
    if (class_decl)
      fatal_error(decl->line,"class class redeclared");
    class_decl=decl;
    debug(debug_pass_3,"Got CLASS declaration\n");
    if (gen_dcode)
      dcode_decl(decl);
    return;
  }
  
  if (class->decl==class_decl)
    {
      if (!compstdlib)
	fatal_error(decl->line,"Got a class declaration while not compiling "
		    "standard library");
      if (gen_dcode)
        dcode_decl(decl);
      if (proto->sort==fcall_es)
	{
	  if (proto->u.fc.ident==putsym("type"))
	    {
              if (type_class)
                fatal_error(decl->line,"class type redeclared");
	      type_class=decl;
	      debug(debug_pass_3,"Got TYPE declaration\n");
	      return;
	    }
	  else if (proto->u.fc.ident==putsym("val"))
	    {
              if (val_class)
                fatal_error(decl->line,"class val redeclared");
	      val_class=decl;
	      debug(debug_pass_3,"Got VAL declaration\n");
	      return;
	    }
	  else if (proto->u.fc.ident==putsym("var"))
	    {
              if (var_class)
                fatal_error(decl->line,"class var redeclared");
	      var_class=decl;
	      debug(debug_pass_3,"Got VAR declaration\n");
	      return;
	    }
	  else if (proto->u.fc.ident==putsym("expr"))
	    {
              if (expr_class)
                fatal_error(decl->line,"class expr redeclared");
	      expr_class=decl;
	      debug(debug_pass_3,"Got EXPR declaration\n");
	      return;
	    }
          else if (proto->u.fc.ident==putsym("lib"))
          {
            if (lib_class)
              fatal_error(decl->line,"class lib redeclared");
            lib_class=decl;
            debug(debug_pass_3,"Got LIB declaration\n");
            return;
          }
          else if (proto->u.fc.ident==putsym("varargs"))
          {
            if (varargs_class)
              fatal_error(decl->line,"class varargs redeclared");
            varargs_class=decl;
            debug(debug_pass_3,"Got VARARGS declaration\n");
            return;
          }
          else if (proto->u.fc.ident==putsym("overloaded"))
          {
            if (overloaded_class)
              fatal_error(decl->line,"class overloaded redeclared");
            overloaded_class=decl;
            debug(debug_pass_3,"Got OVERLOADED declaration\n");
            return;
          }
	}
      fatal_error(decl->line,
	      "Don't understand class declaration %s",
	      proto->u.fc.ident->name);
    }

  decl->nestlevel=currentnestlevel;
  debug(debug_pass_3,"nl %d : ",currentnestlevel);

  if (external && gen_dcode && class->decl!=expr_class
      && class->decl!=lib_class)
  {
    dcode_decl(decl);
  }
    
  if (class->decl==type_class)
    codegen_type_decl(decl,external);
  else if (class->decl==val_class)
    codegen_val_decl(decl);
  else if (class->decl==var_class)
    codegen_var_decl(decl,code,external);
  else if (class->decl==expr_class)
    codegen_expr_decl(decl,external);
  else if (class->decl==lib_class)
    codegen_lib_decl(decl,code);
  else if (class->decl==varargs_class)
        {} /* do nothing */
  else if (class->decl==overloaded_class)
        {} /* do nothing */
  else
    {
      fatal_error(decl->line,
		  "Don't understand class %s",
		  class->u.fc.ident->name);
    }
}

	  
void codegen_type_decl ( struct expr * decl, bool external )
{
  struct expr * proto = get_proto(decl);
  struct expr * defn = get_defn(decl);
  char * * code;

  decl->external=external;

  if (reading_dcode)
    return;
  
  debug(debug_pass_3,"type %s : ",proto->u.fc.ident->name);

  code=&(decl->code);

  if (!defn)
    codegen_type_decl_builtin(proto,code);
  else
  {
    apasprintf(code,"%s%s\t\t# type definition %s\n",
               external?"EXTERN ":"",
               external?bdname('T',decl):dname('T',decl),
               get_proto(decl)->u.fc.ident->name);
    codegen_type_decl_alias(decl,code);
    apasprintf(code,"\tPOP RAS PC\n\n");
  }
}


void codegen_type_decl_builtin ( struct expr * proto, char * * code )
{
  debug(debug_pass_3,"BUILTIN STRUCTURE, builtin size expression\n");
  if (!compstdlib)
    fatal_error(proto->line,"Got builtin type declaration and not "
		"compiling standard library");
}


void codegen_type_decl_alias ( struct expr * decl, char * * code )
{
  struct expr * proto = get_proto(decl);
  struct expr * defn = get_defn(decl);
  int * old_localnump;
  int localnum=0;

  debug(debug_pass_3,"ALIAS, generating size expression\n");
  old_localnump=localnump;
  localnump=&localnum;
  currentnestlevel++;
  apasprintf(code,"\tSETN T1 %d\t\t# allocate new CX\n"
             "\tALLOC T1 T2\n"
             "\tSTORE T2 CXP\n"
             "\tSETR CXP T2\n",
             (count_paramdecls(proto->u.fc.args)+1)*4);
  codegen_paramdecls(proto->u.fc.args,code,FALSE);
  codegen_expr(defn,NULL,code,FALSE);
  codegen_paramdecls_free(proto->u.fc.args,code);
  apasprintf(code,"\tFREE CXP\n");
  currentnestlevel--;
  localnump=old_localnump;
}


void codegen_val_decl ( struct expr * decl )
{
  struct expr * proto = get_proto(decl);
  struct expr * defn = get_defn(decl);

  debug(debug_pass_3,"val %s : ",
        proto->sort==fcall_es?proto->u.fc.ident->name:
        proto->sort==num_es?"(num)":
        proto->sort==str_es?"(string)":
        proto->sort==char_es?"(char)":"???");
  
  if (proto->sort!=fcall_es)
  {
    debug(debug_pass_3,"BUILTIN LITERAL\n");
    return;
  }

  if (defn)
    codegen_val_decl_const(decl,proto,defn);
  else
    
  if (count_args(proto)>0)
  {
    if (compstdlib)
    {
      codegen_val_decl_convfunc(decl,proto);
      return;
    }
    else
    {
      fatal_error(proto->line,
                  "Prototype for a val declaration should have "
                  "no parameters");
    }
  }
  else
    codegen_val_decl_enum(decl);
}


void codegen_val_decl_const ( struct expr * decl,
			      struct expr * proto, struct expr * defn )
{
/*  if (reading_dcode)
    return; */
  
  debug(debug_pass_3,"CONSTANT\n");
  decl->num=(*localnump)++; /* check this */
  fatal_error(decl->line,"Constant declarations are not implemented");
      /* to do */
}


void codegen_val_decl_enum ( struct expr * decl )
{
  if (count_args(arg(1,get_class(decl)))>0)
    fatal_error(decl->line,"Can't specify an enum value for this type");
  decl->enum_ord=(arg(1,get_class(decl))->decl->enum_ord)++;
  debug(debug_pass_3,"ENUM = %d\n",decl->enum_ord);
}


void codegen_val_decl_convfunc ( struct expr * decl, struct expr * proto )
{
  debug(debug_pass_3,"BUILTIN CONV FUNC %s\n",proto->u.fc.ident->name);
  if (proto->u.fc.ident==putsym("read"))
    readdecl=decl;
  else if (proto->u.fc.ident==putsym("eval"))
    evaldecl=decl;
}


void codegen_var_decl ( struct expr * decl, char * * code, bool external )
{
  struct expr * proto = get_proto(decl);
  struct expr * defn = get_defn(decl);

  debug(debug_pass_3,"var %s\n",proto->u.fc.ident->name);
  
  if (count_args(proto)>0)
    {
      if (!compstdlib)
        fatal_error(proto->line,
		  "Prototype for a var declaration should have no parameters");
      else
        return;
    }

  decl->num=(*localnump)++;
  decl->external=external;
  
  if (reading_dcode)
    return;

  apasprintf(code,"\t\t\t\t# variable %s\n",
             get_proto(decl)->u.fc.ident->name);
  if (defn)
  {
    fatal_error(decl->line,"Variable initialisations don't work");
    apasprintf(code,"\t\t\t\t# get initialisation value\n");
    codegen_expr(defn,NULL,code,FALSE);
  }
  apasprintf(code,"\t\t\t\t# get type\n");
  codegen_type(arg(1,get_class(decl)),NULL,code);
  apasprintf(code,"\tPOP PS T1\t\t# variable %s; allocate space\n"
             "\tALLOC T1 T2\n",
             get_proto(decl)->u.fc.ident->name);
/*  if (add_plump)
    apasprintf(code,"\tDEBUG \"STO %%reg %%reg %s\\n\" T2 T1\n",
               get_proto(decl)->u.fc.ident->name); */
  if (external)
  {
    apasprintf(code,"\tSETSYM_D T3 %s\t\t# store pointer in global\n",
               bdname('G',decl));
    apasprintf(&topdecls,"EXTERN %s\tNOP\n",bdname('G',decl));
  }
  else
    apasprintf(code,"\tSETR T3 CXP\t\t# store pointer in CX\n"
               "\tSETN T4 %d\n"
               "\tADD T3 T4\n",
             (decl->num+1)*4);
  apasprintf(code,"\tSTORE T3 T2\n");

  if (defn)
  {
        /* never used */
    apasprintf(code,"\t\t\t\t# copy in initialisation value\n"
               "%s\n"
               "\tPOP PS T5\n"
               "\tSTORE T2 T5\n"
               "\tINC4 T2\n"
               "\tDEC4 T1\n"
               "\tSETSYMNZ T1 PC %s",
               dname('L',decl),dname('L',decl));
  }
}


void codegen_lib_decl ( struct expr * decl, char * * code )
{
  int old_term_nulls=-1;

  if (get_defn(decl))
  {
    if (gen_dcode)
    {
      fputc('F',dcode_file);
      fputs("_._",dcode_file);
      fputc('\0',dcode_file);
      fputc('A',dcode_file);
      fputc('F',dcode_file);
      fputs("_:_=_",dcode_file);
      fputc('\0',dcode_file);
      fputc('A',dcode_file);
      dump_dcode(dcode_file,get_proto(decl));
      fputc('A',dcode_file);
      dump_dcode(dcode_file,get_class(decl));
      fputc('A',dcode_file);
      old_term_nulls=dcode_term_nulls;
      dcode_term_nulls=0;
    }
    
    codegen_expr(get_defn(decl),NULL,code,TRUE);

    if (gen_dcode)
    {
      while (dcode_term_nulls>0)
      {
        fputc('\0',dcode_file);
        dcode_term_nulls--;
      }
      fputc('\0',dcode_file);
      dcode_term_nulls=old_term_nulls;
      fputc('A',dcode_file);
      dcode_term_nulls++;
    }
  } else
  {
    if (gen_dcode)
    {
      fputc('F',dcode_file);
      fputs("_._",dcode_file);
      fputc('\0',dcode_file);
      fputc('A',dcode_file);
      dump_dcode(dcode_file,decl);
      fputc('A',dcode_file);
      dcode_term_nulls++;
    }
  }
}


void codegen_expr_decl ( struct expr * decl, bool external )
{
  struct expr * proto = get_proto(decl);
  struct expr * defn = get_defn(decl);
  char * * code;
  int * old_localnump;
  int localnum=0;

  debug(debug_pass_3,"expr %s : ",proto->u.fc.ident->name);

  if (external && gen_dcode)
  {
    fputc('F',dcode_file);
    fputs("_._",dcode_file);
    fputc('\0',dcode_file);
    fputc('A',dcode_file);
    fputc('F',dcode_file);
    fputs("_:_",dcode_file);
    fputc('\0',dcode_file);
    fputc('A',dcode_file);
    dump_dcode(dcode_file,proto);
    fputc('A',dcode_file);
    dump_dcode(dcode_file,get_class(decl));
    fputc('\0',dcode_file);
    fputc('A',dcode_file);
    dcode_term_nulls++;
  }
  
  if (!defn)
    {
      codegen_expr_decl_builtin(decl,proto);
      return;
    }

  decl->external=external;

  if (reading_dcode)
    return;
  
  debug(debug_pass_3,"\n");

  old_localnump=localnump;
  localnump=&localnum;
  currentnestlevel++;
  code=&(decl->code);
  apasprintf(code,"%s%s\t\t# definition of %s\n",
             external?"EXTERN ":"",
             external?bdname('F',decl):dname('F',decl),
             get_proto(decl)->u.fc.ident->name);

  if (add_prof) {
    apasprintf(code,"\tPOP PTIME T1\t\t# profiling\n"
               "\tPUSH PTIME T1\n"
               "\tGETTIME T2\n"
               "\tSETR T3 T2\n"
               "\tSUB T2 TIME\n"
               "\tSETR TIME T3\n"
               "\tLOAD T1 T4\n"
               "\tADD T4 T2\n"
               "\tSTORE T1 T4\n"
               "\tSETSYM_D T5 TI%s\n"
               "\tPUSH PTIME T5\t\t# end of profiling\n",
                external?bdname('F',decl):dname('F',decl));
  }
  apasprintf(code,"\tSETN T1 %d\t\t# allocate new CX\n"
             "\tALLOC T1 T2\n"
             "\tSTORE T2 CXP\n"
             "\tSETR CXP T2\n",
             (count_paramdecls(proto->u.fc.args)+1)*4);
  codegen_paramdecls(proto->u.fc.args,code,TRUE);
  if (add_plump)
    apasprintf(code,"\tSTEPPOINT\n");
  codegen_expr(defn,NULL,code,FALSE);
  apasprintf(code,"\tPUSH CXS CXP\t\t# evaluate body expression\n"
             "\tSETSYM T1 %s\n"
             "\tPUSH RAS T1\n"
             "\tPOP PS CXP\n"
             "\tPOP PS PC\n"
             "%s\n"
             "\tPOP CXS CXP\n",
             dname('R',decl),dname('R',decl));
  codegen_paramdecls_free(proto->u.fc.args,code);
  apasprintf(code,"\tFREE CXP\t\t# free CX\n");
  if (add_prof) {
    apasprintf(code,"\tPOP PTIME T1\t\t# profiling\n"
               "\tGETTIME T2\n"
               "\tSETR T3 T2\n"
               "\tSUB T2 TIME\n"
               "\tSETR TIME T3\n"
               "\tLOAD T1 T4\n"
               "\tADD T4 T2\n"
               "\tSTORE T1 T4\t\t# end of profiling\n");
  }
  apasprintf(code,"\tPOP RAS PC\t\t# return\n\n");
  if (add_prof) {
    apasprintf(code,"TI%s\t\t# profile counter\n"
               "\tEQN 0\n\n",external?bdname('F',decl):dname('F',decl));
  }
  currentnestlevel--;
  localnump=old_localnump;
}


int count_paramdecls ( struct list * args )
{
  if (args)
    return 1+count_impdecls(arg(2,args->expr))+count_paramdecls(args->next);
  else
    return 0;
}


int count_impdecls ( struct expr * expr )
{
  if (expr)
    if (expr->sort==fcall_es)
	{
      if (expr->u.fc.ident==decl2_ident)
        return(1+count_impdecls(arg(2,expr)));
      else
		  return count_impdecls_args(expr->u.fc.args);
	}
  return 0;
}


int count_impdecls_args ( struct list * args )
{
  if (args)
    return count_impdecls(args->expr) + count_impdecls_args(args->next);
  else
    return 0;
}


int count_decls ( struct expr * expr )
{
  if (expr->sort!=fcall_es)
    return 0;
  else
    if (expr->u.fc.ident==dot_ident)
      return(count_decl(arg(1,expr))+count_decls(arg(2,expr)));
    else
      return 0;
}


int count_decl ( struct expr * decl )
{
  if ((decl->u.fc.ident!=use_ident)
      &&((get_class(decl)->decl==var_class)
         && (count_args(get_proto(decl))==0)))
    return 1;
  else
    return 0;
}


void codegen_expr_decl_builtin ( struct expr * decl, struct expr * proto )
{
  debug(debug_pass_3,"BUILTIN\n");
  if (proto->u.fc.ident==putsym("mkexpr"))
    mkexprdecl=decl;
}


void codegen_type_param_decl ( struct expr * param, char * * code, bool plmp )
{
  debug(debug_pass_3,"type, num=%d\n",param->num);
  apasprintf(code,"\tSETN T1 4\t\t# type parameter %s\n"
             "\tALLOC T1 T2\n"
             "\tSETR T3 CXP\n"
             "\tSETN T4 %d\n"
             "\tADD T3 T4\n"
             "\tSTORE T3 T2\n"
             "\tPOP PS T5\n"
             "\tSTORE T2 T5\n",
             get_proto(param)->u.fc.ident->name,(param->num+1)*4);
/*  if (plmp && add_plump)
    apasprintf(code,"\tDEBUG \"{%s TYPE %%reg} \" T5\n",
               get_proto(param)->u.fc.ident->name); */
}


void codegen_val_param_decl ( struct expr * param, char * * code, bool plmp )
{
  codegen_type(arg(1,get_class(param)),NULL,code);
  debug(debug_pass_3,"val, num=%d\n",param->num);
  apasprintf(code,"\tPOP PS T1\t\t# val parameter %s\n",
             get_proto(param)->u.fc.ident->name);
/*  if (plmp && add_plump)
    apasprintf(code,"\tDEBUG \"{%s VAL {\"\n",
               get_proto(param)->u.fc.ident->name); */
  apasprintf(code,"\tALLOC T1 T2\n"
             "\tSETR T3 CXP\n"
             "\tSETN T4 %d\n"
             "\tADD T3 T4\n"
             "\tSTORE T3 T2\n"
             "\tSETSYMNZ T1 PC %s\n"
             "\tSETSYM PC %s\n"
             "%s\n"
             "\tPOP PS T5\n",
             (param->num+1)*4,
             dname('L',param),
             dname('X',param),
             dname('L',param));
/*  if (plmp && add_plump)
    apasprintf(code,"\tDEBUG \"%%reg \" T5\n"); */
  apasprintf(code,"\tSTORE T2 T5\n"
             "\tINC4 T2\n"
             "\tDEC4 T1\n"
             "\tSETSYMNZ T1 PC %s\n"
             "%s\n",
             dname('L',param),
             dname('X',param));
/*  if (plmp && add_plump)
    apasprintf(code,"\tDEBUG \"}} \"\n"); */
}


void codegen_var_param_decl ( struct expr * param, char * * code, bool plmp )
{
  debug(debug_pass_3,"var, num=%d\n",param->num);
  apasprintf(code,"\tSETR T3 CXP\t\t# var parameter %s\n"
             "\tSETN T4 %d\n"
             "\tADD T3 T4\n"
             "\tPOP PS T5\n"
             "\tSTORE T3 T5\n",
             get_proto(param)->u.fc.ident->name,(param->num+1)*4);
/*  if (plmp && add_plump)
    apasprintf(code,"\tDEBUG \"{%s VAR %%reg} \" T5\n",
             get_proto(param)->u.fc.ident->name); */
}


void codegen_expr_param_decl ( struct expr * param, char * * code, bool plmp )
{
  debug(debug_pass_3,"expr, num=%d\n",param->num);
  apasprintf(code,"\tSETN T1 8\t\t# expr parameter %s\n"
             "\tALLOC T1 T2\n"
             "\tSETR T3 CXP\n"
             "\tSETN T4 %d\n"
             "\tADD T3 T4\n"
             "\tSTORE T3 T2\n"
             "\tPOP PS T5\n"
             "\tPOP PS T6\n"
             "\tSTORE T2 T5\n"
             "\tINC4 T2\n"
             "\tSTORE T2 T6\n",
             get_proto(param)->u.fc.ident->name,(param->num+1)*4);
/*  if (plmp && add_plump)
    apasprintf(code,"\tDEBUG \"{%s EXPR} \" T5 T6\n",
             get_proto(param)->u.fc.ident->name); */
}


void codegen_fcall ( struct expr * expr, struct expr * decls, char * * code,
                     bool external )
{
  struct expr * class;
  int * old_localnump;
  int localnum;
  
  if (expr->u.fc.ident==arrow_ident)
  {
    codegen_arrow(expr,code);
    return;
  }

  if (expr->u.fc.ident==field_ident)
  {
    codegen_expr(arg(2,expr),decls,code,FALSE);
    return;
  }
  
  class=get_class(expr->decl);

  if (compstdlib && !expr_class)
  {
    codegen_exprcall(expr,decls,code,external);
    return;
  }
  
  if (decls && !((class->decl==expr_class && !expr->decl->isparam) ||
                 (class->decl==type_class) ||
                 (class->decl==lib_class)))
  {
    fatal_error(decls->line,"declarations not expected here");
  }

  old_localnump=localnump;
  localnump=&localnum;
  localnum=0;
  
  if (class->decl==type_class)
    codegen_type(expr,decls,code);
  else if (class->decl==val_class)
    codegen_val(expr,code);
  else if (class->decl==var_class)
    codegen_var(expr,code);
  else if (class->decl==expr_class)
    {
		if (expr->decl->isparam)
			codegen_exprparamcall(expr,code);
		else
			codegen_exprcall(expr,decls,code,external);
	}
  else if (class->decl==lib_class)
    codegen_lib(expr,decls,code);
  else
    {
      fatal_error(expr->line,"Don't understand class %s",
		  class->u.fc.ident->name);
    }

  localnump=old_localnump;
}


void codegen_lib ( struct expr * expr, struct expr * decls, char * * code )
{
  if (decls) codegen_decls(decls,code,TRUE);

  if (gen_dcode)
  {
    dump_dcode(dcode_file,expr);
  }
}


void codegen_arrow ( struct expr * expr, char * * code )
{
  apasprintf(code,"\t\t\t\t# ->\n");

  apasprintf(code,"\tSETN T1 %d\t\t# field number for %s\n"
             "\tPUSH PS T1\n",
             arg(2,expr)->decl->num,
             arg(2,expr)->u.fc.ident->name);

  apasprintf(code,"\t\t\t\t# record\n");
  codegen_expr(arg(1,expr),NULL,code,FALSE);
  apasprintf(code,"\t\t\t\t# type for whole record\n");
  codegen_type(arg(1,arg(1,expr)->class),NULL,code);
  apasprintf(code,"\t\t\t\t# type for selected field\n");
  codegen_type(arg(1,expr->class),NULL,code);

  apasprintf(code,"\tPUSH CXS CXP\t\t# save CX\n"
             "\tSETSYM T1 %s\n"
             "\tPUSH RAS T1\n"
             "\tSETSYM PC %s\t\t# call -> function\n"
             "%s\n"
             "\tPOP CXS CXP\t\t# restore CX\n",
             ename('R',expr),
             expr->class->decl==var_class?"VAR_->":
             expr->class->decl==val_class?"VAL_->":
             expr->class->decl==expr_class?"EXPR_->":"???",
             ename('R',expr));
}


void codegen_type ( struct expr * expr, struct expr * decls, char * * code )
{
  int i;

  if (expr->decl->isparam)
  {
    if (decls)
      fatal_error(decls->line,"declarations not allowed here");
    apasprintf(code,"\tSETR T1 CXP\t\t# type %s\n",expr->u.fc.ident->name);
    for(i=expr->decl->nestlevel;i<currentnestlevel;i++)
      apasprintf(code,"\tLOAD T1 T1\n");
    apasprintf(code,"\tSETN T2 %d\n"
               "\tADD T1 T2\n"
               "\tLOAD T1 T3\n"
               "\tLOAD T3 T4\n"
               "\tPUSH PS T4\n",
               (expr->decl->num+1)*4);
  }
  else
  {
    codegen_decls(decls,code,FALSE);
    codegen_exprcall_params(expr->u.fc.args,get_proto(expr->decl)->u.fc.args,
                            code);
    apasprintf(code,"\tPUSH CXS CXP\t\t# type %s\n",expr->u.fc.ident->name);
    for (i=expr->decl->nestlevel;i<currentnestlevel;i++)
      apasprintf(code,"\tLOAD CXP CXP\n");
    apasprintf(code,"\tSETSYM T1 %s\n"
               "\tPUSH RAS T1\n"
               "\tSETSYM PC %s\n"
               "%s\n"
               "\tPOP CXS CXP\n",
               ename('R',expr),
               (get_defn(expr->decl) && !expr->decl->external)?
               dname('T',expr->decl):bdname('T',expr->decl),
               ename('R',expr));
    expr->idcode=idcode++;
  }
}


void codegen_val ( struct expr * expr, char * * code )
{
  int i;

  if (count_args(get_proto(expr->decl))>0)
  {
        /* call val conversion function */
    apasprintf(code,"\t\t\t\t# Val function %s\n",expr->u.fc.ident->name);
    codegen_exprcall_params(expr->u.fc.args,get_proto(expr->decl)->u.fc.args,
                            code);
    apasprintf(code,"\tPUSH CXS CXP\t\t# save CX\n"
               "\tSETSYM T1 %s\n"
               "\tPUSH RAS T1\n"
               "\tSETSYM PC %s\t\t# call val function %s\n"
               "%s\n"
               "\tPOP CXS CXP\t\t# restore CX\n",
               ename('R',expr),bname('V',expr),
               expr->u.fc.ident->name,ename('R',expr));
    expr->idcode=idcode++;
  }
  else if (!expr->decl->isparam && count_args(expr->decl)==2)
  {
        /* enum label value */
    apasprintf(code,"\tSETN T1 %d\t\t# enum label %s\n"
               "\tPUSH PS T1\n",
               expr->decl->enum_ord,expr->u.fc.ident->name);
  }
  else /* constant or val parameter */
  {
    codegen_type(arg(1,expr->class),NULL,code);
    apasprintf(code,"\tSETR T1 CXP\t\t# parameter %s\n",
               expr->u.fc.ident->name);
    for(i=expr->decl->nestlevel;i<currentnestlevel;i++)
      apasprintf(code,"\tLOAD T1 T1\n");
    apasprintf(code,"\tSETN T2 %d\n"
               "\tADD T1 T2\n"
               "\tLOAD T1 T3\n"
               "\tPOP PS T5\n"
               "\tADD T3 T5\n"
               "\tSETSYMNZ T5 PC %s\n"
               "\tSETSYM PC %s\n"
               "%s\n"
               "\tDEC4 T3\n"
               "\tLOAD T3 T4\n"
               "\tPUSH PS T4\n"
               "\tDEC4 T5\n"
               "\tSETSYMNZ T5 PC %s\n"
               "%s\n",
               (expr->decl->num+1)*4,
               ename('L',expr),
               ename('X',expr),
               ename('L',expr),
               ename('L',expr),
               ename('X',expr));
        /* hack: */ expr->idcode=idcode++;
  }
}


void codegen_var ( struct expr * expr, char * * code )
{
  int i;
  
  if (count_args(get_proto(expr->decl))>0)
  {
        /* call var conversion function */
    apasprintf(code,"\t\t\t\t# Var function %s\n",expr->u.fc.ident->name);
    codegen_exprcall_params(expr->u.fc.args,get_proto(expr->decl)->u.fc.args,
                            code);
    apasprintf(code,"\tPUSH CXS CXP\t\t# save CX\n"
               "\tSETSYM T1 %s\n"
               "\tPUSH RAS T1\n"
               "\tSETSYM PC %s\t\t# call var function %s\n"
               "%s\n"
               "\tPOP CXS CXP\t\t# restore CX\n",
               ename('R',expr),bname('V',expr),
               expr->u.fc.ident->name,ename('R',expr));
  } else {
        /* read from real variable */
    if (expr->decl->external)
    {
      apasprintf(code,"\tSETSYM_D T1 %s\t\t# global var %s\n",
                 bname('G',expr),expr->u.fc.ident->name);
    } else {
      apasprintf(code,"\tSETR T1 CXP\t\t# var %s\n",expr->u.fc.ident->name);
      for(i=expr->decl->nestlevel;i<currentnestlevel;i++)
        apasprintf(code,"\tLOAD T1 T1\n");
      apasprintf(code,"\tSETN T2 %d\n"
                 "\tADD T1 T2\n",
                 (expr->decl->num+1)*4);
    }
    apasprintf(code,"\tLOAD T1 T3\n"
               "\tPUSH PS T3\n");
    
  }
}


void codegen_exprcall ( struct expr * expr, struct expr * decls,
                        char * * caller_code, bool external )
{
  char * * code;
  int i, n;

  if (decls)
    n=count_decls(decls);
  else
    n=0;
  code=&(expr->code);
  apasprintf(caller_code,"\tSETSYM T1 %s\t\t# expr %s\n"
             "\tPUSH PS T1\n"
             "\tPUSH PS CXP\n",
             ename('E',expr),expr->u.fc.ident->name);
  apasprintf(code,"%s\t\t\t# (%s:%d) expression %s\n",
             ename('E',expr),infilename,expr->line,expr->u.fc.ident->name);
  expr->idcode=idcode++;
  if (decls)
  {
    currentnestlevel++;
    apasprintf(code,"\tSETN T1 %d\t\t# allocate new CX for local decls\n"
               "\tALLOC T1 T2\n"
               "\tSTORE T2 CXP\n"
               "\tSETR CXP T2\n"
               "\t\t\t\t# allocate local decls\n",
               (n+1)*4);
    codegen_decls(decls,code,external);
  } else {
    apasprintf(code,"\t\t\t\t# no local decls\n");
  }
  apasprintf(code,"\t\t\t\t# push parameters\n");
  codegen_exprcall_params(expr->u.fc.args,get_proto(expr->decl)->u.fc.args,
                          code);
  if (add_plump)
  {
    apasprintf(code,"\tLINE INIT_%s %d\n",
               infilename,expr->line);
  }
  apasprintf(code,"\tPUSH CXS CXP\t\t# save CX\n"
             "\t\t\t\t# find callee's parent's context\n");
  for (i=expr->decl->nestlevel;i<currentnestlevel;i++)
    apasprintf(code,"\tLOAD CXP CXP\n");
  apasprintf(code,"\tSETSYM T1 %s\t\t# save return address\n"
             "\tPUSH RAS T1\n"
             "\tSETSYM PC %s\t\t# call body function\n"
             "%s\n"
             "\tPOP CXS CXP\t\t# restore CX\n",
             ename('R',expr),
             (get_defn(expr->decl) && !expr->decl->external)
             ?dname('F',expr->decl):bdname('F',expr->decl),
             ename('R',expr));
  expr->idcode=idcode++;
  if (decls)
  {
    codegen_decls_free(decls,code);
    apasprintf(code,"\tFREE CXP\t\t# free CX\n");
    currentnestlevel--;
  }
  apasprintf(code,"\tPOP RAS PC\t\t# return\n\n");
}


void codegen_exprcall_params ( struct list * aargs, struct list * fargs,
                               char * * code )
{
  if ( aargs )
    {
      codegen_exprcall_params(aargs->next,fargs->next,code);
      apasprintf(code,"\t\t\t\t# compute parameter %s\n",
                 get_proto(fargs->expr)->u.fc.ident->name);
      codegen_expr(aargs->expr,NULL,code,FALSE);
      codegen_impparams(aargs->expr->class,get_class(fargs->expr),code);
    }
}


void codegen_impparams ( struct expr * actualc, struct expr * formalc,
                         char * * code )
{
  if (formalc->sort==fcall_es)
  {
    if (formalc->u.fc.ident==decl2_ident)
    {
      apasprintf(code,"\t\t\t\t# compute implicit parameter %s\n",
                 get_proto(formalc)->u.fc.ident->name);
      codegen_expr(actualc,NULL,code,FALSE);
/* something more here ? */
    }
    else
      codegen_impparams_args(actualc->u.fc.args,formalc->u.fc.args,code);
  }
}


void codegen_impparams_args ( struct list * actuals, struct list * formals,
                              char * * code )
{
  if (formals)
  {
    codegen_impparams_args(actuals->next,formals->next,code);
    codegen_impparams(actuals->expr,formals->expr,code);
  }
}


void codegen_exprparamcall ( struct expr * expr, char * * code )
{
  int i;
  
  apasprintf(code,"\tSETR T1 CXP\t\t# expr param %s\n",expr->u.fc.ident->name);
  for(i=expr->decl->nestlevel;i<currentnestlevel;i++)
    apasprintf(code,"\tLOAD T1 T1\n");
  apasprintf(code,"\tSETN T2 %d\n"
             "\tADD T1 T2\n"
             "\tLOAD T1 T3\n"
             "\tLOAD T3 T4\n"
             "\tINC4 T3\n"
             "\tLOAD T3 T5\n"
             "\tPUSH PS T5\n"
             "\tPUSH PS T4\n",
             (expr->decl->num+1)*4);
}


void codegen_numlit ( struct expr * expr, char * * code )
{
  apasprintf(code,
             "\tSETN T1 %d\t\t# numeric literal\n"
             "\tPUSH PS T1\n",
             expr->u.n);
}


void codegen_strlit ( struct expr * expr, char * * code )
{
  char * escaped_string;
  escaped_string=escapify_str(expr->u.s);
  apasprintf(&stringcode,
             "STRCR_%d\n"
             "\tEQN 1\n"
             "\tEQN 0\n"
             "STR_%d\n"
             "\tEQS \"%s\"\n"
             "\n",strnum,strnum,escaped_string);
  apasprintf(&stringinits,
             "\tSETSYM_D T1 STRCR_%d\n"
             "\tINC4 T1\n"
             "\tSETSYM_D T2 STR_%d\n"
             "\tSTORE T1 T2\n"
             "\n",
             strnum,strnum);
  apasprintf(code,
             "\tSETSYM_D T1 STRCR_%d\t\t# string literal\n"
             "\tPUSH PS T1\n",
             strnum);
  strnum++;
}


void codegen_charlit ( struct expr * expr, char * * code )
{
  char * escaped_char=escapify_chr(expr->u.c);
  apasprintf(code,
             "\tSETC T1 '%s'\t\t# character literal\n"
             "\tPUSH PS T1\n",
             escaped_char);
}

void codegen_paramdecls ( struct list * decls, char * * code, bool plmp )
{
  if (decls)
    {
      codegen_param_decl(decls->expr,code,plmp);
      codegen_paramdecls(decls->next,code,plmp);
    }
}


void codegen_param_decl ( struct expr * decl, char * * code, bool plmp )
{
  struct expr * proto;
  struct expr * class;
  struct expr * defn;
  
  proto=get_proto(decl);
  class=get_class(decl);
  defn=get_defn(decl);
  
  if (defn)
    {
      fatal_error(decl->line,
		  "Parameter declarations shouldn't have definitions");
    }
  
  decl->isparam=TRUE;
  decl->num=(*localnump)++;
  decl->nestlevel=currentnestlevel;

  codegen_impparam_decls(class,code,plmp);
  
  debug(debug_pass_3,"nl %d :   parameter %s : ",
        currentnestlevel,proto->u.fc.ident->name);

  if (class->decl==type_class)
    codegen_type_param_decl(decl,code,plmp);
  else if (class->decl==val_class)
    codegen_val_param_decl(decl,code,plmp);
  else if (class->decl==var_class)
    codegen_var_param_decl(decl,code,plmp);
  else if (class->decl==expr_class)
    codegen_expr_param_decl(decl,code,plmp);
  else
    {
      fatal_error(decl->line,
		  "Don't understand class %s",
		  class->u.fc.ident->name);
    }
}


void codegen_impparam_decls ( struct expr * expr, char * * code, bool plmp )
{
  if (expr->sort==fcall_es)
  {
    if (expr->u.fc.ident==decl2_ident)
    {
      codegen_param_decl(expr,code,plmp);
      codegen_impparam_decls(arg(2,expr),code,plmp);
    }
    else
      codegen_impparam_decls_args(expr->u.fc.args,code,plmp);
  }
}


void codegen_impparam_decls_args ( struct list * args, char * * code,
                                   bool plmp )
{
  if (args)
  {
    codegen_impparam_decls(args->expr,code,plmp);
    codegen_impparam_decls_args(args->next,code,plmp);
  }
}


void codegen_paramdecls_free ( struct list * decls, char * * code )
{
  if (decls)
    {
      codegen_param_decl_free(decls->expr,code);
      codegen_paramdecls_free(decls->next,code);
    }
}


void codegen_param_decl_free ( struct expr * decl, char * * code )
{
  struct expr * proto;
  struct expr * class;
  
  proto=get_proto(decl);
  class=get_class(decl);

  codegen_impparam_decls_free(class,code);
  
  if (class->decl==type_class || class->decl==val_class
      || class->decl==expr_class)
    apasprintf(code,"\tSETR T3 CXP\t\t# free local %s\n"
               "\tSETN T4 %d\n"
               "\tADD T3 T4\n"
               "\tLOAD T3 T2\n"
               "\tFREE T2\n",
               get_proto(decl)->u.fc.ident->name,
               (decl->num+1)*4);
}


void codegen_impparam_decls_free ( struct expr * expr, char * * code )
{
  if (expr->sort==fcall_es)
  {
    if (expr->u.fc.ident==decl2_ident)
    {
      codegen_param_decl_free(expr,code);
      codegen_impparam_decls_free(arg(2,expr),code);
    }
    else
      codegen_impparam_decls_args_free(expr->u.fc.args,code);
  }
}


void codegen_impparam_decls_args_free ( struct list * args, char * * code )
{
  if (args)
  {
    codegen_impparam_decls_free(args->expr,code);
    codegen_impparam_decls_args_free(args->next,code);
  }
}


void codegen_decls ( struct expr * expr, char * * code, bool external )
{
  if (expr)
  {
    if (expr->sort==fcall_es && expr->u.fc.ident==dot_ident)
    {
      codegen_decl(arg(1,expr),code,external);
      codegen_decls(arg(2,expr),code,external);
    }
  }
}


void codegen_decls_free ( struct expr * expr, char * * code )
{
  if (expr)
  {
    if (expr->sort==fcall_es && expr->u.fc.ident==dot_ident)
    {
      codegen_decl_free(arg(1,expr),code);
      codegen_decls_free(arg(2,expr),code);
    }
  }
}


void codegen_decl_free ( struct expr * decl, char * * code )
{
  if (decl->u.fc.ident==use_ident)
    return;
  
  if ((get_class(decl)->decl==var_class) && (count_args(get_proto(decl))==0))
    codegen_var_decl_free(decl,code);
}


void codegen_var_decl_free ( struct expr * decl, char * * code )
{
  apasprintf(code,"\tSETR T3 CXP\t\t# free variable %s\n"
             "\tSETN T4 %d\n"
             "\tADD T3 T4\n"
             "\tLOAD T3 T3\n"
             "\tFREE T3\n",
             get_proto(decl)->u.fc.ident->name,(decl->num+1)*4);
}


void codegen_top_expr ( struct expr * expr )
{
  if (expr->sort==fcall_es && expr->u.fc.ident==dot_ident)
  {
    codegen_decl(arg(1,expr),&topcode,TRUE);
    codegen_top_expr(arg(2,expr));
    return;
  }

  if (gen_dcode)
  {
    dump_dcode(dcode_file,expr);
    while (dcode_term_nulls>0)
    {
      fputc('\0',dcode_file);
      dcode_term_nulls--;
    }
  }
  
  if (!reading_dcode)
  {
    codegen_expr(expr,NULL,&topcode,FALSE);

    apasprintf(&topcode,
               "\tPUSH CXS CXP\t\t# evaluate body expression\n"
               "\tSETSYM T1 ENT_R_%s\n"
               "\tPUSH RAS T1\n"
               "\tPOP PS CXP\n"
               "\tPOP PS PC\n"
               "ENT_R_%s\n"
               "\tPOP CXS CXP\n",
               infilename,infilename);
  }

}


void dcode_decl ( struct expr * decl )
{
  fputc('F',dcode_file);
  fputs("_._",dcode_file);
  fputc('\0',dcode_file);
  fputc('A',dcode_file);
  dump_dcode(dcode_file,decl);
  fputc('A',dcode_file);
  dcode_term_nulls++;
}
