/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: cxform.c,v 1.1.1.1 1999/07/15 18:47:32 bardslea Exp $


	Implicit class transformations: var -> val, expr -> val ...

	11 Jun 1999, AMULET Group
*/

#include "symtab.h"
#include "parser_structs.h"
#include "codegen.h"
#include "error.h"
#include "decls.h"


void apply_xform_args ( struct list * list );


struct expr * apply_xform ( struct expr * expr )
{
  struct expr * e;
  struct expr * f;

  if (expr->sort==fcall_es)
    apply_xform_args(expr->u.fc.args);
  
  switch (expr->class_xform) {
  case cx_none:
    return expr;
    break;
  case cx_read:
    if (!readdecl)
      fatal_error(expr->line,"implicit read found before read declaration "
                  "processed");
    e=mk_expr(get_proto(readdecl)->u.fc.ident,mk_list(expr,NULL));
    e->class=mk_expr(get_proto(val_class)->u.fc.ident,
                     mk_list(arg(1,expr->class),NULL));
    e->class->decl=val_class;
    e->decl=readdecl;
    e->line=expr->line;
    return e;
    break;
  case cx_eval:
    if (!evaldecl)
      fatal_error(expr->line,"implicit eval found before eval declaration "
                  "processed");
    e=mk_expr(get_proto(evaldecl)->u.fc.ident,mk_list(expr,NULL));
    e->class=mk_expr(get_proto(val_class)->u.fc.ident,
                     mk_list(arg(1,expr->class),NULL));
    e->class->decl=val_class;
    e->decl=evaldecl;
    e->line=expr->line;
    return e;
    break;
  case cx_mkexpr:
    if (!mkexprdecl)
      fatal_error(expr->line,"implicit mkexpr found before mkexpr declaration "
                  "processed");
    e=mk_expr(get_proto(mkexprdecl)->u.fc.ident,mk_list(expr,NULL));
    e->class=mk_expr(get_proto(expr_class)->u.fc.ident,
                     mk_list(arg(1,expr->class),NULL));
    e->class->decl=expr_class;
    e->decl=mkexprdecl;
    e->line=expr->line;
    return e;
    break;
  case cx_mkexpr_read:
    if (!readdecl)
      fatal_error(expr->line,"implicit read found before read declaration "
                  "processed");
    if (!mkexprdecl)
      fatal_error(expr->line,"implicit mkexpr found before read declaration "
                  "processed");
    e=mk_expr(get_proto(readdecl)->u.fc.ident,mk_list(expr,NULL));
    e->class=mk_expr(get_proto(val_class)->u.fc.ident,
                     mk_list(arg(1,expr->class),NULL));
    e->class->decl=val_class;
    e->decl=readdecl;
    e->line=expr->line;
    f=mk_expr(get_proto(mkexprdecl)->u.fc.ident,mk_list(e,NULL));
    f->class=mk_expr(get_proto(expr_class)->u.fc.ident,
                     mk_list(arg(1,expr->class),NULL));
    f->class->decl=expr_class;
    f->decl=mkexprdecl;
    f->line=expr->line;
    return f;
    break;
  }
  return NULL;
}


void apply_xform_args ( struct list * list )
{
  if (list)
  {
    list->expr=apply_xform(list->expr);
    apply_xform_args(list->next);
  }
}



