/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: dcode.c,v 1.4 2001/10/23 11:35:15 bardslea Exp $

	Dcode parser

	11 Jun 1999, AMULET Group
*/

#include <config.h>
#include <stdio.h>
#include <string.h>

#include <alloc.h>
#include <error.h>
#include <simple.h>

#include "dcode.h"
#include "../bytecode/bytecode.h"
#include "infix.h"
#include "dump.h"

int dcode_magic=0x44434f00+BCODE_VERSION;
int dcode_wrong_magic=0x004f4344+(BCODE_VERSION<<24);


void dump_dcode_args ( FILE * f, struct list * args );
char * getzt ( FILE * f );
struct list * read_dcode_args ( FILE * f );


void dump_dcode ( FILE * f, struct expr * expr )
{
  switch(expr->sort)
  {
  case fcall_es:
    fputc('F',f);
    fputs(expr->u.fc.ident->name,f);
    fputc('\0',f);
    dump_dcode_args(f,expr->u.fc.args);
    break;
  case num_es:
    fputc('N',f);
    fwrite(&expr->u.n,sizeof(int),1,f);
    break;
  case str_es:
    fputc('S',f);
    fputs(expr->u.s,f);
    fputc('\0',f);
    break;
  case char_es:
    fputc('C',f);
    fputc(expr->u.c,f);
    break;
  }
}


void dump_dcode_args ( FILE * f, struct list * args )
{
  if (!args)
  {
    fputc('\0',f);
  } else {
    fputc('A',f);
    dump_dcode(f,args->expr);
    dump_dcode_args(f,args->next);
  }
}


char * getzt ( FILE * f )
{
  int buffsize = 2048;
  char *buff = palloc (buffsize);
  char * r;
  char * p;
  bool done;
  
  p=buff;
  done=FALSE;
  while (!done) {
	if (((int) (p - buff)) >= buffsize)
	{
		buffsize *= 2;
		buff = realloc (buff, buffsize);
	}
    (*p)=fgetc(f);
    done=((*p)=='\0');
    p++;   /* FIXME - 4096 limit not acceptable */
  }
  r=palloc(strlen(buff)+1);
  strcpy(r,buff);
  return r;
}


struct expr * read_dcode ( FILE * f )
{
  int c;
  struct symrec * ident;
  int n;
  char * s;
  struct expr * e;
  
  c=fgetc(f);
  if (c==EOF)
  {
    fatal_error(0,"dcode file format error; expecting F, N, S or C but got EOF");
    return NULL;
  }
  switch(c)
  {
  case 'F':
    s=getzt(f);
    ident=putsym(s);
    e=mk_expr(ident,read_dcode_args(f));
/*    dump_expr(e,0);
    fprintf(stderr,"\n---------------------\n"); */
    return e;
    break;
  case 'N':
    fread(&n,sizeof(int),1,f);
    return mk_expr_num(n);
    break;
  case 'S':
    return mk_expr_str(getzt(f));
    break;
  case 'C':
    return mk_expr_char(fgetc(f));
    break;
  default:
    fatal_error(0,"dcode file format error; expecting F, N, S or C but got %c (%d)\n",c?c:' ',(int)c);
    return NULL;
    break;
  }
}


struct list * read_dcode_args ( FILE * f )
{
  int c;
  struct expr * arg;

  c=fgetc(f);
  if (c==EOF)
  {
    fatal_error(0,"dcode file format error; expecting A or \\0 but got EOF");
    return NULL;
  }
  switch(c)
  {
  case 'A':
    arg=read_dcode(f);
    return mk_list(arg,read_dcode_args(f));
    break;
  case '\0':
    return NULL;
    break;
  default:
    fatal_error(0,"dcode file format error; expecting A or \\0 but got %c (%d)\n",c,(int)c);
    return NULL;
    break;
  }
}


void read_dcode_infix ( FILE * f )
{
  int c;
  char * ident;
  int priority;
  char associativity;

  while (TRUE)
  {
    c=fgetc(f);
    if (c==EOF)
    {
      fatal_error(0,"dcode file format error; expecting I or \\0 but got EOF");
      return;
    }
    switch(c)
    {
    case 'I':
      ident=getzt(f);
      fread(&priority,sizeof(int),1,f);
      associativity=fgetc(f);
      add_rule(ident,
               associativity=='L'?left_assoc:
               associativity=='R'?right_assoc:
               associativity=='B'?both_assoc:none_assoc,
               priority);
      break;
    case '\0':
      return;
      break;
    default:
      fatal_error(0,"dcode file format error; expecting I or \\0 but got %c (%d)\n",c,(int)c);
      return;
      break;
    }
  }
}
