/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: decls.c,v 1.2 2001/06/18 14:36:50 bardslea Exp $


	Declaration binding

	11 Jun 1999, AMULET Group
*/

#include <string.h>
#include <alloc.h>
#include <error.h>

#include "debug.h"
#include "parser_structs.h"
#include "scopes.h"
#include "dump.h"
#include "main.h"
#include "ccheck.h"
#include "records.h"
#include "codegen.h"


void bind_fcall ( struct expr * expr, struct expr * parent );
void bind_args ( struct list * args, struct expr * parent );
void do_decl ( struct expr * decl, struct expr * parent );
void do_decls ( struct list * list, struct expr * parent );
void do_impdecls ( struct expr * expr, struct expr * parent );
void do_impdecls_args ( struct list * list, struct expr * parent );
void do_use ( struct expr * decl, struct expr * parent );


struct symrec * decl2_ident;
struct symrec * decl3_ident;
struct symrec * dot_ident;
struct symrec * arrow_ident;
struct symrec * use_ident;
struct symrec * lib_ident;
struct symrec * field_ident;


void init_decls(void)
{
  decl2_ident=putsym("_:_");
  decl3_ident=putsym("_:_=_");
  dot_ident=putsym("_._");
  arrow_ident=putsym("_->_");
  use_ident=putsym("use");
  lib_ident=putsym("lib");
  field_ident=putsym("_::_");
}

struct expr * numlitdecl=NULL;
struct expr * strlitdecl=NULL;
struct expr * charlitdecl=NULL;


/* Utility Functions */

struct expr * get_proto ( struct expr * decl )
{
  return(arg(1,decl));
}

struct expr * get_class ( struct expr * decl )
{
  return(arg(2,decl));
}

struct expr * get_defn ( struct expr * decl )
{
  if (count_args(decl)==3)
    return(arg(3,decl));
  else
    return(NULL);
}



void bind_expr ( struct expr * expr, struct expr * parent )
{
      /* Process an expression.  The world starts here when the
         expression representing the whole program is passed.
         Literals are easy and are dealt with here; fcalls are sent to
         bind_fcall. */

      /* We may reach this point with the details already filled in if
         the parent expression is overloaded and this isn't the first
         attempt at processing the parameters.  In that case we can
         stop straight away as the declaration and class are already
         known. */

  if (expr->decl)
  {
/*    fprintf(stderr,"Not binding ");
    dump_expr(expr,-1);
    fprintf(stderr," because it already has declaration ");
    dump_expr(expr->decl,-1);
    fprintf(stderr," and class ");
    dump_expr(expr->class,-1);
    fprintf(stderr,"\n"); */
    return;
  }
  
      /* Fill in the parent field for all expressions.  NULL is passed
         for the root expression. */
  expr->parent=parent;

  switch(expr->sort)
    {
    case fcall_es:
      bind_fcall(expr,parent);
      break;
    case num_es:
      if (!numlitdecl)
        fatal_error(expr->line,"Numeric literal has not been declared");
      expr->decl=numlitdecl;
      ind_debug(debug_pass_2b,currentscopelevel,
                "computing return class for numeric literal: ");
      get_return_class(expr);
      debug(debug_pass_2b,"\n");
          /* "0" gets decl "0:val(int)" and class "val(int)" */
      break;
    case str_es:
      if (!strlitdecl)
        fatal_error(expr->line,"String literal has not been declared");
      expr->decl=strlitdecl;
      ind_debug(debug_pass_2b,currentscopelevel,
                "computing return class for string literal: ");
      get_return_class(expr);
      debug(debug_pass_2b,"\n");
      break;
    case char_es:
      if (!charlitdecl)
        fatal_error(expr->line,"Character literal has not been declared");
      expr->decl=charlitdecl;
      ind_debug(debug_pass_2b,currentscopelevel,
                "computing return class for character literal: ");
      get_return_class(expr);
      debug(debug_pass_2b,"\n");
      break;
    }
}

void restore_real_args ( struct list * args );

struct expr * restore_real ( struct expr * expr )
{
  if (expr)
  {
    if (expr->real)
    {
/*      fprintf(stderr,"restoring ");
      dump_expr(expr,-1);
      fprintf(stderr," to ");
      dump_expr(expr->real,-1);
      fprintf(stderr,"\n"); */
      if (expr->real->sort==fcall_es)
        restore_real_args(expr->real->u.fc.args);
      return expr->real;
    } else
    {
      if (expr->sort==fcall_es)
        restore_real_args(expr->u.fc.args);
      return expr;
    }
  } else
  {
    return NULL;
  }
}


void restore_real_args ( struct list * args )
{
  if (args)
  {
    args->expr=restore_real(args->expr);
    restore_real_args(args->next);
  }
}


void restore_real_arg_classes ( struct list * args )
{
  if (args)
  {
    args->expr->class=restore_real(args->expr->class);
    restore_real_arg_classes(args->next);
  }
}


void bind_fcall ( struct expr * expr, struct expr * parent )
{
      /* Process an fcall expression.  The expressions _._, _->_, _:_
         and _:_=_ have to be specially recognised. */

  struct expr * decl;
  
  if (expr->u.fc.ident==dot_ident)
    {
          /* _._ has the value of the right hand operand in the
             context of the declaration made by the left hand
             operand. */
      if (count_args(expr)!=2)
	{
	  fatal_error(expr->line,"Parameter count wrong for \"_._\"");
	}
      do_decl(arg(1,expr),expr);
      bind_expr(arg(2,expr),expr);
      expr->decl=NULL;
      expr->class=arg(2,expr)->class;
      return;
    }

  if (expr->u.fc.ident==arrow_ident)
  {
        /* _->_ is the field selection function */
    bind_arrow(expr,parent);
    return;
  }

      /* Check for an expression that is an implicit declaration.
         These occur only in the class expressions of parameter
         declarations, e.g. equal(a:val(T:type),b:val(T)) - T:type is
         the implicit declaration. */

      /* This code doesn't check that the context is suitable for an
         implicit declaration. */

      /* Actually processing the declaration was done when the
         declaration that this is part of was processed. */
  
  if (expr->u.fc.ident==decl2_ident)
    {
      if (!(arg(2,expr)->decl))
      {
        fatal_error(expr->line,"Implicit parameter not expected here");
      }
      expr->decl=expr;
      expr->class=arg(2,expr);
      get_proto(expr)->decl=expr;
      get_proto(expr)->class=arg(2,expr);
      ind_debug(debug_pass_2a,currentscopelevel,
                "%s (line %d) ==> itself\n",
                arg(1,expr)->u.fc.ident->name,expr->line);
      return;
    }

  if (expr->u.fc.ident==field_ident)
  {
    bind_expr(arg(2,expr),parent);
    expr->decl=arg(2,expr)->decl;
    expr->class=arg(2,expr)->class;
    return;
  }
  
  if (expr->u.fc.ident==decl3_ident)
  {
    fatal_error(expr->line,"expression expected but declaration found");
  }
      
      /* Look for matching declaration visible in the current scope. */
  decl=lookup1(expr->u.fc.ident);
  
  if (!decl)
    {
      fatal_error(expr->line,"identifier %s used but not declared",
		  expr->u.fc.ident->name);
    }

  ind_debug(debug_pass_2a,currentscopelevel,
            "%s (line %d)  ==>  %s (line %d)\n",
            expr->u.fc.ident->name, expr->line,
            get_proto(decl)->u.fc.ident->name, decl->line );

      /* If the matching declaration is a varargs declaration, replace
  this fcall with a new one that specifies the number of arguments */

  if (varargs_class && (get_class(decl)->decl==varargs_class))
  {
    char * buff;

    buff=palloc(strlen(expr->u.fc.ident->name)+5);
    sprintf(buff,"%s%d",expr->u.fc.ident->name,count_args(expr));
    expr->u.fc.ident=putsym(buff);
    free(buff);
    decl=lookup1(expr->u.fc.ident);
    if (!decl)
    {
      fatal_error(expr->line,"varargs identifier %s used but not declared",
		  expr->u.fc.ident->name);
    }
    ind_debug(debug_pass_2a,currentscopelevel,
              "%s (line %d)  ==>  %s (line %d) (after varargs)\n",
              expr->u.fc.ident->name, expr->line,
              get_proto(decl)->u.fc.ident->name, decl->line );
  }
  
  if (decl->overloaded)
  {
    bool done=FALSE;
    struct looknrec lnr;

    init_looknrec(&lnr);
    ind_debug(debug_pass_2a,currentscopelevel,
              "%s is overloaded\n",expr->u.fc.ident->name);
    while (!done) {
      decl=lookupn(&lnr,expr->u.fc.ident);
      if (!decl)
      {
        internal_error("didn't reach non-overloaded declaration");
      }
      if (get_class(decl)->decl==overloaded_class)
      {
	    struct expr *last_decl = NULL;
        nonfatal_error(expr->line,
                    "Can't find matching overloaded declaration for %s; "
					"tried (in reverse order):\n",
                    expr->u.fc.ident->name);
        init_looknrec(&lnr);
		do {
			decl=lookupn(&lnr,expr->u.fc.ident);
			if (! last_decl) last_decl = decl;
			if (decl && get_proto(decl)->u.fc.args)
			{
				fprintf (stderr, "    ");
				dump_expr (get_proto(decl),-1);
				fprintf (stderr, "\n");
			}
		} while (decl);
		fprintf (stderr, "\n");
        nonfatal_error(expr->line, "Here is the most recent error:\n");
		check_arg_classes(expr->u.fc.args,get_proto(last_decl)->u.fc.args,1,TRUE);
      }
      ind_debug(debug_pass_2a,currentscopelevel,
                "trying %s (line %d):\n",
                get_proto(decl)->u.fc.ident->name, decl->line);
      bind_args(expr->u.fc.args,expr);
      expr->decl=decl;
      if (count_args(expr)==count_args(get_proto(decl)))
      {
        ind_debug(debug_pass_2b,currentscopelevel,
                  "checking parameter classes for %s: ",
                  expr->u.fc.ident->name);
        if (check_arg_classes(expr->u.fc.args,get_proto(decl)->u.fc.args,1,FALSE))
        {
          debug(debug_pass_2b,"\n");
          done=TRUE;
        } else
        {
/*          fprintf(stderr,"Calling restore...\n"); */
          restore_real_arg_classes(expr->u.fc.args);
        }
      } else
      {
        ind_debug(debug_pass_2a,currentscopelevel,"Parameter count mismatch");
      }
    }

  } else {
  
        /* Process the parameters to the call.  This also gets the class
           of each of the parameters. */
    
    bind_args(expr->u.fc.args,expr);

    expr->decl=decl;

        /* Once a match has been found check that the expression and the
           declaration have matching parameters */
    
    if (count_args(expr)!=count_args(get_proto(decl)))
    {
      fatal_error(expr->line,
                  "Parameter counts mismatch for %s (declaration at line %d)",
                  expr->u.fc.ident->name, decl->line);
    }

    ind_debug(debug_pass_2b,currentscopelevel,
              "checking parameter classes for %s: ",
              expr->u.fc.ident->name);

        /* Check that the classes of the parameters match. */
  
    check_arg_classes(expr->u.fc.args,get_proto(decl)->u.fc.args,1,TRUE);

    debug(debug_pass_2b,"\n");

  }
  
  ind_debug(debug_pass_2b,currentscopelevel,
            "computing return class for %s: ",
            expr->u.fc.ident->name);

      /* Get the class of the expression as a whole. */
  
  get_return_class(expr);
  
  debug(debug_pass_2b,"\n");
}


void bind_args ( struct list * args, struct expr * parent )
{
  if (args)
  {
    if (args->expr->sort==fcall_es && args->expr->u.fc.ident==dot_ident)
    {
      openscope();
      bind_expr(args->expr,parent);
      closescope();
    } else {
      bind_expr(args->expr,parent);
    }
    bind_args(args->next,parent);
  }
}


void do_decl ( struct expr * decl, struct expr * parent )
{
  struct expr * proto;
  struct expr * class;
  struct expr * defn;

      /* Process a declaration. */
  
  decl->parent=parent;

  if (decl->sort!=fcall_es)
    {
      fatal_error(decl->line,"Expecting a declaration but got a literal");
    }

  if (decl->u.fc.ident==use_ident)
  {
    do_use(decl,parent);
    return;
  }
  
  if (decl->u.fc.ident!=decl2_ident && decl->u.fc.ident!=decl3_ident)
    {
      fatal_error(decl->line,"Expecting a declaration but got function %s",
		  decl->u.fc.ident->name);
    }
  
  if (!((decl->u.fc.ident==decl2_ident&&count_args(decl)==2)
	||(decl->u.fc.ident==decl3_ident&&count_args(decl)==3)))
    {
      fatal_error(decl->line,"Declaration has wrong number of arguments");
    }

  proto=get_proto(decl);
  class=get_class(decl);
  defn=get_defn(decl);

  if (proto->sort!=fcall_es && !compstdlib)
    {
      fatal_error(decl->line,
		  "Literal declared (not compiling standard library)");
    }

  if (proto->sort==fcall_es &&
      (proto->u.fc.ident==decl2_ident || proto->u.fc.ident==decl3_ident ||
       proto->u.fc.ident==dot_ident || proto->u.fc.ident==arrow_ident ))
  {
    fatal_error(decl->line,"Special identifier %s cannot be redeclared",
                proto->u.fc.ident->name);
  }
  
      /* Find and process any implicit declarations in the class of
         this declaration. */
  
  do_impdecls(class,decl);
  
  ind_debug(debug_pass_2a,currentscopelevel,"%s :\n",
            proto->sort==fcall_es?proto->u.fc.ident->name:
            proto->sort==num_es?"(all numeric literals)":
            proto->sort==str_es?"(all string literals)":
            proto->sort==char_es?"(all character literals)":"???");
  
      /* Record the declaration.  fcall expressions are stored in the
         scope tables, literals are stored in the xxxlitdecl
         variables. */
  
  switch(proto->sort)
    {
    case fcall_es:
    {
      struct expr * prev_decl;

      prev_decl=lookup1(proto->u.fc.ident);
      if (prev_decl && prev_decl->overloaded)
      {
        addscope(decl);
        decl->overloaded=prev_decl->overloaded+1;
      } else
      {
        check_existing_current(proto);
        addscope(decl);
      }
      break;
    }
    case num_es:
      if (numlitdecl)
        fatal_error(decl->line,"Numeric literal redeclared");
      numlitdecl=decl;
      break;
    case str_es:
      if (strlitdecl)
        fatal_error(decl->line,"String literal redeclared");
      strlitdecl=decl;
      break;
    case char_es:
      if (charlitdecl)
        fatal_error(decl->line,"Character literal redeclared");
      charlitdecl=decl;
      break;
    }

      /* Enter a new scope for the declaration.  Record the
         declarations of the parameters.  Process the class and
         definition, if present.  Check that the class of the
         definition, if present, matches the class. */

  openscope();
  if (proto->sort==fcall_es)
    do_decls(proto->u.fc.args,proto);
  bind_expr(class,decl);
  if (defn)
    {
      if (defn->sort==fcall_es && defn->u.fc.ident==dot_ident)
      {
        openscope();
        bind_expr(defn,decl);
        recordlibscope(decl);
        closescope();
      } else
      {
        bind_expr(defn,decl);
        recordlibscope(decl);
      }
      if (!class_match(&(defn->class),class))
      {
        if (!class_xform(defn,decl))
        {
          nonfatal_error(decl->line,"Definition class mismatch; expecting ");
          dump_expr(class,-1);
          fprintf(stderr,", got ");
          dump_expr(defn->class,-1);
          fprintf(stderr,"\n");
          exit(1);
        }
      }
    }
  closescope();

      /* See if this is an overloaded declaration */

  if (class->decl==overloaded_class)
  {
    decl->overloaded=1;
  }
}  

void do_decls ( struct list * list, struct expr * parent )
{
  if (list)
    {
      do_decl(list->expr,parent);
      do_decls(list->next,parent);
    }
}


void do_impdecls ( struct expr * expr, struct expr * parent )
{
  if (expr && expr->sort==fcall_es)
  {
    if (expr->u.fc.ident==decl2_ident || expr->u.fc.ident==decl3_ident)
      do_decl(expr,parent);
    do_impdecls_args(expr->u.fc.args,expr);
  }
}


void do_impdecls_args ( struct list * list, struct expr * parent )
{
  if (list)
  {
    do_impdecls(list->expr,parent);
    do_impdecls_args(list->next,parent);
  }
}


void do_use ( struct expr * decl, struct expr * parent )
{
  struct expr * libname;
  struct expr * libdecl;
  
  if (count_args(decl)!=1)
    fatal_error(decl->line,"use has wrong number of parameters");

  libname=arg(1,decl);
  bind_expr(libname,decl);
  libdecl=libname->decl;
  if (get_class(libdecl)->u.fc.ident!=lib_ident)
    fatal_error(decl->line,"%s is not a lib",libname->u.fc.ident->name);

  addlibscopeitems(libdecl);
}



