/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: dump.c,v 1.1.1.1 1999/07/15 18:47:33 bardslea Exp $


	Internal structure printing

	11 Jun 1999, AMULET Group
*/

#include "dump.h"
#include <stdio.h>


const int inc=2;

void newline ( int indent, int line )
{
  int i;
  if (indent<0)
    return;
  fprintf(stderr,"\n%4d:",line);
  for(i=0;i<indent;i++)
    fprintf(stderr," ");
}


void dump_list_body ( struct list * list, int indent )
{
  if (list)
    {
      dump_expr(list->expr,indent);
      if (list->next)
	{
	  fprintf(stderr,",");
	}
      dump_list_body(list->next,indent);
    }
}

void dump_list ( struct list * list, int indent )
{
  fprintf(stderr,"(");
  dump_list_body(list,indent<0?indent:(indent+inc));
  fprintf(stderr,")");
}

void dump_expr ( struct expr * expr, int indent )
{
  if (!expr)
  {
    fprintf(stderr,"(NULL)");
    return;
  }
  
  switch(expr->sort)
    {
    case fcall_es:
      newline(indent,expr->line);
      fprintf(stderr,expr->u.fc.ident->name);
      dump_list(expr->u.fc.args,indent);
      break;
    case num_es:
      newline(indent,expr->line);
      fprintf(stderr,"%d",expr->u.n);
      break;
    case str_es:
      newline(indent,expr->line);
      fprintf(stderr,"\"%s\"",expr->u.s);
      break;
    case char_es:
      newline(indent,expr->line);
      fprintf(stderr,"\'%c\'",expr->u.c);
      break;
    }
}

void edc_dump_list ( struct list * list )
{
  if (list)
  {
    edc_dump_expr(list->expr);
    edc_dump_list(list->next);
  }
}

void edc_dump_expr ( struct expr * expr )
{
  fprintf(stderr,"------------------------------------------\n");
  fprintf(stderr,"Expr:%3d:  ",expr->line);
  dump_expr(expr,-1);
  fprintf(stderr,"\nDecl:%3d:  ",expr->decl?expr->decl->line:0);
  dump_expr(expr->decl,-1);
  fprintf(stderr,"\nClass:     ");
  dump_expr(expr->class,-1);
  fprintf(stderr,"\n");
  if (expr->sort==fcall_es)
    edc_dump_list(expr->u.fc.args);
}

