/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: lexer.l,v 1.1.1.1 1999/07/15 18:47:35 bardslea Exp $


	Lard lexer description

	11 Jun 1999, AMULET Group
*/

%{
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <error.h>
#include <strtoint.h>

#include "symtab.h"
#include "parser.h"
#include "infix.h"
#include "constants.h"
#include "../config.h"

/* Read input file.  We do this here in order to have input_buffer handy
   for error messages. */

int line_increment = 1;

#undef YY_INPUT
#define YY_INPUT(buf,result,max_size)  	       	  \
  {						  \
    char * fg_res;				  \
						  \
    fg_res = fgets(input_buffer,                  \
        max_size<MAX_INPUT_LINE_LENGTH?max_size:MAX_INPUT_LINE_LENGTH,yyin); \
    strcpy(buf,input_buffer);			  \
    if (fg_res==NULL)				  \
      result = YY_NULL;				  \
    else					  \
      result = strlen(input_buffer);		  \
  }

int magic_brkt=0;

%}


NUMBER	[0-9]+
SYM	\!|\%|\^|\&|\*|\-|\=|\+|\;|\:|\@|\~|\<|\>|\.|\/|\?|\$|\[|\]|\:\=|\-\>|\!\=|\<\=|\>\=|\|\||\||\\|\.\.|\{|\}|\&\&|\:\:|\>\>|\<\<|\=\>
WORD	[a-zA-Z0-9_]*

NONSPACE        [^ \t\n]
SPACE           [ \t]
QUOTEDCHAR	[^ \t\n`]

%option stack

%x comment
%x string
%x chr

%s rndbrkt
%s sqrbrkt
%s curlbrkt

%%

                /* Numeric literals in various number bases */

2#[01]+         { yylval.number=lard_strtoul(&yytext[2],NULL,2); return(NUM); }
8#[0-7]+        { yylval.number=lard_strtoul(&yytext[2],NULL,8); return(NUM); }
10#[0-9]+       { yylval.number=lard_strtoul(&yytext[3],NULL,10); return(NUM); }
16#[0-9a-fA-F]+ { yylval.number=lard_strtoul(&yytext[3],NULL,16); return(NUM); }
{NUMBER}	{ yylval.number=lard_strtoul(yytext,NULL,10); return(NUM); }

                /* String and character literals */

<INITIAL,rndbrkt,sqrbrkt,curlbrkt>\"     { yy_push_state(string);
                  return (DQUOTE); }

<string>\"      { yy_pop_state();
                  return (DQUOTE); }
                  
<INITIAL,rndbrkt,sqrbrkt,curlbrkt>\'     { yy_push_state(chr);
                  return (SQUOTE); }

<chr>\'         { yy_pop_state();
                  return (SQUOTE); }
                  
<string,chr>\\[0-7]{1,3} { yylval.character = strtol(&yytext[1],NULL,8);
                           if (yylval.character==0)
                             fatal_error(line_num,"\\0 is not allowed");
                           return (CHAR); }
<string,chr>\\n       { yylval.character = '\n'; return (CHAR); }
<string,chr>\\t       { yylval.character = '\t'; return (CHAR); }
<string,chr>\\r       { yylval.character = '\r'; return (CHAR); }
<string,chr>\\b       { yylval.character = '\b'; return (CHAR); }
<string,chr>\\f       { yylval.character = '\f'; return (CHAR); }
<string,chr>\\.       { yylval.character = yytext[1]; return (CHAR); }
<string,chr>\\\n      { line_num += line_increment; yylval.character = '\n'; return (CHAR); }

<string,chr>\n        { fatal_error(line_num,
                          "Unterminated string/character at end of line"); }

<string><<EOF>> |
<chr><<EOF>>           { fatal_error(line_num,
                          "Unterminated string/character at end of file"); }

<string,chr>[^\\\n]   { yylval.character = yytext[0]; return (CHAR); }
                  
                /* Comments */

<INITIAL,rndbrkt,sqrbrkt,curlbrkt,comment>\/\* { yy_push_state(comment); }

^"#".*\n		{
	char *firstToken = strtok (yytext + 1, " \t\n");

	if (isdigit (*firstToken)) /* line directive */
	{
		char *file = strtok (NULL, " \t\n");

		if (file && *file == '"')
		{
			char *endQuote = strchr (file + 1, '"');
			if (endQuote) { *endQuote = '\0'; infilename = strdup (file + 1); }
		}
		line_num = atoi (firstToken);
	} else if (strcmp (firstToken, "pragma") == 0)
	{
		char *arg = strtok (NULL, " \t\n");

		if (strcmp (arg, "noautolines") == 0) line_increment = 0;
	}
}

<comment>\*+\/          { yy_pop_state(); }

<comment>[^*/\n]*

<comment>"*"+[^*/\n]*

<comment>"/"+[^*/\n]*

<comment>\n             { line_num += line_increment; }

<INITIAL>\*\/           { fatal_error(line_num,"Unexpected close comment"); }

<comment><<EOF>>        { fatal_error(line_num,"Unterminated comment"); }

                /* Some primitive tokens */

\(		{ if (magic_brkt==0) yy_push_state(rndbrkt);
		  magic_brkt=0;
                  return (LPAREN); }

\[		{ struct symrec *p;
		  yy_push_state(sqrbrkt);
		  p=putsym(yytext);
                  p->sort=iskeyword;
		  yylval.keyword=p;
                  unput('(');
		  magic_brkt=1;
                  return (KEYWORD); }

\{		{ struct symrec *p;
		  yy_push_state(curlbrkt);
		  p=putsym(yytext);
                  p->sort=iskeyword;
		  yylval.keyword=p;
		  unput('(');
		  magic_brkt=1;
                  return (KEYWORD); }

\)		{ if (YY_START!=rndbrkt)
		    fatal_error(line_num,"Unmatched )");
		  yy_pop_state();
		  return (RPAREN); }

\]		{ struct symrec *p;
                  if (magic_brkt==0)
                  {
		    if (YY_START!=sqrbrkt)
		      fatal_error(line_num,"Unmatched ]");
                    unput(']');
		    magic_brkt=1;
		    return (RPAREN);
		  } else {
		    yy_pop_state();
		    p=putsym(yytext);
                    p->sort=iskeyword;
		    yylval.keyword=p;
		    magic_brkt=0;
                    return (KEYWORD);
                  }
                }

\}		{ struct symrec *p;
		  if (magic_brkt==0)
		  {
		    if (YY_START!=curlbrkt)
		      fatal_error(line_num,"Unmatched }");
		    unput('}');
		    magic_brkt=1;
                    return (RPAREN);
                  } else {
		    yy_pop_state();
		    p=putsym(yytext);
                    p->sort=iskeyword;
		    yylval.keyword=p;
                    magic_brkt=0;
                    return (KEYWORD);
		  }
		}

<rndbrkt>\,	{ return (COMMA); }

<sqrbrkt,curlbrkt>\,	{ struct symrec *p;
		  if (magic_brkt==0)
		  {
		    magic_brkt=1;
                    unput(',');
		    return (RPAREN);
                  } else {
  		    p=putsym(yytext);
                    p->sort=iskeyword;
		    yylval.keyword=p;
                    unput('(');
		    return (KEYWORD);
		  }
                }

                /* Horrible stuff for infix declarations */

{NONSPACE}+{SPACE}*:{SPACE}*infix{SPACE}*\({SPACE}*{NUMBER}{SPACE}*,{SPACE}*left{SPACE}*\){SPACE}*\.                { char p[1024];
                  char * q1;
                  int pri;
                  strcpy(p,yytext);
                  sscanf(strstr(p,"infix"),"infix ( %d",&pri);
                  *strstr(p,"infix")='\0';
                  *strrchr(p,':')='\0';
                  q1=strchr(p,' '); if (q1) *q1='\0';
                  add_rule(p,left_assoc,pri); }

{NONSPACE}+{SPACE}*:{SPACE}*infix{SPACE}*\({SPACE}*{NUMBER}{SPACE}*,{SPACE}*right{SPACE}*\){SPACE}*\.                { char p[1024];
                  char * q1;
                  int pri;
                  strcpy(p,yytext);
                  sscanf(strstr(p,"infix"),"infix ( %d",&pri);
                  *strstr(p,"infix")='\0';
                  *strrchr(p,':')='\0';
                  q1=strchr(p,' '); if (q1) *q1='\0';
                  add_rule(p,right_assoc,pri); }

{NONSPACE}+{SPACE}*:{SPACE}*infix{SPACE}*\({SPACE}*{NUMBER}{SPACE}*,{SPACE}*none{SPACE}*\){SPACE}*\.                { char p[1024];
                  char * q1;
                  int pri;
                  strcpy(p,yytext);
                  sscanf(strstr(p,"infix"),"infix ( %d",&pri);
                  *strstr(p,"infix")='\0';
                  *strrchr(p,':')='\0';
                  q1=strchr(p,' '); if (q1) *q1='\0';
                  add_rule(p,none_assoc,pri); }

{NONSPACE}+{SPACE}*:{SPACE}*infix{SPACE}*\({SPACE}*{NUMBER}{SPACE}*,{SPACE}*both{SPACE}*\){SPACE}*\.                { char p[1024];
                  char * q1;
                  int pri;
                  strcpy(p,yytext);
                  sscanf(strstr(p,"infix"),"infix ( %d",&pri);
                  *strstr(p,"infix")='\0';
                  *strrchr(p,':')='\0';
                  q1=strchr(p,' '); if (q1) *q1='\0';
                  add_rule(p,both_assoc,pri); }

                /* Identifiers and keywords */

{WORD}		{ struct symrec *p;
                  p=putsym(yytext);
		  if (p->sort==isident) { yylval.ident=p; return (IDENT); }
                  else { yylval.keyword=p; return (KEYWORD); } }
                
                /* Identifiers in back-quotes */

`{QUOTEDCHAR}*`	{ char b[1024];
                  struct symrec *p;
                  strcpy(b,yytext+1);
                  b[strlen(b)-1]='\0';
                  p=putsym(b);
		  p->sort=isident;
                  yylval.ident=p;
                  return (IDENT); }

                /* Symbols (which are just keywords) */

{SYM}		{ struct symrec *p;
                  p=putsym(yytext);
                  p->sort=iskeyword;
		  yylval.keyword=p;
                  return (KEYWORD); }

                /* End of file */
                
<<EOF>>         { return (END_OF_FILE); }

                /* Whitespace */

[ \t]+

\n              { line_num += line_increment; }

                /* Anything else is junk */

.		{ if (isprint(yytext[0]))
                    fatal_error(line_num,
			        "Illegal character '%c' in input",
			        yytext[0]);
                  else
                    fatal_error(line_num,
			        "Illegal character ASCII %d in input",
			        (int)yytext[0]);
		}

%%

