/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: main.c,v 1.3 2001/06/18 14:36:50 bardslea Exp $


	The Lard compiler: lcd

	11 Jun 1999, AMULET Group
*/

/* main.c */
/* compiler main program */

#include <string.h>
#include <stdlib.h>

#include <error.h>
#include <simple.h>

#include "constants.h"
#include "debug.h"
#include "symtab.h"
#include "parse.h"
#include "passes.h"
#include "output.h"
#include "scopes.h"
#include "decls.h"
#include "codegen.h"
#include "dcode.h"
#include "dump.h"

bool compstdlib;
char * rcs_version = "$Revision: 1.3 $";

bool reading_dcode;

char * infilename;
int line_num;
char line_input_buffer[MAX_INPUT_LINE_LENGTH];
char * input_buffer;

void parse_arguments_1(int argc, char *argv[])
{
  int a;

  outf=stdout;

  for(a=1;a<argc;a++)
    {
      if (strncmp(argv[a],"-d",2)==0)
	{
              /* nothing */
	}
      else if (strcmp(argv[a],"-v")==0)
	{
	  verbose=TRUE;
	}
      else if (strcmp(argv[a],"-p1")==0)
	{
	  stop_after_1=TRUE;
	}
      else if (strcmp(argv[a],"-p2")==0)
	{
	  stop_after_2=TRUE;
	}
      else if (strcmp(argv[a],"-g")==0)
      {
            /* nothing */
      }
      else if (strcmp(argv[a],"-p")==0)
      {
            /* nothing */
      }
      else if (strcmp(argv[a],"-od")==0)
      {
        a++;
      }
      else if (strcmp(argv[a],"-std")==0)
	{
              /* nothing */
	}
      else if (strcmp(argv[a],"-o")==0)
	{
          if (outf!=stdout)
          {
            usage_error("Multiple output files specified");
          }
	  outf=fopen(argv[a+1],"w");
	  if (!outf)
          {
            usage_error("Can't open output file %s",argv[a+1]);
            exit(1);
          }
	  a++;
	}
      else if (strcmp(argv[a],"-help")==0 || strcmp(argv[a],"--help")==0
               || strcmp(argv[a],"-?")==0)
      {
        char version[10];
        sscanf(rcs_version+1,"Revision: %[^ ]",version);
        fprintf(stderr,
          "LARD Compiler version %s\n"
          "usage: %s [flags] [source files]\n"
          "  -o file   Output to file\n"
          "  -v        Verbose\n"
          "  -std      File is part of standard library\n"
          "  -dX       Various debugging options\n"
          "  -pN       Stop after pass N\n"
          "  -g        Add debugging code in the following file\n"
          "  -p        Add profiling code in the following file\n"
          "  -od file  Output dcode to file\n"
          "  -d file   Read from dcode file\n"
                ,version,argv[0]);
        exit(1);
      }
      else
	{
              /* nothing */
	}
    }
}


void parse_arguments_2(int argc, char *argv[])
{
  int a;

  compstdlib=FALSE;
  reading_dcode=FALSE;
  add_plump=FALSE;
  add_prof=FALSE;
  
  for(a=1;a<argc;a++)
    {
      if (strcmp(argv[a],"-d1a")==0)
	{
	  debug_pass_1a=TRUE;
	}
      else if (strcmp(argv[a],"-d1b")==0)
	{
	  debug_pass_1b=TRUE;
	}
      else if (strcmp(argv[a],"-d1c")==0)
	{
	  debug_pass_1c=TRUE;
	}
      else if (strcmp(argv[a],"-d1d")==0)
	{
	  debug_pass_1d=TRUE;
	}
      else if (strcmp(argv[a],"-d2a")==0)
	{
	  debug_pass_2a=TRUE;
	}
      else if (strcmp(argv[a],"-d2b")==0)
	{
	  debug_pass_2b=TRUE;
	}
      else if (strcmp(argv[a],"-d2c")==0)
	{
	  debug_pass_2c=TRUE;
	}
      else if (strcmp(argv[a],"-d2d")==0)
	{
	  debug_pass_2d=TRUE;
	}
      else if (strcmp(argv[a],"-d2e")==0)
	{
	  debug_pass_2e=TRUE;
	}
      else if (strcmp(argv[a],"-d3")==0)
	{
	  debug_pass_3=TRUE;
	}
      else if (strcmp(argv[a],"-dall")==0)
	{
	  debug_pass_1a=TRUE;
	  debug_pass_1b=TRUE;
	  debug_pass_1c=TRUE;
          debug_pass_1d=TRUE;
	  debug_pass_2a=TRUE;
	  debug_pass_2b=TRUE;
          debug_pass_2c=TRUE;
          debug_pass_2d=TRUE;
          debug_pass_2e=TRUE;
	  debug_pass_3=TRUE;
	}
      else if (strcmp(argv[a],"-v")==0)
	{
              /* nothing */
	}
      else if (strcmp(argv[a],"-p1")==0)
	{
	      /* nothing */
	}
      else if (strcmp(argv[a],"-p2")==0)
	{
	      /* nothing */
	}
      else if (strcmp(argv[a],"-std")==0)
	{
	  compstdlib=TRUE;
	}
      else if (strcmp(argv[a],"-g")==0)
      {
            add_plump=TRUE;
      }
      else if (strcmp(argv[a],"-p")==0)
      {
            add_prof=TRUE;
      }
      else if (strcmp(argv[a],"-od")==0)
      {
        gen_dcode=TRUE;
        dcode_term_nulls=0;
        a++;
        dcode_file=fopen(argv[a],"w");
        if (!dcode_file)
        {
          usage_error("Can't open dcode output file %s",argv[a]);
          exit(1);
        }
        fwrite(&dcode_magic,sizeof(int),1,dcode_file);
      }
      else if (strcmp(argv[a],"-d")==0)
      {
        FILE * f;
        int magic;
        struct expr * root_expr;
        int c;
        
        a++;
        f=fopen(argv[a],"r");
        if (!f)
        {
          usage_error("Can't open dcode input file %s",argv[a]);
          exit(1);
        }

        debug(verbose,"%s:\n",argv[a]);
        infilename=argv[a];
        reading_dcode=TRUE;
        while ((c=fgetc(f))!=EOF)
        {
          ungetc(c,f);
          fread(&magic,sizeof(int),1,f);
          if (magic==dcode_wrong_magic)
          {
            usage_error("%s has the wrong byte sex\n",argv[a]);
          }
          if (magic!=dcode_magic)
          {
            usage_error("%s is not a dcode file\n",argv[a]);
            exit(1);
          }
          infilenum++;
          read_dcode_infix(f);
          root_expr=read_dcode(f);
          pass_2(root_expr);
        }
        reading_dcode=FALSE;
        fclose(f);
      }
      else if (strcmp(argv[a],"-o")==0)
	{
	  a++;
	}
      else if (strcmp(argv[a],"-help")==0 || strcmp(argv[a],"--help")==0
               || strcmp(argv[a],"-?")==0)
      {
            /* nothing */
      }
      else
	{
	  infilename=argv[a];
          infilenum++;
          line_num=1;
	  yyin=fopen(argv[a],"r");
          if (!yyin)
	    {
	      usage_error("Can't open input file %s",argv[a]);
	      exit(1);
	    }
          pass_1();
	  debug_pass_1a=FALSE;
	  debug_pass_1b=FALSE;
	  debug_pass_1c=FALSE;
          debug_pass_1d=FALSE;
	  debug_pass_2a=FALSE;
	  debug_pass_2b=FALSE;
          debug_pass_2c=FALSE;
          debug_pass_2d=FALSE;
          debug_pass_2e=FALSE;
	  debug_pass_3=FALSE;
	  compstdlib=FALSE;
          add_plump=FALSE;
          gen_dcode=FALSE;
          fclose(yyin);
	}
    }
}


int main ( int argc, char *argv[] )
{
  input_buffer = line_input_buffer;
  parse_arguments_1(argc,argv);
  init_decls();
  init_codegen();
  openscope();
  parse_arguments_2(argc,argv);
  if (!stop_after_1 && !stop_after_2)
  {
    output_global_end();
  }
  closescope();
  return(0);
}
