/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: parser.y,v 1.3 2002/10/24 12:24:30 janinl Exp $


	Lard first pass parser

	11 Jun 1999, AMULET Group
*/

%{

/* C DECLARATIONS */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <alloc.h>
#include <apasprintf.h>
#include <error.h>

#include "symtab.h"
#include "parser_structs.h"
#include "passes.h"
#include "infix.h"
#include "output.h"

#define YYERROR_VERBOSE
#define YYMAXDEPTH 100000

extern int yylex (void);
%}


/* BISON DECLARATIONS */

/* The possible types of the semantic value of a token */

%union 
{
  int number;                /* integers */
  char character;            /* characters */
  struct symrec *ident;      /* identifiers */
  struct symrec *keyword;    /* keywords */

  struct expr *		texpr;
  struct infix *	tinfix;
  struct list *		tlist;
  char *                tstr;
}

/* Types for the rules */

%type <texpr>		expr not_infix_expr prefix_expr infix_expr
%type <texpr>		literal
%type <tinfix>		infix
%type <tlist>		list fulllist
%type <tstr>            string

/* The tokens.  First those that have a semantic value : */

%token <number>       NUM
%token <character>    CHAR
%token <ident>        IDENT
%token <keyword>      KEYWORD


/* Now those that don't */

/* Symbols */

%token LPAREN RPAREN COMMA DQUOTE SQUOTE END_OF_FILE

%%


/* GRAMMAR RULES */

input:		  expr END_OF_FILE		{
			if (! stop_after_1 && ! stop_after_2) output_global_start ();
			pass_2($1); YYACCEPT;
		} ;

expr:		  infix_expr		        { $$=$1; }
		| not_infix_expr		{ $$=$1; }
		;

infix_expr:	  KEYWORD infix
			{ struct infix * i = mk_infix_kw($1,$2);
			  $$=infix_to_expr(i); }
		| not_infix_expr KEYWORD infix
			{ struct infix * i =
				mk_infix_expr($1,mk_infix_kw($2,$3));
			  $$=infix_to_expr(i); }
		;

infix:	          /* nothing */			{ $$=NULL; }
		| not_infix_expr		{ $$=mk_infix_expr($1,NULL); }
		| not_infix_expr KEYWORD infix
				{ $$=mk_infix_expr($1,mk_infix_kw($2,$3)); }
		| KEYWORD infix		{ $$=mk_infix_kw($1,$2); }
		;

not_infix_expr:
		  prefix_expr			{ $$=$1; }
		| LPAREN expr RPAREN		{ $$=$2; }
		| literal			{ $$=$1; }
		;

prefix_expr:	  IDENT list			{ $$=mk_expr($1,$2); }
		;

list:		  /* nothing */			{ $$=NULL; }
		| LPAREN RPAREN			{ $$=NULL; }
		| LPAREN fulllist RPAREN	{ $$=$2; }
		;

fulllist:	  expr				{ $$=mk_list($1,NULL); }
		| expr COMMA fulllist		{ $$=mk_list($1,$3); }
		;

literal:	  NUM				{ $$=mk_expr_num($1); }
		| DQUOTE string DQUOTE		{ $$=mk_expr_str($2); }
		| SQUOTE CHAR SQUOTE		{ $$=mk_expr_char($2); }
		;

string:           /* nothing */                 { char * p=palloc(1);
                                                  *p='\0'; $$=p; }
                | string CHAR                   { char * p=$1;
                                                  apasprintf(&p,"%c",$2);
                                                  $$=p; }
                ;

%%





