/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: parser_structs.c,v 1.1.1.1 1999/07/15 18:47:34 bardslea Exp $


	Parsed Lard structure representation types

	11 Jun 1999, AMULET Group
*/

#include <alloc.h>
#include <apasprintf.h>
#include <error.h>
#include <simple.h>

#include "parser_structs.h"
#include "parse.h"


int idcode=1000;


struct expr * mk_expr ( struct symrec * ident, struct list * args )
{
  struct expr * p;
  
  p=palloc(sizeof(struct expr));
  p->sort=fcall_es;
  p->u.fc.ident=ident;
  p->u.fc.args=args;
  p->line=line_num;
  p->idcode=idcode++;
  p->code=NULL;
  p->isparam=FALSE;
  p->decl=NULL;
  p->class=NULL;
  p->real=NULL;
  p->class_xform=cx_none;
  p->inst=NULL;
  p->overloaded=0;
  p->enum_ord=0;
  p->external=FALSE;
  return(p);
}

     
struct infix * mk_infix_kw ( struct symrec * keyword, struct infix * infix )
{
  struct infix * p;
  
  p=palloc(sizeof(struct infix));
  p->sort=kw_is;
  p->u.keyword=keyword;
  p->next=infix;
  p->line=line_num;
  return(p);
}


struct infix * mk_infix_expr ( struct expr * expr, struct infix * infix )
{
  struct infix * p;
  
  p=palloc(sizeof(struct infix));
  p->sort=expr_is;
  p->u.expr=expr;
  p->next=infix;
  p->line=line_num;
  return(p);
}


struct expr * mk_expr_num ( int n )
{
  struct expr * p;
  
  p=palloc(sizeof(struct expr));
  p->sort=num_es;
  p->u.n=n;
  p->line=line_num;
  p->idcode=idcode++;
  p->code=NULL;
  p->decl=NULL;
  p->class=NULL;
  p->real=NULL;
  p->class_xform=cx_none;
  return(p);
}

struct expr * mk_expr_str ( char * s )
{
  struct expr * p;
  
  p=palloc(sizeof(struct expr));
  p->sort=str_es;
  p->u.s=s;
  p->line=line_num;
  p->idcode=idcode++;
  p->code=NULL;
  p->decl=NULL;
  p->class=NULL;
  p->real=NULL;
  p->class_xform=cx_none;
  return(p);
}


struct expr * mk_expr_char ( char c )
{
  struct expr * p;
  
  p=palloc(sizeof(struct expr));
  p->sort=char_es;
  p->u.c=c;
  p->line=line_num;
  p->idcode=idcode++;
  p->code=NULL;
  p->decl=NULL;
  p->class=NULL;
  p->real=NULL;
  p->class_xform=cx_none;
  return(p);
}


struct list * mk_list ( struct expr * expr, struct list * list )
{
  struct list * p;
  
  p=palloc(sizeof(struct list));
  p->expr=expr;
  p->next=list;
  p->line=line_num;
  return(p);
}


struct expr * larg ( int n, struct list * l )
{
  if (!l)
    {
      internal_error("Not enough ARGs\n");
    }
  if (n==1)
    return(l->expr);
  else
    return(larg(n-1,l->next));
}


struct expr * arg ( int n, struct expr * e )
{
  if (e->sort!=fcall_es)
    {
      internal_error("ARG used on a value\n");
    }
  
  return(larg(n,e->u.fc.args));
}


int lcount ( struct list * l )
{
  if (!l)
    return(0);
  else
    return(1+lcount(l->next));
}


int count_args ( struct expr * e )
{
  if (e->sort!=fcall_es)
    {
      internal_error("COUNT_ARGS used on a value\n");
    }
  
  return(lcount(e->u.fc.args));
}


