/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: parser_structs.h,v 1.1.1.1 1999/07/15 18:47:34 bardslea Exp $

    
	Parsed Lard structure representation types

	11 Jun 1999, AMULET Group
*/

#ifndef __PARSER_STRUCTS__
#define __PARSER_STRUCTS__

#include "symtab.h"
#include "simple.h"

extern int idcode;

enum exprsort { fcall_es, num_es, str_es, char_es };

enum class_x { cx_none, cx_read, cx_eval, cx_mkexpr, cx_mkexpr_read };

struct expr
{
      /* The following fields are filled in during pass 1 as the
         structure is built using mk_expr and related functions */

  enum exprsort sort;          /* what sort of expression */
  union {
    struct {                   /* this member if sort==fcall_es */
      struct symrec * ident;   /* fcall identifier */
      struct list * args;      /* list of parameters */
    } fc;
    int n;                     /* this member if sort==num_es */
    char *s;                   /* this member if sort==str_es */
    char c;                    /* this member if sort==char_es */
  } u;
  int idcode;                  /* a unique code assigned during
                                  parsing and used in code generation */
  int line;                    /* line number on which the expression
                                  occurs for error messages etc. */

      /* The following fields are filled in during pass 2 as the
         semantics are established */
  
  struct expr * parent;        /* pointer to the expression for which
                                  this is a sub-expression */
  struct expr * decl;          /* pointer to the declaration
                                  coresponding to this expression */
  struct expr * class;         /* pointer to an expression giving the
                                  class of this expression */
  struct expr * real;          /* copy of class that isn't changed by expand
                                  and is restored if the declaration doesn't
                                  match so the next overloaded def can be
                                  tried */
  struct expr * inst;          /* used during type checking to point
                                  from a formal parameter to the
                                  coresponding actual parameter */
  enum class_x class_xform;    /* indicates which class transformation
                                  should be applied to this expression */
  struct expr * libdecls;      /* for library declarations stores a
                                  pointer to the scope records
                                  recrding the declarations */
  int overloaded;              /* non-zero means this declaration is
                                  overloaded; incremented for each
                                  declaration */
  
      /* The following fields are filled in during pass 3 as code is
         generated */

  int nestlevel;               /* level of nesting at which the
                                  expression or declaration is found */
  int num;                     /* for local declarations, indicates
                                  this declaration's position */
  int enum_ord;                /* if this is an enumerated type
                                  declaration, this holds the ordinal
                                  for the next enumeration label
                                  encountered.  If this is an
                                  enumeration label, it holds its
                                  ordinal. */
  bool isparam;                /* set for parameter declarations */
  bool external;               /* set if the declaration is external */
  char * code;                 /* pointer to space for generated code,
                                  printed into using apasprintf() */
};

struct list
{
  struct expr * expr;
  struct list * next;

  int line;
};

enum infixsort { kw_is, expr_is };

struct infix
{
  enum infixsort sort;
  union {
    struct expr * expr;
    struct symrec * keyword;
  } u;
  struct infix * next;

  int line;
};


/* Functions to construct the internal representation of the file,
   called from the bison generated parser. */

struct expr * mk_expr ( struct symrec * ident, struct list * args );
     
struct infix * mk_infix_kw ( struct symrec * keyword, struct infix * infix );

struct infix * mk_infix_expr ( struct expr * expr, struct infix * infix );

struct expr * mk_expr_num ( int n );

struct expr * mk_expr_str ( char * s );

struct expr * mk_expr_char ( char c );

struct list * mk_list ( struct expr * expr, struct list * list );

/* Convenience functions to count and get arguments of an expression */

struct expr * larg ( int n, struct list * l );

struct expr * arg ( int n, struct expr * e );

int lcount ( struct list * l );

int count_args ( struct expr * e );


#endif
