/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: passes.c,v 1.1.1.1 1999/07/15 18:47:34 bardslea Exp $


	Sequencing compiler passes

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>

#include <apasprintf.h>
#include <error.h>
#include <simple.h>

#include "debug.h"
#include "parser_structs.h"
#include "dump.h"
#include "symtab.h"
#include "parse.h"
#include "decls.h"
#include "scopes.h"
#include "codegen.h"
#include "output.h"
#include "passes.h"
#include "cxform.h"
#include "linenos.h"
#include "main.h"


bool stop_after_1=FALSE;
bool stop_after_2=FALSE;


void pass_1(void)
{
  yydebug=debug_pass_1a;
  debug(verbose,"%s:\n",infilename);
  debug(verbose,"(1) Parsing...\n");
  debug(debug_pass_1a,"BISON debug output switched on\n");
  yyparse();
}

void pass_2(struct expr * expr)
{
  struct expr * cx_expr;
  
  fix_linenos(expr);
  if (debug_pass_1b)
    {
      fprintf(stderr,"Dumping symbol table\n");
      dump_symtable(stderr);
    }
  if (debug_pass_1c)
    {
      fprintf(stderr,"Dumping parsed program at end of pass 1\n");
      dump_expr(expr,0);
      fprintf(stderr,"\n");
    }
  if (stop_after_1)
  {
    fprintf(stderr,"End of pass 1, Stopping.\n");
    return;
  }
  debug(verbose,
	    "(2) Recording declarations, binding expressions and "
	    "checking classes...\n");
  
  bind_expr(expr,NULL);
  cx_expr=apply_xform(expr);
  if (debug_pass_2c)
  {
    fprintf(stderr,"Dumping typed and bound program at end of pass 2\n");
    edc_dump_expr(cx_expr);
  }
  if (stop_after_2)
  {
    fprintf(stderr,"End of pass 2, Stopping.\n");
    return;
  }

  if (gen_dcode)
    fputc('\0',dcode_file);
  
  pass_3(cx_expr);

  if (reading_dcode)
    return;

  pass_4(cx_expr);
}

void pass_3(struct expr * expr)
{
  debug(verbose,"(3) Code generation...\n");

  if (!reading_dcode)
  {
    apasprintf(&topdecls,"\t\t\t\t# Globals for file %s\n",
               infilename);
    if (add_plump)
    {
      apasprintf(&topcode,"\tFILEREF INIT_%s \"%s\"\n"
                 "\tSTEPPOINT\n",
                 infilename,infilename);
    }
  }
  codegen_top_expr(expr);
}

void pass_4(struct expr * expr)
{
  debug(verbose,"(4) Output...\n");

  fprintf(outf,"\n# File: %s\n\n",infilename);
  output_expr(expr);
}
