/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: records.c,v 1.1.1.1 1999/07/15 18:47:34 bardslea Exp $


	Special handling for records and the -> operator

	11 Jun 1999, AMULET Group
*/

#include <string.h>

#include <alloc.h>
#include <error.h>

#include "debug.h"
#include "parser_structs.h"
#include "scopes.h"
#include "dump.h"
#include "main.h"
#include "ccheck.h"
#include "records.h"
#include "decls.h"
#include "codegen.h"

struct expr * record_class ( struct expr * rec_class,
                             struct expr * field_class );
struct expr * lookup_field ( struct expr * recdecl, struct symrec * field );
struct expr * lookup_field_args ( struct list * args, struct symrec * field );


int fieldnum;

void bind_arrow ( struct expr * expr, struct expr * parent )
{
  struct expr * lhs;
  struct expr * rhs;
  struct expr * fieldtype;
  
      /* The -> operator is used to access fields of a record. */

  debug(debug_pass_2e,"Records: converted ");
  if (debug_pass_2e) dump_expr(expr,-1);
  debug(debug_pass_2e," to ");
  
  if (count_args(expr)!=2)
    fatal_error(expr->line,"Parameter count wrong for \"_->_\"");
  lhs=arg(1,expr);
  rhs=arg(2,expr);

  ind_debug(debug_pass_2a,currentscopelevel,
            "_->_ (line %d) ==> special\n", expr->line);

  bind_expr(lhs,expr);
  if (rhs->sort!=fcall_es)
    fatal_error(rhs->line,"field name must be an identifier");
  if (count_args(rhs)!=0)
    fatal_error(rhs->line,"field name must not have parameters");
  if (count_args(lhs->class)!=1)
    fatal_error(lhs->line,"lhs of -> has unsuitable class");
      /* The right hand parameter must be an identifier declared in
         the definition of the left hand side. */
  fieldtype=lookup_field(arg(1,lhs->class),rhs->u.fc.ident);
  if (!fieldtype)
    fatal_error(rhs->line,"field name %s not declared",rhs->u.fc.ident->name);

  ind_debug(debug_pass_2a,currentscopelevel,
            "%s (line %d) => %s (line %d)\n",
            rhs->u.fc.ident->name,rhs->line,
            fieldtype->u.fc.ident->name,fieldtype->line);
  
  if (lhs->class->decl==val_class)
  {
    expr->u.fc.ident=putsym("val_index_record");
  } else if (lhs->class->decl==var_class)
  {
    expr->u.fc.ident=putsym("var_index_record");
  } else if (lhs->class->decl==expr_class)
  {
    expr->u.fc.ident=putsym("val_index_record");
  }
  expr->u.fc.args=mk_list(mk_expr_num(fieldnum),
                          mk_list(lhs,
                                  mk_list(fieldtype,NULL)));
  if (debug_pass_2e) dump_expr(expr,-1);
  debug(debug_pass_2e,"\n");

  bind_expr(expr,parent);
}

struct expr * lookup_field ( struct expr * recdecl, struct symrec * field )
{
  debug(debug_pass_2e,"looking for %s in ",field->name);
  if (debug_pass_2e) dump_expr(recdecl,-1);
  debug(debug_pass_2e,"\n");

  if (strncmp(recdecl->u.fc.ident->name,"record",6)==0)
  {
    fieldnum=0;
    return(lookup_field_args(recdecl->u.fc.args,field));
  }
  else
    if (get_defn(recdecl->decl))
      return(lookup_field(get_defn(recdecl->decl),field));
    else
      return NULL;
}


struct expr * lookup_field_args ( struct list * args, struct symrec * field )
{
  if (args)
  {
    if (args->expr->u.fc.ident==field_ident)
    {
      if (field==arg(1,args->expr)->u.fc.ident)
      {
        return(mksubst(arg(2,args->expr)));
/*         if (args->expr->decl->inst) */
/*         { */
/*           debug(debug_pass_2e,"args->expr->decl->inst->decl = "); */
/*           if (debug_pass_2e) dump_expr(args->expr->decl->inst->decl,-1); */
/*           debug(debug_pass_2e,"\n"); */
/*           return(arg(1,args->expr->decl->inst->decl)); */
/*         } else */
/*         { */
/*           return (arg(2,args->expr)); */
/*         } */
      }
    }
    fieldnum++;
    return lookup_field_args(args->next,field);
  }
  return NULL;
}

