/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: scopes.c,v 1.1.1.1 1999/07/15 18:47:34 bardslea Exp $


	Symbol table handling

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>

#include <alloc.h>
#include <error.h>

#include "parser_structs.h"
#include "symtab.h"
#include "decls.h"
#include "scopes.h"

struct scope
{
  struct scope * parent;
  struct scopeitems * scopeitems;
};

struct scopeitems
{
  struct scopeitem * scopeitem;
  struct scopeitems * scopeitems;
};

struct scopeitem
{
  struct symrec * ident;
  struct expr * decl;
};

struct scope * currentscope;
int currentscopelevel;

void openscope(void)
{
  struct scope * p;
  
  p=palloc(sizeof(struct scope));
  p->parent=currentscope;
  p->scopeitems=NULL;
  currentscope=p;
  currentscopelevel++;
}

void closescope(void)
{
  currentscope=currentscope->parent;
  currentscopelevel--;
}

void addscope ( struct expr * decl )
{
  struct scopeitems * p;
  struct scopeitem * q;
  
  p=palloc(sizeof(struct scopeitems));
  q=palloc(sizeof(struct scopeitem));
  p->scopeitem=q;
  p->scopeitems=currentscope->scopeitems;
  currentscope->scopeitems=p;
  q->ident=decl->u.fc.args->expr->u.fc.ident;
  q->decl=decl;
}

struct scopeitem * search_scopeitems ( struct scopeitems * scopeitems, 
			               struct symrec * ident )
{
  if (!scopeitems)
    return(NULL);
  else
    if (scopeitems->scopeitem->ident==ident)
      return(scopeitems->scopeitem);
    else
      return(search_scopeitems(scopeitems->scopeitems,ident));
}


void check_existing_current ( struct expr * proto )
{
  if (search_scopeitems(currentscope->scopeitems,proto->u.fc.ident))
    fatal_error(proto->line,
                "multiple declarations of identifier %s in current scope\n",
                proto->u.fc.ident->name);
}


struct scopeitem * search_scopes ( struct scope * scope, 
				   struct symrec * ident )
{
  struct scopeitem * p;

  if (!scope)
    return(NULL);
  else
    {
      p=search_scopeitems(scope->scopeitems,ident);
      if (p)
	return(p);
      else
	return(search_scopes(scope->parent,ident));
    }
}


struct expr * lookup1 ( struct symrec * ident )
{
  struct scopeitem * p;

  p=search_scopes(currentscope,ident);
  if (!p)
    return(NULL);
  else
    return(p->decl);
}


void init_looknrec ( struct looknrec * lnr )
{
  lnr->scope=currentscope;
  lnr->scopeitems=currentscope->scopeitems;
}


struct expr * lookupn ( struct looknrec * lnr,
                        struct symrec * ident )
{
  struct expr * p;

  if (!lnr->scope)
    return(NULL);
  if (!lnr->scopeitems)
    {
      lnr->scope=lnr->scope->parent;
      if (!lnr->scope)
	return(NULL);
      lnr->scopeitems=lnr->scope->scopeitems;
      return(lookupn(lnr,ident));
    }
  if (lnr->scopeitems->scopeitem->ident==ident)
    {
      p=lnr->scopeitems->scopeitem->decl;
      lnr->scopeitems=lnr->scopeitems->scopeitems;
      return(p);
    }
  else
    {
      lnr->scopeitems=lnr->scopeitems->scopeitems;
      return(lookupn(lnr,ident));
    }
}


void do_addlibscopeitems ( struct scopeitems * scopeitems )
{
  if (scopeitems)
  {
/*    fprintf(stderr,"Copying decl %s\n",scopeitems->scopeitem->decl->u.fc.args->expr->u.fc.ident->name); */
    do_addlibscopeitems(scopeitems->scopeitems);
    addscope(scopeitems->scopeitem->decl);
  }
}


void addlibscopeitems ( struct expr * libdecl )
{
      /* Copy scope items from the given library declaration into the
         current scope. */

  do_addlibscopeitems(((struct scope *)(libdecl->libdecls))->scopeitems);
}


void recordlibscope ( struct expr * libdecl )
{
      /* Record the declarations made in the current scope in the
         given library declaration. */

  (struct scope *)(libdecl->libdecls)=currentscope;
}


