/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: main.c,v 1.1.1.1 1999/07/15 18:47:36 bardslea Exp $


	Bcode dump utility

	11 Jun 1999, AMULET Group
*/


#include <config.h>
#include "../bytecode/bytecode.h"
#include <stdio.h>
#include <unistd.h>
#include <strings.h>
#include <ctype.h>
#include <memory.h>
#include <sys/mman.h>
#include <stdlib.h>


void dumpstdsect(char * sect, char * sect_end)
{
  char * p;

  p=sect;
  while(p<sect_end)
  {
    printf("%-10d%-20s\n",*(int *)p,p+4);
    p=p+4+((strlen(p+4)+4)&~3);
  }
}


int main (int argc, char * argv[])
{
  FILE * F;
  struct section_desc sd;
  struct section_desc * descs;
  int num_descs;
  int i;
  
  if (argc!=2 && argc!=3)
  {
    fprintf(stderr,"Usage: %s filename [section number]\n",argv[0]);
    exit(1);
  }

  F=fopen(argv[1],"r");
  if (!F)
  {
    fprintf(stderr,"Can't open file %s\n",argv[1]);
    exit(1);
  }
  
  fread(&sd,sizeof(struct section_desc),1,F);

  if (sd.sect_type==WRONG_MAGIC)
  {
    printf("Wrong byte sex.\n");
    exit(0);
  }
  if (sd.sect_type!=MAGIC_NUMBER)
  {
    printf("Magic number is wrong - not a bcode file?\n");
    exit(0);
  }

  num_descs=sd.end/sizeof(struct section_desc);
  descs=malloc(sd.end-sd.start);
  fseek(F,0,0);
  fread(descs,1,sd.end-sd.start,F);

  printf("  Type                Start     End\n"
         "  ----                -----     ---\n");
  
  for(i=0;i<num_descs;i++)
  {
    printf("%1d %-20s%-10d%-10d\n",
           i,
           descs[i].sect_type==HEADER_SECTTYPE?"Header":
           descs[i].sect_type==CODE_SECTTYPE?"Code":
           descs[i].sect_type==RELOC_SECTTYPE?"Relocations":
           descs[i].sect_type==EXTERNSYM_SECTTYPE?"External symbols":
           descs[i].sect_type==INTERNSYM_SECTTYPE?"Internal symbols":
           descs[i].sect_type==OTHERFILES_SECTTYPE?"Other files":
           "Unknown",
           descs[i].start, descs[i].end);
  }

  if (argc==2)
    exit(0);


  {
    int sectnum;
    char * sect;
    int sect_len;
    
    sectnum=strtol(argv[2],NULL,10);
    printf("\nDumping section %d: ",sectnum);
    sect_len=descs[sectnum].end-descs[sectnum].start;
    sect=malloc(sect_len);
    fseek(F,descs[sectnum].start,SEEK_SET);
    fread(sect,1,sect_len,F);
    if (descs[sectnum].sect_type==RELOC_SECTTYPE)
    {
      printf("Relocations\n\n"
             "Address   Symbol\n"
             "-------   ------\n");
      dumpstdsect(sect,sect+sect_len);
    } else if (descs[sectnum].sect_type==EXTERNSYM_SECTTYPE)
    {
      printf("External symbols\n\n"
             "Address   Symbol\n"
             "-------   ------\n");
      dumpstdsect(sect,sect+sect_len);
    } else if (descs[sectnum].sect_type==INTERNSYM_SECTTYPE)
    {
      printf("Internal symbols\n\n"
             "Address   Symbol\n"
             "-------   ------\n");
      dumpstdsect(sect,sect+sect_len);
    } else
    {
      printf("Can't dump this section - sorry\n");
    }
  }

  return 0;
}

  
