/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: basic.c,v 1.2 2000/04/25 00:16:19 bardslea Exp $


	Simple bytecode interpretation

	11 Jun 1999, AMULET Group
*/

#include "regs.h"
#include "exec.h"
#include "simple.h"
#include "load_link.h"
#include "lierror.h"

#include <stdio.h>

void f_SETN ( int dest_reg, int src_val )
{
  regs[dest_reg]=src_val;
}

void f_SETSYM ( int dest_reg, int src_val )
{
  regs[dest_reg]=PC+src_val-2;
}

void f_SETSYM_D ( int dest_reg, int src_val )
{
  regs[dest_reg]=(int)(&(code[PC+src_val-2]));
}

void f_EQS ( char * str )
{
  li_fatal("mustn't execute EQS");
}

void f_SETC ( int dest_reg, int src_val )
{
  regs[dest_reg]=src_val;
}

void f_SETR ( int dest_reg, int src_reg )
{
  regs[dest_reg]=regs[src_reg];
}

void f_ADD ( int dest_reg, int src_reg )
{
  regs[dest_reg]+=regs[src_reg];
}

void f_SUB ( int dest_reg, int src_reg )
{
  regs[dest_reg]-=regs[src_reg];
}

void f_RSB ( int dest_reg, int src_reg )
{
  regs[dest_reg]=regs[src_reg]-regs[dest_reg];
}

void f_MUL ( int dest_reg, int src_reg )
{
  regs[dest_reg]*=regs[src_reg];
}

void f_DIV ( int dest_reg, int src_reg )
{
  regs[dest_reg]/=regs[src_reg];
}

void f_MOD ( int dest_reg, int src_reg )
{
  regs[dest_reg]%=regs[src_reg];
}

void f_NOT ( int reg )
{
  regs[reg]=~regs[reg];
}

void f_AND ( int dest_reg, int src_reg )
{
  regs[dest_reg]&=regs[src_reg];
}

void f_OR ( int dest_reg, int src_reg )
{
  regs[dest_reg]|=regs[src_reg];
}

void f_XOR ( int dest_reg, int src_reg )
{
  regs[dest_reg]^=regs[src_reg];
}

void f_SHL ( int dest_reg, int src_reg )
{
  int sh_amt;

  sh_amt=regs[src_reg];
  if (sh_amt>=32)
  {
    regs[dest_reg]=0;
  } else
  {
    regs[dest_reg]<<=regs[src_reg];
  }
}

void f_SHR ( int dest_reg, int src_reg )
{
  regs[dest_reg]>>=regs[src_reg];
}

void f_INC ( int reg )
{
  regs[reg]++;
}

void f_DEC ( int reg )
{
  regs[reg]--;
}

void f_INC4 ( int reg )
{
  regs[reg]+=4;
}

void f_DEC4 ( int reg )
{
  regs[reg]-=4;
}

void f_CMPEQ ( int dest_reg, int src_reg_1, int src_reg_2 )
{
  regs[dest_reg]=regs[src_reg_1]==regs[src_reg_2]?1:0;
}

void f_CMPNEQ ( int dest_reg, int src_reg_1, int src_reg_2 )
{
  regs[dest_reg]=regs[src_reg_1]!=regs[src_reg_2]?1:0;
}

void f_CMPLT ( int dest_reg, int src_reg_1, int src_reg_2 )
{
  regs[dest_reg]=regs[src_reg_1]<regs[src_reg_2]?1:0;
}

void f_CMPLTEQ ( int dest_reg, int src_reg_1, int src_reg_2 )
{
  regs[dest_reg]=regs[src_reg_1]<=regs[src_reg_2]?1:0;
}

void f_CMPGT ( int dest_reg, int src_reg_1, int src_reg_2 )
{
  regs[dest_reg]=regs[src_reg_1]>regs[src_reg_2]?1:0;
}

void f_CMPGTEQ ( int dest_reg, int src_reg_1, int src_reg_2 )
{
  regs[dest_reg]=regs[src_reg_1]>=regs[src_reg_2]?1:0;
}

void f_SETNNZ ( int cond_reg, int dest_reg, int src_val )
{
  if (regs[cond_reg]!=0) regs[dest_reg]=src_val;
}

void f_SETSYMNZ ( int cond_reg, int dest_reg, int src_val )
{
  if (regs[cond_reg]!=0) regs[dest_reg]=PC+src_val-2;
}

void f_END ( void )
{
  li_fatal("Program terminated\n");
}

void f_NOP ( void )
{
}
