/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: cosimulation.c,v 1.1 2002/10/24 12:24:41 janinl Exp $

	Co-simulation routines

	22 oct 2002, AMULET Group
*/


#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gmp.h>

#include <alloc.h>
#include <simple.h>

#include "cosimulation.h"
#include "stacks.h"
#include "lierror.h"
#include "stringops.h"
#include "threads.h"


#define BEGIN_WITH(x,y) (!strncmp(x,y,strlen(y)))


int readfd, writefd;

void startCosimulationProcess (char *cmd)
{
  int pid;

  unlink ("/tmp/fifo.1");
  unlink ("/tmp/fifo.2");
  if (mkfifo("/tmp/fifo.1", S_IRWXU)) {
    fprintf (stderr, "pb creating /tmp/fifo.1\n");
    exit (-1);
  }
  if (mkfifo("/tmp/fifo.2", S_IRWXU)) {
    fprintf (stderr, "pb creating /tmp/fifo.2\n");
    exit (-1);
  }

  pid = fork();
  if (pid==0) { //child
    //    int status = 
    system (cmd);
    //    if (WEXITSTATUS(status) != 0) {
    printf ("startCosimulationProcess finished\n");
      //    }
    _exit (0);
  }

  writefd = open ("/tmp/fifo.1", O_WRONLY);
  readfd = open ("/tmp/fifo.2", O_RDONLY);
}


int waitingList[100];
int waitingList_nbPorts = 0;
char* portValue[100];
void initThreadWaitingList (int nbPorts)
{
  int i;

  for (i=0; i<nbPorts; i++) {
    waitingList[i] = 0;
    portValue[i] = 0;
  }

  waitingList_nbPorts = nbPorts;
}
void addThreadToWaitingList (int tid, int portnum)
{
  waitingList[portnum] = tid;
}
void wakeUpThread (int portnum)
{
  if (waitingList[portnum]) {
    setpriority (waitingList[portnum], PRI_RUN);
    //    printf("wakeUpThread: port %d waken up\n", portnum);
  }
  else {
    printf("wakeUpThread error: port %d is not waiting\n", portnum);
    exit (-1);
  }

  waitingList[portnum] = 0;
}

void savePortValue (int portnum, char *val)
{
  if (portValue[portnum])
    free (portValue[portnum]);

  portValue[portnum] = strdup(val);
}
char *getPortValue (int portnum)
{
  if (!portValue[portnum]) {
    printf("getPortValue %d error: non existant value\n", portnum);
    exit(-1);
  }

  return portValue[portnum];
}


void cosimulation_cmd (void)
{
  char * cmd;
  struct crec * cr;
  int typeSize;
  int result = 0;

  cr=(struct crec *)param_pop();
  cmd=cr->str;
  if (!cmd)
  {
    li_fatal ("uninitialised string passed to cosimulation_cmd");
    return;
  }
  typeSize=param_pop();


  //  printf ("cosimulation_cmd: %s %d\n", cmd, typeSize);
  if (BEGIN_WITH (cmd, "timestep")) {
    char buf[1];
    if (write (writefd, cmd, 1) <= 0) {
      printf("lard side: cosimulation closed without notice!\n");
      exit (0);
    }
  loop:
    if (read (readfd, buf, 1) <= 0) {
      printf("lard side: cosimulation closed without notice!\n");
      exit (0);
    }
    if (buf[0]=='t')
      goto timestep;
    else if (buf[0]=='s') { //send_*
      char buf2[100];
      int pos = 0;
      buf2[0] = 's';
      do {
	pos++;
	read (readfd, &buf2[pos], 1);
      } while (buf2[pos]);

      //      printf("lard side: received send command \"%s\"\n", buf2);

      if (BEGIN_WITH (buf2, "send_requp ")) {
	int portnum = atoi (buf2+11);
	wakeUpThread (portnum);
      }
      else if (BEGIN_WITH (buf2, "send_requp_with_valmpint ")) {
	int portnum = atoi (buf2+25);
	char *ptrval = strchr (buf2+25, ' ') + 1;
	savePortValue (portnum, ptrval);
	wakeUpThread (portnum);
      }
      else if (BEGIN_WITH (buf2, "send_requp_with_valint ")) {
	int portnum = atoi (buf2+23);
	char *ptrval = strchr (buf2+23, ' ') + 1;
	savePortValue (portnum, ptrval);
	wakeUpThread (portnum);
      }
      else if (BEGIN_WITH (buf2, "send_reqdown ")) {
	int portnum = atoi (buf2+13);
	wakeUpThread (portnum);
      }
      else {
	li_fatal ("Error: Unknown received command \"%s\"\n", buf2);
      }
    }
    else {
      printf("lard side: received '%c'\n", buf[0]);
      exit(-1);
    }
    goto loop;

  timestep: ;

    //    printf("lard simulating\n");// sleep(1);
  }
  else if (BEGIN_WITH (cmd, "WRU")) {
    //    write (writefd, cmd, strlen(cmd)+1);
    addThreadToWaitingList (current_tid, atoi(cmd+3));
    setpriority (current_tid, PRI_SUSP);
    f_YIELD ();
    if (typeSize) {
      mpz_ptr newMpint = palloc (sizeof (mpz_t));
      mpz_init_set_ui (newMpint, 3);
      result = (int)newMpint;
    }
  }
  else if (BEGIN_WITH (cmd, "WRD")) {
    //    write (writefd, cmd, strlen(cmd)+1);
    addThreadToWaitingList (current_tid, atoi(cmd+3));
    setpriority (current_tid, PRI_SUSP);
    f_YIELD ();
    if (typeSize) {
      mpz_ptr newMpint = palloc (sizeof (mpz_t));
      mpz_init_set_ui (newMpint, 3);
      result = (int)newMpint;
    }
  }
  else if (BEGIN_WITH (cmd, "get")) {
    int portnum = atoi (cmd+3);
    mpz_ptr newMpint = palloc (sizeof (mpz_t));
    mpz_init (newMpint);
    mpz_set_str (newMpint, getPortValue(portnum), 10);
    result = (int)newMpint;
  }
  else if (BEGIN_WITH (cmd, "SAU_mpint ")) {
    int portnum = atoi (cmd+10);
    char *addrptr = strchr (cmd+10, ' ') + 1;
    mpz_ptr theMpint = (mpz_ptr) atoi (addrptr);
    char *valtxt = mpz_get_str (0, 10, theMpint);

    char str[200];
    sprintf (str, "SAU_mpint %d %s", portnum, valtxt);
    write (writefd, str, strlen(str)+1);

    free (valtxt);
  }
  else if (BEGIN_WITH (cmd, "SAD ")) {
    write (writefd, cmd, strlen(cmd)+1);
  }
  else if (BEGIN_WITH (cmd, "SAU ")) {
    write (writefd, cmd, strlen(cmd)+1);
  }
  else if (BEGIN_WITH (cmd, "start")) {
    initThreadWaitingList (10);
    startCosimulationProcess (cmd+6);
  }
  else if (BEGIN_WITH (cmd, "end")) {
    close (readfd);
    close (writefd);
  }
  else {
    li_fatal ("Error: Unknown cosimulation command \"%s\"\n", cmd);
  }
 
  free_if_zero(cr);

  if (typeSize) {
    //    printf("push result %d\n",result);
    param_push (result);
  }
}
