/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: i2s.c,v 1.3 2002/03/11 10:53:45 bardslea Exp $


	Conversion between ints and strings

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include <error.h>
#include <alloc.h>

#include "i2s.h"
#include "io.h"
#include "stacks.h"
#include "lierror.h"
#include "stringops.h"

#define MACH_INT_BITS (CHAR_BIT * sizeof(int))

void i2s (void)
{
	/* the builtin takes three arguments, the value, the
	radix in which it is to be printed, and the character set
	that should be used */

	/* the character set is the characters in ascending order
	with the characters for + and - on the front.  If the + or
	- characters is _ [underscore] then they will not be printed */

	/* radices of -infinity < radix <= -2 result in printing of
	values with + or - characters attached */

	/* radices in the range 2 <= radix < infinity will result in
	printing of the unsigned int value (of course without + or -
	prepended) */

	/* radix = 1, behaves as a radix 1 number, i.e. repeat count of
	symbol indicates value */
	
	int val = param_pop ();
	int radix = param_pop ();
	int absradix = abs(radix);
	struct crec * cr=(struct crec *)param_pop();
	char *charset;
	struct crec * rcr = NULL;
	char *r = NULL;
	int i = 0;
	int j = 0;
	
	if (! checkstrcr (cr, "i2s")) return;
	charset = cr->str;

	/* check length of format string */
	if (strlen(charset) < (absradix + 2))
	{li_fatal("i2s: character set passed was shorter than radix"); return;}

	if (radix == 0) {li_fatal("i2s: radix==0 not permitted"); return;}
		
	if (absradix >= 1)
	{
		int posneg = 0;
		char *s = NULL;
		unsigned int uval = 0;
		
		/* if the value is negative and we want that to be important */
		if (val < 0 && radix < 0)
		{
			posneg = 1;
			val = -val;
		}

		/* find the value we should be working with */
		uval = (unsigned) val;

		/* if we want a unary number the allocation must be different */
		if (absradix == 1)
		{
			s = palloc(uval+1);
			for (i = 0; i < uval; i++) s[i] = charset[2];
			s[uval] = '\0';
		} else {
			s = palloc(MACH_INT_BITS+1);

			do
			{
				/* put the appropriate character from the character
				set in the appropriate slot in the string (but in reverse order) */
				s[i++] = charset[(uval % absradix)+2];
				uval /= absradix;
			} while (uval);
		}

		r = palloc(j+i+1+(posneg ? 1 : 0));

		if (radix <= -1)
		{
			/* put the positive or negative sign in */
			if (charset[posneg] != '_') {r[j++] = charset[posneg];}
		}
		
		/* reverse the bits */
		if (absradix == 1)
		{
			/* if we are unary, all the chars will be the same */
			strcpy(r+j, s);
		} else {
			/* otherwise copythem back to front */
			do {r[j++] = s[--i];} while (i>0);
			r[j] = '\0';
		}
		free(s);
	}

	rcr = palloc(sizeof(struct crec));
	rcr->refcount = 0;
	rcr->str=r;

	free_if_zero (cr);

	param_push((int)rcr);
}
