/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: io.c,v 1.6 2000/04/25 00:16:20 bardslea Exp $


	Lard I/O handling

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#include <alloc.h>
#include <simple.h>
#include <strtoint.h>

#include "io.h"
#include "stacks.h"
#include "load_link.h"
#include "lierror.h"
#include "regs.h"
#include "tcl_if.h"
#include "escapify.h"
#include "stringops.h"

#ifndef HAVE_SNPRINTF
#define snprintf(buf, size, fmt, args...) sprintf(buf,fmt , ## args)
#endif

int definpbase=10;

char * stdin_line=NULL;

void output ( FILE * file, char * str )
{
  char * escapified_out_buf;
  char * cmd_buf;

  if (file==stdout)
  { escapified_out_buf=escapify_str(str);
    cmd_buf=palloc(strlen(escapified_out_buf)+32);
    strcpy(cmd_buf,"stdio_outline \"");
    strcat(cmd_buf,escapified_out_buf);
    strcat(cmd_buf,"\"");
    tcl_cmd(cmd_buf);
    free(escapified_out_buf);
    free(cmd_buf);
  } else {
    if (fputs(str,file)==EOF)
    {
      li_fatal("write failed\n");
      return;
    }
  }
}


void l_stdin ( void )
{
  param_push((int)stdin);
}


void l_stdout ( void )
{
  param_push((int)stdout);
}


void l_stderr ( void )
{
  param_push((int)stderr);
}


void openin ( void )
{
  struct crec * cr;
  bool nonBlocking = (bool) param_pop();
  char * fn;

  cr=(struct crec *)param_pop();
  fn=cr->str;

  if (nonBlocking)
  {
	int f = open (fn, O_RDONLY | O_NONBLOCK);

	if (!f)
	{
		li_fatal("openin failed for file %s\n",fn);
		return;
	}
	param_push((int)f);
  } else {
	FILE *f=fopen(fn,"r");
	if (!f)
	{
		li_fatal("openin failed for file %s\n",fn);
		return;
	}
	param_push((int)f);
  }
  free_if_zero(cr);
}


void openout ( void )
{
  struct crec * cr;
  char * fn;
  FILE * f;
  
  cr=(struct crec *)param_pop();
  fn=cr->str;
  f=fopen(fn,"w");
  if (!f)
  {
    li_fatal("openout failed for file %s\n",fn);
    return;
  }
  /*
  if (fseek(f,0,0)==-1)
  {
    li_fatal("fseek failed in openout");
    return;
  }
  */
  param_push((int)f);
  free_if_zero(cr);
}

void fsetunbuffered ( void )
{
	FILE *f = (FILE *) param_pop ();
	setbuf (f, NULL);
}

void openinout ( void )
{
  struct crec * cr;
  char * fn;
  FILE * f;
  
  cr=(struct crec *)param_pop();
  fn=cr->str;
  f=fopen(fn,"a+");
  if (!f)
  {
    li_fatal("openinout failed for file %s\n",fn);
    return;
  }
  if (fseek(f,0,0)==-1)
  {
    li_fatal("fseek failed in openinout");
    return;
  }
  param_push((int)f);
  free_if_zero(cr);
}


void l_close ( void )
{
  bool nonBlocking = (bool) param_pop ();
  int f;

  f=param_pop();
  if (nonBlocking)
  {
	if (close (f)) li_fatal("close failed\n");
  } else if (fclose((FILE *) f)) li_fatal("close failed\n");
}


void fprintint ( void )
{
  FILE * f;
  char b[128];
  
  f=(FILE *)param_pop();
  sprintf(b,"%d",param_pop());
  output(f,b);
}


void fprintstr ( void )
{
  FILE * f;
  struct crec * cr;
  char * s;
  
  f=(FILE *)param_pop();
  cr=(struct crec *)param_pop();
  s=cr->str;
  if (!s)
  {
    li_fatal("Uninitialised string passed to fprintstr");
    return;
  }
  output(f,s);
  free_if_zero(cr);
}


void fprintchar ( void )
{
  FILE * f;
  char c[2];
  
  f=(FILE *)param_pop();
  c[0]=(char)param_pop();
  c[1]='\0';
  output(f,c);
}


int readbasen ( char * buf, int base )
{
  int n;
  char * tail;

  errno=0;
  n=lard_strtoul(buf,&tail,base);
  if (errno)
  {
    li_fatal("readint failed: can't parse \"%s\" in base %d\n",buf,base);
    return 0;
  }
  while (isspace(*tail))
    tail++;
  if (*tail)
  {
    li_fatal("readint failed: unexpected junk after number in \"%s\"\n",buf);
    return 0;
  }
  return n;
}
    

int readanybase ( char * buf )
{
  char * p;
  bool neg;
  int n;
  
  p=buf;
  while (isspace(*p)  && (*p!='\n'))
    p++;
  if (*p=='\n')
  {
    li_fatal("can't find number in blank line");
    return 0;
  }
  if ((*p)=='-')
  {
    neg=TRUE;
    p++;
  } else
  {
    neg=FALSE;
  }
  if (strncmp(p,"2#",2)==0)
  {
    n=readbasen(p+2,2);
  } else if (strncmp(p,"8#",2)==0)
  {
    n=readbasen(p+2,8);
  } else if (strncmp(p,"10#",3)==0)
  {
    n=readbasen(p+3,10);
  } else if (strncmp(p,"16#",3)==0)
  {
    n=readbasen(p+3,16);
  } else
  {
    n=readbasen(p,definpbase);
  }
  if (neg)
    return -n;
  else
    return n;
}


bool redo_input=FALSE;

void freadint ( void )
{
  int n;
  FILE * f;
  char buf[1024];
  
  if (redo_input)
  {
    redo_input=FALSE;
    n=readanybase(stdin_line);
    param_push(n);
    return;
  }
  
  f=(FILE *)param_pop();
  if (f==stdin)
  {
    tcl_cmd("stdio_inline");
    redo_input=TRUE;
    PC--;
    return;
  }

  if (!fgets(buf,1024,f))
  {
    li_fatal("read past end of file\n");
    return;
  }
  n=readanybase(buf);
  param_push(n);
}

void freadstr ( void )
{
  FILE * f;
  char * p;
  char buf[1024];
  struct crec * cr;
  
  if (redo_input)
  {
    redo_input=FALSE;
    p=palloc(strlen(stdin_line)+1);
    strcpy(p,stdin_line);
    cr=palloc(sizeof(struct crec));
    cr->refcount=0;
    cr->str=p;
    param_push((int)cr);
    return;
  }
  
  f=(FILE *)param_pop();
  if (f==stdin)
  {
    tcl_cmd("stdio_inline");
    redo_input=TRUE;
    PC--;
    return;
  }

  if (!fgets(buf,1024,f))
  {
    li_fatal("read past end of file\n");
    return;
  }
  p=palloc(strlen(buf)+1);
  strcpy(p,buf);
  cr=palloc(sizeof(struct crec));
  cr->refcount=0;
  cr->str=p;
  param_push((int)cr);
}


void freadchar ( void )
{
  int c;
  FILE * f;

  if (redo_input)
  {
    redo_input=FALSE;
    c=stdin_line[0];
    param_push((int)c);
    return;
  }
  
  f=(FILE *)param_pop();
  if (f==stdin)
  {
    tcl_cmd("stdio_inline");
    redo_input=TRUE;
    PC--;
    return;
  }

  if ((c=fgetc(f))==EOF)
  {
    li_fatal("read past end of file\n");
    return;
  }
  param_push(c);
}


void teof ( void )
{
  FILE * f;
  int c;
  
  f=(FILE *)param_pop();
  if (f==stdin || f==stdout)
  {
    li_fatal("Can't do eof on stdin or stdout\n");
    return;
  }

  c=' ';
  while (isspace(c))
  {
    c=fgetc(f);
  }
  
  if (c==EOF)
  {
    param_push(1);
  } else {
    if (fseek(f,-1,1)==-1)
    {
      li_fatal("fseek failed in teof");
      return;
    }
    param_push(0);
  }
}


void beof ( void )
{
  FILE * f;
  int c;
  
  f=(FILE *)param_pop();
  c=fgetc(f);
  if (c==EOF)
  {
    param_push(1);
  } else {
    if (fseek(f,-1,1)==-1)
    {
      li_fatal("fseek failed in beof");
      return;
    }
    param_push(0);
  }
}


void setdefinpbase ( void )
{
  int b;

  b=param_pop();
  if (b!=2 && b!=8 && b!=10 && b!=16)
  {
    li_fatal("Illegal input base\n");
    return;
  }
  definpbase=b;
}


void l_fputc ( void )
{
  FILE * f;
  char c;

  f=(FILE *)param_pop();
  c=(char)param_pop();
  if (fputc(c,f)==EOF)
  {
    li_fatal("fputc failed");
    return;
  }
}


void l_fgetc ( void )
{
  FILE * f;
  int c;

  f=(FILE *)param_pop();
  c=fgetc(f);
  if (c==EOF)
  {
    li_fatal("fgetc read past end of file");
    return;
  }
  param_push(c);
}


void l_fwrite ( void )
{
  FILE * f;
  int wsz;
  int bsz;
  int * data;
  int i;
  
  f=(FILE *)param_pop();
  bsz=param_pop();
  data=palloc(bsz);
  wsz=bsz/4;
  for(i=0;i<wsz;i++)
  {
    data[i]=param_pop();
  }
  if (!fwrite(data,1,bsz,f))
  {
    li_fatal("fwrite failed");
    return;
  }
  free(data);
}
  

void l_fread ( void )
{
  int f; /* NB awful 32 bitism */
  bool nonBlocking;
  int bsz;
  int * data;

  nonBlocking = (bool) param_pop ();
  f=param_pop();
  bsz=param_pop();
  data=(int *)param_pop();

  if (nonBlocking)
  {
  	int readRet, committedToRead = FALSE;
	/* Non blocking I/O will not block if no data is available, if some data
		is ready then this will block untill the whole bsz is received */

	do {
		errno = 0;
		readRet = read (f, data, bsz);

		if (errno || readRet < 0)
		{
			if (errno != EAGAIN)
			{
				li_fatal("fread failed");
				return;
			}
		} else {
			bsz -= readRet;
			data += readRet;
			committedToRead = TRUE;
		}
	} while (committedToRead && bsz != 0);
	param_push (errno != EAGAIN);
  } else if (!fread(data,1,bsz,(FILE *) f))
  {
    li_fatal("fread failed");
    return;
  }
}


void l_fseek ( void )
{
  FILE * f;
  int pos;

  f=(FILE *)param_pop();
  pos=param_pop();
  if (fseek(f,pos,0)==-1)
  {
    li_fatal("fseek failed");
    return;
  }
}


void l_ftell ( void )
{
  FILE * f;
  int pos;

  f=(FILE *)param_pop();
  pos=ftell(f);
  param_push(pos);
}


void l_flen ( void )
{
  FILE * f;
  int fd;
  struct stat st;
  int len;
  
  f=(FILE *)param_pop();
  fd=fileno(f);
  if (fstat(fd,&st)==-1)
  {
    li_fatal("fstat failed in flen");
    return;
  }
  len=st.st_size;
  param_push(len);
}


void l_ftrunc ( void )
{
  FILE * f;
  int pos;
  int fd;
  
  f=(FILE *)param_pop();
  fflush(f);
  pos=ftell(f);
  fd=fileno(f);
  if (ftruncate(fd,pos)==-1)
  {
    li_fatal("ftruncate failed");
    return;
  }
}

