/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: main.c,v 1.5 2001/12/06 14:48:44 bardslea Exp $


	Main program for li

	11 Jun 1999, AMULET Group
*/

#include <tcl.h>
#include <tk.h>
#include <tclExtend.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include "simple.h"
#include "linenos.h"

#include "tcl_if.h"

char *input_buffer = "";
char *infilename = "";
int line_num = 0;

int main_argc;
char ** main_argv;

int use_tk;


void tcl_init_error ( Tcl_Interp * interp )
{
  char * msg;
  
  msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
  if (msg == NULL) {
    msg = interp->result;
  }
  fprintf(stderr, "%s\n", msg);
  exit(1);
}


int li_AppInit ( Tcl_Interp * interp )
{
  char *args;
  char buf[128];
  
  args=Tcl_Merge(main_argc,main_argv);
  Tcl_SetVar(interp,"argv",args, TCL_GLOBAL_ONLY);
  sprintf(buf, "%d", main_argc);
  Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "argv0", main_argv[0], TCL_GLOBAL_ONLY);
  

  if (Tcl_Init(interp) == TCL_ERROR) {
    return(TCL_ERROR);
  }
  if (Tclx_Init(interp) == TCL_ERROR) {
    return(TCL_ERROR);
  }
  if (use_tk) {
    if (Tk_Init(interp) == TCL_ERROR) {
      return(TCL_ERROR);
    }
    if (Tkx_Init(interp) == TCL_ERROR) {
      return(TCL_ERROR);
    }
  }
  
  create_commands(interp);
  
  if (Tcl_VarEval(interp,
#include "init.tcl"
                  ,NULL) == TCL_ERROR)
  {
    tcl_init_error(interp);
  }

  return(TCL_OK);
}

void segv (int a)
{
	fprintf (stderr, "Segmentation fault at line %d of file `%s'\n",
		current_linenum, current_filename);
	fflush (NULL);
	_exit (1);
}

void quit (int a)
{
	exit (1);
}

void flush_io (int a)
{
	fflush (0);
}

int main(int argc,char **argv)
{
  struct sigaction action;

  action.sa_handler = (void (*)()) segv;
  sigemptyset (&action.sa_mask);
  action.sa_flags = 0;
  sigaction (SIGSEGV, &action, NULL);

  action.sa_handler = (void (*)()) quit;
  sigemptyset (&action.sa_mask);
  action.sa_flags = 0;
  sigaction (SIGINT, &action, NULL);

  action.sa_handler = (void (*)()) quit;
  sigemptyset (&action.sa_mask);
  action.sa_flags = 0;
  sigaction (SIGHUP, &action, NULL);

  action.sa_handler = (void (*)()) flush_io;
  sigemptyset (&action.sa_mask);
  action.sa_flags = 0;
  sigaction (SIGUSR1, &action, NULL);

  if (argc>=2 && strcmp(argv[1],"-notk")==0) {
    use_tk=0;
    main_argc=argc-1;
    argv[1]=argv[0];
    main_argv=&argv[1];
  } else {
    use_tk=1;
    main_argc=argc;
    main_argv=argv;
  }

  if (use_tk) {
    Tk_Main(0,argv,li_AppInit);
  } else {
    Tcl_Main(0,argv,li_AppInit);
  }
  return 0;
}

