#!/bin/sh
exec perl -w -x $0 ${1+"$@"}
    if 0;
#!perl


# mkexec
# read descriptions of instructions and generate exec_body file
# format of instruction description is
#     OPCODE    OPERAND OPERAND ...
# where each operand is one of REG, STK, NUM, SYM, CHAR, BUILTIN, or STR
# format of each part of exec_body is 
#    case BC_OPCODE:
#      debug("OPCODE OPERAND_FMTS\n",OPERANDS);
#      PC+=N;
#      f_OPCODE(OPERANDS);
#      break;


open(INSTRS,"<../bytecode/instrs.in") || die "Can't open instrs.in";
while (<INSTRS>)
{
    if (m/^\s*$/)
    {
	next;
    }
    (($opcode,$operands)=m/^(\S+)\s+(.*)$/) || die "Don't understand $_";
    @oplist=split(/\s+/,$operands);
    @debug_fmtlist=();
    @debug_arglist=();
    @funccall_arglist=();

    $tpnum=0;
    $strfound=0;
    foreach $op (@oplist)
    {
	if ($op eq "NUM" || $op eq "SYM" || $op eq "CHAR" || $op eq "STR")
	{
	    if ($strfound)
	    {
		die "String must be last operand in $_";
	    }
	    $tpnum++;
	    if ($op eq "STR")
	    {
		$strfound=1;
	    }
	}
    }

    $opnum=1;
    $pnum=0;
    $strpos=-1;
    foreach $op (@oplist)
    {
	$ppos=$tpnum-$pnum;
	if ($op eq "REG")
	{
	    push(@debug_fmtlist,"%s");
	    push(@debug_arglist,"regnames[op$opnum]");
	    push(@funccall_arglist,"op$opnum");
	    $opnum++;
	} elsif ($op eq "STK")
	{
	    push(@debug_fmtlist,"%s");
	    push(@debug_arglist,"stacknames[op$opnum]");
	    push(@funccall_arglist,"op$opnum");
	    $opnum++;
	} elsif ($op eq "NUM")
	{
	    push(@debug_fmtlist,"%d");
	    push(@debug_arglist,"code[PC-$ppos]");
	    push(@funccall_arglist,"code[PC-$ppos]");
	    $pnum++;
	} elsif ($op eq "SYM")
	{
	    push(@debug_fmtlist,"%s");
	    push(@debug_arglist,"printsym(code[PC-$ppos]+PC-1)");
	    push(@funccall_arglist,"code[PC-$ppos]");
	    $pnum++;
	} elsif ($op eq "CHAR")
	{
	    push(@debug_fmtlist,"%c");
	    push(@debug_arglist,"code[PC-$ppos]");
	    push(@funccall_arglist,"code[PC-$ppos]");
	    $pnum++;
	} elsif ($op eq "BUILTIN")
	{
	    push(@debug_fmtlist,"%d");
	    push(@debug_arglist,"op$opnum");
	    push(@funccall_arglist,"op$opnum");
	    $opnum++;
	} elsif ($op eq "STR")
	{
	    push(@debug_fmtlist,"%s");
	    push(@debug_arglist,"(char *)(&code[PC-$ppos])");
	    push(@funccall_arglist,"(char *)(&code[PC-$ppos])");
	    $strpos=$ppos;
	    $pnum++;
	} else
	{
	    die "Don't understand operand type $op";
	}
    }

    $pc_inc=$pnum;
    print "    case BC_$opcode:\n";
    if ($pc_inc>0)
    {
	print "      PC+=$pc_inc;\n";
    }
    print "      debug(\"$opcode";
    foreach $fmt (@debug_fmtlist)
    {
	print " $fmt";
    }
    print "\\n\"";
    foreach $arg (@debug_arglist)
    {
	print ",$arg";
    }
    print ");\n";
    print "      f_$opcode(";
    print join(",",@funccall_arglist);
    print ");\n";
    if ($strpos>-1)
    {
	print "      PC+=((strlen((char *)(&code[PC-$strpos]))+4)&~3)/4-1;\n";
    }
    print "      break;\n";
}




