/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: mpint.c,v 1.4 2000/10/23 11:00:16 bardslea Exp $

    
	Multi-precision int function defns

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <alloc.h>

#include "mpint.h"
#include "io.h"
#include "stacks.h"
#include "lierror.h"
#include "stringops.h"
#include "../config.h"

#ifndef HAVE_LIBGMP
void mpintop (void)
{
    li_fatal ("mpintop: this installation of LARD was compiled without mpint support\n");
}
#else
void mpintop (void)
{
	int operation = param_pop ();

	switch (operation)
	{
		case 0: { /* mpint_init () */
			mpz_ptr newMpint = palloc (sizeof (mpz_t));
			mpz_init_set_ui (newMpint, 0);
			param_push ((int) newMpint);
		} break;
		case 1: { /* mpint_clear (arg) */
			mpz_ptr mp = (mpz_ptr) param_pop ();
			mpz_clear (mp);
			free (mp);
		} break;

		case 100: { /* mpint_add (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_add (result, left, right);
		} break;
		case 101: { /* mpint_add_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_add_ui (result, left, right);
		} break;
		case 102: { /* mpint_sub (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_sub (result, left, right);
		} break;
		case 103: { /* mpint_sub_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_sub_ui (result, left, right);
		} break;
		case 104: { /* mpint_mul (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_mul (result, left, right);
		} break;
		case 105: { /* mpint_mul_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_mul_ui (result, left, right);
		} break;
		case 106: { /* mpint_mul_2exp (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_mul_2exp (result, left, right);
		} break;
		case 107: { /* mpint_neg (result,arg) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg = (mpz_ptr) param_pop ();
			mpz_neg (result, arg);
		} break;
		case 108: { /* mpint_abs (result,arg) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg = (mpz_ptr) param_pop ();
			mpz_abs (result, arg);
		} break;
		case 109: { /* mpint_fac_ui (result,arg) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			int arg = param_pop ();
			mpz_fac_ui (result, arg);
		} break;
		case 110: { /* mpint_tdiv_q (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_tdiv_q (result, left, right);
		} break;
		case 111: { /* mpint_tdiv_q_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_tdiv_q_ui (result, left, right);
		} break;
		case 112: { /* mpint_tdiv_r (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_tdiv_r (result, left, right);
		} break;
		case 113: { /* mpint_tdiv_r_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_tdiv_r_ui (result, left, right);
		} break;
		case 114: { /* mpint_tdiv_qr (quo,rem,left,right) */
			mpz_ptr quo = (mpz_ptr) param_pop ();
			mpz_ptr rem = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_tdiv_qr (quo, rem, left, right);
		} break;
		case 115: { /* mpint_tdiv_qr_ui (quo,rem,left,right) */
			mpz_ptr quo = (mpz_ptr) param_pop ();
			mpz_ptr rem = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_tdiv_qr_ui (quo, rem, left, right);
		} break;
		case 116: { /* mpint_fdiv_q (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_fdiv_q (result, left, right);
		} break;
		case 117: { /* mpint_fdiv_q_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_fdiv_q_ui (result, left, right);
		} break;
		case 118: { /* mpint_fdiv_r (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_fdiv_r (result, left, right);
		} break;
		case 119: { /* mpint_fdiv_r_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			param_push ((int) mpz_fdiv_r_ui (result, left, right));
		} break;
		case 120: { /* mpint_fdiv_qr (quo,rem,left,right) */
			mpz_ptr quo = (mpz_ptr) param_pop ();
			mpz_ptr rem = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_fdiv_qr (quo, rem, left, right);
		} break;
		case 121: { /* mpint_fdiv_qr_ui (quo,rem,left,right) */
			mpz_ptr quo = (mpz_ptr) param_pop ();
			mpz_ptr rem = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			param_push ((int) mpz_fdiv_qr_ui (quo, rem, left, right));
		} break;
		case 122: { /* mpint_fdiv_ui (left,right) */
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			param_push ((int) mpz_fdiv_ui (left, right));
		} break;
		case 123: { /* mpint_cdiv_q (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_cdiv_q (result, left, right);
		} break;
		case 124: { /* mpint_cdiv_q_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_cdiv_q_ui (result, left, right);
		} break;
		case 125: { /* mpint_cdiv_r (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_cdiv_r (result, left, right);
		} break;
		case 126: { /* mpint_cdiv_r_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			param_push ((int) mpz_cdiv_r_ui (result, left, right));
		} break;
		case 127: { /* mpint_cdiv_qr (quo,rem,left,right) */
			mpz_ptr quo = (mpz_ptr) param_pop ();
			mpz_ptr rem = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_cdiv_qr (quo, rem, left, right);
		} break;
		case 128: { /* mpint_cdiv_qr_ui (quo,rem,left,right) */
			mpz_ptr quo = (mpz_ptr) param_pop ();
			mpz_ptr rem = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			param_push ((int) mpz_cdiv_qr_ui (quo, rem, left, right));
		} break;
		case 129: { /* mpint_cdiv_ui (left,right) */
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			param_push ((int) mpz_cdiv_ui (left, right));
		} break;
		case 130: { /* mpint_mod (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_mod (result, left, right);
		} break;
		case 131: { /* mpint_mod_ui (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			param_push ((int) mpz_mod_ui (result, left, right));
		} break;
		case 132: { /* mpint_divexact (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_divexact (result, left, right);
		} break;
		case 133: { /* mpint_tdiv_q_2exp (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_tdiv_q_2exp (result, left, right);
		} break;
		case 134: { /* mpint_tdiv_r_2exp (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_tdiv_r_2exp (result, left, right);
		} break;
		case 135: { /* mpint_fdiv_q_2exp (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_fdiv_q_2exp (result, left, right);
		} break;
		case 136: { /* mpint_fdiv_r_2exp (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			mpz_fdiv_r_2exp (result, left, right);
		} break;
		case 137: { /* mpint_powm (result,base,expo,modu) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr base = (mpz_ptr) param_pop ();
			mpz_ptr expo = (mpz_ptr) param_pop ();
			mpz_ptr modu = (mpz_ptr) param_pop ();
			mpz_powm (result, base, expo, modu);
		} break;
		case 138: { /* mpint_powm_ui (result,base,expo,modu) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr base = (mpz_ptr) param_pop ();
			int expo = param_pop ();
			mpz_ptr modu = (mpz_ptr) param_pop ();
			mpz_powm_ui (result, base, expo, modu);
		} break;
		case 139: { /* mpint_pow_ui (result,base,expo) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr base = (mpz_ptr) param_pop ();
			int expo = param_pop ();
			mpz_pow_ui (result, base, expo);
		} break;
		case 140: { /* mpint_ui_pow_ui (result,base,expo) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			int base = param_pop ();
			int expo = param_pop ();
			mpz_ui_pow_ui (result, base, expo);
		} break;
		case 141: { /* mpint_sqrt (result,arg) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg = (mpz_ptr) param_pop ();
			mpz_sqrt (result, arg);
		} break;
		case 142: { /* mpint_sqrtrem (result,rem,arg) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr rem = (mpz_ptr) param_pop ();
			mpz_ptr arg = (mpz_ptr) param_pop ();
			mpz_sqrtrem (result, rem, arg);
		} break;
		case 143: { /* mpint_perfect_square_p (arg) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			param_push ((mpz_perfect_square_p (arg) ? 1 : 0));
		} break;
		case 144: { /* mpint_probab_prime_p (arg,reps) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			int reps = param_pop ();
			param_push ((mpz_probab_prime_p (arg, reps) ? 1 : 0));
		} break;
		case 145: { /* mpint_gcd (result,arg1,arg2) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg1 = (mpz_ptr) param_pop ();
			mpz_ptr arg2 = (mpz_ptr) param_pop ();
			mpz_gcd (result, arg1, arg2);
		} break;
		case 146: { /* mpint_gcd_ui (result,arg1,arg2) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg1 = (mpz_ptr) param_pop ();
			int arg2 = param_pop ();
			param_push ((int) mpz_gcd_ui (result, arg1, arg2));
		} break;
		case 147: { /* mpint_gcdext (g,s,t,a,b) */
			mpz_ptr g = (mpz_ptr) param_pop ();
			mpz_ptr s = (mpz_ptr) param_pop ();
			mpz_ptr t = (mpz_ptr) param_pop ();
			mpz_ptr a = (mpz_ptr) param_pop ();
			mpz_ptr b = (mpz_ptr) param_pop ();
			mpz_gcdext (g, s, t, a, b);
		} break;
		case 148: { /* mpint_invert (result,arg,modu) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg = (mpz_ptr) param_pop ();
			mpz_ptr modu = (mpz_ptr) param_pop ();
			mpz_invert (result, arg, modu);
		} break;
		case 149: { /* mpint_jacobi (result,arg) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg = (mpz_ptr) param_pop ();
			mpz_jacobi (result, arg);
		} break;
		case 150: { /* mpint_legendre (result,arg) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg = (mpz_ptr) param_pop ();
			mpz_legendre (result, arg);
		} break;

		case 200: { /* mpint_cmp (left,right) */
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			int cmpResult = mpz_cmp (left, right);
			param_push ((cmpResult < 0 ? 0 : (cmpResult == 0 ? 1 : 2)));
		} break;
		case 201: { /* mpint_cmp_ui (left,right) */
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			int cmpResult = mpz_cmp_ui (left, right);
			param_push ((cmpResult < 0 ? 0 : (cmpResult == 0 ? 1 : 2)));
		} break;
		case 202: { /* mpint_cmp_si (left,right) */
			mpz_ptr left = (mpz_ptr) param_pop ();
			int right = param_pop ();
			int cmpResult = mpz_cmp_si (left, right);
			param_push ((cmpResult < 0 ? 0 : (cmpResult == 0 ? 1 : 2)));
		} break;
		case 203: { /* mpint_sgn (arg) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			int cmpResult = mpz_sgn (arg);
			param_push ((cmpResult < 0 ? 0 : (cmpResult == 0 ? 1 : 2)));
		} break;

		case 300: { /* mpint_and (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_and (result, left, right);
		} break;
		case 301: { /* mpint_ior (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
			mpz_ior (result, left, right);
		} break;
		case 302: { /* mpint_com (result,arg) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr arg = (mpz_ptr) param_pop ();
			mpz_com (result, arg);
		} break;
		case 303: { /* mpint_popcount (arg) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			param_push ((int) mpz_popcount (arg));
		} break;
		case 304: { /* mpint_hamdist (arg1,arg2) */
			mpz_ptr arg1 = (mpz_ptr) param_pop ();
			mpz_ptr arg2 = (mpz_ptr) param_pop ();
			param_push ((int) mpz_hamdist (arg1, arg2));
		} break;
		case 305: { /* mpint_scan0 (arg,bitNo) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			int bitNo = param_pop ();
			param_push ((int) mpz_scan0 (arg, bitNo));
		} break;
		case 306: { /* mpint_scan1 (arg,bitNo) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			int bitNo = param_pop ();
			param_push ((int) mpz_scan1 (arg, bitNo));
		} break;
		case 307: { /* mpint_setbit (arg,bitNo) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			int bitNo = param_pop ();
			mpz_setbit (arg, bitNo);
		} break;
		case 308: { /* mpint_clrbit (arg,bitNo) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			int bitNo = param_pop ();
			mpz_clrbit (arg, bitNo);
		} break;
		case 309: { /* mpint_xor (result,left,right) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			mpz_ptr left = (mpz_ptr) param_pop ();
			mpz_ptr right = (mpz_ptr) param_pop ();
#ifdef NEW_LIBGMP
			mpz_xor (result, left, right);
#else
			fatal ("can't do XOR on mpints without GMP version 3.0 or greater\n");
#endif
		} break;

		case 400: { /* mpint_out_str (stream,base,mpint) */
			FILE *stream = (FILE *) param_pop ();	
			int base = param_pop ();
			int ret;
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			if (stream != stdout)
				ret = mpz_out_str (stream, base, mpint);
			else {
				char *outputString = mpz_get_str (NULL, base, mpint);
				ret = strlen (outputString);
				output (stream, outputString);
				free (outputString);
			}
			param_push ((int) ret);
		} break;
		case 401: { /* mpint_inp_str (stream,base,mpint) */
			FILE *stream = (FILE *) param_pop ();	
			int base = param_pop ();
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			param_push ((int) mpz_inp_str (mpint, stream, base));
		} break;
		case 402: { /* mpint_out_raw (stream,mpint) */
			FILE *stream = (FILE *) param_pop ();	
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			param_push ((int) mpz_out_raw (stream, mpint));
		} break;
		case 403: { /* mpint_inp_raw (stream,mpint) */
			FILE *stream = (FILE *) param_pop ();	
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			param_push ((int) mpz_inp_raw (mpint, stream));
		} break;

		case 500: { /* mpint_set (mpint,value) */
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			mpz_ptr value = (mpz_ptr) param_pop ();
			mpz_set (mpint, value);
		} break;
		case 501: { /* mpint_set_ui (mpint,int) */
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			int setValue = param_pop ();
			mpz_set_ui (mpint, setValue);
		} break;
		case 502: { /* mpint_set_si (mpint,int) */
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			int setValue = param_pop ();
			mpz_set_si (mpint, setValue);
		} break;
		case 503: { /* mpint_set_str (mpint,value,base) */
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			struct crec *cr = (struct crec *) param_pop ();
			char *value = cr->str;
			int base = param_pop ();
			if (checkstrcr (cr, "mpint_set_str"))
			{
				mpz_set_str (mpint, value, base);
				free_if_zero (cr);
			}
		} break;
		case 504: { /* mpint_get_ui (mpint) */
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			param_push ((int) mpz_get_ui (mpint));
		} break;
		case 505: { /* mpint_get_si (mpint) */
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			param_push ((int) mpz_get_si (mpint));
		} break;
		case 506: { /* mpint_get_str (mpint,base) */
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			int base = param_pop ();
			char *str = mpz_get_str (NULL, base, mpint);
			struct crec *stringRecord = palloc (sizeof (struct crec));
			stringRecord->refcount = 0;
			stringRecord->str = str;
			param_push ((int) stringRecord);
		} break;
		case 507: { /* mpint_copy (mpint) */
			mpz_ptr newMpint = palloc (sizeof (mpz_t));
			mpz_ptr mpint = (mpz_ptr) param_pop ();
			mpz_init_set (newMpint, mpint);
			param_push ((int) newMpint);
		} break;

		case 600: { /* mpint_random (result,size) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			int size = param_pop ();
			mpz_random (result, size);
		} break;
		case 601: { /* mpint_random2 (result,size) */
			mpz_ptr result = (mpz_ptr) param_pop ();
			int size = param_pop ();
			mpz_random2 (result, size);
		} break;
		case 602: { /* mpint_size (arg) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			param_push ((int) mpz_size (arg));
		} break;
		case 603: { /* mpint_sizeinbase (arg,base) */
			mpz_ptr arg = (mpz_ptr) param_pop ();
			int base = param_pop ();
			param_push ((int) mpz_sizeinbase (arg, base));
		} break;
	}
}
#endif
