/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: stacks.c,v 1.2 2000/04/25 00:16:21 bardslea Exp $

    
	Machine stack handling

	11 Jun 1999, AMULET Group
*/

#include <stdlib.h>
#include <stdio.h>

#include <alloc.h>

#include "lierror.h"
#include "../bytecode/bytecode.h"
#include "regs.h"
#include "stacks.h"


char * stacknames [NUM_STKS] = {
#include "stacknames"
};


void init_stack ( struct stack * stack )
{
  stack->top_chunk=NULL;
  stack->index=STK_CHUNK_SZ-1;
}


void push ( struct stack * stack, int data )
{
  struct stack_chunk * newchunk;

  stack->index++;
  if (stack->index==STK_CHUNK_SZ)
  {
    newchunk=palloc(sizeof(struct stack_chunk));
    newchunk->prev_chunk=stack->top_chunk;
    stack->top_chunk=newchunk;
    stack->index=0;
  }
  stack->top_chunk->data[stack->index]=data;
}


int pop ( struct stack * stack )
{
  struct stack_chunk * newchunk;
  int data;
  
  if (!stack->top_chunk)
  {
    li_fatal("stack underflow\n");
    return 0;
  }
  data=stack->top_chunk->data[stack->index];
  if (stack->index==0)
  {
    newchunk=stack->top_chunk->prev_chunk;
    free(stack->top_chunk);
    stack->top_chunk=newchunk;
    stack->index=STK_CHUNK_SZ;
  }
  stack->index--;
  return data;
}


void popforget ( struct stack * stack, int popcount )
{
  if (popcount==0)
    return;

  if (!stack->top_chunk)
  {
    li_fatal("stack underflow (popforget)\n");
    return;
  }
  if (popcount>stack->index)
  {
    struct stack_chunk * newchunk;
    int left_to_pop;

    left_to_pop=popcount-stack->index-1;
    newchunk=stack->top_chunk->prev_chunk;
    free(stack->top_chunk);
    stack->top_chunk=newchunk;
    stack->index=STK_CHUNK_SZ-1;
    popforget(stack,left_to_pop);
    return;
  }

  stack->index=stack->index-popcount;
}


struct stack * stacks;


void init_stacks ( struct stack * stacks )
{
  int i;
  for(i=0;i<NUM_STKS;i++)
    init_stack(&stacks[i]);
}


void f_PUSH ( int stk_num, int data_reg )
{
  push(&stacks[stk_num],regs[data_reg]);
}
  

void f_POP ( int stk_num, int data_reg )
{
  regs[data_reg]=pop(&stacks[stk_num]);
}


void f_POPFORGET ( int stk_num, int popcountreg )
{
  popforget(&stacks[stk_num],regs[popcountreg]/4);
}


int param_pop(void)
{
  return(pop(&stacks[STK_PS]));
}


void param_push(int n)
{
  push(&stacks[STK_PS],n);
}



