/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: stringops.c,v 1.2 2000/04/25 00:16:21 bardslea Exp $


    Strings library builtins

	11 Jun 1999, AMULET Group
*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <alloc.h>
#include <simple.h>

#include "stringops.h"
#include "stacks.h"
#include "lierror.h"

/* stringop: despatch calls to the builtin `stringops' providing
	functions for string manipulation */
void stringop ( void )
{
	int service = param_pop ();

	switch (service)
	{
		case 0: /* c2s */
			l_c2s ();
			break;	
		case 1: /* strindex, string[int] */
			l_strindex ();
			break;
		case 2: /* strlen */
			l_strlen ();
			break;
		case 3: /* strcat, @ */
			l_strcat ();
			break;
		case 4: /* strcmp */
			l_strcmp ();
			break;
		case 5: /* substr, string[range(int)] */
			l_substr ();
			break;
		case 6: /* strdchr */
			l_strdchr ();
			break;
		case 7: /* strdstr */
			l_strdstr ();
			break;
	}
}

void free_if_zero ( struct crec * cr )
{
      /* free the control record and string if ref count is zero */
  if (cr->refcount==0)
  {
    if (cr->str)
    {
      free(cr->str);
    }
    free(cr);
  }
}


void l_c2s ( void )
{
      /* create a new singleton string containing a supplied character */
  char c;
  struct crec * cr;
  char * s;
  
  c=(char)param_pop();
  cr=palloc(sizeof(struct crec));
  cr->refcount=0;
  s=palloc(2);
  s[0]=c;
  s[1]='\0';
  cr->str=s;
  param_push((int)cr);
}


bool checkstrcr ( struct crec * cr, char * func )
{
  if (!cr)
  {
    li_fatal("%s: uninitialised string (newstringvar not called?)\n",func);
    return FALSE;
  }
  if (!cr->str)
  {
    li_fatal("%s: undefined string (newstringvar called, but no assignment before this read?)\n",func);
    return FALSE;
  }
  return TRUE;
}

void l_strindex ( void )
{
      /* return the nth character of a string */
  struct crec * cr;
  int i;
  int len;
  char c;

  cr=(struct crec *)param_pop();
  i=param_pop();
  if (!checkstrcr(cr,"strindex"))
  {
    return;
  }
  len=strlen(cr->str);
  if (i>=len || (i<0 && -i>len))
  {
    li_fatal("strindex: index %d out of range (string length %d)",i,len);
    return;
  }
  c=cr->str[(i<0 ? len+i : i)];
  param_push((int)c);
  free_if_zero(cr);
}


void l_strlen ( void )
{
      /* return the length of a string */
  struct crec * cr;
  int len;

  cr=(struct crec *)param_pop();
  if (!checkstrcr(cr,"strlen"))
  {
    return;
  }
  len=strlen(cr->str);
/*  fprintf(stderr,"*** STRLEN: refcount is %d, string is %s, length is %d\n",cr->refcount,cr->str,len); */
  param_push(len);
  free_if_zero(cr);
}


void l_strcat ( void )
{
      /* concatenate two strings */
  struct crec * cr1;
  struct crec * cr2;
  struct crec * cr3;
  char * s;

  cr1=(struct crec *)param_pop();
  cr2=(struct crec *)param_pop();

  if (!checkstrcr(cr1,"strcat (string 1)"))
  {
    return;
  }
  if (!checkstrcr(cr2,"strcat (string 2)"))
  {
    return;
  }
  s=palloc(strlen(cr1->str)+strlen(cr2->str)+1);
  strcpy(s,cr1->str);
  strcat(s,cr2->str);
  cr3=palloc(sizeof(struct crec));
  cr3->refcount=0;
  cr3->str=s;
  param_push((int)cr3);
  free_if_zero(cr1);
  free_if_zero(cr2);
}


void int2str ( void )
{
  int i;
  char b[128];
  char * s;
  struct crec * cr;

  i=param_pop();
  sprintf(b,"%d",i);
  s=palloc(strlen(b)+1);
  strcpy(s,b);
  cr=palloc(sizeof(struct crec));
  cr->refcount=0;
  cr->str=s;
  param_push((int)cr);
}


void int2hexstr ( void )
{
  int i;
  char b[128];
  char * s;
  struct crec * cr;
  
  i=param_pop();
  sprintf(b,"%08X",i);
  s=palloc(strlen(b)+1);
  strcpy(s,b);
  cr=palloc(sizeof(struct crec));
  cr->refcount=0;
  cr->str=s;
  param_push((int)cr);
}


void l_strcmp ( void )
{
  struct crec * cr1;
  struct crec * cr2;
  char * s1;
  char * s2;
  int cmp_res;
  
  cr1=(struct crec *)param_pop();
  cr2=(struct crec *)param_pop();
  if (!checkstrcr(cr1,"strcmp (string 1)"))
  {
    return;
  }
  if (!checkstrcr(cr2,"strcmp (string 2)"))
  {
    return;
  }
  s1=cr1->str;
  s2=cr2->str;
  cmp_res=strcmp(s1,s2);
  param_push(cmp_res);
  free_if_zero(cr1);
  free_if_zero(cr2);
}

/* l_substr : l_substr(str,lowindex,highindexplus1) returns a
	string containing the portion of the given string between indices
	lowindex and highindexplus1-1. Barf on any index errors. Setting
	highindexplus1 to the same value as lowindex returns the empty
	string */
void l_substr ( void )
{
	struct crec *str = (struct crec *) param_pop ();
	int lowIndex = param_pop ();
	int highIndexPlus1 = param_pop ();

	struct crec *ret = palloc (sizeof (struct crec));
	ret->str = NULL;
	ret->refcount = 0;

	if (highIndexPlus1 < lowIndex || lowIndex < 0)
	{
		li_fatal ("substring: index error lowindex: %d, highindexplus1: %d\n",
			lowIndex, highIndexPlus1);
		return;
	}
	if (! checkstrcr (str, "substring")) return;
	else {
		int strLen = strlen (str->str);

		if ((highIndexPlus1 - 1) > strLen)
		{
			li_fatal ("substring: string arg length (%d) less than high index (%d)\n",
				strLen, highIndexPlus1);
			return;
		} else {
			ret->str = palloc (1 + highIndexPlus1 - lowIndex);
			strncpy (ret->str, str->str + lowIndex, highIndexPlus1 - lowIndex);
			(ret->str)[highIndexPlus1 - lowIndex] = '\0';
		}
		free_if_zero (str);
	}
	param_push ((int) ret);
}

/* l_strdchr : find a character in the given string starting at the given index
	(which may be negative indicating a count from the end of the string) and in
	the given direction (true=forward, false=backwards) */
void l_strdchr ( void )
{
	struct crec *str = (struct crec *) param_pop ();
	char chr = (char) param_pop ();
	int startIndex = param_pop ();
	int strLen, ret;
	bool forwardNbackwards = (bool) param_pop ();
	char *chrPos;
	char tempChar;

	if (! checkstrcr (str, "substring")) return;

	strLen = strlen (str->str);

	if (startIndex >= strLen || (startIndex < 0 && -startIndex > strLen))
	{
    	li_fatal("strdchr: start index %d out of range (string length %d)",startIndex,strLen);
		return;
	}
	startIndex = (startIndex < 0 ? strLen+startIndex : startIndex);
	if (forwardNbackwards) chrPos = strchr (str->str + startIndex, chr);
	else {
		tempChar = str->str[startIndex + 1];	
		str->str[startIndex + 1] = '\0';
		chrPos = strrchr (str->str, chr);
		str->str[startIndex + 1] = tempChar;
	}

	ret = (chrPos ? chrPos - str->str : -1);
	free_if_zero (str);

	param_push (ret);
}

/* l_strdstr: like l_strdchr but for strings */
void l_strdstr ( void )
{
	struct crec *lhs = (struct crec *) param_pop ();
	struct crec *rhs = (struct crec *) param_pop ();
	int startIndex = param_pop ();
	int strLenLhs, strLenRhs, ret;
	bool forwardNbackwards = (bool) param_pop ();
	char *chrPos;
	char tempChar;

	if (! checkstrcr (lhs, "substring")) return;
	if (! checkstrcr (rhs, "substring")) return;

	strLenLhs = strlen (lhs->str);
	strLenRhs = strlen (rhs->str);

	if (startIndex >= (strLenLhs - strLenRhs)
		|| (startIndex < 0 && -startIndex > (strLenLhs - strLenRhs)))
	{
    	li_fatal("strdstr: start index %d out of range (string length %d, test string length %d)",
			startIndex, strLenLhs, strLenRhs);
		return;
	}
	startIndex = (startIndex < 0 ? strLenLhs+startIndex : startIndex);
	if (forwardNbackwards) chrPos = strstr (lhs->str + startIndex, rhs->str);
	else {
		char *lastMatch = chrPos;
		startIndex += strLenRhs;
		tempChar = lhs->str[startIndex];	
		lhs->str[startIndex] = '\0';

		lastMatch = NULL;
		chrPos = lhs->str;
		do {
			chrPos = strstr (chrPos+1, rhs->str);
			if (chrPos) lastMatch = chrPos;
		} while (chrPos);

		lhs->str[startIndex] = tempChar;
		chrPos = lastMatch;	
	}

	ret = (chrPos ? chrPos - lhs->str : -1);
	free_if_zero (lhs);
	free_if_zero (rhs);

	param_push (ret);

}
