/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: tcl_if.c,v 1.4 2000/10/23 11:00:16 bardslea Exp $


	Tcl/interpreter interface

	11 Jun 1999, AMULET Group
*/

#include <tcl.h>
#include <tk.h>
#include <tclExtend.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <alloc.h>

#include "stacks.h"
#include "exec.h"
#include "start.h"
#include "lierror.h"
#include "debug.h"
#include "io.h"
#include "threads.h"
#include "linenos.h"
#include "stringops.h"
#include "profile.h"
#include "format.h"
#include "lard_link.h"

Tcl_Interp * bci_interp;


/* tcl functions called from lard */

void export_var ( void )
{
  int * v;
  char * name;
  struct crec * cr;
  
  v=(int *)param_pop();
  cr=(struct crec *)param_pop();
  name=cr->str;
  if (!name)
  {
    li_fatal("Uninitialised string passed to export_var");
    return;
  }
  if (Tcl_LinkVar(bci_interp,name,(char *)v,TCL_LINK_INT)==TCL_ERROR)
  {
    li_fatal(Tcl_GetStringResult(bci_interp));
  }
  free_if_zero(cr);
}
  

void update_export_var ( void )
{
  char * name;
  struct crec * cr;

  cr=(struct crec *)param_pop();
  name=cr->str;
  if (!name)
  {
    li_fatal("Uninitialised string passed to update_export_var");
    return;
  }
  Tcl_UpdateLinkedVar(bci_interp,name);
  free_if_zero(cr);
}


void tcl_cmd ( char * cmd )
{
  char * msg;

  if (Tcl_VarEval(bci_interp,cmd,NULL)==TCL_ERROR)
  {
    msg = Tcl_GetVar(bci_interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL) {
      msg = Tcl_GetStringResult(bci_interp);
    }
    li_fatal(msg);
  }
}


void tcl_cmd_builtin ( void )
{
  char * cmd;
  struct crec * cr;

  cr=(struct crec *)param_pop();
  cmd=cr->str;
  if (!cmd)
  {
    li_fatal("uninitialised string passed to tcl_cmd");
    return;
  }
  tcl_cmd(cmd);
  free_if_zero(cr);
}


void tcl_cmd_noerr ( char * cmd )
{
  Tcl_VarEval(bci_interp,cmd,NULL);
}


void tcl_cmd_noerr_builtin ( void )
{
  char * cmd;
  struct crec * cr;

  cr=(struct crec *)param_pop();
  cmd=cr->str;
  if (!cmd)
  {
    li_fatal("uninitialised string passed to tcl_cmd_noerr");
    return;
  }
  tcl_cmd_noerr(cmd);
  free_if_zero(cr);
}


int tcl_cmd_int ( char * cmd )
{
  char * msg;
  int r;
  Tcl_Obj * obj;
  
  if (Tcl_VarEval(bci_interp,cmd,NULL)==TCL_ERROR)
  {
    msg = Tcl_GetVar(bci_interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL) {
      msg = Tcl_GetStringResult(bci_interp);
    }
    li_fatal(msg);
  }
  obj=Tcl_GetObjResult(bci_interp);
  if (Tcl_GetIntFromObj(bci_interp,obj,&r)==TCL_ERROR)
  {
    msg=Tcl_GetStringResult(bci_interp);
    li_fatal(msg);
  }
  return r;
}


void tcl_cmd_int_builtin ( void )
{
  char * cmd;
  struct crec * cr;
  int r;
  
  cr=(struct crec *)param_pop();
  cmd=cr->str;
  if (!cmd)
  {
    li_fatal("uninitialised string passed to tcl_cmd_int");
    return;
  }
  r=tcl_cmd_int(cmd);
  free_if_zero(cr);
  param_push(r);
}


char * tcl_cmd_str ( char * cmd )
{
  char * msg;
  char * s;
  char * r;
  
  if (Tcl_VarEval(bci_interp,cmd,NULL)==TCL_ERROR)
  {
    msg = Tcl_GetVar(bci_interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL) {
      msg = Tcl_GetStringResult(bci_interp);
    }
    li_fatal(msg);
  }
  s=Tcl_GetStringResult(bci_interp);
  r=palloc(strlen(s)+1);
  strcpy(r,s);
  return r;
}


void tcl_cmd_str_builtin ( void )
{
  char * cmd;
  struct crec * cr;
  char * r;
  struct crec * rcr;
  
  cr=(struct crec *)param_pop();
  cmd=cr->str;
  if (!cmd)
  {
    li_fatal("uninitialised string passed to tcl_cmd_str");
    return;
  }
  r=tcl_cmd_str(cmd);
  free_if_zero(cr);
  rcr=palloc(sizeof(struct crec));
  rcr->refcount=0;
  rcr->str=r;
  param_push((int)rcr);
}

int bci_load_unlinked(ClientData clientData, Tcl_Interp * interp,
              int argc, char * argv[])
{
  if (argc!=2)
  {
    Tcl_SetResult(interp,"Wrong # args to bci_load_unlinked",TCL_VOLATILE);
    return TCL_ERROR;
  }
	lard_norelocs = TRUE;
	lard_istrip = TRUE;
	lard_estrip = TRUE;

	lard_link_load_file (argv[1]);
	return TCL_OK;
}

/* bcode interpreter functions called from tcl */

int bci_load_internal(ClientData clientData, Tcl_Interp * interp,
              int argc, char * argv[])
{
  bool r;

  if (argc!=2)
  {
    Tcl_SetResult(interp,"Wrong # args to bci_load_internal",TCL_VOLATILE);
    return TCL_ERROR;
  }
  Tcl_SetVar(interp,"bci_filename",argv[1],TCL_GLOBAL_ONLY);

  r=start(argv[1]);
  llink ();

  if (lard_norelocs && lard_relocsectsize>0)
  {
    li_fatal("Unresolved relocations found\n");
  }

  Tcl_UpdateLinkedVar(interp,"bci_ready");
  if (r)
  {
    return TCL_OK;
  } else {
    Tcl_SetResult(interp,bci_error_msg,TCL_VOLATILE);
    return TCL_ERROR;
  }
}


int bci_exec(ClientData clientData, Tcl_Interp * interp,
             int argc, char * argv[])
{
  bool r;

  if (argc!=2)
  {
    Tcl_SetResult(interp,"Wrong # args to bci_exec",TCL_VOLATILE);
    return TCL_ERROR;
  }
  bci_interp=interp;
  steppoint_break=FALSE;
/*  fprintf(stderr,"before exec: steppoint_break=%d\n",steppoint_break); */
  r=exec(atoi(argv[1]));
/*  fprintf(stderr,"after exec: steppoint_break=%d\n",steppoint_break); */
  if (!bci_ready)
  {
    Tcl_UpdateLinkedVar(interp,"bci_ready");
  }
  if (steppoint_break)
  {
    Tcl_UpdateLinkedVar(interp,"line_steppoint");
  }
  if (r)
  {
    return TCL_OK;
  } else {
    Tcl_SetResult(interp,bci_error_msg,TCL_VOLATILE);
    return TCL_ERROR;
  }
}


int bci_kill(ClientData clientData, Tcl_Interp * interp,
             int argc, char * argv[])
{
  if (argc!=1)
  {
    Tcl_SetResult(interp,"Wrong # args to bci_kill",TCL_VOLATILE);
    return TCL_ERROR;
  }

  bci_ready=FALSE;
  Tcl_UpdateLinkedVar(interp,"bci_ready");

      /* This isn't implemented.  We need some way of freeing all the
         memory allocated by the last instance of the interpreter.  */
  
  return TCL_OK;
}


int bci_profile_on(ClientData clientData, Tcl_Interp * interp,
                   int argc, char * argv[])
{
  if (argc!=1)
  {
    Tcl_SetResult(interp,"Wrong # args to bci_profile_on",TCL_VOLATILE);
    return TCL_ERROR;
  }

  if (!bci_ready)
  {
    Tcl_SetResult(interp,"Please call bci_load before bci_profile_on",TCL_VOLATILE);
    return TCL_ERROR;
  }

  init_profile();
  
  return TCL_OK;
}


int bci_profile_off(ClientData clientData, Tcl_Interp * interp,
                   int argc, char * argv[])
{
  if (argc!=1)
  {
    Tcl_SetResult(interp,"Wrong # args to bci_profile_off",TCL_VOLATILE);
    return TCL_ERROR;
  }

  if (!do_profile)
  {
    Tcl_SetResult(interp,"Profiling not enabled",TCL_VOLATILE);
    return TCL_ERROR;
  }

  print_profile();
  
  return TCL_OK;
}

/* lard_format: formal a value for display in the channel viewer */
int lard_format (ClientData clientData, Tcl_Interp * interp,
	     int argc, char *argv[])
{
	void **formatArgs;
	char *formattedVal;
	int argCount;

    if (argc != 4)
    {
		Tcl_SetResult (interp, "Wrong # args to lard_format: "
			"lard_format <format> <argCount> <data>", TCL_VOLATILE);
		return TCL_ERROR;
    }

	argCount = atoi (argv[2]);
	formatArgs = make_format_array (argCount, argv[3]);
	formattedVal = lard_format_from_array (argv[1], argCount, formatArgs);
	free (formatArgs);

	Tcl_SetResult (interp, formattedVal, TCL_VOLATILE);

    return TCL_OK;
}

void create_commands ( Tcl_Interp * interp )
{
  Tcl_CreateCommand(interp,"bci_load_internal",bci_load_internal,NULL,NULL);
  Tcl_CreateCommand(interp,"bci_load_unlinked",bci_load_unlinked,NULL,NULL);
  Tcl_CreateCommand(interp,"bci_exec",bci_exec,NULL,NULL);
  Tcl_CreateCommand(interp,"bci_kill",bci_kill,NULL,NULL);
  Tcl_CreateCommand(interp,"bci_profile_on",bci_profile_on,NULL,NULL);
  Tcl_CreateCommand(interp,"bci_profile_off",bci_profile_off,NULL,NULL);
  Tcl_CreateCommand(interp,"lard_format",lard_format,NULL,NULL);
  
  Tcl_LinkVar(interp,"bci_debug",(char *)&bci_debug,TCL_LINK_BOOLEAN);
  Tcl_LinkVar(interp,"bci_ready",(char *)&bci_ready,TCL_LINK_BOOLEAN);
  Tcl_LinkVar(interp,"stdin_line",(char *)&stdin_line,TCL_LINK_STRING);
  Tcl_LinkVar(interp,"bci_current_thread",(char *)&current_tid,TCL_LINK_INT);
  Tcl_LinkVar(interp,"line_linenum",(char *)&current_linenum,TCL_LINK_INT);
  Tcl_LinkVar(interp,"line_fileref",(char *)&current_fileref,TCL_LINK_INT);
  Tcl_LinkVar(interp,"line_steppoint",(char *)&steppoint_break,TCL_LINK_BOOLEAN);

  
}
