/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: threads.c,v 1.7 2000/10/23 11:44:22 bardslea Exp $

	Thread management

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>
#include <string.h>
#include <time.h>

#include <alloc.h>
#include <simple.h>

#include "stacks.h"
#include "regs.h"
#include "lierror.h"
#include "../bytecode/bytecode.h"
#include "threads.h"

int yields = 0;

struct thread
{
    int *regs;
    struct stack *stacks;
    int tid;
    int priority;
    int timebase;
    struct thread *next;
    struct thread *prev;
};

/*
struct thread * root_thread=NULL;
*/
struct thread *currentthread;

struct thread *root_thread[] = { NULL, NULL, NULL, NULL, NULL };

int max_tid = 1;

int current_tid;

struct thread **thread_array = NULL;

#define THREAD_ARRAY_CHUNK_SIZE 1024

int thread_array_size = 0;

#undef DEBUGGGG

int new_thread (void)
{
    struct thread *p;

    if (thread_array_size <= max_tid)
    {
        thread_array_size += THREAD_ARRAY_CHUNK_SIZE;
        thread_array =
          (struct thread **) prealloc (thread_array,

          sizeof (struct thread *) * thread_array_size);
    }

    p = palloc (sizeof (struct thread));
    p->regs = palloc (NUM_REGS * sizeof (int));
    p->stacks = palloc (NUM_STKS * sizeof (struct stack));

    init_stacks (p->stacks);
    p->tid = max_tid;
    p->priority = PRI_RUN;
    p->timebase = 0;            /*clock(); */
    p->next = root_thread[PRI_RUN];
    if (root_thread[PRI_RUN])
        root_thread[PRI_RUN]->prev = p;
    p->prev = NULL;
    root_thread[PRI_RUN] = p;

    thread_array[max_tid] = p;

    max_tid++;

#ifdef DEBUGGGG
    fprintf (stderr, "New thread %d\n", p->tid);
#endif

    return p->tid;
}

void reschedule (void)
{
    int pri;

    for (pri = PRI_RUN; pri > PRI_DEMWAIT; pri--)
    {
        if (root_thread[pri])
        {
            currentthread = root_thread[pri];
            regs = currentthread->regs;
            stacks = currentthread->stacks;
            current_tid = currentthread->tid;
#ifdef DEBUGGGG
            fprintf (stderr, "Choosing thread %d %d %d\n", current_tid,
              currentthread->priority, pri);
            fprintf (stderr, "Choosing %d %d\n", (int) currentthread->prev,
              (int) currentthread->next);
#endif
            return;
        }
    }
    li_fatal ("No more unsuspended threads to schedule\n");
}

void init_threads (void)
{

    new_thread ();
    currentthread = root_thread[PRI_RUN];
    regs = currentthread->regs;
    memset (regs, 0, sizeof (int) * NUM_REGS);

    regs[REG_TID] = 1;
    stacks = currentthread->stacks;
    current_tid = 1;
    currentthread->timebase = clock ();
}

void f_FORK (int tid_reg)
{
    int tid;

    tid = new_thread ();

#ifdef DEBUGGGG
    fprintf (stderr, "Forking %d\n", tid);
#endif

    memcpy (root_thread[PRI_RUN]->regs, regs, NUM_REGS * sizeof (int));

    regs[tid_reg] = tid;
    root_thread[PRI_RUN]->regs[tid_reg] = 0;
    root_thread[PRI_RUN]->regs[REG_TID] = tid;
}

void f_DIE (void)
{
    struct thread *nx;
    struct thread *px;
    struct thread *oldthread;

    free (currentthread->regs);
    free (currentthread->stacks);

    nx = currentthread->next;
    px = currentthread->prev;
    oldthread = currentthread;
    if (!nx && !px)
    {
        li_fatal ("No threads left!");
        return;
    }
    if (nx)
        nx->prev = px;
    if (px)
        px->next = nx;
    else
    {
        root_thread[currentthread->priority] = nx;
        nx->prev = NULL;
    }

    reschedule ();
    currentthread->timebase += clock ();

    thread_array[oldthread->tid] = NULL;

    free (oldthread);
}

void f_YIELD (void)
{
    int clk;

    yields++;
    clk = clock ();
    currentthread->timebase -= clk;
    reschedule ();
    currentthread->timebase += clk;
}

void setthreadpriority (struct thread *t, int pri)
{
#ifdef DEBUGGGG
    fprintf (stderr, "B: Setting priority of %d to %d (from %d)\n", t->tid, pri,
      t->priority);
    fprintf (stderr, "B: [%d] %d %d %d\n", (int) t, (int) t->prev,
      (int) t->next, (int) root_thread[t->priority]);
#endif
    if (t->next)
        t->next->prev = t->prev;
    if (t->prev)
        t->prev->next = t->next;
    else
        root_thread[t->priority] = t->next;

    if (root_thread[pri])
        root_thread[pri]->prev = t;
    t->next = root_thread[pri];
    t->prev = NULL;
    root_thread[pri] = t;

    t->priority = pri;
}

void f_SETMYPRI (int pri_reg)
{
#ifdef DEBUGGGG
    fprintf (stderr, "C: Setting priority of %d to %d\n", currentthread->tid,
      regs[pri_reg]);
#endif
    setthreadpriority (currentthread, regs[pri_reg]);
}

void setpriority (int tid, int pri)
{
    if (thread_array[tid])
    {
        setthreadpriority (thread_array[tid], pri);
    } else
        li_fatal ("setpriority failed - thread %d doesn't exist\n", tid);
}

void f_SETPRI (int tid_reg, int pri_reg)
{
    setpriority (regs[tid_reg], regs[pri_reg]);
}

void f_RUNCHECK (int tid_reg, int dest_reg)
{
	regs[dest_reg] = (thread_array[regs[tid_reg]] ? 1 : 0);
}

void promote_waits (void)
{
    struct thread *p;

    p = root_thread[PRI_DEMWAIT];

#ifdef DEBUGGGG
    fprintf (stderr, "Promoting waits\n");
#endif
    while (p && p->next)
    {
#ifdef DEBUGGGG
        fprintf (stderr, "A: Setting priority of %d to %d\n", p->tid, PRI_WAIT);
#endif
        p->priority = PRI_WAIT;
        p = p->next;
    }

    if (p)
    {
        p->priority = PRI_WAIT;
        if (root_thread[PRI_WAIT])
            root_thread[PRI_WAIT]->prev = p;
        p->next = root_thread[PRI_WAIT];
        root_thread[PRI_WAIT] = root_thread[PRI_DEMWAIT];

        root_thread[PRI_DEMWAIT] = NULL;
    }
}

void f_GETTIME (int reg)
{
    regs[reg] = clock () - currentthread->timebase;
}
