/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: error.c,v 1.1.1.1 1999/07/15 18:47:29 bardslea Exp $


	Error message printing

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>
#include <stdlib.h>
#include "error.h"

void error_msg ( char * filename, int linenum, char * msg )
{
  fprintf(stderr,"%s:%d:%s\n",filename,linenum,msg);
}

int yyerror(char *s)
{
  fprintf(stderr,"%s",input_buffer);
  fatal_error(line_num,s);
}

void fatal_error ( int linenum, char * fmt, ...)
{
  va_list ap;
  char buff[2048];
  
  va_start(ap,fmt);
  vsprintf(buff,fmt,ap);
  va_end(ap);
  error_msg(infilename,linenum,buff);
  exit(1);
}

void fatal ( char * fmt, ...)
{
  va_list ap;
  
  va_start(ap,fmt);
  vfprintf(stderr,fmt,ap);
  va_end(ap);
  exit(1);
}

void veryfatal ( char * msg )
{
  fputs(msg,stderr);
  exit(1);
}

void usage_error ( char * fmt, ...)
{
  va_list ap;
  char buff[2048];
  
  va_start(ap,fmt);
  vsprintf(buff,fmt,ap);
  va_end(ap);
  fprintf(stderr,"Error: %s\n",buff);
  exit(1);
}

void nonfatal_error ( int linenum, char * fmt, ...)
{
  va_list ap;
  char buff[2048];
  
  va_start(ap,fmt);
  vsprintf(buff,fmt,ap);
  va_end(ap);
  error_msg(infilename,linenum,buff);
}

void internal_error ( char * fmt, ...)
{
  va_list ap;
  char buff[2048];
  
  va_start(ap,fmt);
  vsprintf(buff,fmt,ap);
  va_end(ap);
  fprintf(stderr,"Panic!  Internal Error: %s\n",buff);
  exit(1);
}

void debug ( bool doit, char * fmt, ...)
{
  va_list ap;
  char buff[2048];
  
  if (doit)
    {
      va_start(ap,fmt);
      vsprintf(buff,fmt,ap);
      va_end(ap);
      fprintf(stderr,"%s",buff);
    }
}

void ind_debug ( bool doit, int indent, char * fmt, ...)
{
  va_list ap;
  char buff[2048];
  int i;
  
  if (doit)
    {
      va_start(ap,fmt);
      vsprintf(buff,fmt,ap);
      va_end(ap);
      for(i=0;i<indent;i++)
        fprintf(stderr,"  ");
      fprintf(stderr,"%s",buff);
    }
}
