/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: strtoint.c,v 1.1.1.1 1999/07/15 18:47:29 bardslea Exp $


	Portable strtoul for libcs without one

	11 Jun 1999, AMULET Group
*/

#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include "simple.h"
#include "../config.h"

#ifndef HAVE_STRTOUL
/* lard_strtoul: a portable strtoul, or use the system func if present */
static char *LardDigitChars = "0123456789abcdefghijklmnopqrstuvwxyz";

unsigned long lard_strtoul (char *ptr, char **endPtr, int radix)
{
	unsigned long ret = 0;
	bool negate = FALSE;
	char *found;

	if (*ptr && *ptr == '-') { negate = TRUE; ptr ++; }
	while (*ptr && (found = strchr (LardDigitChars, tolower (*ptr)))
		&& found <= LardDigitChars + radix)
	{
		unsigned long oldRet = ret;
		ret = (ret * radix) + (found - LardDigitChars);
		if (oldRet != 0 && ret <= oldRet) { errno = -ERANGE; return 0; }
		ptr ++;
	}
	if (endPtr) *endPtr = ptr;
	if (negate) ret = -ret;
	return ret;
}

#endif
