/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: files.c,v 1.1 2000/04/26 00:55:28 bardslea Exp $


	File handling

	11 Jun 1999, AMULET Group
*/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <alloc.h>
#include <error.h>
#include <simple.h>

#include "../bytecode/bytecode.h"
#include "lard_link.h"


void lard_link_load_file ( char * fn )
{
  FILE * hf;
  FILE * f;
  struct section_desc sd;
  int num_sects;
  int i;
  int code_offset;
  char * p;
   
  hf=fopen(fn,"r");
  if (!hf)
  {
    fatal("Can't open %s\n",fn);
  }
  fread(&sd,sizeof(struct section_desc),1,hf);
  if (sd.sect_type==WRONG_MAGIC)
  {
    fatal("%s has the wrong byte sex\n",fn);
  }
  if (sd.sect_type!=MAGIC_NUMBER)
  {
    fatal("%s is not a bcode file\n",fn);
  }

  f=fopen(fn,"r");
  if (!f)
  {
    fatal("Can't open %s (2)\n",fn);
  }
  
  num_sects=sd.end/sizeof(struct section_desc);
  code_offset=lard_codesectsize/4;
  for(i=1;i<num_sects;i++)
  {
    fread(&sd,sizeof(struct section_desc),1,hf);
    if (sd.sect_type==CODE_SECTTYPE)
    {
      debug(lard_link_verbose,"Loading %s at %d - %d\n",fn,code_offset,
            code_offset+(sd.end-sd.start)/4);
      lard_codesect=prealloc(lard_codesect,lard_codesectsize+sd.end-sd.start);
      fseek(f,sd.start,SEEK_SET);
      fread(lard_codesect+lard_codesectsize,1,sd.end-sd.start,f);
      lard_codesectsize+=sd.end-sd.start;
    } else if (sd.sect_type==RELOC_SECTTYPE)
    {
      fseek(f,sd.start,SEEK_SET);
      p=palloc(sd.end-sd.start);
      fread(p,1,sd.end-sd.start,f);
      readsymtree(&lard_relocs,p,p+sd.end-sd.start,code_offset,TRUE,"");
    } else if (sd.sect_type==EXTERNSYM_SECTTYPE)
    {
      fseek(f,sd.start,SEEK_SET);
      p=palloc(sd.end-sd.start);
      fread(p,1,sd.end-sd.start,f);
      readsymtree(&lard_extsyms,p,p+sd.end-sd.start,code_offset,FALSE,"");
    } else if (sd.sect_type==INTERNSYM_SECTTYPE)
    {
      if (!lard_istrip)
      {
        char * start;
        char * end;
        char * leafname;

        start=strrchr(fn,'/');
        if (!start)
          start=fn;
        else
          start++;
        end=strchr(start,'.');
        if (!end) end=start+strlen(start);
        leafname=palloc(end-start+2);
        memcpy(leafname+1,start,end-start);
        leafname[0]='_';
        leafname[end-start+1]='\0';
        
        fseek(f,sd.start,SEEK_SET);
        p=palloc(sd.end-sd.start);
        fread(p,1,sd.end-sd.start,f);
        readsymtree(&lard_intsyms,p,p+sd.end-sd.start,code_offset,FALSE,leafname);
      }
    } else
    {
      fatal("Can't load this section!!!\n");
    }
  }
  fclose(hf);
  fclose(f);
}


void makesymsect(char ** symsect, int * symsectsize,
                 struct symtree * symtree)
{
  int slen;
  
  if (symtree)
  {
    slen=(strlen(symtree->sym)+4)&~3;
    (*symsect)=prealloc(*symsect,*symsectsize+4+slen);
    *((int *)(*symsect+*symsectsize))=symtree->addr;
    strcpy(*symsect+*symsectsize+4,symtree->sym);
    *symsectsize=*symsectsize+4+slen;
    makesymsect(symsect,symsectsize,symtree->left);
    makesymsect(symsect,symsectsize,symtree->right);
  }
}


void make_sectheader ( FILE * F, int secttype, int start, int end,
                       int p1, int p2)
{
  struct section_desc sd;

  sd.sect_type=secttype;
  sd.start=start;
  sd.end=end;
  sd.private1=p1;
  sd.private2=p2;
  fwrite(&sd,sizeof(struct section_desc),1,F);
}


void lard_link_save_file ( char * fn )
{
  FILE * F;
  int e, f;

  char * externsymsect=malloc(0);
  char * internsymsect=malloc(0);
  int externsymsectsize=0;
  int internsymsectsize=0;

  bool hascodesect;
  bool hasexternsymsect;
  bool hasinternsymsect;
  bool hasrelocsect;

  int num_sects;
  
  F=fopen(fn,"w");
  if (!F)
  {
    fatal("Can't open output file %s\n",fn);
  }

  if (!lard_estrip)
  {
    makesymsect(&externsymsect,&externsymsectsize,lard_extsyms);
  }
  
  makesymsect(&internsymsect,&internsymsectsize,lard_intsyms);

  hascodesect=(lard_codesectsize>0);
  hasexternsymsect=(externsymsectsize>0);
  hasinternsymsect=(internsymsectsize>0);
  hasrelocsect=(lard_relocsectsize>0);

  num_sects= 1 + (hascodesect?1:0) + (hasexternsymsect?1:0)
             + (hasinternsymsect?1:0) + (hasrelocsect?1:0) ;
  
  e=num_sects*sizeof(struct section_desc);
  make_sectheader(F,HEADER_SECTTYPE,0,e,0,0);
  f=e+lard_codesectsize;
  if (hascodesect) make_sectheader(F,CODE_SECTTYPE,e,f,0,0);
  e=f+externsymsectsize;
  if (hasexternsymsect) make_sectheader(F,EXTERNSYM_SECTTYPE,f,e,0,0);
  f=e+internsymsectsize;
  if (hasinternsymsect) make_sectheader(F,INTERNSYM_SECTTYPE,e,f,0,0);
  e=f+lard_relocsectsize;
  if (hasrelocsect) make_sectheader(F,RELOC_SECTTYPE,f,e,0,0);
  
  if (hascodesect) fwrite(lard_codesect,lard_codesectsize,1,F);
  if (hasexternsymsect) fwrite(externsymsect,externsymsectsize,1,F);
  if (hasinternsymsect) fwrite(internsymsect,internsymsectsize,1,F);
  if (hasrelocsect) fwrite(lard_relocsect,lard_relocsectsize,1,F);
}

  

  
