/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: symtrees.c,v 1.1 2000/04/26 00:55:30 bardslea Exp $


	Symbol trees

	11 Jun 1999, AMULET Group
*/

#include <stdio.h>
#include <string.h>

#include <alloc.h>
#include <error.h>
#include <simple.h>

#include "lard_link.h"


void addsymtotree ( struct symtree * * t, char * sym, int addr,
                    bool allow_dups )
{
  int c;

  if (!(*t))
  {
    (*t)=palloc(sizeof(struct symtree));
    (*t)->sym=sym;
    (*t)->addr=addr;
    (*t)->left=NULL;
    (*t)->right=NULL;
    (*t)->chain=NULL;
  } else
  {
    c=strcmp(sym,(*t)->sym);
    if (c==0)
    {
      if (allow_dups)
      {
        struct chainrec * c;

        c=palloc(sizeof(struct chainrec));
        c->addr=addr;
        c->chain=(*t)->chain;
        (*t)->chain=c;
      } else
      {
        fatal("Duplicate symbol %s\n",sym);
      }
    } else if (c<0)
    {
      addsymtotree(&((*t)->left),sym,addr,allow_dups);
    } else
    {
      addsymtotree(&((*t)->right),sym,addr,allow_dups);
    }
  }
}
      

void readsymtree ( struct symtree * * init, char * start, char * end,
                   int offset, bool allow_dups, char * symsuffix )
{
  char * p;
  int addr;
  char * sym;
  int suffixlen;

  suffixlen=strlen(symsuffix);
  p=start;
  while (p<end)
  {
    addr=*(int *)p;
    if (suffixlen==0)
    {
      sym=p+4;
    } else
    {
      sym=palloc(strlen(p+4)+suffixlen+1);
      strcpy(sym,p+4);
      strcat(sym,symsuffix);
    }
/*    fprintf(stderr,"adding %s at %d\n",sym,addr); */
    addsymtotree(init,sym,addr+offset,allow_dups);
    p=p+4+((strlen(p+4)+4)&~3);
  }
}


struct symtree * symtree_find_by_sym ( struct symtree * t, char * sym )
{
  int c;

  if (!t)
  {
    return(NULL);
  }
  
  c=strcmp(sym,t->sym);
  if (c==0)
  {
    return t;
  } else if (c<0)
  {
    return symtree_find_by_sym(t->left,sym);
  } else
  {
    return symtree_find_by_sym(t->right,sym);
  }
}

struct symtree * symtree_find_by_addr ( struct symtree * t, int addr )
{
  struct symtree *ret;

  if (!t)
  {
    return(NULL);
  }
  
  if (addr == t->addr)
  {
    ret = t;
  } else {
  	ret = symtree_find_by_addr (t->left, addr);
  	if (! ret) ret = symtree_find_by_addr (t->right, addr);
  }

  return ret;
}
