/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: io.l,v 1.4 2000/03/25 13:53:23 bardslea Exp $


	LARD I/O operations

	11 Jun 1999, AMULET Group
*/

io:lib=(

  /* tfile: text files */

  tfile:type=scalar.

  stdin:expr(tfile).
  stdout:expr(tfile).
  stderr:expr(tfile).

  topenin(fn:val(string)) : expr(tfile).
  topenout(fn:val(string)) : expr(tfile).

  close:overloaded.
  close(f:val(tfile)) : expr(void).

  fprintint(f:val(tfile),n:val(int)) : expr(void).
  fprintchar(f:val(tfile),c:val(char)) : expr(void).
  fprintstr(f:val(tfile),s:val(string)) : expr(void).

  freadint(f:val(tfile)) : expr(int) .
  freadchar(f:val(tfile)) : expr(char) .
  freadstr(f:val(tfile)) : expr(string) .

  printint(n:val(int)) : expr(void) = fprintint(stdout,n).
  printchar(c:val(char)) : expr(void) = fprintchar(stdout,c).
  printstr(s:val(string)) : expr(void) = fprintstr(stdout,s).

  readint : expr(int) = freadint(stdin).
  readchar : expr(char) = freadchar(stdin).
  readstr : expr(string) = freadstr(stdin).

  eof:overloaded.
  eof(f:val(tfile)) : expr(bool).

  set_default_input_base(base:val(int)) : expr(void).

  fprinthex(f:val(tfile),i:val(int)):expr(void)=fprintstr(f,i2hex(i)).
  printhex(i:val(int)):expr(void)=fprinthex(stdout,i).

  print:varargs.
  
  print1:overloaded.

  print1(s1:val(string)):expr(void)=(
      printstr(s1)
  ).

  print1(i1:val(int)):expr(void)=(
      printint(i1)
  ).

  print1(c1:val(char)):expr(void)=(
      printchar(c1)
  ).

  print1(b1:val(bool)):expr(void)=(
      if b1 then printstr("true") else printstr("false")
  ).

  print1(iv:val(int^(n:val(int)))):expr(void)=(
      c:var(int).
      
      printstr("[");
      if n!=0 then (
        forseq c in 0 to (n-1) do (
            if (c!=0) then printstr(",");
            printint(iv[c])
        )
      );
      printstr("]")
  ).

  print2(s1:val(string),s2:val(string)):expr(void)=(
      printstr(s1);
      printstr(s2)
  ).

  print3(s1:val(string),s2:val(string),s3:val(string)):expr(void)=(
      printstr(s1);
      printstr(s2);
      printstr(s3)
  ).

  print4(s1:val(string),s2:val(string),s3:val(string),
          s4:val(string)):expr(void)=(
      printstr(s1);
      printstr(s2);
      printstr(s3);
      printstr(s4)
  ).

  print5(s1:val(string),s2:val(string),s3:val(string),
          s4:val(string),s5:val(string)):expr(void)=(
      printstr(s1);
      printstr(s2);
      printstr(s3);
      printstr(s4);
      printstr(s5)
  ).

  print6(s1:val(string),s2:val(string),s3:val(string),
          s4:val(string),s5:val(string),s6:val(string)):expr(void)=(
      printstr(s1);
      printstr(s2);
      printstr(s3);
      printstr(s4);
      printstr(s5);
      printstr(s6)
  ).

  print7(s1:val(string),s2:val(string),s3:val(string),
          s4:val(string),s5:val(string),s6:val(string),
          s7:val(string)):expr(void)=(
      printstr(s1);
      printstr(s2);
      printstr(s3);
      printstr(s4);
      printstr(s5);
      printstr(s6);
      printstr(s7)
  ).

  
  /* bfile: binary files */

  bfile:type=scalar.

  bopenin(fn:val(string)) : expr(bfile).
  bopenout(fn:val(string)) : expr(bfile).
  bopeninout(fn:val(string)) : expr(bfile).

  fsetunbuffered(f:val(bfile)) : expr(void).

  close(f:val(bfile)) : expr(void).

  fputc(f:val(bfile),c:val(char)) : expr(void).
  fgetc(f:val(bfile)) : expr(char) .

  fwrite(f:val(bfile),data:val(T:type)) : expr(void).
  fread(f:val(bfile),data:var(T:type)) : expr(void).

  eof(f:val(bfile)) : expr(bool).
  
  fseek(f:val(bfile),pos:val(int)):expr(void).
  ftell(f:val(bfile)):expr(int).
  flen(f:val(bfile)):expr(int).
  ftrunc(f:val(bfile)):expr(void).

  /* nbfile: non blocking binary file input, polls each tick */

  nbfile:type=scalar.

  nbopenin(fn:val(string)) : expr(nbfile).
  nbfread(f:val(nbfile),data:var(T:type)) : expr(bool).
  close(f:val(nbfile)) : expr(void).

  islib
).


null

