/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: bitfields.l,v 1.1.1.1 1999/07/15 18:47:46 bardslea Exp $


    Bitfield operations and miscellaneous new numerical ops.

	11 Jun 1999, AMULET Group
*/

/* Misc. integer ops */

abs : overloaded.
abs (a:val(int)) : expr(int) = (
	if a<0 then -a else a
).

min : overloaded.
min(l:val(int),r:val(int)) : expr(int) = (
	if l<r then l else r
).

max : overloaded.
max(l:val(int),r:val(int)) : expr(int) = (
	if l>r then l else r
).

setbit : overloaded.
setbit (v:var(int),i:val(int)) : expr(void) = (
  if (i<32) && (i>0)
  then ( v := v or (1<<i) )
).

/* Misc. range ops */

min(r:val(range(int))) : expr(int) = (
  min(r->left_index,r->right_index)
).

max(r:val(range(int))) : expr(int) = (
  max(r->left_index,r->right_index)
).

rangewidth : overloaded.
rangewidth(r:val(range(int))) : expr(int) = (
  1 + abs (r->left_index - r->right_index)
).

bigendrange : overloaded.
bigendrange(r:val(range(int))) : expr(range(int)) = (
  if r->left_index>r->right_index
  then r
  else (cast(range(int), {r->right_index,r->left_index}))
).

bitmask : overloaded.
bitmask(r:val(range(int))) : expr(int) = (
  mask : var(int) .
  mask := 0;
  setbit(mask,rangewidth(r));
  (mask - 1) << min(r)
).

(v:val(int)) [ (r:val(range(int))) ] : expr(int) = (
  (v and bitmask(r)) >> min(r)
).

(v:val(int)) [ (i:val(int)) ] : expr(int) = (
  (v >> i) and 1
).

signextend : overloaded.
signextend (value:val(int), width:val(int)) : expr(int) =
(
	mask : var(int) .
	mask := bitmask (width-1 to 0);
	if value[width - 1] = 0
	then ( mask and value )
	else ( (not mask) or value )
) .

strnormindex (str:val(string),i:val(int)) : expr(int) = (
	ret : var(int) .
	newstringvalparam (str);
	ret := (if i >= 0 then i else ( strlen (str) + i ));
	endstringvalparam (str);
	ret
) .

/* It's convenient to define this here */

(a:val(string)) [ (r:val(range(int))) ] : expr(string) = (
	left : var(int) .
	right : var(int) .
	ret : var(string) .
	newstringvar (ret);
	newstringvalparam (a);
	left := strnormindex (a, r->left_index);
	right := strnormindex (a, r->right_index);
	ret := substr (a, min (left, right), max (left, right) + 1);
	endstringvalparam (a);
	returnstringvar (ret)
) .

(a:val(string)) [ (r:val(int)) ] : expr(char) = (
	strindex (a, r)
) .

null
