/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: ctrl.l,v 1.4 2000/03/25 13:42:11 bardslea Exp $


   This file declares the basic flow of control structures of the
   standard language.

	11 Jun 1999, AMULET Group
*/

/* sequential flow-of-control */

_;_ : infix(40,right).
(S1:expr(void)) ; (S2:expr(T:type)) : expr(T).

if_then_ : infix(60,none).
if (c:val(bool)) then (e1:expr(void)) : expr(void).

if_then_else_ : infix(60,none).
if (c:val(bool)) then (e1:expr(T:type)) else (e2:expr(T)) : expr(T).

while_do_ : infix(60,none).
while (c:expr(bool)) do (S:expr(void)) : expr(void).

forever (e:expr(void)) : expr(void) = while true do e.

repeat_until_ : infix(60,none).
repeat (S:expr(void)) until (c:expr(bool)) : expr(void) = (
    S;
    while (!c) do S
).

forseq_in_do_ : infix(60,none).
forseq (v:var(scalar)) in (r:val(range(scalar))) do
       (e:expr(void)) : expr(void) = (
  v:=r->left_index;
  while v<=r->right_index do (
    e;
    v:=succ(v)
  )
).

forseq_in_step_do_ : infix(60,none).
forseq (v:var(scalar)) in (r:val(range(scalar))) step (s:val(int)) do
       (e:expr(void)) : expr(void) = (
  v:=r->left_index;
  if s>0 then (
    while v<=r->right_index do (
      e;
      v:=cast(int,v)+s
	)
  ) else if s<0 then (
    while v>=r->right_index do (
      e;
      v:=cast(int,v)+s
    )
  )
).

foreach_in_do_ : infix(60,none).
foreach (v:var(T:type)) in (A:val(T^(n:val(int)))) do (e:expr(void))
        : expr(void) = (
 i:var(int).
 forseq i in 0 to (n-1) do (
   v:=A[i];
   e
 )
).
 

/* Exit */

fatal(msg:val(string)):expr(void).

warning(msg:val(string)):expr(void).

atexit(e:expr(void)):expr(void).

exit(x:val(int)):expr(void).

lardpos:expr(string).

/* concurrent flow of control */

_is_ : infix(60,right).
(label:val(string)) is (e:expr(T:type)) : expr(T) = (
    r:var(T).
    
    newstringvalparam(label);
    tcl_cmd_noerr("pushblockname {"@label@"}");
    r:=e;
    tcl_cmd_noerr("popblockname");
    endstringvalparam(label);
    r
).

spawn(e:expr(void),fpnum:val(int)) : expr(int).

running(tid:val(int)) : expr(bool).

mytid:expr(int).

yield : expr(void).

priority:type=scalar.
PRI_SUSP:val(priority).
PRI_DEMWAIT:val(priority).
PRI_TIMEADV:val(priority).
PRI_WAIT:val(priority).
PRI_RUN:val(priority).

setmypriority(pri:val(priority)):expr(void).

setpriority(tid:val(int),pri:val(priority)):expr(void).

lastrunning:expr(bool).

forparnum : expr(int).

promote_waits : expr(void).

badwaits:var(int).

wait_until(e:expr(bool)) : expr(void) = (
  promote_waits;
  while !e do (
    badwaits:=badwaits+1;
    setmypriority(PRI_DEMWAIT);
    yield
  );
/*  promote_waits; */
  setmypriority(PRI_RUN)
).

_|_ : infix(50,right).
(S1:expr(void)) | (S2:expr(void)) : expr(void) = (
  ltid:var(int).
  rtid:var(int).
  ptid:var(int).

  ptid:=mytid;
  ltid:=spawn( (S1;setpriority(ptid,PRI_RUN)) ,forparnum);
  rtid:=spawn( (S2;setpriority(ptid,PRI_RUN)) ,forparnum);

  repeat (
    setmypriority(PRI_SUSP);
    yield;
    promote_waits
  ) until ((!running(ltid)) && (!running(rtid)))
).

forpar_do_ : infix(60,none).
forpar (n:val(int)) do (e:expr(void)) : expr(void) = (
  x:var(int).
  tids:var(int^n).
  ptid:var(int).
  
  ptid:=mytid;
  forseq x in 0 to n-1 do (
    tids[x]:=spawn((e;setpriority(ptid,PRI_RUN)),x)
  );
  forseq x in 0 to n-1 do (
    while (running(tids[x])) do (
      setmypriority(PRI_SUSP);
      yield;
      promote_waits
    )
  )
).


/* assignment on arrays of strings
   see strings.l */

(lhs:var(string^(i:val(int)))) := (rhs:val(string^i)) : expr(void) = (
  a:var(int).
  forseq a in 0 to (i-1) do (
    lhs[a]:=rhs[a]
  )
).


/* Main program */

badwaits:=0;
atexit(null)


