/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: ops.l,v 1.1.1.1 1999/07/15 18:47:47 bardslea Exp $

   
    This file declares the basic operations of the standard language.

	11 Jun 1999, AMULET Group
*/

/* assignment */

_:=_ : infix(70,none).
`_:=_` : overloaded.
(lhs:var(T:type)) := (rhs:val(T)) : expr(void).


/* sizeof functions */

sizeof_type(T:type) : expr(int).
sizeof_var(V:var(T:type)) : expr(int).
sizeof_val(V:val(T:type)) : expr(int).


/* peek and poke */

peek(V:var(T:type),offset:val(int)) : expr(int).
poke(V:var(T:type),offset:val(int),value:val(int)) : expr(void).


/* casting */

cast : overloaded.
cast(T2:type,X:val(T1:type)) : expr(T2).
cast(T2:type,X:var(T1:type)) : var(T2).


/* sink */

sink(v:val(T:type)):expr(void).


/* arithmetic */

-_ : infix(180,none).
`-_` : overloaded.
- (a:val(int)) : expr(int).

_+_ : infix(160,left).
`_+_` : overloaded.
(a:val(int)) + (b:val(int)) : expr(int).

_-_ : infix(160,left).
`_-_` : overloaded.
(a:val(int)) - (b:val(int)) : expr(int).

_*_ : infix(170,left).
`_*_` : overloaded.
(a:val(int)) * (b:val(int)) : expr(int).

_/_ : infix(170,left).
`_/_` : overloaded.
(a:val(int)) / (b:val(int)) : expr(int).

_mod_ : infix(170,left).
`_mod_` : overloaded.
(a:val(int)) mod (b:val(int)) : expr(int).


/* bitwise operations */

not_ : infix(180,none).
`not_` : overloaded.
not (a:val(int)) : expr(int).

_and_ : infix(130,left).
`_and_` : overloaded.
(a:val(int)) and (b:val(int)) : expr(int).

_or_ : infix(120,left).
`_or_` : overloaded.
(a:val(int)) or (b:val(int)) : expr(int).

_xor_ : infix(125,left).
`_xor_` : overloaded.
(a:val(int)) xor (b:val(int)) : expr(int).

_<<_ : infix(155,left).
`_<<_` : overloaded.
(a:val(int)) << (b:val(int)) : expr(int).

_>>_ : infix(155,left).
`_>>_` : overloaded.
(a:val(int)) >> (b:val(int)) : expr(int).

bitsdiff : overloaded.
bitsdiff(v1:val(T:type),v2:val(T)) : expr(int).

/* scalar operations */

succ(a:val(scalar)) : expr(scalar).
pred(a:val(scalar)) : expr(scalar).
ord(a:val(scalar)) : expr(int).


/* relations */

_=_ : infix(140,left).
`_=_` : overloaded.
((a:val(T:type)) = (b:val(T))) : expr(bool).

_!=_ : infix(140,left).
`_!=_` : overloaded.
(a:val(T:type)) != (b:val(T)) : expr(bool).

_<_ : infix(150,left).
`_<_` : overloaded.
(a:val(scalar)) < (b:val(scalar)) : expr(bool).

_>_ : infix(150,left).
`_>_` : overloaded.
(a:val(scalar)) > (b:val(scalar)) : expr(bool).

_<=_ : infix(150,left).
`_<=_` : overloaded.
(a:val(scalar)) <= (b:val(scalar)) : expr(bool).

_>=_ : infix(150,left).
`_>=_` : overloaded.
(a:val(scalar)) >= (b:val(scalar)) : expr(bool).


/* boolean operators */

_&&_ : infix(110,left).
(a:val(bool)) && (b:expr(bool)) : expr(bool).

_||_ : infix(100,left).
(a:val(bool)) || (b:expr(bool)) : expr(bool).

!_ : infix(180,none).
! (a:val(bool)) : expr(bool).


/* Misc */

rand : overloaded.
rand(max:val(int)) : expr(int).


/* Main program */

null
