/*
    The Language for Asynchronous Research and Development -- LARD
    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	$Id: strings.l,v 1.1.1.1 1999/07/15 18:47:47 bardslea Exp $


    LARD Strings package

	11 Jun 1999, AMULET Group
*/

/* string type */

string : type = scalar.


/* string literal */

"" : val(string).


/* constructor and destructor functions */

newstringvar(s:var(string)):expr(void).
        /* call this for variables at the start of their scope */
        /* make a new control record with a ref count of one and a
           null string */

newstringvalparam(s:val(string)):expr(void).
        /* call this for val parameters at the start of their scope */
        /* increments reference count */

newstringvarparam(s:var(string)):expr(void).
        /* call this for var parameters at the start of their scope */
        /* does nothing */

endstringvar(s:val(string)):expr(void).
        /* call this for variables at the end of their scope, except
           when they are returned as the result of the function */
        /* decrement the reference count and free if zero */

endstringvalparam(s:val(string)):expr(void).
        /* call this for val parameters at the end of their scope, except
           when they are returned as the result of the function */
        /* decrement the reference count and free if zero */

endstringvarparam(s:val(string)):expr(void).
        /* call this for var parameters at the end of their scope, except
           when they are returned as the result of the function */
        /* does nothing */

returnstringvar(s:val(string)):expr(string).
        /* call this to return a local string variable as the result of the
           function. */
        /* decrement the reference count, but don't free */
        
returnstringvalparam(s:val(string)):expr(string).
        /* call this to return a local string val parameter as the
           result of the function. */
        /* decrement the reference count, but don't free */
        
returnstringvarparam(s:val(string)):expr(string).
        /* call this to return a local string var parameter as the
           result of the function. */
        /* does nothing */


/* assignment */

(lhs:var(string)) := (rhs:val(string)) : expr(void).
        /* string assignment */
        /* decrement ref count and possibly free string previously
           pointed to by lhs, make it point to rhs, and increment ref
           count */

/* assignment on arrays of strings
   can't do this here as forseq hasn't been declared yet
   real declaration is in ctrl.l

(lhs:var(string^(i:val(int)))) := (rhs:val(string^i)) : expr(void) = (
  a:var(int).
  forseq a in 0 to (i-1) do (
    lhs[a]:=rhs[a]
  )
).
    
*/


/* string operations */

c2s(c:val(char)) : expr(string).

strindex(s:val(string),i:val(int)) : expr(char).

strlen(s:val(string)) : expr(int).

`_@_` : overloaded.
_@_ : infix(160,left).
(s1:val(string)) @ (s2:val(string)) : expr(string).

strcmp(a:val(string),b:val(string)) : expr(int).

((a:val(string)) = (b:val(string))) : expr(bool) = (
  strcmp(a,b)=0
).

(a:val(string)) != (b:val(string)) : expr(bool) = (
  strcmp(a,b)!=0
).

(a:val(string)) < (b:val(string)) : expr(bool) = (
  strcmp(a,b)<0
).

(a:val(string)) > (b:val(string)) : expr(bool) = (
  strcmp(a,b)>0
).

(a:val(string)) <= (b:val(string)) : expr(bool) = (
  strcmp(a,b)<=0
).

(a:val(string)) >= (b:val(string)) : expr(bool) = (
  strcmp(a,b)>=0
).

substr (a:val(string),lowindex:val(int),highindexplus1:val(int)) : expr(string) .

strdchr (a:val(string),c:val(char),i:val(int),d:val(bool)) : expr(int) .
strdstr (a:val(string),b:val(string),i:val(int),d:val(bool)) : expr(int) .

strchr : overloaded .
strrchr : overloaded .

strchr (a:val(string),c:val(char)) : expr (int) = strdchr (a,c,0,true) .
strchr (a:val(string),c:val(char),i:val(int)) : expr (int) = strdchr (a,c,i,true) .
strrchr (a:val(string),c:val(char)) : expr (int) = strdchr (a,c,-1,false) .
strrchr (a:val(string),c:val(char),i:val(int)) : expr (int) = strdchr (a,c,i,false) .

strstr : overloaded .
strrstr : overloaded .

strstr (a:val(string),b:val(string)) : expr(int) = strdstr (a,b,0,true) .
strstr (a:val(string),b:val(string),i:val(int)) : expr(int) = strdstr (a,b,i,true) .
strrstr (a:val(string),b:val(string)) : expr(int) = strdstr (a,b,-1,false) .
strrstr (a:val(string),b:val(string),i:val(int)) : expr(int) = strdstr (a,b,i,false) .

formatdata (format:val(string),value:val(T:type)) : expr(string) .
parsedata (format:val(string),value:val(string),ptr:var(T:type)) : expr(string) .

null
