#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: bview.tcl,v 1.2 2000/03/24 17:06:42 bardslea Exp $
#
#
#   Block view
#
#	11 Jun 1999, AMULET Group

li_require windows
li_require winutils

toplevel .bview
wm title .bview "Block View"
wm iconname .bview "Block View"

frame .bview.menubar
menubutton .bview.menubar.window -text "Window" \
    -menu .bview.menubar.window.menu -underline 0
pack .bview.menubar.window -side left
menu .bview.menubar.window.menu
.bview.menubar.window.menu add command -label "Close" \
    -command "set bview_open 0" -underline 0

global main_menubar

$main_menubar.windows.menu add checkbutton -label "Block View" \
    -variable bview_open -underline 0

$main_menubar.session.menu.modules entryconfigure {Block View} \
    -state disabled

menubutton .bview.menubar.edit -text "Edit" \
    -menu .bview.menubar.edit.menu -underline 0
pack .bview.menubar.edit -side left
menu .bview.menubar.edit.menu
.bview.menubar.edit.menu add command -label "Set tag" \
    -command "bview_set_sel_tag" -state disabled -underline 0
.bview.menubar.edit.menu add command -label "Edit text" \
    -command "bview_edit_text" -state disabled -underline 0
.bview.menubar.edit.menu add command -label "Delete" \
    -command "bview_del_sel" -state disabled -underline 0
.bview.menubar.edit.menu add command -label "Delete All" \
    -command "bview_del_all"
.bview.menubar.edit.menu add command -label "Lower" \
    -command "bview_lower_sel" -state disabled -underline 0
.bview.menubar.edit.menu add command -label "Raise" \
    -command "bview_raise_sel" -state disabled -underline 0

menubutton .bview.menubar.options -text "Options" \
    -menu .bview.menubar.options.menu -underline 0
pack .bview.menubar.options -side left
menu .bview.menubar.options.menu
.bview.menubar.options.menu add checkbutton -label "Snap" \
    -variable bview_snap_on -underline 0
.bview.menubar.options.menu add checkbutton -label "Grid" \
    -variable bview_grid -underline 0
.bview.menubar.options.menu add checkbutton -label "Orthogonal" \
    -variable bview_ortho -underline 0
.bview.menubar.options.menu add command -label "Arrow shape..." \
    -command "bview_arrow_shape" -underline 0
.bview.menubar.options.menu add cascade -label "Font size" \
    -menu .bview.menubar.options.menu.fontsize -underline 0

set bview_bigfont [option get .bview bigfont Bigfont]
set bview_mediumfont [option get .bview mediumfont Mediumfont]
set bview_smallfont [option get .bview smallfont Smallfont]
set bview_font $bview_mediumfont

menu .bview.menubar.options.menu.fontsize
.bview.menubar.options.menu.fontsize add radiobutton -label "Big" \
    -variable bview_font -value "$bview_bigfont" -underline 0
.bview.menubar.options.menu.fontsize add radiobutton -label "Medium" \
    -variable bview_font -value "$bview_mediumfont" -underline 0
.bview.menubar.options.menu.fontsize add radiobutton -label "Small" \
    -variable bview_font -value "$bview_smallfont" -underline 0

frame .bview.buttons
winutils::make_image bvrect
winutils::make_image bvpoly
winutils::make_image bvtext
winutils::make_image bvarrow
winutils::make_image bvslider
winutils::make_image bvpointer
radiobutton .bview.buttons.rect -image bvrect_img -command bview_add_rect \
    -variable bview_mode -value rect -indicatoron false -selectcolor ""
radiobutton .bview.buttons.poly -image bvpoly_img -command bview_add_poly \
    -variable bview_mode -value poly -indicatoron false -selectcolor ""
radiobutton .bview.buttons.txt -image bvtext_img -command bview_add_text \
    -variable bview_mode -value text -indicatoron false -selectcolor ""
radiobutton .bview.buttons.arrow -image bvarrow_img -command bview_add_arrow \
    -variable bview_mode -value arrow -indicatoron false -selectcolor ""
radiobutton .bview.buttons.slider -image bvslider_img \
    -command bview_add_slider -variable bview_mode -value slider \
    -indicatoron false -selectcolor ""
radiobutton .bview.buttons.pointer -image bvpointer_img \
    -command bview_pointer_mode -variable bview_mode -value pointer \
    -indicatoron false -selectcolor ""
pack .bview.buttons.rect .bview.buttons.poly .bview.buttons.txt \
    .bview.buttons.arrow .bview.buttons.slider .bview.buttons.pointer -side top

scrollbar .bview.xscr -orient horiz -command ".bview.blocks xview"
scrollbar .bview.yscr -command ".bview.blocks yview"

canvas .bview.blocks -xscrollcommand ".bview.xscr set" \
    -yscrollcommand ".bview.yscr set"

label .bview.status -textvariable bview_status

grid .bview.menubar -row 0 -column 0 -columnspan 3 -sticky ew
grid .bview.status -row 1 -column 0 -columnspan 3 -sticky w
grid .bview.buttons -row 2 -column 0 -rowspan 2 -sticky n
grid .bview.blocks -row 2 -column 1 -sticky nsew
grid .bview.yscr -row 2 -column 2 -sticky ns
grid .bview.xscr -row 3 -column 1 -sticky ew

grid columnconfigure .bview 0 -weight 0
grid columnconfigure .bview 1 -weight 1
grid columnconfigure .bview 2 -weight 0

grid rowconfigure .bview 0 -weight 0
grid rowconfigure .bview 1 -weight 0
grid rowconfigure .bview 2 -weight 1
grid rowconfigure .bview 3 -weight 0

set bview_newtag ""

trace variable bview_open w bview_open_changed

proc bview_open_changed {name1 name2 op} {
    global bview_open

    if {$bview_open} {
	wm deiconify .bview
    } else {
	wm withdraw .bview
    }
}

set bview_open 1


proc bview_snap {a} {
    global bview_snap_on

    if {$bview_snap_on} {
	return [expr $a-(int($a+5)%10)+5]
    } else {
	return $a
    }
}

proc bview_conv_snap_pair {x y} {
    return [list [bview_snap [.bview.blocks canvasx $x]] \
		[bview_snap [.bview.blocks canvasy $y]]]
}

proc bview_conv_snap_pair_ortho {prevx prevy x y} {
    global bview_ortho

    if {!$bview_ortho} {
	return [bview_conv_snap_pair $x $y]
    }

    set cx [bview_snap [.bview.blocks canvasx $x]]
    set cy [bview_snap [.bview.blocks canvasy $y]]
    set dx [expr $cx-$prevx]
    set dy [expr $cy-$prevy]
    if {$dx>=0 && $dy>=0} {
	if {$dx>$dy} {
	    return [list $cx $prevy]
	} else {
	    return [list $prevx $cy]
	}
    } elseif {$dx>=0 && $dy<0} {
	if {$dx>-$dy} {
	    return [list $cx $prevy]
	} else {
	    return [list $prevx $cy]
	}
    } elseif {$dx<0 && $dy>=0} {
	if {-$dx>$dy} {
	    return [list $cx $prevy]
	} else {
	    return [list $prevx $cy]
	}
    } elseif {$dx<0 && $dy<0} {
	if {-$dx>-$dy} {
	    return [list $cx $prevy]
	} else {
	    return [list $prevx $cy]
	}
    }
}


proc bview_add_rect {} {
    global bview_status

    bview_selection_off
    bind .bview.blocks <ButtonPress-1> "bview_add_rect_first_corner %x %y"
    set bview_status "Select first corner and press button 1"
}

proc bview_add_rect_first_corner {x y} {
    global bview_corner1x
    global bview_corner1y
    global bview_status

    lassign [bview_conv_snap_pair $x $y] bview_corner1x bview_corner1y
    bind .bview.blocks <ButtonRelease-1> "bview_add_rect_second_corner %x %y"
    bind .bview.blocks <Button1-Motion> "bview_add_rect_update %x %y"
    set bview_status "Select second corner and release button 1"
}

proc bview_add_rect_update {x y} {
    global bview_corner1x
    global bview_corner1y

    .bview.blocks delete rubberrect
    lassign [bview_conv_snap_pair $x $y] corner2x corner2y
    .bview.blocks create rectangle $bview_corner1x $bview_corner1y \
	$corner2x $corner2y -outline black -tags "rubberrect"
}

proc bview_add_rect_second_corner {x y} {
    global bview_corner1x
    global bview_corner1y
    global bview_mode
    global bview_status

    .bview.blocks delete rubberrect
    lassign [bview_conv_snap_pair $x $y] corner2x corner2y
    .bview.blocks create rectangle $bview_corner1x $bview_corner1y \
	$corner2x $corner2y -outline black -fill white -tags "newshape"
    bind .bview.blocks <ButtonPress-1> {}
    bind .bview.blocks <Button1-Motion> {}
    bind .bview.blocks <ButtonRelease-1> {}
    bview_set_new_tag
    bview_pointer_mode
}

proc bview_add_poly {} {
    global bview_status

    bview_selection_off
    bind .bview.blocks <Button-1> "bview_add_poly_first_corner %x %y"
    set bview_status "Select first corner and click button 1"
}

proc bview_add_poly_first_corner {x y} {
    global bview_corners
    global bview_corner1x
    global bview_corner1y
    global bview_prevx
    global bview_prevy
    global bview_status

    lassign [bview_conv_snap_pair $x $y] bview_corner1x bview_corner1y
    set bview_prevx $bview_corner1x
    set bview_prevy $bview_corner1y
    set bview_corners "$bview_corner1x $bview_corner1y" 
    bind .bview.blocks <Button-1> "bview_add_poly_next_corner %x %y"
    bind .bview.blocks <Button-2> "bview_add_poly_last_corner %x %y"
    bind .bview.blocks <Motion> "bview_add_poly_update %x %y"
    set bview_status "Select next corner and click button 1, or last corner and click button 2"
}

proc bview_add_poly_update {x y} {
    global bview_corners
    global bview_corner1x
    global bview_corner1y
    global bview_prevx
    global bview_prevy

    lassign [bview_conv_snap_pair_ortho $bview_prevx $bview_prevy $x $y] cx cy
    .bview.blocks delete rubberpoly
    eval ".bview.blocks create line $bview_corners $cx $cy \
              $bview_corner1x $bview_corner1y -fill black -tags rubberpoly"
}

proc bview_add_poly_next_corner {x y} {
    global bview_corners
    global bview_prevx
    global bview_prevy

    lassign [bview_conv_snap_pair_ortho $bview_prevx $bview_prevy $x $y] cx cy
    append bview_corners " $cx $cy"
    set bview_prevx $cx
    set bview_prevy $cy
}

proc bview_add_poly_last_corner {x y} {
    global bview_corners
    global bview_mode
    global bview_prevx
    global bview_prevy
    global bview_status

    lassign [bview_conv_snap_pair_ortho $bview_prevx $bview_prevy $x $y] cx cy
    append bview_corners " $cx $cy"
    .bview.blocks delete rubberpoly
    eval ".bview.blocks create poly $bview_corners $cx $cy -outline black \
	-fill white -tags newshape"
    bind .bview.blocks <Button-1> ""
    bind .bview.blocks <Button-2> ""
    bind .bview.blocks <Motion> ""
    bview_set_new_tag
    bview_pointer_mode
}

proc bview_add_text {} {
    global bview_status

    bview_selection_off
    set bview_status "Select position of text and click button 1"
    bind .bview.blocks <Button-1> "bview_add_text_pt %x %y"
}

proc bview_add_text_pt {x y} {
    global bview_mode
    global bview_status
    global bview_text_mode
    global bview_tagtext
    global bview_font

    lassign [bview_conv_snap_pair $x $y] cx cy
    .bview.blocks create text $cx $cy -text "?" -tags newshape \
	-font $bview_font
    bind .bview.blocks <Button-1> ""
    set bview_status "Enter text or tag"
    toplevel .bview_text_dialog
    radiobutton .bview_text_dialog.tagbutton -variable bview_text_mode \
	-value "Tag"
    radiobutton .bview_text_dialog.textbutton -variable bview_text_mode \
	-value "Text"
    label .bview_text_dialog.tagbuttonlabel -text "Tag for animated text"
    label .bview_text_dialog.textbuttonlabel -text "Fixed text"
    label .bview_text_dialog.tagtextlabel -textvariable bview_text_mode
    entry .bview_text_dialog.tagtext -textvariable bview_tagtext
    button .bview_text_dialog.ok -text "OK" \
	-command "destroy .bview_text_dialog"
    grid .bview_text_dialog.tagbutton -row 0 -column 0 -sticky e
    grid .bview_text_dialog.textbutton -row 1 -column 0 -sticky e
    grid .bview_text_dialog.tagbuttonlabel -row 0 -column 1 -sticky w
    grid .bview_text_dialog.textbuttonlabel -row 1 -column 1 -sticky w
    grid .bview_text_dialog.tagtextlabel -row 2 -column 0 -sticky e
    grid .bview_text_dialog.tagtext -row 2 -column 1 -sticky w
    grid .bview_text_dialog.ok -row 3 -column 0 -columnspan 2
    tkwait visibility .bview_text_dialog
    grab .bview_text_dialog
    tkwait window .bview_text_dialog
    if {$bview_text_mode=="Tag"} {
	.bview.blocks addtag $bview_tagtext withtag "newshape"
	.bview.blocks addtag tagtext withtag "newshape"
    } else {
	.bview.blocks itemconfigure newshape -text $bview_tagtext
	.bview.blocks addtag annottext withtag "newshape"
    }
    .bview.blocks dtag "newshape"
    set bview_status ""
    bview_pointer_mode
}

set bview_text_mode "Text"
set bview_tagtext ""


proc bview_add_arrow {} {
    global bview_status

    bview_selection_off
    bind .bview.blocks <Button-1> "bview_add_arrow_start %x %y"
    set bview_status "Select first corner and click button 1"
}

proc bview_add_arrow_start {x y} {
    global bview_corners
    global bview_corner1x
    global bview_corner1y
    global bview_prevx
    global bview_prevy
    global bview_status

    lassign [bview_conv_snap_pair $x $y] bview_corner1x bview_corner1y
    set bview_prevx $bview_corner1x
    set bview_prevy $bview_corner1y
    set bview_corners "$bview_corner1x $bview_corner1y"
    bind .bview.blocks <Button-1> "bview_add_arrow_corner %x %y"
    bind .bview.blocks <Button-2> "bview_add_arrow_end %x %y"
    bind .bview.blocks <Motion> "bview_add_arrow_update %x %y"
    set bview_status "Select next corner and click button 1, or select end and click button 2"
}

proc bview_add_arrow_update {x y} {
    global bview_corners
    global bview_corner1x
    global bview_corner1y
    global bview_prevx
    global bview_prevy

    lassign [bview_conv_snap_pair_ortho $bview_prevx $bview_prevy $x $y] cx cy
    .bview.blocks delete rubberarrow
    set arrow_vertices [bview_compute_arrow_vertices "$bview_corners $cx $cy"]
    eval ".bview.blocks create line $bview_corner1x $bview_corner1y \
             $arrow_vertices $bview_corner1x $bview_corner1y \
             -fill black -tags rubberarrow"
}

proc bview_add_arrow_corner {x y} {
    global bview_corners
    global bview_prevx
    global bview_prevy

    lassign [bview_conv_snap_pair_ortho  $bview_prevx $bview_prevy $x $y] cx cy
    append bview_corners " $cx $cy"
    set bview_prevx $cx
    set bview_prevy $cy
}

proc bview_add_arrow_end {x y} {
    global bview_corners
    global bview_mode
    global bview_prevx
    global bview_prevy
    global bview_status

    lassign [bview_conv_snap_pair_ortho $bview_prevx $bview_prevy $x $y] cx cy
    append bview_corners " $cx $cy"
    .bview.blocks delete rubberarrow
    set arrow_vertices [bview_compute_arrow_vertices $bview_corners]
    eval ".bview.blocks create poly $arrow_vertices -outline black \
	-fill white -tags newshape"
    bind .bview.blocks <Button-1> ""
    bind .bview.blocks <Button-2> ""
    bind .bview.blocks <Motion> ""
    bview_set_new_tag
    bview_pointer_mode
}

proc bview_compute_arrow_vertices {pointlist} {
    global bview_arrow_width
    global bview_arrow_head_len
    global bview_arrow_head_width

    # remove colinear points
    set prevx [lvarpop pointlist]
    set prevy [lvarpop pointlist]
    lappend nclpoints $prevx $prevy
    set x [lvarpop pointlist]
    set y [lvarpop pointlist]
    foreach {nextx nexty} $pointlist {
	set dx [expr $x-$prevx]
	set dy [expr $y-$prevy]
	set ex [expr $nextx-$x]
	set ey [expr $nexty-$y]
	if {$ex==0 && $ey==0} {
	    set colin 1
	} elseif {$dx==0 && $ex==0} {
	    set colin 1
	} elseif {$dx==0 || $ex==0} {
	    set colin 0
	} elseif {$dy==0 && $ey==0} {
	    set colin 1
	} elseif {$dy==0 || $ey==0} {
	    set colin 0
	} else {
	    set colin [expr double($dx)/$ex==double($dy)/$ey]
	}
	if {!$colin} {
	    lappend nclpoints $x $y
	    set prevx $x
	    set prevy $y
	}
	set x $nextx
	set y $nexty
    }
    lappend nclpoints $x $y

    # If there's only one point left, return empty
    if {[llength $nclpoints]==2} {
	return {}
    }

    # Keep a list of the vertices on the left of the arrow, and a list of
    # the vertices on the right of the arrow, and eventually join them up.
    set leftlist {}
    set rightlist {}

    # deal with the first point
    set x [lvarpop nclpoints]
    set y [lvarpop nclpoints]
    set nextx [lvarpop nclpoints]
    set nexty [lvarpop nclpoints]
    set ex [expr $nextx-$x]
    set ey [expr $nexty-$y]
    set el [bview_length $ex $ey]
    set vx [expr $bview_arrow_width/2*-$ey/$el]
    set vy [expr $bview_arrow_width/2*$ex/$el]
    lappend leftlist [expr $x+$vx] [expr $y+$vy]
    lappend rightlist [expr $x-$vx] [expr $y-$vy]

    # deal with the intermediate points
    set prevx $x
    set prevy $y
    set x $nextx
    set y $nexty
    foreach {nextx nexty} $nclpoints {
	set dx [expr $x-$prevx]
	set dy [expr $y-$prevy]
	set dl [bview_length $dx $dy]
	set dxu [expr $dx/$dl]
	set dyu [expr $dy/$dl]
	set ex [expr $nextx-$x]
	set ey [expr $nexty-$y]
	set el [bview_length $ex $ey]
	set exu [expr $ex/$el]
	set eyu [expr $ey/$el]
	lassign [bview_line_intersect [expr $x-$dyu*$bview_arrow_width/2] \
		     [expr $y+$dxu*$bview_arrow_width/2] $dxu $dyu \
		     [expr $x-$eyu*$bview_arrow_width/2] \
		     [expr $y+$exu*$bview_arrow_width/2] $exu $eyu] ix iy
	lappend leftlist $ix $iy
	lassign [bview_line_intersect [expr $x+$dyu*$bview_arrow_width/2] \
		     [expr $y-$dxu*$bview_arrow_width/2] $dxu $dyu \
		     [expr $x+$eyu*$bview_arrow_width/2] \
		     [expr $y-$exu*$bview_arrow_width/2] $exu $eyu] ix iy
	lappend rightlist $ix $iy
	set prevx $x
	set prevy $y
	set x $nextx
	set y $nexty
    }	
    
    # deal with the last point
    set dx [expr $x-$prevx]
    set dy [expr $y-$prevy]
    set dl [bview_length $dx $dy]
    set dxu [expr $dx/$dl]
    set dyu [expr $dy/$dl]
    set ix [expr $dxu]
    set iy [expr $dyu]
    set jx [expr -$dyu]
    set jy [expr $dxu]
    lappend leftlist \
	[expr $x-$ix*$bview_arrow_head_len/2+$jx*$bview_arrow_width/2] \
	[expr $y-$iy*$bview_arrow_head_len/2+$jy*$bview_arrow_width/2]
    lappend leftlist \
	[expr $x-$ix*$bview_arrow_head_len/2+$jx*$bview_arrow_head_width/2] \
	[expr $y-$iy*$bview_arrow_head_len/2+$jy*$bview_arrow_head_width/2]
    lappend rightlist \
	[expr $x-$ix*$bview_arrow_head_len/2-$jx*$bview_arrow_width/2] \
	[expr $y-$iy*$bview_arrow_head_len/2-$jy*$bview_arrow_width/2]
    lappend rightlist \
	[expr $x-$ix*$bview_arrow_head_len/2-$jx*$bview_arrow_head_width/2] \
	[expr $y-$iy*$bview_arrow_head_len/2-$jy*$bview_arrow_head_width/2]
    lappend leftlist $x $y

    set vlist {}
    foreach {x y} $rightlist {
	lvarpush vlist $y
	lvarpush vlist $x
    }
    append leftlist " $vlist"

    return $leftlist
}

proc bview_length {x y} {
    if {$x==0.0 && $y==0.0} {
	# puts "bview_length returning 1 to avoid div0!!"
	return 1.0
    }
    return [expr sqrt($x*$x+$y*$y)]
}

proc bview_line_intersect {x0 y0 dx dy i0 j0 di dj} {
    set divisor [expr $dy*$di-$dx*$dj]
    if {$divisor==0.0} {
	puts "bview_line_intersect $x0 $y0 $dx $dy $i0 $j0 $di $dj"
    }
    set b [expr ($dx*($j0-$y0)-$dy*($i0-$x0))/double($divisor)]
    set x [expr $i0+$b*$di]
    set y [expr $j0+$b*$dj]
    return [list $x $y]
}


proc bview_set_new_tag {} {
    global bview_newtag
    global bview_status

    set bview_status "Enter tag for new element"
    bview_dialog "Tag for new shape" "Tag:" bview_newtag
    set bview_status ""
    .bview.blocks addtag $bview_newtag withtag "newshape"
    .bview.blocks dtag "newshape"
}

proc bview_set_sel_tag {} {
    global bview_seltag

    set oldtags [.bview.blocks gettags selection]
    if {[lindex $oldtags 0]=="selection"} {
	set oldtag [lindex $oldtags 1]
    } else {
	set oldtag [lindex $oldtags 0]
    }
    set bview_seltag $oldtag
    bview_dialog "Tag for selected shape" "Tag:" bview_seltag
    .bview.blocks dtag selection $oldtag
    .bview.blocks addtag $bview_seltag withtag "selection"
}

proc bview_edit_text {} {
    global bview_newtext

    set bview_newtext [.bview.blocks itemcget selection -text]
    bview_dialog "New text for selected text" "Text:" bview_newtext
    .bview.blocks itemconfigure selection -text $bview_newtext
}


proc config_save_go_bview {f} {

    .bview.blocks delete grid
    foreach window [winfo children .bview.blocks] {
	puts -nonewline $f "[translit A-Z a-z [winfo class $window]] $window "
	set a [$window configure]
	foreach b $a {
	    set name [lindex $b 0]
	    if {$name!="-bg" && $name!="-fg" && $name!="-bd"} {
		puts -nonewline $f "$name \{[lindex $b 4]\} "
	    }
	}
	puts $f ""
	foreach bindevent [bind $window] {
	    set bindaction [bind $window $bindevent]
	    puts $f "bind $window $bindevent \"$bindaction\""
	}
    }
    foreach tag [.bview.blocks find all] {
	puts -nonewline $f ".bview.blocks create "
	puts -nonewline $f [.bview.blocks type $tag]
	puts -nonewline $f " "
	puts -nonewline $f [.bview.blocks coords $tag]
	puts -nonewline $f [bview_getconfig $tag]
	puts $f ""
    }
    config_save_var $f bview_snap_on
    config_save_var $f bview_grid
    config_save_var $f bview_ortho
    config_save_var $f bview_font
    config_save_var $f bview_arrow_width
    config_save_var $f bview_arrow_head_len
    config_save_var $f bview_arrow_head_width
    config_save_var $f bview_slider_id
    bview_grid_changed a b c
}

proc bview_getconfig {tag} {

    set a [.bview.blocks itemconfigure $tag]
    set b ""
    foreach thisone $a {
	set name [lindex $thisone 0]
	set value [lindex $thisone 4]
	append b " $name \{$value\}"
    }
    return $b
}

lappend config_save_list {{bview} {Save Block View Display}}


proc bview_del_all {} {
    .bview.blocks delete all
    bview_grid_changed a b c
}

proc bview_selection_off {} {
    set items [.bview.blocks find withtag selection]
    foreach item $items {
	set type [.bview.blocks type $item]
	if {$type=="text"} {
	    .bview.blocks itemconfigure $item -fill black
	} elseif {$type=="window"} {
	    set win [.bview.blocks itemcget $item -window]
	    set default [lindex [$win configure -background] 3]
	    $win configure -background $default
	} else {
	    .bview.blocks itemconfigure $item -fill white
	}
    }
    .bview.blocks dtag selection
    .bview.menubar.edit.menu entryconfigure Delete -state disabled
    .bview.menubar.edit.menu entryconfigure "Set tag" -state disabled
    .bview.menubar.edit.menu entryconfigure "Edit text" -state disabled
    .bview.menubar.edit.menu entryconfigure Lower -state disabled
    .bview.menubar.edit.menu entryconfigure Raise -state disabled
}


proc bview_pointer_mode {} {
    global bview_mode

    set bview_mode pointer
    bind .bview.blocks <ButtonPress-1> "bview_set_selection %x %y"
    bind .bview.blocks <ButtonPress-2> "bview_add_selection %x %y"
}

proc bview_set_selection {x y} {
    global bview_prevx
    global bview_prevy

    lassign [bview_conv_snap_pair $x $y] bview_prevx bview_prevy
    bview_selection_off
    .bview.blocks addtag selection withtag current
    .bview.blocks dtag grid selection
    if {[.bview.blocks type selection]!=""} {
	.bview.menubar.edit.menu entryconfigure Delete -state normal
	.bview.menubar.edit.menu entryconfigure "Set tag" -state normal
	.bview.menubar.edit.menu entryconfigure Lower -state normal
	.bview.menubar.edit.menu entryconfigure Raise -state normal
	.bview.blocks itemconfigure selection -fill red
	bind .bview.blocks <Motion> "bview_move_update %x %y"
	bind .bview.blocks <ButtonRelease-1> "bview_move_done %x %y"
    }
    if {[.bview.blocks type selection]=="text"} {
	.bview.menubar.edit.menu entryconfigure "Edit text" -state normal
    }
}


proc bview_add_selection {x y} {
    global bview_prevx
    global bview_prevy

    lassign [bview_conv_snap_pair $x $y] bview_prevx bview_prevy
    .bview.blocks addtag selection withtag current
    .bview.blocks dtag grid selection
    if {[.bview.blocks type selection]!=""} {
	.bview.menubar.edit.menu entryconfigure Delete -state normal
	.bview.menubar.edit.menu entryconfigure "Set tag" -state normal
	.bview.menubar.edit.menu entryconfigure Lower -state normal
	.bview.menubar.edit.menu entryconfigure Raise -state normal
	.bview.blocks itemconfigure selection -fill red
	bind .bview.blocks <Motion> "bview_move_update %x %y"
	bind .bview.blocks <ButtonRelease-2> "bview_move_done %x %y"
    }
    if {[.bview.blocks type selection]=="text"} {
	.bview.menubar.edit.menu entryconfigure "Edit text" -state normal
    }
}


proc bview_move_update {x y} {
    global bview_prevx
    global bview_prevy

    set newx [bview_snap [.bview.blocks canvasx $x]]
    set newy [bview_snap [.bview.blocks canvasx $y]]
    .bview.blocks move selection [expr $newx-$bview_prevx] \
	[expr $newy-$bview_prevy]
    set bview_prevx $newx
    set bview_prevy $newy
}

proc bview_move_done {x y} {
    bind .bview.blocks <Motion> ""
    bind .bview.blocks <ButtonRelease-1> ""
}

proc bview_del_sel {} {
    foreach item [.bview.blocks find withtag selection] {
	if {[.bview.blocks type $item]=="window"} {
	    set win [.bview.blocks itemcget $item -window]
	    destroy $win
	}
    }
    .bview.blocks delete selection
    bview_selection_off
}

proc bview_lower_sel {} {
    .bview.blocks lower selection
    bview_selection_off
}

proc bview_raise_sel {} {
    .bview.blocks raise selection
    bview_selection_off
}


proc bview_set_colour {tag colour} {
    catch [.bview.blocks itemconfigure $tag -fill $colour]
}

proc bview_set_text {tag text} {
    catch [.bview.blocks itemconfigure $tag -text $text]
}


proc bview_dialog {title prompt var} {
    global $var
    
    toplevel .dialogbox
    wm title .dialogbox $title
    wm iconname .dialogbox $title
    frame .dialogbox.f
    label .dialogbox.f.prompt -text $prompt
    pack .dialogbox.f.prompt -side left
    entry .dialogbox.f.entry
    pack .dialogbox.f.entry -side left
    .dialogbox.f.entry insert 0 [set $var]
    bind .dialogbox.f.entry <Return> "bview_dialog_done $var"
    pack .dialogbox.f
    
    focus .dialogbox.f.entry
    tkwait visibility .dialogbox
    grab .dialogbox
    tkwait window .dialogbox
}


proc bview_dialog_done {var} {
  global $var

  set $var [.dialogbox.f.entry get]
  destroy .dialogbox
}

bview_pointer_mode

trace variable bview_grid w bview_grid_changed

proc bview_grid_changed {name1 name2 op} {
    global bview_grid

    if {$bview_grid} {
	bview_draw_grid
    } else {
	.bview.blocks delete grid
    }
}


proc bview_draw_grid {} {

    lassign [.bview.blocks cget -scrollregion] xmin ymin xmax ymax
    for {set x $xmin} {$x<$xmax} {incr x 10} {
	.bview.blocks lower [.bview.blocks create line \
				 $x $ymin $x $ymax -fill grey60 -tags grid]
    }
    for {set y $ymin} {$y<$ymax} {incr y 10} {
	.bview.blocks lower [.bview.blocks create line \
				 $xmin $y $xmax $y -fill grey60 -tags grid]
    }
}


set bview_status ""

set bview_snap_on [option get .bview snap Snap]
set bview_ortho [option get .bview ortho Ortho]
set bview_grid [option get .bview grid Grid]


proc bview_arrow_shape {} {
    toplevel .bview_arrow_dialog
    wm title .bview_arrow_dialog "Arrow shape"
    wm iconname .bview_arrow_dialog "Arrow shape"
    label .bview_arrow_dialog.shaftwidthlabel -text "Shaft width"
    label .bview_arrow_dialog.headlengthlabel -text "Head length"
    label .bview_arrow_dialog.headwidthlabel -text "Head width"
    entry .bview_arrow_dialog.shaftwidth -textvariable bview_arrow_width
    entry .bview_arrow_dialog.headlength -textvariable bview_arrow_head_len
    entry .bview_arrow_dialog.headwidth -textvariable bview_arrow_head_width
    button .bview_arrow_dialog.ok -text "OK" \
	-command "destroy .bview_arrow_dialog"
    grid .bview_arrow_dialog.shaftwidthlabel -row 0 -column 0 -sticky e
    grid .bview_arrow_dialog.headlengthlabel -row 1 -column 0 -sticky e
    grid .bview_arrow_dialog.headwidthlabel -row 2 -column 0 -sticky e
    grid .bview_arrow_dialog.shaftwidth -row 0 -column 1 -sticky w
    grid .bview_arrow_dialog.headlength -row 1 -column 1 -sticky w
    grid .bview_arrow_dialog.headwidth -row 2 -column 1 -sticky w
    grid .bview_arrow_dialog.ok -row 3 -column 0 -columnspan 2
    tkwait visibility .bview_arrow_dialog
    grab .bview_arrow_dialog
    tkwait window .bview_arrow_dialog
}
    

set bview_arrow_width 20.0
set bview_arrow_head_len 50.0
set bview_arrow_head_width 50.0


proc bview_add_slider {} {
    global bview_status

    bview_selection_off
    set bview_status "Select position for slider and click button 1"
    bind .bview.blocks <Button-1> "bview_add_slider_pt %x %y"
}


proc bview_add_slider_pt {x y} {
    global bview_mode
    global bview_status
    global bview_slider_varname
    global bview_slider_minval
    global bview_slider_maxval
    global bview_slider_orient
    global bview_slider_ok
    global bview_slider_id
    
    lassign [bview_conv_snap_pair $x $y] cx cy
    set bview_status "Enter attributes for slider"
    toplevel .bview_slider_dialog
    label .bview_slider_dialog.varname_label -text "Variable name"
    entry .bview_slider_dialog.varname -textvariable bview_slider_varname
    label .bview_slider_dialog.minval_label -text "Min value"
    entry .bview_slider_dialog.minval -textvariable bview_slider_minval
    label .bview_slider_dialog.maxval_label -text "Max value"
    entry .bview_slider_dialog.maxval -textvariable bview_slider_maxval
    radiobutton .bview_slider_dialog.horizbutton -text "Horizontal" \
	-variable bview_slider_orient -value horiz
    radiobutton .bview_slider_dialog.vertbutton -text "Vertical" \
	-variable bview_slider_orient -value vert
    button .bview_slider_dialog.ok -text "OK" -command "bview_slider_ok"
    button .bview_slider_dialog.cancel -text "cancel" \
	-command "bview_slider_cancel"
    grid .bview_slider_dialog.varname_label -row 0 -column 0 -sticky e
    grid .bview_slider_dialog.varname -row 0 -column 1 -sticky w
    grid .bview_slider_dialog.minval_label -row 1 -column 0 -sticky e
    grid .bview_slider_dialog.minval -row 1 -column 1 -sticky w
    grid .bview_slider_dialog.maxval_label -row 2 -column 0 -sticky e
    grid .bview_slider_dialog.maxval -row 2 -column 1 -sticky w
    grid .bview_slider_dialog.horizbutton -row 3 -column 0 -columnspan 2 \
	-sticky w
    grid .bview_slider_dialog.vertbutton -row 4 -column 0 -columnspan 2 \
	 -sticky w
#    frame .bview_slider_dialog.buttons
#    pack .bview_slider_dialog.ok .bview_slider_dialog.cancel \
	-in .bview_slider_dialog.buttons
#    grid .bview_slider_dialog.buttons -row 5 -column 0 -columnspan 2 -sticky n
    grid .bview_slider_dialog.ok -row 5 -column 0
    grid .bview_slider_dialog.cancel -row 5 -column 1
    tkwait visibility .bview_slider_dialog
    grab .bview_slider_dialog
    tkwait window .bview_slider_dialog
    if {$bview_slider_ok} {
	scale .bview.blocks.slider$bview_slider_id -from $bview_slider_minval \
	    -to $bview_slider_maxval -variable $bview_slider_varname \
	    -showvalue true -orient $bview_slider_orient
	.bview.blocks create window $cx $cy \
	    -tags "window slider$bview_slider_id" \
	    -window .bview.blocks.slider$bview_slider_id
	bind .bview.blocks.slider$bview_slider_id <Control-ButtonPress-1> \
	    "bview_set_selection_window slider$bview_slider_id \
            .bview.blocks.slider$bview_slider_id %X %Y"
	incr bview_slider_id
    }
    set bview_status ""
    bview_pointer_mode
}


proc bview_slider_ok {} {
    global bview_slider_ok

    set bview_slider_ok 1
    destroy .bview_slider_dialog
}

proc bview_slider_cancel {} {
    global bview_slider_ok

    set bview_slider_ok 0
    destroy .bview_slider_dialog
}


proc bview_set_selection_window {tag win X Y} {
    global bview_prevx
    global bview_prevy

    set x [expr {$X - [winfo rootx .bview.blocks]}]
    set y [expr {$Y - [winfo rootx .bview.blocks]}]
    lassign [bview_conv_snap_pair $x $y] bview_prevx bview_prevy
    bview_selection_off
    .bview.blocks addtag selection withtag $tag
    .bview.menubar.edit.menu entryconfigure Delete -state normal
    $win configure -background red
    bind $win <Motion> "bview_move_window_update %X %Y"
    bind $win <ButtonRelease-1> "bview_move_window_done $win"
}


proc bview_move_window_update {X Y} {
    set x [expr {$X - [winfo rootx .bview.blocks]}]
    set y [expr {$Y - [winfo rootx .bview.blocks]}]
    bview_move_update $x $y
}


proc bview_move_window_done {win} {
    bind $win <Motion> ""
    bind .bview.blocks <Motion> ""
    bind $win <ButtonRelease-1> ""
}



set bview_slider_minval 0
set bview_slider_maxval 100
set bview_slider_orient horiz
set bview_slider_id 0

