#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: channels.tcl,v 1.6 2000/05/10 14:03:25 bardslea Exp $
#
#
#   Channels
#
#	11 Jun 1999, AMULET Group

li_require tview
li_require threads
li_require selector

global selector_toplevel
global cands_menubar
global main_menubar

$main_menubar.session.menu.modules entryconfigure "Channels" -state disabled

menubutton $cands_menubar.channels -text "Channels" \
    -menu $cands_menubar.channels.menu -underline 0
pack $cands_menubar.channels -side left
menu $cands_menubar.channels.menu

$cands_menubar.channels.menu add command -label "Add selection" \
    -command chan_sel_tview_add -underline 0
$cands_menubar.channels.menu add command -label "Delete selection" \
    -command chan_sel_tview_delete -underline 0
$cands_menubar.channels.menu add separator
$cands_menubar.channels.menu add cascade -label "Default format" \
    -menu $cands_menubar.channels.menu.default_format -underline 8
$cands_menubar.channels.menu add checkbutton -label "Trace by default" \
    -variable chan_default_trace -underline 0
$cands_menubar.channels.menu add checkbutton \
    -label "Show databox by default" -variable chan_default_show_data \
    -underline 11
$cands_menubar.channels.menu add checkbutton \
    -label "Show blobs by default" -variable chan_default_show_blobs \
    -underline 6

menu $cands_menubar.channels.menu.default_format
$cands_menubar.channels.menu.default_format add radiobutton -label "Blank" \
    -variable chan_default_format -value " " -underline 0
$cands_menubar.channels.menu.default_format add radiobutton -label "Hex" \
    -variable chan_default_format -value " %* X" -underline 0
$cands_menubar.channels.menu.default_format add radiobutton \
    -label "Decimal" -variable chan_default_format -value " %* d" -underline 0

$cands_menubar.channels.menu add separator
$cands_menubar.channels.menu add command \
    -label "Show incomplete handshakes" -command complete
$cands_menubar.channels.menu add command \
    -label "Hide incomplete handshakes" -command uncomplete
$cands_menubar.channels.menu add separator
$cands_menubar.channels.menu add command \
    -label "Show all data boxes" -command show_all_data
$cands_menubar.channels.menu add command \
    -label "Show all blobs" -command show_all_blobs
$cands_menubar.channels.menu add command \
    -label "Show all channels" -command show_all_sigs



proc chan_sel_tview_add {} {
	global selector_toplevel
    
    set wins [selector_sel_windows]
    while {![lempty $wins]} {
	set win [lvarpop wins 0]
	if {[regexp {chanbutton(c[0-9]*)$} $win foo cid]} {
	    global chan_traced_$cid
	    if {![set chan_traced_$cid]} {
		set chan_traced_$cid 1
	    }
	}
    }
}


proc chan_sel_tview_delete {} {
	global selector_toplevel
    
    set wins [selector_sel_windows]
    while {![lempty $wins]} {
	set win [lvarpop wins 0]
	if {[regexp {chanbutton(c[0-9]*)$} $win foo cid]} {
	    global chan_traced_$cid
	    if {[set chan_traced_$cid]} {
		set chan_traced_$cid 0
	    }
	}
    }
}


proc chan_selector_pre_update {attrlist} {
	global selector_toplevel

    set cid [lindex $attrlist [lsearch -regexp $attrlist {^c[0-9]*$}]]
    checkbutton $selector_toplevel.text.chanbutton$cid -text "tview" \
	-variable chan_traced_$cid
    $selector_toplevel.text window create insert \
	-window $selector_toplevel.text.chanbutton$cid
}


append selector_pre_update_hook {
	global selector_toplevel

    if {[lsearch -exact $attrlist chan] != -1} {
	chan_selector_pre_update $attrlist
    }
}


proc chan_add_tview {cid} {
    global chan_names
    global chan_tvids
    global time
    global tview_toplevel

    if {![catch {set tvid $chan_tvids($cid)}]} {
	return
    }

    set tvid [tview_new $chan_names($cid)]
    set chan_tvids($cid) $tvid

    $tview_toplevel.tracenames.$tvid.menu add command -label "Delete" \
	-command "set chan_traced_$cid 0" -underline 0
    $tview_toplevel.tracenames.$tvid.menu add checkbutton -label "Show databox" \
	-variable chan_show_data($cid) -underline 11
    $tview_toplevel.tracenames.$tvid.menu add checkbutton -label "Show blobs" \
	-variable chan_show_blobs($cid) -underline 5
    $tview_toplevel.tracenames.$tvid.menu add command -label "Format..." \
	-command "chan_format $cid" -underline 0

    set y [tview_tvid_to_canvasy $tvid]
    $tview_toplevel.traces lower [ $tview_toplevel.traces create rectangle \
			      0 [expr $y-12] [tview_time_to_canvasx $time] \
			      [expr $y+12] -fill grey40 -outline grey40 \
			      -tags "$tvid stretchzoom" ]
}


proc chan_format {cid} {
    global chan_sizes
    global chan_names
    global chan_default_format

    tview_format "chan_format_$cid" $chan_sizes($cid) $chan_names($cid) $cid \
	$chan_default_format
}


proc chan_tview_delete {cid} {
    global chan_tvids

    if [catch {set tvid $chan_tvids($cid)}] {
	return
    }
    tview_delete $tvid
    unset chan_tvids($cid)
}


proc chan_new {name sz form} {
    global threadnames
    global bci_current_thread
    global chan_id
    global chan_seq_id
    global chan_names
    global chan_show_data
    global chan_show_blobs
    global chan_sizes
    global chan_default_format
    global chan_default_show_data
    global chan_default_show_blobs
    global chan_default_trace
    global chan_trace_thisone
    global chan_pathnames
    global chan_new_hook

    set pathname [concat $threadnames($bci_current_thread) channels \
					  "{$name}"]
    set attrs [struct_get_attrs $pathname]
    if {$attrs=="notfound"} {
		set chan_id $chan_seq_id
		set cid c$chan_id
		incr chan_seq_id
		struct_add block [concat $threadnames($bci_current_thread) channels ]
		struct_add "chan $cid" $pathname
		global chan_traced_$cid
		set chan_names($cid) $name
		set chan_pathnames($cid) $pathname
		set chan_sizes($cid) $sz
		set chan_show_data($cid) $chan_default_show_data
		set chan_show_blobs($cid) $chan_default_show_blobs

		set fv "chan_format_${cid}"
		global $fv
		if {$form==""} {
			set $fv $chan_default_format
		} else {
			set $fv $form
		}
		set chan_trace_thisone $chan_default_trace
		chan_trace_vars $cid
    } else {
		set chan_id [crange \
			 [lindex $attrs [lsearch -regexp $attrs {^c[0-9]*$}]] 1 end]
		set cid c$chan_id
		struct_rm_attr unconf $pathname
		global chan_traced_$cid
		if {$chan_sizes($cid)!=$sz} {
			error "Channel size for $name has changed from $chan_size($cid) to $sz"
		}
		set traced [set chan_traced_$cid]
		set chan_trace_thisone $traced
    }
    eval $chan_new_hook
}

proc chan_trace_vars {cid} {
    global chan_traced_$cid
    global chan_trace_vars_hook
    trace variable chan_traced_$cid w "chan_traced_changed $cid"
    eval $chan_trace_vars_hook
}


append chan_new_hook {}

append chan_trace_vars_hook {}


proc chan_traced_changed {cid name1 name2 op} {
    global chan_traced_$cid

    if {[set chan_traced_$cid]} {
	chan_add_tview $cid
    } else {
	chan_tview_delete $cid
    }
}


set chan_seq_id 0
set chan_default_format "%* d"
set chan_default_show_data 0
set chan_default_show_blobs 0
set chan_default_trace 1



proc chan_get_data {cid} {
    global chan_sizes

    set sz $chan_sizes($cid)
    set data ""
	set ret ""
    for {set i 0} {$i<$sz} {incr i} {
	set dv "chan_data_${cid}_${i}"
	global $dv
	append data [format "%08x" [set $dv]]
    }
    if {$sz != 0} {
	set fv "chan_format_${cid}"
	global $fv
	set ret [lard_format [set $fv] $sz $data]
	return " ${ret}"
    }
    return ""
}


proc chan_data {cid} {
    global chan_show_data
    global chan_show_blobs
    global chan_tvids
    global time

    if {!$chan_show_data($cid) && !$chan_show_blobs($cid)} {
	return
    }
    set data [chan_get_data $cid]
    set tvid $chan_tvids($cid)
    if {$chan_show_data($cid)} {
	tview_databox $tvid $time $data
    }
    if {$chan_show_blobs($cid)} {
	blob $tvid $time $data
    }
}

proc chan_times {cid} {

    global time
    global chan_stime_$cid
    global chan_rtime_$cid
    global chan_tvids
	global chan_last_stime
	global chan_last_rtime

	set chan_last_stime($cid) [set chan_stime_$cid]
	set chan_last_rtime($cid) [set chan_rtime_$cid] 
    global tview_toplevel

    set etime $time
    set stime [set chan_stime_$cid]
    set rtime [set chan_rtime_$cid]

    set tvid $chan_tvids($cid)
    set y [tview_tvid_to_canvasy $tvid]

    $tview_toplevel.traces lower \
		[ $tview_toplevel.traces create rectangle \
			 [tview_time_to_canvasx $stime] [expr $y-12] \
			 [tview_time_to_canvasx $etime] [expr $y-2] \
			 -fill red -outline black -tags "$tvid stretchzoom"]

    $tview_toplevel.traces lower \
		[ $tview_toplevel.traces create rectangle \
			 [tview_time_to_canvasx $rtime] [expr $y+2] \
			 [tview_time_to_canvasx $etime] [expr $y+12] \
			 -fill blue -outline black -tags "$tvid stretchzoom"]
}


lappend config_save_list {{channels} {Save Channel Formats}}

proc config_save_go_channels {f} {
    global chan_names
    global chan_pathnames
    global chan_sizes
    global chan_show_data
    global chan_show_blobs
    global chan_tvids
    global chan_seq_id
    global tview_ypos

    if {![array exists chan_names]} {
	puts $f "\# No Channels"
	return
    }

    config_save_var $f chan_seq_id
    for_array_keys cid chan_names {
	puts $f "\# Channel \"$chan_names($cid)\""
	puts $f "struct_add_attr unconf {$chan_pathnames($cid)}"
	puts $f "set chan_names($cid) {$chan_names($cid)}"
	puts $f "set chan_pathnames($cid) {$chan_pathnames($cid)}"
	puts $f "set chan_sizes($cid) $chan_sizes($cid)"
	puts $f "set chan_show_data($cid) $chan_show_data($cid)"
	puts $f "set chan_show_blobs($cid) $chan_show_blobs($cid)"
	set fv "chan_format_${cid}"
	global $fv
	set fvv [set $fv]
	puts $f "set $fv {$fvv}"
	global chan_traced_$cid
	set traced [set chan_traced_$cid]
	if {!$traced} {
	    puts $f "set chan_traced_$cid $traced"
	}
	puts $f "chan_trace_vars $cid"
	puts $f ""
    }
    
    puts $f "\# Traced channels in trace position order"
    if {[catch {
	for_array_keys cid chan_tvids {
	    set tvid $chan_tvids($cid)
	    set ypos $tview_ypos($tvid)
	    lappend unsorted "$ypos $cid"
	}
    }]} {
	puts $f "\# No traced channels"
	return
    }
    set sorted [lsort -command chan_save_sort $unsorted]
    foreach p $sorted {
	lassign $p ypos cid
	puts $f "set chan_traced_$cid 1"
    }
}


proc chan_save_sort {a b} {
    set a0 [lindex $a 0]
    set b0 [lindex $b 0]
    if {$a0<$b0} {return -1} elseif {$a0>$b0} {return 1} else {return 0}
}


proc config_save_channels_update {name1 name2 op} {
    global config_save_channels
    global config_save_structure

    if {$config_save_channels} {
	set config_save_structure 1
    }
}

proc show_all_data {} {
    global chan_tvids
    global chan_show_data
	catch {
		for_array_keys cid chan_tvids {
			set chan_show_data($cid) 1
		}
	}
}

proc show_all_blobs {} {
    global chan_tvids
    global chan_show_blobs
	catch {
        for_array_keys cid chan_tvids {
		    set chan_show_blobs($cid) 1
        }
	}
}

proc show_all_sigs {} {
    global sig_tvids
    global sig_show_boolean
	catch {
        for_array_keys sid sig_tvids {
		    set sig_show_boolean($sid) 1
        }
	}
}

proc complete {} {
    global time
    global chan_seq_id
    global chan_tvids
    global chan_last_stime
    global chan_last_rtime
    global tview_toplevel

	catch {
		for_array_keys cid chan_tvids {
			global chan_stime_$cid
			global chan_rtime_$cid
			set stime [set chan_stime_$cid]
			set rtime [set chan_rtime_$cid]
			set tvid $chan_tvids($cid)
			set y [tview_tvid_to_canvasy $tvid]
			if {![info exists chan_last_stime($cid)]} {
				set chan_last_stime($cid) 0 
			}
			if {![info exists chan_last_rtime($cid)]} {
				set chan_last_rtime($cid) 0 
			}
			set last_stime [set chan_last_stime($cid)]
			set last_rtime [set chan_last_rtime($cid)] 
			if {$stime!=$last_stime} {
				$tview_toplevel.traces lower \
					[ $tview_toplevel.traces create rectangle \
						 [tview_time_to_canvasx [set chan_stime_$cid]] [expr $y-12] \
						 [tview_time_to_canvasx [expr $time+1]] [expr $y-2] \
						 -fill yellow -outline black -tags "$tvid stretchzoom complete"]
			}
			if {$rtime!=$last_rtime} {
				$tview_toplevel.traces lower \
					[$tview_toplevel.traces create rectangle \
						 [tview_time_to_canvasx [set chan_rtime_$cid]] [expr $y+2] \
						 [tview_time_to_canvasx [expr $time+1]] [expr $y+12] \
						 -fill orange -outline black -tags "$tvid stretchzoom complete"]
			}
		}
	}
}

proc uncomplete {} {
    global tview_toplevel

	$tview_toplevel.traces delete withtag complete
}

proc autocomplete {name1 name2 op} {
	global bci_running
	global time
	if {$time>0} {
		if {$bci_running} {
			uncomplete
		} else {
			complete
		}
	}
} 

set config_save_channels 0

trace variable config_save_channels w config_save_channels_update
trace variable config_save_structure w config_save_channels_update
trace variable bci_running w autocomplete

