#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: power.tcl,v 1.2 2000/03/24 17:06:43 bardslea Exp $
#
#
#   Power channels
#
#	11 Jun 1999, AMULET Group

li_startup_module power
li_require selector
li_require windows
li_require channels

proc power_selector_pre_update {attrlist} {
   	global selector_toplevel 

    set cid [lindex $attrlist [lsearch -regexp $attrlist {^c[0-9]*$}]]
    checkbutton $selector_toplevel.text.power$cid -text "power" \
	    -variable power_traced_$cid
    $selector_toplevel.text window create insert \
	-window $selector_toplevel.text.power$cid
}


append selector_pre_update_hook {
    if {[lsearch -exact $attrlist chan] != -1} {
	power_selector_pre_update $attrlist
    }
}


append chan_trace_vars_hook {
    global power_traced_$cid
    trace variable power_traced_$cid w "power_traced_changed $cid"
#    set power_traced_$cid 0
}


proc power_traced_changed {cid n1 n2 op} {
    global power_traced_$cid
    global chan_names

    if {[set power_traced_$cid]} {
	power_new $chan_names($cid) chan_pwr_$cid
    } else {
#	power_delete ???
    }
}

# Power viewer

toplevel .power
wm title .power "Power"
wm iconname .power "Power"

frame .power.menubar

menubutton .power.menubar.window -text "Window" \
    -menu .power.menubar.window.menu -underline 0
pack .power.menubar.window -side left
menu .power.menubar.window.menu
.power.menubar.window.menu add command -label "Close" \
    -command "set power_open 0" -underline 0

frame .power.zoom
button .power.zoom.in -image zoom_in_img -command {power_change_zoom 2.0}
button .power.zoom.out -image zoom_out_img -command {power_change_zoom 0.5}
menubutton .power.zoom.label -textvariable power_zoom_scale \
    -menu .power.zoom.label.menu
menu .power.zoom.label.menu
.power.zoom.label.menu add command -label "8" -command {power_set_zoom 8}
.power.zoom.label.menu add command -label "4" -command {power_set_zoom 4}
.power.zoom.label.menu add command -label "2" -command {power_set_zoom 2}
.power.zoom.label.menu add command -label "1" -command {power_set_zoom 1}
.power.zoom.label.menu add command -label "0.5" -command {power_set_zoom 0.5}
.power.zoom.label.menu add command -label "0.25" -command {power_set_zoom 0.25}
.power.zoom.label.menu add command -label "0.125" -command {power_set_zoom 0.125}
pack .power.zoom.in .power.zoom.out -side left -fill y
pack .power.zoom.label -side left -fill both

scrollbar .power.yscr -command ".power.traces yview"
canvas .power.traces -yscrollcommand ".power.yscr set"

grid .power.menubar -row 0 -column 0 -columnspan 3 -sticky ew
grid .power.traces -row 1 -column 0 -columnspan 2 -sticky nsew
grid .power.yscr -row 1 -column 2 -sticky ns
grid .power.zoom -row 2 -column 0 -sticky ew

grid columnconfigure .power 0 -weight 0
grid columnconfigure .power 1 -weight 1
grid columnconfigure .power 2 -weight 0

grid rowconfigure .power 0 -weight 0
grid rowconfigure .power 1 -weight 1
grid rowconfigure .power 2 -weight 0

global main_menubar

$main_menubar.windows.menu add checkbutton -label "Power" \
    -variable power_open

# .menubar.session.menu.modules entryconfigure {Power} -state disabled


set power_tracenames_width [option get .power.traces namewidth ""]


proc power_open_changed {name1 name2 op} {
    global power_open

    if {$power_open} {
	wm deiconify .power
    } else {
	wm withdraw .power
    }
}

trace variable power_open w power_open_changed

set power_open 1


proc power_change_zoom {change} {
    global power_zoom_scale

    set power_zoom_scale [expr $power_zoom_scale * $change]
    power_update_all
}


proc power_set_zoom {scale} {
    global power_zoom_scale

    power_change_zoom [expr $scale/$power_zoom_scale]
}


set power_zoom_scale 1.0
set power_traces_height 0
set power_traces_max 0


proc power_update_scrollregion {name1 name2 op} {
    global power_traces_height
    global power_traces_max
    global infinity

    .power.traces configure \
	-scrollregion [list 0 0 $infinity $power_traces_height]
}


trace variable power_traces_height w power_update_scrollregion
trace variable power_traces_max w power_update_scrollregion

set infinity 10000000

set power_id 0
set power_next_ypos 0
set power_trace_spacing 30


proc power_hmove_start {pvid xpos} {
    global power_hmove_prev_xpos

    bind .power.traces <Button2-Motion> "power_hmove_update %x"
    bind .power.traces <ButtonRelease-2> "power_hmove_done"
    .power.traces configure -cursor sb_h_double_arrow
    .power.traces addtag hmoving withtag current
    set power_hmove_prev_xpos $xpos
}


proc power_hmove_update {xpos} {
    global power_hmove_prev_xpos

    .power.traces move hmoving [expr $xpos - $power_hmove_prev_xpos] 0
    set power_hmove_prev_xpos $xpos
}


proc power_hmove_done {} {
    bind .power.traces <Button2-Motion> {}
    bind .power.traces <ButtonRelease-2> {}
    .power.traces configure -cursor top_left_arrow
    .power.traces dtag hmoving
}


proc power_vmove_start {pvid} {
    global power_vmove_pvid

    set power_vmove_pvid $pvid
    bind .power.traces <Button1-Motion> "power_vmove_update"
    bind .power.traces <ButtonRelease-1> "power_vmove_done %y"
    .power.traces configure -cursor sb_v_double_arrow
}


proc power_vmove_update {} {
}


proc power_vmove_done {y} {
    global power_vmove_pvid
    global power_next_ypos
    global power_ypos

    bind .power.traces <Button1-Motion> {}
    bind .power.traces <ButtonRelease-1> {}
    .power.traces configure -cursor top_left_arrow

    set from_ypos $power_ypos($power_vmove_pvid)
    set to_ypos [power_screeny_to_ypos $y]

    if {($to_ypos<0) || ($to_ypos>=$power_next_ypos)} {
	bell
	return
    }

    set dist 0
    if {$from_ypos<$to_ypos} {
	for_array_keys t power_ypos {
	    if {($power_ypos($t)>$from_ypos) && ($power_ypos($t)<=$to_ypos)} {
		incr power_ypos($t) -1
		.power.traces addtag moveupone withtag $t
		.power.traces addtag moveupone withtag $t
	    }
	}
	set dist [expr $to_ypos-$from_ypos]
	set power_ypos($power_vmove_pvid) $to_ypos
	.power.traces addtag movedownlots withtag $power_vmove_pvid
	.power.traces addtag movedownlots withtag $power_vmove_pvid
    } elseif {$from_ypos>$to_ypos} {
	for_array_keys t power_ypos {
	    if {($power_ypos($t)<$from_ypos) && ($power_ypos($t)>=$to_ypos)} {
		incr power_ypos($t)
		.power.traces addtag movedownone withtag $t
		.power.traces addtag movedownone withtag $t
	    }
	}
	set dist [expr $from_ypos-$to_ypos]
	set power_ypos($power_vmove_pvid) $to_ypos
	.power.traces addtag moveuplots withtag $power_vmove_pvid
	.power.traces addtag moveuplots withtag $power_vmove_pvid
    }

    power_move_movethem .power.traces $dist
    power_move_movethem .power.traces $dist
}

proc power_move_movethem {win dist} {
    global power_trace_spacing

    $win move moveupone 0 -$power_trace_spacing
    $win move movedownone 0 $power_trace_spacing
    $win move moveuplots 0 [expr -$dist*$power_trace_spacing]
    $win move movedownlots 0 [expr $dist*$power_trace_spacing]
    $win dtag moveupone
    $win dtag movedownone
    $win dtag moveuplots
    $win dtag movedownlots
}


proc power_new {name variable} {
    global power_id
    global power_ypos
    global power_next_ypos
    global power_traces_height
    global power_trace_spacing
    global power_traces_width
    global infinity
    global power_vars
    global power_label_font

    set pvid p$power_id
    incr power_id
    set power_ypos($pvid) $power_next_ypos
    incr power_next_ypos
    set power_traces_height [expr $power_next_ypos * $power_trace_spacing]

    set y [power_pvid_to_canvasy $pvid]
    set txtid [.power.traces create text 0 $y -anchor w -text $name \
		   -tags "$pvid hmoveable" -font $power_label_font ]

    .power.traces bind $txtid <ButtonPress-1> \
	"power_vmove_start $pvid"
    .power.traces bind $txtid <ButtonPress-2> \
	"power_hmove_start $pvid %x"

    set yy [expr $y+($power_trace_spacing/2)]
    .power.traces create line 0 $yy $infinity $yy \
	-fill grey60 -tags $pvid

    set power_vars($pvid) $variable

    global $variable
    trace variable $variable w "power_update $pvid"

    return $pvid
}


proc power_update {pvid name1 name2 op} {
    global $name1
    global power_tracenames_width

    set pwr [set $name1]
    set y [power_pvid_to_canvasy $pvid]
    .power.traces delete bar_$pvid
    .power.traces create rectangle $power_tracenames_width [expr $y-12] \
	[expr [power_pwr_to_canvasx $pwr]+$power_tracenames_width] [expr $y+12] \
	-tags bar_$pvid -fill red -outline red
}


proc power_update_all {} {
    global power_vars

    for_array_keys pvid power_vars {
	power_update $pvid $power_vars($pvid) "" w
    }
}


proc power_delete {pvid} {
    global power_ypos
    global power_next_ypos
    global power_trace_spacing
    global power_traces_height
    global power_vars

    .power.traces delete $pvid
    set ypos $power_ypos($pvid)
    for_array_keys t power_ypos {
	if {$power_ypos($t)>$ypos} {
	    incr power_ypos($t) -1
	    .power.traces addtag moveupone withtag $t
	}
    }
    power_move_movethem .power.traces 0
    unset power_ypos($pvid)
    incr power_next_ypos -1
    set power_traces_height [expr $power_next_ypos * $power_trace_spacing]
    unset power_vars($pvid)
}


proc power_ypos_to_canvasy {ypos} {
    global power_trace_spacing

    return [expr ($ypos*$power_trace_spacing)+($power_trace_spacing/2)]
}


proc power_pvid_to_canvasy {pvid} {
    global power_ypos

    return [power_ypos_to_canvasy $power_ypos($pvid)]
}


proc power_screenx_to_canvasx {scrx} {
    return [.power.traces canvasx $scrx]
}


proc power_screeny_to_canvasy {scry} {
    return [.power.traces canvasy $scry]
}

proc power_screeny_to_ypos {scry} {
    global power_trace_spacing

    return [expr int([power_screeny_to_canvasy $scry]/$power_trace_spacing)]
}

proc power_pwr_to_canvasx {t} {
    global power_zoom_scale

    return [expr $t * $power_zoom_scale]
}

proc power_canvas_to_pwr {x} {
    global power_zoom_scale

    return [expr $x / $power_zoom_scale]
}

proc power_canvasx_to_screenx {x} {

    set rootx [winfo rootx .power.traces]
    set xoffset [.power.traces canvasx 0]
    return [expr int($x+$rootx-$xoffset)]
}

proc power_canvasy_to_screeny {y} {

    set rooty [winfo rooty .power.traces]
    set yoffset [.power.traces canvasy 0]
    return [expr int($y+$rooty-$yoffset)]
}


proc power_pwr_to_screenx {t} {

    return [power_canvasx_to_screenx [power_pwr_to_canvasx $t]]
}


proc power_pvid_to_screeny {pvid} {

    return [power_canvasy_to_screeny [power_pvid_to_canvasy $pvid]]
}


set power_label_font [option get .power.traces font Font]
