#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: realtime.tcl,v 1.1.1.1 1999/07/15 18:47:52 bardslea Exp $
#
#
#   realtime module
#   delay during executuion so that simulated time and real time elapse
#   at some fixed relative rate.
#
#	11 Jun 1999, AMULET Group

namespace eval realtime {
    
    variable start_rtime
    variable scale 100
    variable enabled 0
    variable was_enabled 0

    trace variable enabled w [namespace code enabled_changed]

    proc enabled_changed {name1 name2 op} {
	variable enabled
	variable was_enabled

	if {$enabled!=$was_enabled} {
	    set was_enabled $enabled
	    if {$enabled} {
		enable
	    } else {
		disable
	    }
	}
    }

    proc enable {} {
	rename ::bci_run real_bci_run
	rename realtime_run ::bci_run
    }

    proc disable {} {
	rename ::bci_run realtime_run
	rename real_bci_run ::bci_run
    }

    proc realtime_run {} {
	::realtime::run
    }

    proc run {} {
	global time
	variable start_rtime

	set start_rtime [clock clicks]
	trace variable time w [namespace code timecheck]
	real_bci_run
    }

    proc timecheck {name1 name2 op} {
	global time
	
	set desired_rtime [calc_desired_rtime $time]
	set actual_rtime [clock clicks]
	# puts "timecheck: desired=${desired_rtime}, actual=${actual_rtime}"

	if {$desired_rtime>$actual_rtime} {
	    set delay [expr int(($desired_rtime-$actual_rtime)/1000.0)]
	    # puts "waiting for ${delay} ms"
	    after $delay [namespace code resume]
	    suspend
	}
    }

    proc suspend {} {
	global bci_run_stop

	set bci_run_stop 1
    }

    proc resume {} {
	real_bci_run
    }

    proc calc_desired_rtime {time} {
	variable start_rtime
	variable scale

	return [expr $time*$scale*1000 + $start_rtime]
    }

    li_require windows
    li_require winutils

    ::winutils::maketoplevel realtime "Realtime"

    radiobutton .realtime.enablebutton \
	-text "Tie simulated time to real time" \
	-variable [namespace current]::enabled -value 1
    radiobutton .realtime.disablebutton \
	-text "Run flat out" \
	-variable [namespace current]::enabled -value 0

    label .realtime.scalelabel -text "Real ms per simulated time unit"
    entry .realtime.scale -textvariable [namespace current]::scale

    pack .realtime.enablebutton -side top -anchor nw
    pack .realtime.disablebutton -side top -anchor nw
    pack .realtime.scalelabel -side left -anchor nw
    pack .realtime.scale -side right -anchor nw

    lappend ::config_save_list {{realtime} {Save realtime setup}}

    namespace eval :: {
	proc config_save_go_realtime {f} {
	    config_save_var $f ::realtime::scale
	    config_save_var $f ::realtime::enabled
	}
    }
}
