#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: selector.tcl,v 1.3 2000/05/04 13:07:02 bardslea Exp $
#
#
#   Channel selector
#
#	11 Jun 1999, AMULET Group

li_require windows
li_require winutils
li_require tview

# Selector window

winutils::make_image hidden
winutils::make_image visible

global cands_window
global cands_menubar
global cands_notebook

global selector_toplevel
global main_menubar

scrollbar $selector_toplevel.scr -command "$selector_toplevel.text yview"
pack $selector_toplevel.scr -side right -fill y

text $selector_toplevel.text -yscrollcommand "$selector_toplevel.scr set"
pack $selector_toplevel.text -expand yes -fill both

$main_menubar.windows.menu add checkbutton -label "Selector" \
    -variable selector_open -underline 0

# .menubar.session.menu.modules entryconfigure Selector -state disabled


# Maintain information about structure

proc struct_add {attrs name} {
    global struct

#    puts "Adding $attrs $name"
    set struct [struct_add_i $attrs $struct [lrange $name 1 end] ]
}

proc struct_add_i {attrs substruct name} {

    if {[llength $name] == 0} {
	return $substruct
    }

    set subblocklist [lindex $substruct 1]
    set l [llength $subblocklist]
    for {set i 0} {$i<$l} {incr i} {
	if [cequal [lindex [lindex $subblocklist $i] 0] [lindex $name 0] ] {
	    return [lreplace $substruct 1 1 \
			[lreplace $subblocklist $i $i \
			      [struct_add_i $attrs \
				   [lindex $subblocklist $i] \
				   [lrange $name 1 end] ] ] ]
	}
    }

    return [lreplace $substruct 1 1 \
		[ concat $subblocklist \
		      [list [list [lindex $name 0] \
				 [struct_add_i $attrs {} \
				      [lrange $name 1 end] ] $attrs ] ] ] ]

}


proc struct_rm_attr {attr pathname} {
    global struct

#    puts "removing attribute $attr from $pathname"
    set struct [struct_rm_attr_i $attr $struct [lrange $pathname 1 end] ]
}


proc struct_rm_attr_i {attr substruct name} {

    if {[llength $name] ==0} {
	set attrs [lindex $substruct 2]
	set pos [lsearch -exact $attrs $attr]
	while {$pos != -1} {
	    set attrs [lreplace $attrs $pos $pos]
	    set pos [lsearch -exact $attrs $attr]
	}
	return [lreplace $substruct 2 2 $attrs]
    }

    set subblocklist [lindex $substruct 1]
    set l [llength $subblocklist]
    for {set i 0} {$i<$l} {incr i} {
	if [cequal [lindex [lindex $subblocklist $i] 0] [lindex $name 0] ] {
	    return [lreplace $substruct 1 1 \
			[lreplace $subblocklist $i $i \
			     [struct_rm_attr_i $attr \
				  [lindex $subblocklist $i] \
				  [lrange $name 1 end] ] ] ]
	}
    }

    error "panic!"
}


proc struct_add_attr {attr pathname} {
    global struct

#    puts "adding attribute $attr to $pathname"
    set struct [struct_add_attr_i $attr $struct [lrange $pathname 1 end] ]
}

proc struct_add_attr_i {attr substruct name} {

    if {[llength $name] ==0} {
	return [lreplace $substruct 2 2 [concat [lindex $substruct 2] $attr] ]
    }

    set subblocklist [lindex $substruct 1]
    set l [llength $subblocklist]
    for {set i 0} {$i<$l} {incr i} {
	if [cequal [lindex [lindex $subblocklist $i] 0] [lindex $name 0] ] {
	    return [lreplace $substruct 1 1 \
			[lreplace $subblocklist $i $i \
			     [struct_add_attr_i $attr \
				  [lindex $subblocklist $i] \
				  [lrange $name 1 end] ] ] ]
	}
    }

    error "panic!"
}


proc struct_get_attrs {pathname} {
    global struct

    return [struct_get_attrs_i $struct [lrange $pathname 1 end]]
}


proc struct_get_attrs_i {substruct pathname} {
    
    if {[llength $pathname] ==0} {
	return [lindex $substruct 2]
    }

    set subblocklist [lindex $substruct 1]
    set l [llength $subblocklist]
    for {set i 0} {$i<$l} {incr i} {
	if [cequal [lindex [lindex $subblocklist $i] 0] [lindex $pathname 0] ] {
	    return [struct_get_attrs_i [lindex $subblocklist $i] \
			[lrange $pathname 1 end]]
	}
    }
    return "notfound"
}

set struct {toplevel {} {block}}

proc selector_open_changed {name1 name2 op} {
    global selector_open
    global cands_window

    if {$selector_open} {
	selector_update
	wm deiconify $cands_window
    } else {
	wm withdraw $cands_window
    }
}

trace variable selector_open w selector_open_changed


proc selector_update {} {
    global struct
    global selector_update_pending
    global selector_toplevel

    set top_scr [lindex [$selector_toplevel.text yview] 0]
    $selector_toplevel.text delete 1.0 end
    selector_update_i $struct 0 ""
    $selector_toplevel.text yview moveto $top_scr
    set selector_update_pending 0
    after cancel selector_update
}


proc selector_update_i {struct indent pathname} {
    global selector_pre_update_hook
    global selector_post_update_hook
    global selector_toplevel

    lappend pathname [lindex $struct 0]

    set insertline [lindex [split [$selector_toplevel.text index insert] .] 0]

    for {set i 0} {$i<$indent} {incr i} {
	$selector_toplevel.text insert insert "\t"
    }

    set attrlist [lindex $struct 2]
    set isblock [expr [lsearch -exact $attrlist block] != -1]

    if {$isblock} {
	set hidden [expr [lsearch -exact $attrlist hidden] != -1]
	if {$hidden} {
	    button $selector_toplevel.text.visbutton$insertline -image hidden_img \
		-command "struct_rm_attr hidden {$pathname}; selector_update"
	    $selector_toplevel.text window create insert \
		-window $selector_toplevel.text.visbutton$insertline
	} else {
	    button $selector_toplevel.text.hidebutton$insertline -image visible_img \
		-command "struct_add_attr hidden {$pathname}; selector_update"
	    $selector_toplevel.text window create insert \
		-window $selector_toplevel.text.hidebutton$insertline
	}
    }

    eval $selector_pre_update_hook

    $selector_toplevel.text insert insert [lindex $struct 0]

    eval $selector_post_update_hook

    $selector_toplevel.text insert insert "\n"

    if {$isblock} {
	if {!$hidden} {
	    set subblocklist [lindex $struct 1]
	    set l [llength $subblocklist]
	    for {set i 0} {$i<$l} {incr i} {
		selector_update_i [lindex $subblocklist $i] [expr $indent+1] \
		    $pathname
	    }
	}
    }
}

append selector_pre_update_hook ""
append selector_post_update_hook ""


set selector_update_pending 0

proc selector_update_later {name1 name2 op} {
    global selector_update_pending
    global selector_open

    if {!$selector_open} {
	return
    }
    if {$selector_update_pending} {
	return
    }
    set selector_update_pending 1
    after 2000 selector_update
}


trace variable struct w selector_update_later

set selector_open 1


proc selector_sel_windows {} {
    global selector_toplevel

    set wininfo [$selector_toplevel.text dump -window sel.first sel.last]
    set wins ""
    while {![lempty $wininfo]} {
	set key [lvarpop wininfo 0]
	set val [lvarpop wininfo 0]
	set ind [lvarpop wininfo 0]
	lappend wins $val
    }
    return $wins
}

set config_save_structure 0

lappend config_save_list {{structure} {Save Structure}}

proc config_save_go_structure {f} {
    config_save_var $f struct
}


append selector_post_update_hook {
    global selector_toplevel

    if {[lsearch -exact $attrlist unconf] != -1} {
	$selector_toplevel.text insert insert "  (unconfirmed)"
    }
}

