#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: signals.tcl,v 1.2 2000/03/24 17:06:43 bardslea Exp $
#
#
#   Signals
#
#	11 Jun 1999, AMULET Group

li_require tview
li_require threads
li_require selector

global cands_menubar
global main_menubar

$main_menubar.session.menu.modules entryconfigure "Signals" -state disabled

global selector_toplevel

menubutton $cands_menubar.signals -text "Signals" \
    -menu $cands_menubar.signals.menu
pack $cands_menubar.signals -side left
menu $cands_menubar.signals.menu

$cands_menubar.signals.menu add command -label "Add selection" \
    -command sig_sel_add
$cands_menubar.signals.menu add command -label "Delete selection" \
    -command sig_sel_delete
$cands_menubar.signals.menu add separator
$cands_menubar.signals.menu add cascade -label "Default format" \
    -menu $cands_menubar.signals.menu.default_format
$cands_menubar.signals.menu add checkbutton -label "Trace by default" \
    -variable sig_default_trace
$cands_menubar.signals.menu add checkbutton \
    -label "Show databox by default" -variable sig_default_show_data
$cands_menubar.signals.menu add checkbutton \
    -label "Show blobs by default" -variable sig_default_show_blobs
$cands_menubar.signals.menu add checkbutton \
    -label "Show as boolean by default" -variable sig_default_show_boolean

menu $cands_menubar.signals.menu.default_format
$cands_menubar.signals.menu.default_format add radiobutton -label "Blank" \
    -variable sig_default_format -value " "
$cands_menubar.signals.menu.default_format add radiobutton -label "Hex" \
    -variable sig_default_format -value " %X"
$cands_menubar.signals.menu.default_format add radiobutton \
    -label "Decimal" -variable sig_default_format -value " %d"


proc sig_sel_add {} {
    
    set wins [selector_sel_windows]
    while {![lempty $wins]} {
	set win [lvarpop wins 0]
	if {[regexp {sigbutton(s[0-9]*)$} $win foo sid]} {
	    global sig_traced_$sid
	    if {![set sig_traced_$sid]} {
		set sig_traced_$sid 1
	    }
	}
    }
}


proc sig_sel_delete {} {
    
    set wins [selector_sel_windows]
    while {![lempty $wins]} {
	set win [lvarpop wins 0]
	if {[regexp {sigbutton(s[0-9]*)$} $win foo sid]} {
	    global sig_traced_$sid
	    if {[set sig_traced_$sid]} {
		set sig_traced_$sid 0
	    }
	}
    }
}



proc sig_selector_pre_update {attrlist} {
	global selector_toplevel
    set sid [lindex $attrlist [lsearch -regexp $attrlist {^s[0-9]*$}]]
    
    checkbutton $selector_toplevel.text.sigbutton$sid -text "tview" \
	-variable sig_traced_$sid
    $selector_toplevel.text window create insert \
	-window $selector_toplevel.text.sigbutton$sid
}


append selector_pre_update_hook {
    if {[lsearch -exact $attrlist sig] != -1} {
	sig_selector_pre_update $attrlist
    }
}



proc sig_add_tview {sid} {
    global sig_names
    global sig_tvids
    global time
    global tview_toplevel

    if {![catch {set tvid $sig_tvids($sid)}]} {
	return
    }

    set tvid [tview_new $sig_names($sid)]
    set sig_tvids($sid) $tvid

    $tview_toplevel.tracenames.$tvid.menu add command -label "Delete" \
	-command "set sig_traced_$sid 0" -underline 0
    $tview_toplevel.tracenames.$tvid.menu add checkbutton -label "Show databox" \
	-variable sig_show_data($sid) -underline 11
    $tview_toplevel.tracenames.$tvid.menu add checkbutton -label "Show blobs" \
	-variable sig_show_blobs($sid) -underline 5
    $tview_toplevel.tracenames.$tvid.menu add checkbutton -label "Show as boolean" \
	-variable sig_show_boolean($sid)
    $tview_toplevel.tracenames.$tvid.menu add command -label "Format..." \
	-command "sig_format $sid" -underline 0

    set y [tview_tvid_to_canvasy $tvid]
    $tview_toplevel.traces lower [ $tview_toplevel.traces create rectangle \
			      0 [expr $y-12] [tview_time_to_canvasx $time] \
			      [expr $y+12] -fill grey40 -outline grey40 \
			      -tags "$tvid stretchzoom" ]
}


proc sig_format {sid} {
    global sig_sizes
    global sig_names
    global sig_default_format

    tview_format "sig_format_$sid" $sig_sizes($sid) $sig_names($sid) $sid \
	$sig_default_format
}


proc sig_tview_delete {sid} {
    global sig_tvids

    if [catch {set tvid $sig_tvids($sid)}] {
	return
    }
    tview_delete $tvid
    unset sig_tvids($sid)
}


proc sig_new {name sz} {
    global threadnames
    global bci_current_thread
    global sig_id
    global sig_seq_id
    global sig_names
    global sig_pathnames
    global sig_show_data
    global sig_show_blobs
    global sig_show_boolean
    global sig_sizes
    global sig_default_format
    global sig_default_show_data
    global sig_default_show_blobs
    global sig_default_show_boolean
    global sig_default_trace
    global sig_trace_thisone
    global sig_new_hook

    set pathname [concat $threadnames($bci_current_thread) signals \
		      "{$name}"]
    set attrs [struct_get_attrs $pathname]
    if {$attrs=="notfound"} {
	set sig_id $sig_seq_id
	set sid s$sig_id
	incr sig_seq_id
	struct_add block [concat $threadnames($bci_current_thread) signals ]
	struct_add "sig $sid" $pathname
	global sig_traced_$sid
	set sig_names($sid) $name
	set sig_pathnames($sid) $pathname
	set sig_sizes($sid) $sz
	set sig_show_data($sid) $sig_default_show_data
	set sig_show_blobs($sid) $sig_default_show_blobs
	set sig_show_boolean($sid) $sig_default_show_boolean
	for {set i 0} {$i<$sz} {incr i} {
	    set fv "sig_format_${sid}_${i}"
	    global $fv
	    set $fv $sig_default_format
	}
	set sig_trace_thisone $sig_default_trace
	sig_trace_vars $sid
    } else {
	set sig_id [crange \
			[lindex $attrs [lsearch -regexp $attrs {^s[0-9]*$}]] \
			1 end]
	set sid s$sig_id
	struct_rm_attr unconf $pathname
	global sig_traced_$sid
	if {$sig_sizes($sid)!=$sz} {
	    error "Signal size for $name has changed from $sig_size($sid) to $sz"
	}
	set traced [set sig_traced_$sid]
	set sig_trace_thisone $traced
    }
    eval $sig_new_hook
    set odv "sig_old_data_${sid}"
    global $odv
    set $odv -1
}


proc sig_trace_vars {sid} {
    global sig_traced_$sid
    global sig_trace_vars_hook
    trace variable sig_traced_$sid w "sig_traced_changed $sid"
    eval $sig_trace_vars_hook
}

append sig_new_hook {}

append sig_trace_vars_hook {}


proc sig_traced_changed {sid name1 name2 op} {
    global sig_traced_$sid

    if {[set sig_traced_$sid]} {
	sig_add_tview $sid
    } else {
	sig_tview_delete $sid
    }
}



set sig_seq_id 0
set sig_default_format " %d"
set sig_default_show_data 0
set sig_default_show_blobs 0
set sig_default_show_boolean 0
set sig_default_trace 0



proc sig_get_data {sid} {
    global sig_sizes

    set sz $sig_sizes($sid)
    set data ""
    for {set i 0} {$i<$sz} {incr i} {
	set dv "sig_data_${sid}_${i}"
	set fv "sig_format_${sid}_${i}"
	global $dv
	global $fv
	append data [format [set $fv] [set $dv]]
    }
    return $data
}


proc sig_data {sid} {
    global sig_show_data
    global sig_show_blobs
    global sig_tvids
    global time
    global sig_show_boolean
    global tview_toplevel

    set tvid $sig_tvids($sid)

    if {$sig_show_boolean($sid)} {
	set x [tview_time_to_canvasx $time]
	set y [tview_tvid_to_canvasy $tvid]
	set ymax [expr $y+12]
	set ymin [expr $y-12]
	set odv "sig_old_data_${sid}"
	global $odv
	set od [set $odv]
	set otv "sig_oldt_${sid}"
	global $otv
	if {$od==-1} {
	} else {
	    set oldx [tview_time_to_canvasx [set $otv]]
	    if {$od} {
		$tview_toplevel.traces create line $oldx $ymin $x $ymin \
		    -tags "$tvid stretchzoom"
	    } else {
		$tview_toplevel.traces create line $oldx $ymax $x $ymax \
		    -tags "$tvid stretchzoom"
	    }
	}
	set dv "sig_data_${sid}_0"
	global $dv
	set d [set $dv]
	set $odv $d
	set $otv $time
	if {$od!=$d} {
	    $tview_toplevel.traces create line $x $ymin $x $ymax \
		-tags "$tvid stretchzoom"
	}
    }

    if {!$sig_show_data($sid) && !$sig_show_blobs($sid)} {
	return
    }

    set data [sig_get_data $sid]
    if {$sig_show_data($sid)} {
	tview_databox $tvid $time $data
    }
    if {$sig_show_blobs($sid)} {
	blob $tvid $time $data
    }
}


lappend config_save_list {{signals} {Save Signal Formats}}

proc config_save_go_signals {f} {
    global sig_names
    global sig_pathnames
    global sig_sizes
    global sig_show_data
    global sig_show_blobs
    global sig_show_boolean
    global sig_tvids
    global sig_seq_id
    global tview_ypos

    if {![array exists sig_names]} {
	puts $f "\# No Signals"
	return
    }

    puts $f "set sig_seq_id $sig_seq_id"
    for_array_keys sid sig_names {
	puts $f "\# Signal \"$sig_names($sid)\""
	puts $f "struct_add_attr unconf {$sig_pathnames($sid)}"
	puts $f "set sig_names($sid) {$sig_names($sid)}"
	puts $f "set sig_pathnames($sid) {$sig_pathnames($sid)}"
	puts $f "set sig_sizes($sid) $sig_sizes($sid)"
	puts $f "set sig_show_data($sid) $sig_show_data($sid)"
	puts $f "set sig_show_blobs($sid) $sig_show_blobs($sid)"
	puts $f "set sig_show_boolean($sid) $sig_show_boolean($sid)"
	for {set i 0} {$i<$sig_sizes($sid)} {incr i} {
	    set fv "sig_format_${sid}_${i}"
	    global $fv
	    set fvv [set $fv]
	    puts $f "set $fv {$fvv}"
	}
	global sig_traced_$sid
	set traced [set sig_traced_$sid]
	if {!$traced} {
	    puts $f "set sig_traced_$sid $traced"
	}
	puts $f "sig_trace_vars $sid"
	puts $f ""
    }
    
    puts $f "\# Traced signals in trace position order"
    if {[catch {
	for_array_keys sid sig_tvids {
	    set tvid $sig_tvids($sid)
	    set ypos $tview_ypos($tvid)
	    lappend unsorted "$ypos $sid"
	}
    }]} {
	puts $f "\# No traced signals"
	return
    }
    set sorted [lsort -command sig_save_sort $unsorted]
    foreach p $sorted {
	lassign $p ypos sid
	puts $f "set sig_traced_$sid 1"
    }
}


proc sig_save_sort {a b} {
    set a0 [lindex $a 0]
    set b0 [lindex $b 0]
    if {$a0<$b0} {return -1} elseif {$a0>$b0} {return 1} else {return 0}
}


set config_save_signals 0

trace variable config_save_signals w config_save_signals_update
trace variable config_save_structure w config_save_signals_update

proc config_save_signals_update {name1 name2 op} {
    global config_save_signals
    global config_save_structure

    if {$config_save_signals} {
	set config_save_structure 1
    }
}

