#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: simctrl.tcl,v 1.2 2000/03/24 17:06:43 bardslea Exp $
#
#
#   Simulation controls window
#
#	11 Jun 1999, AMULET Group

li_require windows

global main_menubar

toplevel .simctrl
wm title .simctrl "Simulation controls"
wm iconname .simctrl "Simulation controls"

frame .simctrl.menubar

menubutton .simctrl.menubar.window -text "Window" \
    -menu .simctrl.menubar.window.menu -underline 0
pack .simctrl.menubar.window -side left
menu .simctrl.menubar.window.menu
.simctrl.menubar.window.menu add command -label "Close" \
    -command "set simctrl_open 0" -underline 0

pack .simctrl.menubar -fill x

frame .simctrl.buttons

button .simctrl.run -text "Run" -command "bci_run"
button .simctrl.stop -text "Stop" -command {set bci_run_stop 1}
button .simctrl.timestep -text "Timestep" -command "simctrl_timestep"
button .simctrl.runfor -text "Run for" -command {simctrl_run_for $simctrl_time}
button .simctrl.rununtil -text "Run until" \
    -command {simctrl_run_until $simctrl_time}
entry .simctrl.timeentry -textvariable simctrl_time

grid .simctrl.run .simctrl.stop .simctrl.timestep \
    -in .simctrl.buttons -row 0 -sticky ew
grid .simctrl.runfor .simctrl.rununtil .simctrl.timeentry \
    -in .simctrl.buttons -row 1 -sticky ew

pack .simctrl.buttons

$main_menubar.windows.menu add checkbutton -label "Simulation controls" \
    -variable simctrl_open -underline 11

$main_menubar.sim.menu add command -label "Controls..." \
    -command {set simctrl_open 1}

$main_menubar.session.menu.modules entryconfigure {Simulation Controls} \
    -state disabled

proc simctrl_open_changed {name1 name2 op} {
    global simctrl_open

    if {$simctrl_open} {
	wm deiconify .simctrl
    } else {
	wm withdraw .simctrl
    }
}

trace variable simctrl_open w simctrl_open_changed

set simctrl_open 1


trace variable bci_running w simctrl_cancel

proc simctrl_cancel {name1 name2 op} {
    global time
    global bci_running

    if {!$bci_running} {
	trace vdelete time w simctrl_timerun_check
    }
}


proc simctrl_timestep {} {
    global simctrl_timestop_expr
    global time

    set simctrl_timestop_expr "\$time>$time"
    simctrl_timerun
}


proc simctrl_run_for {t} {
    global time

    set target_time [expr $time+$t]
    simctrl_run_until $target_time
}


proc simctrl_run_until {t} {
    global simctrl_timestop_expr

    set simctrl_timestop_expr "\$time>=$t"
    simctrl_timerun
}


proc simctrl_timerun {} {
    global time

    trace variable time w simctrl_timerun_check
    bci_run
}


proc simctrl_timerun_check {name1 name2 op} {
    global simctrl_timestop_expr
    global time
    global bci_run_stop

    if $simctrl_timestop_expr {
	set bci_run_stop 1
    }
}



proc simctrl_update {name1 name2 op} {
    global bci_ready
    global bci_running
    global stdio_input

    if {$stdio_input} {
	.simctrl.run configure -state disabled
	.simctrl.timestep configure -state disabled
	.simctrl.runfor configure -state disabled
	.simctrl.rununtil configure -state disabled
	.simctrl.stop configure -state disabled
    } elseif {$bci_running} {
	.simctrl.run configure -state disabled
	.simctrl.timestep configure -state disabled
	.simctrl.runfor configure -state disabled
	.simctrl.rununtil configure -state disabled
	.simctrl.stop configure -state normal
     } elseif {$bci_ready} {
	.simctrl.run configure -state normal
	.simctrl.timestep configure -state normal
	.simctrl.runfor configure -state normal
	.simctrl.rununtil configure -state normal
	.simctrl.stop configure -state disabled
    } else {
	.simctrl.run configure -state disabled
	.simctrl.timestep configure -state disabled
	.simctrl.runfor configure -state disabled
	.simctrl.rununtil configure -state disabled
	.simctrl.stop configure -state disabled
    }
}

trace variable bci_ready w simctrl_update
trace variable bci_running w simctrl_update
trace variable stdio_input w simctrl_update

simctrl_update a b c
