#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: sourceview.tcl,v 1.2 2000/03/24 17:06:44 bardslea Exp $
#
#
#   Source view
#
#	11 Jun 1999, AMULET Group

li_require windows
li_require winutils
li_require threadwin

winutils::make_image pc_pointer -foreground darkgreen
winutils::make_image breakpoint -foreground red

global sandt_window
global sandt_menubar
global sandt_notebook

global source_toplevel

global main_menubar

$main_menubar.session.menu.modules entryconfigure "Source View" -state disabled

menubutton $sandt_menubar.buffers -text "Buffers" \
    -menu $sandt_menubar.buffers.menu -underline 1
pack $sandt_menubar.buffers -side left
menu $sandt_menubar.buffers.menu

menubutton $sandt_menubar.breakpoints -text "Breakpoints" \
    -menu $sandt_menubar.breakpoints.menu -underline 0
pack $sandt_menubar.breakpoints -side left
menu $sandt_menubar.breakpoints.menu
$sandt_menubar.breakpoints.menu add command -label "Add" \
    -command "source_add_breakpoint" -underline 0
$sandt_menubar.breakpoints.menu add command -label "Delete" \
    -command "source_delete_breakpoint" -underline 0

frame $source_toplevel.buttonbar
button $source_toplevel.buttonbar.run -text "Run" \
    -command "bci_run"
button $source_toplevel.buttonbar.stop -text "Stop" \
    -command "set bci_run_stop 1"
button $source_toplevel.buttonbar.step -text "Step" \
    -command "source_step"
button $source_toplevel.buttonbar.timestep -text "Time step" \
    -command "source_time_step"
pack $source_toplevel.buttonbar.run $source_toplevel.buttonbar.stop $source_toplevel.buttonbar.step \
    $source_toplevel.buttonbar.timestep -side left

frame $source_toplevel.statusline
label $source_toplevel.statusline.filename -textvariable source_filename
pack $source_toplevel.statusline.filename


# grid $sandt_menubar -row 0 -column 0 -sticky ew
# What's this? AB
grid $source_toplevel.buttonbar -row 1 -column 0 -sticky ew
grid $source_toplevel.statusline -row 2 -column 0 -sticky ew


grid columnconfigure $source_toplevel 0 -weight 1
grid rowconfigure $source_toplevel 3 -weight 1


$main_menubar.windows.menu add checkbutton -label "Source View" \
    -variable source_open

# .menubar.session.menu.load entryconfigure {Source View} -state disabled



proc source_open_changed {name1 name2 op} {
    global source_open
    global sandt_window

    if {$source_open} {
	wm deiconify $sandt_window
    } else {
	wm withdraw $sandt_window
    }
}

trace variable source_open w source_open_changed

set source_open 1


proc source_buttonbar_update {a b c} {
    global bci_ready
    global bci_running
    global stdio_input
	global source_toplevel

    if {$stdio_input} {
	$source_toplevel.buttonbar.run configure -state disabled
	$source_toplevel.buttonbar.stop configure -state disabled
	$source_toplevel.buttonbar.step configure -state disabled
    } elseif {$bci_running} {
	$source_toplevel.buttonbar.run configure -state disabled
	$source_toplevel.buttonbar.stop configure -state normal
	$source_toplevel.buttonbar.step configure -state disabled
    } elseif {$bci_ready} {
	$source_toplevel.buttonbar.run configure -state normal
	$source_toplevel.buttonbar.stop configure -state disabled
	$source_toplevel.buttonbar.step configure -state normal
    } else {
	$source_toplevel.buttonbar.run configure -state disabled
	$source_toplevel.buttonbar.stop configure -state disabled
	$source_toplevel.buttonbar.step configure -state disabled
    }
}

trace variable bci_ready w source_buttonbar_update
trace variable bci_running w source_buttonbar_update
trace variable stdio_input w source_buttonbar_update

source_buttonbar_update a b c


proc source_current_update {a b c} {
    global bci_running
    global line_linenum
    global line_fileref
    global source_filename
    global line_filenames
    global source_fileref
    global threads_traced
    global thread_filerefs
    global thread_linenums
    global bci_current_thread
	global source_toplevel

    if {$bci_running} {
	return
    }

    if {$line_fileref==0} {
	return
    }

    set source_filename $line_filenames($line_fileref)
    set source_fileref $line_fileref
    for_array_keys tid thread_filerefs {
	if {$threads_traced($tid)} {
	    set fileref $thread_filerefs($tid)
	    set linenum $thread_linenums($tid)
	    $source_toplevel.t$fileref.text mark set arrow $linenum.0
	    catch {destroy $source_toplevel.pc$tid}
	    label $source_toplevel.pc$tid -text "$tid >" -fg grey40
	    # -image pc_pointer_img
	    $source_toplevel.t$fileref.text window create arrow \
		-window $source_toplevel.pc$tid
	    # $source_toplevel.t$fileref.text insert arrow "$tid >"
	    if {$tid==$bci_current_thread} {
		$source_toplevel.pc$tid configure -fg red
	    }
	}
    }
    $source_toplevel.t$line_fileref.text see $line_linenum.0
    raise $source_toplevel.t$line_fileref
    foreach w [$source_toplevel.t$line_fileref.text window names] {
	raise $w
    }
}


trace variable bci_running w source_current_update


proc source_remove_pc {name1 name2 op} {
    global threads_traced
	global source_toplevel

    if {!$threads_traced($name2)} {
	 catch {destroy $source_toplevel.pc$name2}
    }
}

trace variable threads_traced w source_remove_pc


proc source_end_thread {name1 name2 op} {
    global thread_filerefs
    global thread_linenums
	global source_toplevel

    unset thread_filerefs($name2)
    unset thread_linenums($name2)
    catch {destroy $source_toplevel.pc$name2}
}

trace variable threads_traced u source_end_thread


proc source_atsteppoint {a b c} {
    global bci_run_stop
    global source_stepmode
    global source_breakpoint_expr
    global line_linenum
    global line_fileref
    global threads_traced
    global bci_current_thread
    global thread_linenums
    global thread_filerefs
    global step_start_time
    global time

    if {$threads_traced($bci_current_thread)} {
	set thread_linenums($bci_current_thread) $line_linenum
	set thread_filerefs($bci_current_thread) $line_fileref
	if {$source_stepmode} {
		if {$source_stepmode==2} {
			if {$step_start_time!=$time} {
				set bci_run_stop 1
				set source_stepmode 0
			}
		} else {
			set bci_run_stop 1
			set source_stepmode 0
		}
	}
	eval $source_breakpoint_expr
    }
}


set source_stepmode 0

trace variable line_steppoint w source_atsteppoint


proc source_step {} {
    global source_stepmode

    set source_stepmode 1
    bci_run
}

proc source_time_step {} {
    global source_stepmode
    global step_start_time
    global time

    set source_stepmode 2
    set step_start_time $time
    bci_run
}


proc line_setfileref {fileref filename} {
    global line_filenames
    global source_filename
    global bci_filename
	global source_toplevel
	global sandt_menubar 

    if {[file mtime $filename] > [file mtime $bci_filename]} {
	set filename_prefix "WARNING: source is newer than binary  "
    } else {
	set filename_prefix ""
    }
    set line_filenames($fileref) "$filename_prefix $filename"
    frame $source_toplevel.t$fileref
    grid $source_toplevel.t$fileref -row 3 -column 0 -sticky nsew
    scrollbar $source_toplevel.t$fileref.yscr \
	-command "$source_toplevel.t$fileref.text yview"
    pack $source_toplevel.t$fileref.yscr -side right -fill y
    text $source_toplevel.t$fileref.text \
	-yscrollcommand "$source_toplevel.t$fileref.yscr set"
    pack $source_toplevel.t$fileref.text -side left -expand y -fill both
    $sandt_menubar.buffers.menu add command -label $filename \
	-command "source_show_buffer $fileref"
    set f [open $filename]
    while {![eof $f]} {
	$source_toplevel.t$fileref.text insert end "\t\t"
	$source_toplevel.t$fileref.text insert end [gets $f]
	$source_toplevel.t$fileref.text insert end "\n"
    }
    close $f
    set source_filename "$filename_prefix $filename"
    $source_toplevel.t$fileref.text mark set arrow 1.1
    $source_toplevel.t$fileref.text mark gravity arrow left
}

label $source_toplevel.pc_pointer -image pc_pointer_img


proc source_show_buffer {fileref} {
    global line_filenames
    global source_filename
    global source_fileref
	global source_toplevel

    raise $source_toplevel.t$fileref
    foreach w [$source_toplevel.t$fileref.text window names] {
	raise $w
    }
    set source_filename $line_filenames($fileref)
    set source_fileref $fileref
}


proc source_add_breakpoint {} {
    global source_fileref
    global source_breakpoint_list
    global source_bpnum
	global source_toplevel

    set bpline [lindex [split [$source_toplevel.t$source_fileref.text index insert] \
			    "."] 0]
    label $source_toplevel.bp$source_bpnum -image breakpoint_img
    $source_toplevel.t$source_fileref.text window create $bpline.0 \
	-window $source_toplevel.bp$source_bpnum
    lappend source_breakpoint_list [list $source_bpnum $source_fileref $bpline]
    append source_breakpoints "
	if {\$line_linenum==$bpline && \$line_fileref==$source_fileref} {
            set bci_run_stop 1
        }
    "
    incr source_bpnum
}

set source_breakpoint_expr ""
set source_bpnum 0


proc source_compute_breakpoint_expr {a b c} {
    global source_breakpoint_list
    global source_breakpoint_expr

    set source_breakpoint_expr ""
    foreach p $source_breakpoint_list {
	lassign $p bpnum fileref linenum
	append source_breakpoint_expr "
	    if {\$line_linenum==$linenum && \$line_fileref==$fileref} {
                set bci_run_stop 1
            }
        "
    }
}


trace variable source_breakpoint_list w source_compute_breakpoint_expr


proc source_delete_breakpoint {} {
    global source_breakpoint_list
    global source_fileref
	global source_toplevel

    set bpline [lindex [split [$source_toplevel.t$source_fileref.text index insert] \
			    "."] 0]
    set found 0
    set new_bp_list ""
    foreach p $source_breakpoint_list {
	lassign $p bpnum fileref linenum
	if {$linenum==$bpline && $fileref==$source_fileref} {
	    set found 1
	    destroy $source_toplevel.bp$bpnum
	} else {
	    lappend new_bp_list $p
	}
    }
    if {$found} {
	set source_breakpoint_list $new_bp_list
    } else {
	error "No breakpoint here to delete"
    }
}



set thread_filerefs(a) 1
unset thread_filerefs(a)
