#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: threadwin.tcl,v 1.2 2000/03/24 17:06:44 bardslea Exp $
#
#
#   Threads
#
#	11 Jun 1999, AMULET Group


li_require windows
li_require threads

global sandt_window
global sandt_menubar
global sandt_notebook

global source_toplevel
global threads_toplevel

global main_menubar

set sandt_window ".sandt"
set sandt_menubar "$sandt_window.menubar"

toplevel $sandt_window
wm title $sandt_window "Source view"
wm iconname $sandt_window "Source view"

frame $sandt_menubar
pack $sandt_menubar -fill x
set sandt_menubar "$sandt_menubar"

set sandt_notebook [NoteBook::create $sandt_window.notebook]
set source_toplevel [$sandt_notebook insert end source -text "Source"]
set threads_toplevel [$sandt_notebook insert end threads -text "Threads"]
$sandt_notebook raise source
pack $sandt_notebook -expand true -fill both -padx 2 -pady 2

menubutton $sandt_menubar.window -text "Window" \
    -menu $sandt_menubar.window.menu -underline 0
pack $sandt_menubar.window -side left
menu $sandt_menubar.window.menu
$sandt_menubar.window.menu add command -label "Close" \
    -command "set threadwin_open 0" -underline 0

menubutton $sandt_menubar.traces -text "Traces" \
    -menu $sandt_menubar.traces.menu -underline 0
pack $sandt_menubar.traces -side left
menu $sandt_menubar.traces.menu
$sandt_menubar.traces.menu add command -label "Trace All" \
    -command threadwin_trace_all -underline 0
$sandt_menubar.traces.menu add command -label "Untrace All" \
    -command threadwin_untrace_all -underline 0

scrollbar $threads_toplevel.scr -command "$threads_toplevel.text yview"
pack $threads_toplevel.scr -side right -fill y

text $threads_toplevel.text -yscrollcommand "$threads_toplevel.scr set"
pack $threads_toplevel.text -expand yes -fill both

$main_menubar.windows.menu add checkbutton -label "Threads" \
    -variable threadwin_open

# .menubar.session.menu.modules entryconfigure Threads -state disabled


proc threadwin_open_changed {name1 name2 op} {
    global threadwin_open
	global sandt_window

    if {$threadwin_open} {
	threadwin_update
	wm deiconify $sandt_window
    } else {
	wm withdraw $sandt_window
    }
}

trace variable threadwin_open w threadwin_open_changed


proc threadwin_update {} {
    global threadnames
    global threadwin_update_pending
    global threads_traced
	global threads_toplevel

    set top_scr [lindex [$threads_toplevel.text yview] 0]
    $threads_toplevel.text delete 1.0 end
    foreach tid [lsort -integer [array names threadnames]] {
	set name [lrange $threadnames($tid) 1 end]
	checkbutton $threads_toplevel.tracebutton$tid -variable threads_traced($tid)
	$threads_toplevel.text window create insert \
	    -window $threads_toplevel.tracebutton$tid
	$threads_toplevel.text insert insert "$tid\t$name\n"
    }
    $threads_toplevel.text yview moveto $top_scr
    set threadwin_update_pending 0
}


set threadwin_update_pending 0

proc threadwin_update_later {a b c} {
    global threadwin_update_pending
    global threadwin_open

    if {!$threadwin_open} {
	return
    }
    if {$threadwin_update_pending} {
	return
    }
    set threadwin_update_pending 1
    after 1000 threadwin_update
}


trace variable threadnames wu threadwin_update_later

set threadwin_open 1

proc threadwin_trace_all {} {
    global threads_traced
    
    foreach tid [array names threads_traced] {
	set threads_traced($tid) 1
    }
}


proc threadwin_untrace_all {} {
    global threads_traced
    
    foreach tid [array names threads_traced] {
	set threads_traced($tid) 0
    }
}

