#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: tview.tcl,v 1.3 2000/08/17 14:30:32 bardslea Exp $
#
#
#   Time View window
#
#	11 Jun 1999, AMULET Group

li_require windows
li_require winutils

winutils::make_image zoom_in
winutils::make_image zoom_out
winutils::make_image format_menu

global cands_window
global cands_menubar
global cands_notebook

global tview_toplevel
global selector_toplevel

global main_menubar

set cands_window ".cands"
set cands_menubar "$cands_window.menubar"

toplevel $cands_window
wm title $cands_window "Channel time view"
wm iconname $cands_window "Channel time view"

frame $cands_menubar
pack $cands_menubar -fill x
set cands_menubar "$cands_menubar"

set cands_notebook [NoteBook::create $cands_window.notebook]
set tview_toplevel [$cands_notebook insert end tview -text "Time view"]
set selector_toplevel [$cands_notebook insert end selector -text "Selector"]
$cands_notebook raise tview
# Make me a predictably named tview_toplevel
frame $tview_toplevel.tview
set tview_toplevel $tview_toplevel.tview
pack $tview_toplevel -expand true -fill both
pack $cands_notebook -expand true -fill both -padx 2 -pady 2

menubutton $cands_menubar.window -text "Window" \
    -menu $cands_menubar.window.menu -underline 0
pack $cands_menubar.window -side left
menu $cands_menubar.window.menu
$cands_menubar.window.menu add command -label "Close" \
    -command "set tview_open 0" -underline 0

menubutton $cands_menubar.options -text "Options" \
    -menu $cands_menubar.options.menu -underline 0
pack $cands_menubar.options -side left
menu $cands_menubar.options.menu
$cands_menubar.options.menu add checkbutton -label "Cursor" \
    -variable tview_hairline -underline 0
$cands_menubar.options.menu add checkbutton -label "Stick Right" \
    -variable tview_stick_right -underline 0
$cands_menubar.options.menu add separator
$cands_menubar.options.menu add radiobutton -label "Unlimited history" \
    -variable tview_history -value 0 -underline 0
$cands_menubar.options.menu add radiobutton -label "History 100000" \
    -variable tview_history -value 100000
$cands_menubar.options.menu add radiobutton -label "History 10000" \
    -variable tview_history -value 10000
$cands_menubar.options.menu add radiobutton -label "History 1000" \
    -variable tview_history -value 1000
$cands_menubar.options.menu add radiobutton -label "History 100" \
    -variable tview_history -value 100

menubutton $cands_menubar.unhide -text "Unhide" \
    -menu $cands_menubar.unhide.menu -underline 0
pack $cands_menubar.unhide -side left
menu $cands_menubar.unhide.menu

frame $tview_toplevel.statusline

frame $tview_toplevel.statusline.cursortime
label $tview_toplevel.statusline.cursortime.label -text "Time at cursor:"
label $tview_toplevel.statusline.cursortime.time -textvariable tview_cursortime
pack $tview_toplevel.statusline.cursortime.label $tview_toplevel.statusline.cursortime.time \
    -side left
pack $tview_toplevel.statusline.cursortime -side top -anchor e
frame $tview_toplevel.statusline.baselinetime
label $tview_toplevel.statusline.baselinetime.label -text "Cursor - Baseline:"
label $tview_toplevel.statusline.baselinetime.time -textvariable tview_baselinetime
pack $tview_toplevel.statusline.baselinetime.label $tview_toplevel.statusline.baselinetime.time \
    -side left
pack $tview_toplevel.statusline.baselinetime -side top -anchor e

frame $tview_toplevel.zoom
button $tview_toplevel.zoom.in -image zoom_in_img -command {tview_change_zoom 2.0}
button $tview_toplevel.zoom.out -image zoom_out_img -command {tview_change_zoom 0.5}
menubutton $tview_toplevel.zoom.label -textvariable tview_zoom_scale \
    -menu $tview_toplevel.zoom.label.menu
menu $tview_toplevel.zoom.label.menu

if {[catch {set tview_scalefactors}]} {
    set tview_scalefactors {32 16 8 4 2 1 0.5 0.25 0.125}
}

proc tview_scalefactors_changed {a b c} {
    global tview_scalefactors
	global tview_toplevel

    $tview_toplevel.zoom.label.menu delete 0 end
    foreach scale $tview_scalefactors {
	$tview_toplevel.zoom.label.menu add command -label $scale \
	    -command "tview_set_zoom $scale"
    }
}

tview_scalefactors_changed a b c

trace variable tview_scalefactors w tview_scalefactors_changed

pack $tview_toplevel.zoom.in $tview_toplevel.zoom.out -side left -fill y
pack $tview_toplevel.zoom.label -side left -fill both

scrollbar $tview_toplevel.xscr -orient horiz -command "tview_xscr"
scrollbar $tview_toplevel.yscr -command "tview_yscr"
canvas $tview_toplevel.scale
canvas $tview_toplevel.tracenames
canvas $tview_toplevel.traces -xscrollcommand "$tview_toplevel.xscr set" \
    -yscrollcommand "$tview_toplevel.yscr set"

bind $tview_toplevel.traces <Enter> "tview_showhairline %x %y"
bind $tview_toplevel.traces <Leave> tview_removehairline
bind $tview_toplevel.traces <Motion> "tview_movehairline %x %y"
bind $tview_toplevel.scale <Enter> "tview_showhairline %x 0"
bind $tview_toplevel.scale <Leave> tview_removehairline
bind $tview_toplevel.scale <Motion> "tview_movehairline %x 0"

bind $tview_toplevel.traces <Button-1> "tview_setbaseline %x"
bind $tview_toplevel.scale <Button-1> "tview_setbaseline %x"

# grid $cands_menubar -row 0 -column 0 -columnspan 3 -sticky ew
grid $tview_toplevel.statusline -row 1 -column 0 -columnspan 3 -sticky ew
grid $tview_toplevel.scale -row 2 -column 1 -sticky ew
grid $tview_toplevel.tracenames -row 3 -column 0 -sticky nsew
grid $tview_toplevel.traces -row 3 -column 1 -sticky nsew
grid $tview_toplevel.yscr -row 3 -column 2 -sticky ns
grid $tview_toplevel.zoom -row 4 -column 0 -sticky ew
grid $tview_toplevel.xscr -row 4 -column 1 -sticky ew

grid columnconfigure $tview_toplevel 0 -weight 0
grid columnconfigure $tview_toplevel 1 -weight 1
grid columnconfigure $tview_toplevel 2 -weight 0

grid rowconfigure $tview_toplevel 0 -weight 0
grid rowconfigure $tview_toplevel 1 -weight 0
grid rowconfigure $tview_toplevel 2 -weight 0
grid rowconfigure $tview_toplevel 3 -weight 1
grid rowconfigure $tview_toplevel 4 -weight 0


$main_menubar.windows.menu add checkbutton -label "Time View" \
    -variable tview_open -underline 0

# .menubar.session.menu.modules entryconfigure {Time View} -state disabled


set tview_tracenames_width [option get $tview_toplevel.tracenames width ""]
set tview_tracenames_align [option get $tview_toplevel.tracenames align ""]


proc tview_open_changed {name1 name2 op} {
    global tview_open
	global cands_window

    if {$tview_open} {
	wm deiconify $cands_window
	tview_extend_scale x y z
    } else {
	wm withdraw $cands_window
    }
}

trace variable tview_open w tview_open_changed

proc tview_yscr {args} {
	global tview_toplevel

    eval $tview_toplevel.tracenames yview $args
    eval $tview_toplevel.traces yview $args
}

proc tview_xscr {args} {
	global tview_toplevel

    eval $tview_toplevel.scale xview $args
    eval $tview_toplevel.traces xview $args
}

proc tview_showhairline {x y} {
    global tview_hairline
    global tview_cursortime
    global tview_traces_height
    global tview_stick_right
	global tview_toplevel

    set cx [tview_screenx_to_canvasx $x]
    if {!$tview_stick_right} {
	if {$tview_hairline} {
	    #	set cy [tview_screeny_to_canvasy $y]
	    set topthings [$tview_toplevel.traces find withtag current]
	    $tview_toplevel.traces create line $cx 0 $cx $tview_traces_height \
		-fill darkgreen -tags cursorline
	    foreach topthing $topthings {
		if {[lsearch -exact \
		     [$tview_toplevel.traces gettags $topthing] blob]!=-1} {
		    $tview_toplevel.traces raise $topthing
		}
	    }
	    #	$tview_toplevel.traces create line $cx [expr $cy+4] $cx $maxypos \
		\#	    -fill darkgreen -tags cursorline
	    $tview_toplevel.scale create line $cx 0 $cx 30 -fill darkgreen \
		-tags cursorline
	}
	set tview_cursortime [format "%.0f" [tview_canvasx_to_time $cx]]
    }
}

proc tview_removehairline {} {
    global tview_cursortime
	global tview_toplevel

    $tview_toplevel.traces delete cursorline
    $tview_toplevel.scale delete cursorline
    set tview_cursortime ""
}

proc tview_movehairline {x y} {
    tview_removehairline
    tview_showhairline $x $y
}


proc tview_setbaseline {x} {
    global tview_traces_height
    global tview_baselinebasetime
	global tview_toplevel

    set cx [tview_screenx_to_canvasx $x]
    set tview_baselinebasetime [format "%.0f" [tview_canvasx_to_time $cx]]
    $tview_toplevel.traces delete baseline
    $tview_toplevel.scale delete baseline
    $tview_toplevel.traces create line $cx 0 $cx $tview_traces_height \
	-fill green -tags {baseline stretchzoom}
    $tview_toplevel.scale create line $cx 0 $cx 30 -fill green \
	-tags {baseline stretchzoom}
}


set tview_baselinebasetime 0

trace variable tview_cursortime w tview_baselinetime_update
trace variable tview_baselinebasetime w tview_baselinetime_update

proc tview_baselinetime_update {name1 name2 op} {
    global tview_cursortime
    global tview_baselinetime
    global tview_baselinebasetime

    if {$tview_cursortime==""} {
	set tview_baselinetime ""
    } else {
	set tview_baselinetime [expr $tview_cursortime-$tview_baselinebasetime]
    }
}



proc tview_change_zoom {change} {
    global tview_zoom_scale
	global tview_toplevel

    set tview_zoom_scale [expr $tview_zoom_scale * $change]
    tview_scale_update_zoom
    $tview_toplevel.scale scale stretchzoom 0 0 $change 1
    $tview_toplevel.traces scale stretchzoom 0 0 $change 1
    
    set rectmovezooms [$tview_toplevel.traces find withtag rectmovezoom]
    foreach item $rectmovezooms {
	lassign [$tview_toplevel.traces coords $item] xmin ymin xmax ymax
	set x $xmin
	set newx [expr ($change-1)*$x]
	$tview_toplevel.traces move $item $newx 0
    }

    set ovalmovezooms [$tview_toplevel.traces find withtag ovalmovezoom]
    foreach item $ovalmovezooms {
	lassign [$tview_toplevel.traces coords $item] xmin ymin xmax ymax
	set x [expr ($xmin+$xmax)/2]
	set newx [expr ($change-1)*$x]
	$tview_toplevel.traces move $item $newx 0
    }
}


proc tview_set_zoom {scale} {
    global tview_zoom_scale

    tview_change_zoom [expr $scale/$tview_zoom_scale]
}


set tview_time_min 0


proc tview_scale_update_zoom {} {
    global tview_last_scale_end
    global tview_scale_tic_count
    global tview_zoom_scale
    global tview_scale_incr
    global tview_time_min
    global tview_traces_max
	global tview_toplevel

    set tview_scale_incr 1
    set z 5
    while {$z>$tview_zoom_scale} {
	set z [expr $z/10.0]
	set tview_scale_incr [expr $tview_scale_incr*10]
    }
    $tview_toplevel.scale delete scale
    set tview_last_scale_end \
	[expr $tview_time_min - $tview_time_min%(10*$tview_scale_incr)]
    set tview_scale_tic_count 0
    set tview_traces_max 0
    tview_extend_scale a b c
}



set tview_zoom_scale 1.0
set tview_scale_incr 10
set tview_traces_height 0
set tview_traces_max 0
set tview_stick_right 0
set tview_hairline 0
set tview_cursortime ""



proc tview_extend_scale {name1 name2 op} {
    global time
    global tview_last_scale_end
    global tview_open
	global tview_toplevel

    if (!$tview_open) {
	return;
    }
    if {$time<$tview_last_scale_end} {
	return
    }

    global tview_scale_incr
    global tview_scale_tic_count
    global tview_traces_max
    global tview_stick_right

    set x [tview_time_to_canvasx $tview_last_scale_end]
    for {set t $tview_last_scale_end} {$t<=($time)} \
	{set t [expr $t+$tview_scale_incr]} {
	set x [tview_time_to_canvasx $t]
	if {$tview_scale_tic_count==0} {
	    $tview_toplevel.scale create line $x 0 $x 12 -tags scale
	    $tview_toplevel.scale create text $x 15 -anchor n -text [format "%.0f" $t] \
		-tags scale
	} else {
	    $tview_toplevel.scale create line $x 0 $x 7 -tags scale
	}
	set tview_scale_tic_count [expr ($tview_scale_tic_count+1)%10]
    }
    $tview_toplevel.scale create line \
	[tview_time_to_canvasx $tview_last_scale_end] 0 \
	[tview_time_to_canvasx $t] 0 -tags scale
    set tview_last_scale_end $t
    if {$x>$tview_traces_max} {
	set tview_traces_max $x
    }
    if {$tview_stick_right} {
	$tview_toplevel.scale xview moveto 1
	$tview_toplevel.traces xview moveto 1
    }
}

set tview_last_scale_end 0
set tview_scale_tic_count 0

set tview_stick_right 0
set tview_history 0

trace variable time w tview_extend_scale


proc tview_update_scrollregion {name1 name2 op} {
    global tview_traces_height
    global tview_traces_max
    global tview_tracenames_width
    global tview_time_min
    global tview_traces_min
	global tview_toplevel

    set tview_traces_min [tview_time_to_canvasx $tview_time_min]
    $tview_toplevel.scale configure \
	-scrollregion [list $tview_traces_min 0 $tview_traces_max 30]
    $tview_toplevel.traces configure \
	-scrollregion [list $tview_traces_min 0 $tview_traces_max \
			   $tview_traces_height]
    $tview_toplevel.tracenames configure \
	-scrollregion [list 0 0 $tview_tracenames_width $tview_traces_height]
}

set tview_traces_min 0

trace variable tview_traces_height w tview_update_scrollregion
trace variable tview_traces_max w tview_update_scrollregion

set infinity 10000000

set tview_id 0
set tview_next_ypos 0
set tview_trace_spacing 30


proc tview_hmove_start {tvid xpos} {
    global tview_hmove_prev_xpos
	global tview_toplevel

    bind $tview_toplevel.tracenames <Button2-Motion> "tview_hmove_update %x"
    bind $tview_toplevel.tracenames <ButtonRelease-2> "tview_hmove_done"
    $tview_toplevel.tracenames configure -cursor sb_h_double_arrow
    $tview_toplevel.tracenames addtag hmoving withtag current
    set tview_hmove_prev_xpos $xpos
}


proc tview_hmove_update {xpos} {
    global tview_hmove_prev_xpos
	global tview_toplevel

    $tview_toplevel.tracenames move hmoving [expr $xpos - $tview_hmove_prev_xpos] 0
    set tview_hmove_prev_xpos $xpos
}


proc tview_hmove_done {} {
	global tview_toplevel

    bind $tview_toplevel.tracenames <Button2-Motion> {}
    bind $tview_toplevel.tracenames <ButtonRelease-2> {}
    $tview_toplevel.tracenames configure -cursor top_left_arrow
    $tview_toplevel.tracenames dtag hmoving
}


proc tview_vmove_start {tvid} {
    global tview_vmove_tvid
	global tview_toplevel

    set tview_vmove_tvid $tvid
    bind $tview_toplevel.tracenames <Button1-Motion> "tview_vmove_update"
    bind $tview_toplevel.tracenames <ButtonRelease-1> "tview_vmove_done %y"
    $tview_toplevel.tracenames configure -cursor sb_v_double_arrow
}


proc tview_vmove_update {} {
}


proc tview_vmove_done {y} {
    global tview_vmove_tvid
    global tview_next_ypos
    global tview_ypos
	global tview_toplevel

    bind $tview_toplevel.tracenames <Button1-Motion> {}
    bind $tview_toplevel.tracenames <ButtonRelease-1> {}
    $tview_toplevel.tracenames configure -cursor top_left_arrow

    set from_ypos $tview_ypos($tview_vmove_tvid)
    set to_ypos [tview_screeny_to_ypos $y]

    if {($to_ypos<0) || ($to_ypos>=$tview_next_ypos)} {
	bell
	return
    }

    set dist 0
    if {$from_ypos<$to_ypos} {
	for_array_keys t tview_ypos {
	    if {($tview_ypos($t)>$from_ypos) && ($tview_ypos($t)<=$to_ypos)} {
		incr tview_ypos($t) -1
		$tview_toplevel.traces addtag moveupone withtag $t
		$tview_toplevel.tracenames addtag moveupone withtag $t
	    }
	}
	set dist [expr $to_ypos-$from_ypos]
	set tview_ypos($tview_vmove_tvid) $to_ypos
	$tview_toplevel.traces addtag movedownlots withtag $tview_vmove_tvid
	$tview_toplevel.tracenames addtag movedownlots withtag $tview_vmove_tvid
    } elseif {$from_ypos>$to_ypos} {
	for_array_keys t tview_ypos {
	    if {($tview_ypos($t)<$from_ypos) && ($tview_ypos($t)>=$to_ypos)} {
		incr tview_ypos($t)
		$tview_toplevel.traces addtag movedownone withtag $t
		$tview_toplevel.tracenames addtag movedownone withtag $t
	    }
	}
	set dist [expr $from_ypos-$to_ypos]
	set tview_ypos($tview_vmove_tvid) $to_ypos
	$tview_toplevel.traces addtag moveuplots withtag $tview_vmove_tvid
	$tview_toplevel.tracenames addtag moveuplots withtag $tview_vmove_tvid
    }

    tview_move_movethem $tview_toplevel.tracenames $dist
    tview_move_movethem $tview_toplevel.traces $dist
}

proc tview_move_movethem {win dist} {
    global tview_trace_spacing

    $win move moveupone 0 -$tview_trace_spacing
    $win move movedownone 0 $tview_trace_spacing
    $win move moveuplots 0 [expr -$dist*$tview_trace_spacing]
    $win move movedownlots 0 [expr $dist*$tview_trace_spacing]
    $win dtag moveupone
    $win dtag movedownone
    $win dtag moveuplots
    $win dtag movedownlots
}


set tview_truncate_pending 0

proc tview_truncate_history_later {name1 name2 op} {
    global tview_truncate_pending
    global tview_history

    if {$tview_history==0} {
	return
    }
    if {$tview_truncate_pending} {
	return
    }
    set tview_truncate_pending 1
    after 10000 tview_truncate_history
}

trace variable time w tview_truncate_history_later

proc tview_truncate_history {} {
    global tview_truncate_pending
    global time
    global tview_history
    global infinity
    global tview_time_min
	global tview_toplevel

    set tview_truncate_pending 0
    if {$tview_history==0} {
	return
    }
    set tview_time_min [expr $time-$tview_history]
    $tview_toplevel.traces addtag old enclosed -10 0 \
	[tview_time_to_canvasx $tview_time_min] $infinity
    $tview_toplevel.traces delete withtag old
    $tview_toplevel.scale addtag old enclosed -10 0 \
	[tview_time_to_canvasx $tview_time_min] $infinity
    $tview_toplevel.scale delete withtag old
}


proc tview_new {name} {
    global tview_id
    global tview_ypos
    global tview_next_ypos
    global tview_traces_height
    global tview_trace_spacing
    global tview_tracenames_width
    global infinity
    global tview_tracenames_align
    global tview_label_font
	global tview_toplevel

    set tvid t$tview_id
    incr tview_id
    set tview_ypos($tvid) $tview_next_ypos
    incr tview_next_ypos
    set tview_traces_height [expr $tview_next_ypos * $tview_trace_spacing]

    set y [tview_tvid_to_canvasy $tvid]
    if {$tview_tracenames_align=="right"} {
	set txtid [$tview_toplevel.tracenames create text \
		       [expr $tview_tracenames_width-19] \
		       $y -anchor e -text $name -tags "$tvid hmoveable" \
		       -font $tview_label_font ]
    } else {
	set txtid [$tview_toplevel.tracenames create text 0 \
		       $y -anchor w -text $name -tags "$tvid hmoveable" \
		       -font $tview_label_font ]
    }
    $tview_toplevel.tracenames bind $txtid <ButtonPress-1> \
	"tview_vmove_start $tvid"
    $tview_toplevel.tracenames bind $txtid <ButtonPress-2> \
	"tview_hmove_start $tvid %x"

    menubutton $tview_toplevel.tracenames.$tvid -menu $tview_toplevel.tracenames.$tvid.menu \
	-image format_menu_img
    menu $tview_toplevel.tracenames.$tvid.menu
    $tview_toplevel.tracenames create window $tview_tracenames_width $y -anchor e \
	-window $tview_toplevel.tracenames.$tvid -tags $tvid
    $tview_toplevel.tracenames.$tvid.menu add command -label "Next right" \
	-command "tview_skip_right $tvid" -underline 5
    $tview_toplevel.tracenames.$tvid.menu add command -label "Next left" \
	-command "tview_skip_left $tvid" -underline 5
    $tview_toplevel.tracenames.$tvid.menu add command -label "Hide" \
	-command "tview_hide $tvid {$name}" -underline 0


    set yy [expr $y+($tview_trace_spacing/2)]
    $tview_toplevel.tracenames create line 0 $yy $tview_tracenames_width $yy \
	-fill grey60 -tags $tvid
    $tview_toplevel.traces create line 0 $yy $infinity $yy \
	-fill grey60 -tags $tvid

    return $tvid
}


proc tview_delete {tvid} {
    global tview_ypos
    global tview_next_ypos
    global tview_trace_spacing
    global tview_traces_height
	global tview_toplevel

    $tview_toplevel.tracenames delete $tvid
    $tview_toplevel.traces delete $tvid
    set ypos $tview_ypos($tvid)
    if {$ypos>=0} {
	for_array_keys t tview_ypos {
	    if {$tview_ypos($t)>$ypos} {
		incr tview_ypos($t) -1
		$tview_toplevel.traces addtag moveupone withtag $t
		$tview_toplevel.tracenames addtag moveupone withtag $t
	    }
	}
	tview_move_movethem $tview_toplevel.tracenames 0
	tview_move_movethem $tview_toplevel.traces 0
	incr tview_next_ypos -1
	set tview_traces_height [expr $tview_next_ypos * $tview_trace_spacing]
    } else {
	# $cands_menubar.unhide.menu delete ???
    }
    unset tview_ypos($tvid)
}


set tview_hidden_next_ypos -1

proc tview_hide {tvid name} {
    global tview_ypos
    global tview_next_ypos
    global tview_trace_spacing
    global tview_traces_height
	global tview_toplevel
	global cands_menubar

    set ypos $tview_ypos($tvid)
    set tview_ypos($tvid) -1
    $tview_toplevel.traces addtag moveuplots withtag $tvid
    $tview_toplevel.tracenames addtag moveuplots withtag $tvid
    set dist [expr $ypos+1]
    for_array_keys t tview_ypos {
	if {$tview_ypos($t)>$ypos} {
	    incr tview_ypos($t) -1
	    $tview_toplevel.traces addtag moveupone withtag $t
	    $tview_toplevel.tracenames addtag moveupone withtag $t
	}
    }
    tview_move_movethem $tview_toplevel.tracenames $dist
    tview_move_movethem $tview_toplevel.traces $dist
    incr tview_next_ypos -1
    set tview_traces_height [expr $tview_next_ypos * $tview_trace_spacing]
    $cands_menubar.unhide.menu add command -label $name \
	-command "tview_unhide $tvid"
}


proc tview_unhide {tvid} {
    global tview_ypos
    global tview_next_ypos
    global tview_trace_spacing
    global tview_traces_height
	global tview_toplevel
	global cands_menubar

    $cands_menubar.unhide.menu delete active
    set ypos $tview_next_ypos
    incr tview_next_ypos
    set tview_ypos($tvid) $ypos
    $tview_toplevel.traces addtag movedownlots withtag $tvid
    $tview_toplevel.tracenames addtag movedownlots withtag $tvid
    set dist [expr $ypos+1]
    tview_move_movethem $tview_toplevel.tracenames $dist
    tview_move_movethem $tview_toplevel.traces $dist
    set tview_traces_height [expr $tview_next_ypos * $tview_trace_spacing]
}


proc tview_skip_right {tvid} {
    global infinity
    global tview_trace_spacing
    global tview_traces_min
    global tview_traces_max
	global tview_toplevel

    lassign [$tview_toplevel.traces xview] minx maxx
    set rightx [expr $tview_traces_min+($tview_traces_max-$tview_traces_min)*$maxx]
    set y [tview_tvid_to_canvasy $tvid]
    set topy [expr $y-($tview_trace_spacing/2)]
    set bottomy [expr $y+($tview_trace_spacing/2)]

    # look for things that are entirely off the right of the screen
    # and find their left hand edge

    $tview_toplevel.traces addtag rightofcenter \
	enclosed $rightx $topy $infinity $bottomy
    lassign [$tview_toplevel.traces bbox rightofcenter] nextx junk1 junk2 junk3
    $tview_toplevel.traces dtag rightofcenter

    # look for things whose right hand edge is off the right of the screen

    foreach item [$tview_toplevel.traces find \
		      overlapping $rightx $topy $infinity $bottomy] {
	lassign [$tview_toplevel.traces bbox $item] junk1 junk2 thisx junk3
	if {$nextx=={} || $thisx<$nextx} {
	    set nextx $thisx
	}
    }
    if {$nextx=={}} {
	bell
	return
    }
    set leftx [expr ($nextx-$tview_traces_min)/($tview_traces_max-$tview_traces_min)-($maxx-$minx)/2]
    tview_xscr moveto $leftx
}


proc tview_skip_left {tvid} {
    global infinity
    global tview_trace_spacing
    global tview_traces_min
    global tview_traces_max
	global tview_toplevel

    lassign [$tview_toplevel.traces xview] minx maxx
    set leftx [expr $tview_traces_min+($tview_traces_max-$tview_traces_min)*$minx]
    set y [tview_tvid_to_canvasy $tvid]
    set topy [expr $y-($tview_trace_spacing/2)]
    set bottomy [expr $y+($tview_trace_spacing/2)]

    # look for things that are entirely off the left of the screen
    # and find their right hand edge

    $tview_toplevel.traces addtag leftofcenter \
	enclosed 0 $topy $leftx $bottomy
    lassign [$tview_toplevel.traces bbox rightofcenter] junk1 junk2 prevx junk3
    $tview_toplevel.traces dtag leftofcenter

    # look for things whose left hand edge is off the left of the screen

    foreach item [$tview_toplevel.traces find \
		      overlapping 0 $topy $leftx $bottomy] {
	lassign [$tview_toplevel.traces bbox $item] thisx junk1 junk2 junk3
	if {$prevx=={} || $thisx>$prevx} {
	    set prevx $thisx
	}
    }
    if {$prevx=={}} {
	bell
	return
    }
    set leftx [expr ($prevx-$tview_traces_min)/($tview_traces_max-$tview_traces_min)-($maxx-$minx)/2]
    tview_xscr moveto $leftx
}



proc tview_ypos_to_canvasy {ypos} {
    global tview_trace_spacing

    return [expr ($ypos*$tview_trace_spacing)+($tview_trace_spacing/2)]
}


proc tview_tvid_to_canvasy {tvid} {
    global tview_ypos

    return [tview_ypos_to_canvasy $tview_ypos($tvid)]
}


proc tview_screenx_to_canvasx {scrx} {
	global tview_toplevel

    return [$tview_toplevel.traces canvasx $scrx]
}


proc tview_screeny_to_canvasy {scry} {
	global tview_toplevel

    return [$tview_toplevel.traces canvasy $scry]
}

proc tview_screeny_to_ypos {scry} {
    global tview_trace_spacing

    return [expr int([tview_screeny_to_canvasy $scry]/$tview_trace_spacing)]
}

proc tview_time_to_canvasx {t} {
    global tview_zoom_scale

    return [expr $t * $tview_zoom_scale]
}

proc tview_canvasx_to_time {x} {
    global tview_zoom_scale

    return [expr $x / $tview_zoom_scale]
}

proc tview_canvasx_to_screenx {x} {
	global tview_toplevel

    set rootx [winfo rootx $tview_toplevel.traces]
    set xoffset [$tview_toplevel.traces canvasx 0]
    return [expr int($x+$rootx-$xoffset)]
}

proc tview_canvasy_to_screeny {y} {
	global tview_toplevel

    set rooty [winfo rooty $tview_toplevel.traces]
    set yoffset [$tview_toplevel.traces canvasy 0]
    return [expr int($y+$rooty-$yoffset)]
}


proc tview_time_to_screenx {t} {

    return [tview_canvasx_to_screenx [tview_time_to_canvasx $t]]
}


proc tview_tvid_to_screeny {tvid} {

    return [tview_canvasy_to_screeny [tview_tvid_to_canvasy $tvid]]
}


proc tview_databox {tvid t text} {
    global tview_traces_max
    global tview_databox_font
	global tview_toplevel

    set txtid [$tview_toplevel.traces create text \
		   [tview_time_to_canvasx $t] \
		   [tview_tvid_to_canvasy $tvid] \
		   -anchor w -text $text -tags "$tvid stretchzoom" \
		   -font $tview_databox_font ]
    lassign [$tview_toplevel.traces bbox $txtid] xmin ymin xmax ymax
    incr xmin
    incr ymin -1
    set boxid [$tview_toplevel.traces create rectangle $xmin $ymin $xmax $ymax \
		   -fill white -outline black -tags "$tvid rectmovezoom"]
    $tview_toplevel.traces lower $boxid $txtid
    if {$xmax>$tview_traces_max} {
	set tview_traces_max $xmax
    }
}


proc blob {tvid t msg} {
	global tview_toplevel
    
    set tags [$tview_toplevel.traces find withtag "${tvid}blobat${t}"]
    if {[llength $tags]==1} {
	set blobid $tags
	set cmd [$tview_toplevel.traces bind $blobid <Enter>]
	set oldmsg [lindex $cmd 3]
	set msg "$oldmsg\n$msg"
	$tview_toplevel.traces bind $blobid <Enter> "blob_show $tvid $t {$msg}"
	$tview_toplevel.traces itemconfigure $blobid -fill orange
	$tview_toplevel.traces raise $blobid
    } else {
	set x [tview_time_to_canvasx $t]
	set y [tview_tvid_to_canvasy $tvid]
	set blobrad 4
	set blobid [$tview_toplevel.traces create oval \
			[expr $x-$blobrad] [expr $y-$blobrad] \
			[expr $x+$blobrad] [expr $y+$blobrad] \
			-fill red -outline black \
			-tags "$tvid blob ${tvid}blobat${t} ovalmovezoom" ]
	$tview_toplevel.traces bind $blobid <Enter> "blob_show $tvid $t {$msg}"
	$tview_toplevel.traces bind $blobid <Leave> "wm withdraw .blob"
    }
}


proc blob_show {tvid t msg} {
    .blob.label configure -text $msg 
    set scrx [expr [tview_time_to_screenx $t]+10]
    set scry [expr [tview_tvid_to_screeny $tvid]+10]
    wm geometry .blob +$scrx+$scry
    wm deiconify .blob
    raise .blob
}


toplevel .blob -class Transient
wm overrideredirect .blob 1
wm withdraw .blob
label .blob.label
pack .blob.label


proc tview_format {prefix sz name id default_fmt} {
    global tview_deffmt_$id
    global tview_deffmt_width_$id
    global tview_deffmt_base_$id
    global tview_deffmt_ext_$id

    toplevel .format$id
    wm title .format$id "Format: $name"

    label .format$id.title -text "Format for $name"
    pack .format$id.title

    frame .format$id.deffmt

    label .format$id.deffmt.label -text "default format:"
    pack .format$id.deffmt.label -side left

    menubutton .format$id.deffmt.width -text "Width" \
	-menu .format$id.deffmt.width.menu -underline 0
    menu .format$id.deffmt.width.menu
    .format$id.deffmt.width.menu add radiobutton \
	-variable tview_deffmt_width_$id -label "blank" -value "blank" \
	-underline 0
    .format$id.deffmt.width.menu add radiobutton \
	-variable tview_deffmt_width_$id -label "fit" -value "_" \
	-underline 0
    .format$id.deffmt.width.menu add radiobutton \
	-variable tview_deffmt_width_$id -label "1" -underline 0
    .format$id.deffmt.width.menu add radiobutton \
	-variable tview_deffmt_width_$id -label "2" -underline 0
    .format$id.deffmt.width.menu add radiobutton \
	-variable tview_deffmt_width_$id -label "4" -underline 0
    .format$id.deffmt.width.menu add radiobutton \
	-variable tview_deffmt_width_$id -label "8" -underline 0

    menubutton .format$id.deffmt.base -text "Base" \
	-menu .format$id.deffmt.base.menu -underline 0
    menu .format$id.deffmt.base.menu
    .format$id.deffmt.base.menu add radiobutton \
	-variable tview_deffmt_base_$id -label "decimal" -value "d" \
	-underline 0
    .format$id.deffmt.base.menu add radiobutton \
	-variable tview_deffmt_base_$id -label "hex" -value "X" \
	-underline 0

    menubutton .format$id.deffmt.ext -text "Extension" \
	-menu .format$id.deffmt.ext.menu -underline 0
    menu .format$id.deffmt.ext.menu
    .format$id.deffmt.ext.menu add radiobutton \
	-variable tview_deffmt_ext_$id -label "blanks" -value "_" -underline 0
    .format$id.deffmt.ext.menu add radiobutton \
	-variable tview_deffmt_ext_$id -label "zeros" -value "0" -underline 0

    pack .format$id.deffmt.width .format$id.deffmt.base \
	.format$id.deffmt.ext -side left

    trace variable tview_deffmt_width_$id w "tview_deffmt_update $id"
    trace variable tview_deffmt_base_$id w "tview_deffmt_update $id"
    trace variable tview_deffmt_ext_$id w "tview_deffmt_update $id"

    if [catch {set tview_deffmt_$id}] {
	set tview_deffmt_$id $default_fmt
    }

    pack .format$id.deffmt

    frame .format$id.formats

    for {set i 0} {$i<$sz} {incr i} {
	set fv "${prefix}_${i}"
	global $fv
	frame .format$id.formats.e$i
	label .format$id.formats.e$i.label -text $i
	button .format$id.formats.e$i.set -text "set" \
	    -command "set $fv \$tview_deffmt_$id"
	entry .format$id.formats.e$i.ent -textvariable $fv
	pack .format$id.formats.e$i.label .format$id.formats.e$i.set \
	    .format$id.formats.e$i.ent -side left
	grid .format$id.formats.e$i -row [expr $i%10] -column [expr $i/10]
	bind .format$id.formats.e$i.ent <Return> \
	    ".format$id.formats.e$i.ent insert insert {\n}"
    }

    pack .format$id.formats

    button .format$id.ok -text "OK" -command "destroy .format$id"
    pack .format$id.ok
}


proc tview_deffmt_update {id name1 name2 op} {
    global tview_deffmt_width_$id
    global tview_deffmt_base_$id
    global tview_deffmt_ext_$id
    global tview_deffmt_$id

    set width [set tview_deffmt_width_$id]
    set base [set tview_deffmt_base_$id]
    set ext [set tview_deffmt_ext_$id]
    if {$ext!="0"} {set ext ""} else {set ext "0"}
    if {$width=="_"} {set width ""}
    if {$width=="blank"} {
	set tview_deffmt_$id ""
    } else {
	set tview_deffmt_$id " %* $ext$width$base"
    }
}


set tview_label_font [option get $tview_toplevel.traces font Font]
set tview_databox_font $tview_label_font

set tview_open 1
