#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: tviewps.tcl,v 1.3 2001/06/18 13:29:42 bardslea Exp $
#
#
#   Provide postscript plotting facility for the tview display
#
#	11 Jun 1999, AMULET Group

li_require tview

namespace eval tviewps {
	global cands_menubar
	# !
	global cands_notebook
	global tview_toplevel

    variable dialog_ok
    variable outtype cmd
    variable cmd "ghostview -a4 -"
    variable whichtraces all
    variable whichtime all
    variable scale 1.0

    # note about scale:  variables ending _pt mean points, but only when
    # scale is 1.  Scales other than 1 are fixed up by doing a global
    # scale in the postscript and imagining that the paper is bigger/smaller.


    # add plot menu to tview window

    menubutton $cands_menubar.plot -text "Plot" \
	-menu $cands_menubar.plot.menu -underline 0

    pack $cands_menubar.plot -side left

    menu $cands_menubar.plot.menu

    $cands_menubar.plot.menu add command -label "Plot..." \
	-command [namespace code iplot] -underline 0


    proc getbbox {ps} {
	# find the bounding box for postscript in ps and return as a list
	# values are in points

	if {![regexp \
		  "\n%%BoundingBox: (\[-0-9\]+) (\[-0-9\]+) (\[-0-9\]+) (\[-0-9\]+)\n" \
		  $ps junk llx lly urx ury]} {
	    error "Can't find BoundingBox in postscript"
	}
	return "$llx $lly $urx $ury"
    }


    proc iplot {} {
	# interactive plot : display dialog box then call plot
	variable dialog_ok
	variable outtype
	variable filename
	variable cmd

	plot_dialog
	if {!$dialog_ok} {
	    return
	}
	if {$outtype=="file"} {
	    set fname $filename
	} elseif {$outtype=="cmd"} {
	    set fname "|$cmd"
	}

	plot $fname
    }

    proc mm_to_points {mm} {
	variable scale
	return [expr 72/25.4*$mm/$scale]
    }
    
    variable pxperpt
    # set pxperpt [winfo fpixels $tview_toplevel 1p]
    set pxperpt [winfo fpixels $cands_notebook 1p]

    proc points_to_px {pt} {
	variable pxperpt
	return [expr $pxperpt*$pt]
    }

    proc px_to_pt {px} {
	variable pxperpt
	return [expr $px/$pxperpt]
    }
    
    proc plot {fname args} {
	# do the plot
	variable whichtraces
	variable whichtime
	global tview_traces_min
	global tview_traces_max
	global tview_traces_height
	global bci_filename
	global tview_toplevel
	variable pageheight_pt
	variable pageborder_pt

	foreach arg $args {
	    switch -exact $arg {
		"-alltraces" {set whichtraces all}
		"-vistraces" {set whichtraces visible}
		"-alltime" {set whichtime all}
		"-vistime" {set whichtime visible}
	    }
	}

	set f [open $fname "w"]

	set pagewidth_mm 291
	set pageheight_mm 210
	set pageborder_mm 10

	set pagewidth_pt [mm_to_points $pagewidth_mm]
	set pageheight_pt [mm_to_points $pageheight_mm]
	set pageborder_pt [mm_to_points $pageborder_mm]

	set pagewidth_px [points_to_px $pagewidth_pt]
	set pageheight_px [points_to_px $pageheight_pt]
	set pageborder_px [points_to_px $pageborder_pt]

	set scaleheight_px [$tview_toplevel.scale cget -height]
	set nameswidth_px [$tview_toplevel.tracenames cget -width]

	set scaleheight_pt [px_to_pt $scaleheight_px]
	set nameswidth_pt [px_to_pt $nameswidth_px]

	set availx_px [expr $pagewidth_px-2*$pageborder_px-$nameswidth_px]
	set availy_px [expr $pageheight_px-2*$pageborder_px-$scaleheight_px]

	if {$whichtraces=="all"} {
	    set ystart_px 0
	    set yend_px $tview_traces_height
	} else {
	    lassign [$tview_toplevel.traces yview] yminpr ymaxpr
	    set ystart_px [expr $yminpr*$tview_traces_height]
	    set yend_px [expr $ymaxpr*$tview_traces_height]
	}

	set ysize_px [expr $yend_px-$ystart_px]
	set yslices [expr int(ceil($ysize_px/$availy_px))]

	if {$whichtime=="all"} {
	    set xstart_px $tview_traces_min
	    set xend_px $tview_traces_max
	} else {
	    lassign [$tview_toplevel.traces xview] xminpr xmaxpr
	    set xstart_px [expr $tview_traces_min+ \
			       $xminpr*($tview_traces_max-$tview_traces_min)]
	    set xend_px [expr $tview_traces_min+ \
			     $xmaxpr*($tview_traces_max-$tview_traces_min)]
	}

	set xsize_px [expr $xend_px-$xstart_px]
	set xslices [expr int(ceil($xsize_px/$availx_px))]

	puts "x slices: $xslices"
	puts "y slices: $yslices"


	ps_preamble $f [expr $xslices*$yslices]
	set pgnum 1
	set xcurr_px $xstart_px
	loop xnum 0 $xslices {
	    set ycurr_px $ystart_px
	    loop ynum 0 $yslices {
		puts "doing page ($xnum,$ynum)"
		ps_pagesetup $f $pgnum "($xnum,$ynum)" \
		    "$bci_filename page ($xnum,$ynum)"
		
		set namesps [eval $tview_toplevel.tracenames postscript \
				 -y $ycurr_px -height $availy_px]
		set scaleps [eval $tview_toplevel.scale postscript \
				 -x $xcurr_px -width $availx_px]
		set tracesps_in [eval $tview_toplevel.traces postscript \
				     -x $xcurr_px -width $availx_px \
				     -y $ycurr_px -height $availy_px]

		# the following is a horrible hack because the lines
		# that divide one trace from the next are "too long"
		regsub -all {10000000} $tracesps_in {10000} tracesps

		ps_printat $f $namesps names 0 0
		ps_printat $f $scaleps scale $nameswidth_pt \
		    [expr $pageheight_pt-2*$pageborder_pt-$scaleheight_pt]
		ps_printat $f $tracesps traces $nameswidth_pt 0
		
		ps_pagedone $f

		set ycurr_px [expr $ycurr_px+$availy_px]
		incr pgnum
	    }
	    set xcurr_px [expr $xcurr_px+$availx_px]
	}
	ps_postamble $f
	close $f
	puts "done"
    }

    proc ps_preamble {f pages} {
	variable pageheight_pt
	variable pageborder_pt
	variable scale
	
	puts $f "%!PS-Adobe-3.0"
	puts $f "%%Creator: li's tviewps module"
	puts $f "%%CreationDate: [clock format [clock seconds]]"
	puts $f "%%Orientation: Landscape"
	puts $f "%%PageOrder: Ascend"
	puts $f "%%Pages: $pages"
	puts $f "%%EndComments"
	puts $f "%%BeginProlog"
	puts $f "%%EndProlog"
	puts $f "%%BeginSetup"
	puts $f "$scale $scale scale"
	# landscape
	puts $f "90 rotate 0 -$pageheight_pt translate"
	# border
	puts $f "$pageborder_pt $pageborder_pt translate"
	puts $f "%%EndSetup"
    }

    proc ps_printat {f ps name x y} {
	lassign [getbbox $ps] llx lly urx ury
	# puts "$name has bbox $llx $lly $urx $ury; moving to $x $y"
	puts $f "gsave"
	puts $f "[expr $x-$llx] [expr $y-$lly] translate"
	# puts $f "$llx $lly moveto ($name) show"
        ps_encapdocument $f $ps $name
	puts $f "grestore"
    }

    proc ps_encapdocument {f ps name} {
	puts $f "%%BeginDocument: $name"
	puts $f $ps
	puts $f "%%EndDocument"
    }

    proc ps_pagesetup {f n desc prdesc} {
	variable pageheight_pt 
	variable pageborder_pt
	puts $f "%%Page: $desc $n"
	puts $f "1 dict begin"
	puts $f "/showpage {} def";
	puts $f "/Times-Roman findfont 12 scalefont setfont"
	puts $f "0 [expr $pageheight_pt-2*$pageborder_pt] moveto"
	puts $f "0 -15 rmoveto"
	puts $f "($prdesc) show"
    }

    proc ps_pagedone {f} {
	puts $f "end"
	puts $f "gsave showpage grestore"
    }


    proc ps_postamble {f} {
	puts $f "%%Trailer"
	puts $f "%%EOF"
    }


    proc plot_dialog {} {
	variable dialog_ok

	toplevel .tviewps
	wm title .tviewps "Plot"
    
	radiobutton .tviewps.filebutton \
	    -variable [namespace current]::outtype -value file \
	    -text "Send plot to file" -underline 13
	radiobutton .tviewps.cmdbutton \
	    -variable [namespace current]::outtype -value cmd \
	    -text "Send plot to command" -underline 13
	entry .tviewps.filename -textvariable [namespace current]::filename
	entry .tviewps.cmd -textvariable [namespace current]::cmd
	button .tviewps.browse -command [namespace code browse] \
	    -text "Browse..." -underline 0
	menubutton .tviewps.cmdmenu -menu .tviewps.cmdmenu.menu \
	    -text "Defaults" -relief raised -underline 0

	menu .tviewps.cmdmenu.menu
	.tviewps.cmdmenu.menu add command -label "ghostview -a4 -" \
	    -command "set [namespace current]::cmd \"ghostview -a4 -\"" \
	    -underline 0
	.tviewps.cmdmenu.menu add command -label "lpr" \
	    -command "set [namespace current]::cmd \"lpr\"" -underline 0

	radiobutton .tviewps.vistracesbutton \
	    -variable [namespace current]::whichtraces -value visible \
	    -text "Only visible traces" -underline 5
	radiobutton .tviewps.alltracesbutton \
	    -variable [namespace current]::whichtraces -value all \
	    -text "All traces" -underline 0

	radiobutton .tviewps.vistimebutton \
	    -variable [namespace current]::whichtime -value visible \
	    -text "Only visible time"
	radiobutton .tviewps.alltimebutton \
	    -variable [namespace current]::whichtime -value all \
	    -text "All time"

	label .tviewps.scalelbl -text "Scale"
	entry .tviewps.scale -textvariable [namespace current]::scale

	frame .tviewps.okcancel
	button .tviewps.ok -text "OK" -underline 0 \
	    -command "set [namespace current]::dialog_ok 1"
	button .tviewps.cancel -text "cancel" -underline 0 \
	    -command "set [namespace current]::dialog_ok 0"
	pack .tviewps.ok .tviewps.cancel -in .tviewps.okcancel -side left

	grid .tviewps.filebutton -row 0 -column 0 -sticky w
	grid .tviewps.cmdbutton -row 1 -column 0 -sticky w
	grid .tviewps.filename -row 0 -column 1 -sticky w
	grid .tviewps.cmd -row 1 -column 1 -sticky w
	grid .tviewps.browse -row 0 -column 2 -sticky w
	grid .tviewps.cmdmenu -row 1 -column 2 -sticky w
	grid .tviewps.vistracesbutton -row 2 -column 0 -sticky w
	grid .tviewps.alltracesbutton -row 3 -column 0 -sticky w
	grid .tviewps.vistimebutton -row 4 -column 0 -sticky w
	grid .tviewps.alltimebutton -row 5 -column 0 -sticky w
	grid .tviewps.scalelbl -row 6 -column 0 -sticky e
	grid .tviewps.scale -row 6 -column 1 -sticky w
	grid .tviewps.okcancel -row 7 -column 0 -columnspan 3
	
	tkwait visibility .tviewps
	grab .tviewps
	tkwait variable [namespace current]::dialog_ok
	destroy .tviewps
    }


    proc browse {} {
	variable filename

	set filename [tk_getSaveFile -initialfile $filename \
			  -title "Plot filename" ]
    }
}



