#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: winbci.tcl,v 1.2 2000/03/24 17:06:45 bardslea Exp $
#
#
#   Winbci module
#   Windows interface to the bcode interpreter
#
#	11 Jun 1999, AMULET Group

li_require windows
li_require winstdio

global main_menubar

menubutton $main_menubar.sim -text "Simulation" -menu $main_menubar.sim.menu \
    -underline 0
pack $main_menubar.sim -side left
menu $main_menubar.sim.menu

$main_menubar.sim.menu add command -label "Load..." -command "bci_winload" \
    -underline 0
$main_menubar.sim.menu add command -label "Reload" -command "bci_reload"
$main_menubar.sim.menu add command -label "Run" -command "bci_run" \
    -underline 0
$main_menubar.sim.menu add command -label "Stop" -command {set bci_run_stop 1} \
    -underline 0
$main_menubar.sim.menu add command -label "Abandon" -command "bci_kill"

set bci_running 0

# Code to update the state of the simulation menu entries

proc simmenu_update {name1 name2 op} {
    global bci_ready
    global bci_running
    global bci_filename
    global stdio_input
    global main_menubar

    if {$stdio_input} {
	$main_menubar.sim.menu entryconfigure Load... -state disabled
	$main_menubar.sim.menu entryconfigure Reload -state disabled
	$main_menubar.sim.menu entryconfigure Run -state disabled
	$main_menubar.sim.menu entryconfigure Stop -state disabled
	$main_menubar.sim.menu entryconfigure Abandon -state disabled
    } elseif {$bci_running} {
	$main_menubar.sim.menu entryconfigure Load... -state disabled
	$main_menubar.sim.menu entryconfigure Reload -state disabled
	$main_menubar.sim.menu entryconfigure Run -state disabled
	$main_menubar.sim.menu entryconfigure Stop -state normal
	$main_menubar.sim.menu entryconfigure Abandon -state disabled
    } elseif {$bci_ready} {
	$main_menubar.sim.menu entryconfigure Load... -state disabled
	$main_menubar.sim.menu entryconfigure Reload -state disabled
	$main_menubar.sim.menu entryconfigure Run -state normal
	$main_menubar.sim.menu entryconfigure Stop -state disabled
	$main_menubar.sim.menu entryconfigure Abandon -state normal
    } else {
	$main_menubar.sim.menu entryconfigure Load... -state normal
	$main_menubar.sim.menu entryconfigure Run -state disabled
	$main_menubar.sim.menu entryconfigure Stop -state disabled
	$main_menubar.sim.menu entryconfigure Abandon -state disabled
	if {$bci_filename != {}} {
	    $main_menubar.sim.menu entryconfigure Reload -state normal
	} else {
	    $main_menubar.sim.menu entryconfigure Reload -state disabled
	}
    }
}

trace variable bci_ready w simmenu_update
trace variable bci_running w simmenu_update
trace variable stdio_input w simmenu_update

simmenu_update a b c


# Run bcode interpreter with occasional calls to update so menus
# etc. continue to work.

proc bci_run {} {
    global bci_run_stop
    global bci_running
    global bci_responsiveness
    global li_startup

    set bci_run_stop 0
    set bci_running 1
    set li_startup 0

    while {!$bci_run_stop} {
	if [catch {bci_exec $bci_responsiveness} errmsg] {
	    set bci_running 0
	    error $errmsg
	}
	update
    }
    set bci_running 0
}


# Load new bcode file

proc bci_winload {} {
    global bci_filename
    global bci_basename
    global config_save_filename

    set bci_winload_filename [tk_getOpenFile -title "Load bcode" \
				  -initialfile $bci_filename]
    if {$bci_winload_filename==""} {
	return
    }
    set bci_filename $bci_winload_filename
    bci_load $bci_filename
    if [file exists $bci_basename.lirc] {
	uplevel \#0 source $bci_basename.lirc
    }
    set config_save_filename "$bci_basename.lirc"
}



if {[catch {set bci_responsiveness}]} {
    set bci_responsiveness 1000
}


