#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: winhelp.tcl,v 1.2 2000/03/24 17:06:46 bardslea Exp $
#
#   Winhelp module
#   interactive help for graphical interface
#
#	11 Jun 1999, AMULET Group

li_require windows

global main_menubar

menubutton $main_menubar.help -text "Help" -menu $main_menubar.help.menu \
    -underline 0
pack $main_menubar.help -side right
menu $main_menubar.help.menu

$main_menubar.help.menu add command -label "Help on Help" \
    -command "help_on_help" -underline 0
$main_menubar.help.menu add command -label "About li" \
    -command "help_aboutli" -underline 0
$main_menubar.help.menu add command -label "On..." \
    -command "help_start" -underline 0

toplevel .helpgrabwin -cursor question_arrow -bg {} -width 1 -height 1 \
    -class Transient
wm overrideredirect .helpgrabwin 1
lower .helpgrabwin
wm geometry .helpgrabwin -1500-1500
bind .helpgrabwin <ButtonRelease-1> "help_getxy %X %Y"

toplevel .helpwin
wm title .helpwin "Help"
text .helpwin.text -yscrollcommand ".helpwin.scr set"
.helpwin.text tag configure heading1 -font \
    [option get .helpwin.text heading1font heading1font]
.helpwin.text tag configure bold -font \
    [option get .helpwin.text boldfont boldfont]
scrollbar .helpwin.scr -command ".helpwin.text yview"
button .helpwin.closebutton -text "Close" -command "wm withdraw .helpwin" \
    -underline 0
pack .helpwin.closebutton -side bottom
pack .helpwin.scr -side right -fill y
pack .helpwin.text -side left -expand yes -fill both
wm withdraw .helpwin

proc help_start {} {
    grab -global .helpgrabwin
}

proc help_getxy {x y} {
    set win [winfo containing $x $y]
    if {$win==""} {
	set help "No help available"
    } else {
	set help [option get $win helpText helpText]
	if {$help==""} {
	    set help "No help available for window $win"
	}
    }
    help_message $help
    grab release .helpgrabwin
}

proc help_message {msg} {
    .helpwin.text delete 1.0 end
    help_insert_html $msg
    wm deiconify .helpwin
}

proc help_insert_html {msg} {
    set tags "body"
    while {$msg!=""} {
	if {[crange $msg 0 3]=="<H1>"} {
	    set tags "heading1"
	    set msg [crange $msg 4 end]
	} elseif {[crange $msg 0 4]=="</H1>"} {
	    set tags "body"
	    set msg [crange $msg 5 end]
	} elseif {[crange $msg 0 2]=="<B>"} {
	    set tags "bold"
	    set msg [crange $msg 3 end]
	} elseif {[crange $msg 0 3]=="</B>"} {
	    set tags "body"
	    set msg [crange $msg 4 end]
	} elseif {[crange $msg 0 2]=="<P>"} {
	    .helpwin.text insert end "\n\n"
	    set msg [crange $msg 3 end]
	} else {
	    set p [string first "<" $msg]
	    if {$p==-1} {
		set c $msg
		set msg ""
	    } else {
		set c [crange $msg 0 {$p-1}]
		set msg [crange $msg $p end]
	    }
	    .helpwin.text insert end $c $tags
	}
    }
}

proc help_aboutli {} {
    help_message [option get . aboutText aboutText]
}

proc help_on_help {} {
    help_message [option get . helpHelp helphelp]
}


