#    The Language for Asynchronous Research and Development -- LARD
#    Copyright (C) 1996-1999 AMULET Group, Department of Computer Science
#        The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	$Id: winutils.tcl,v 1.2 2000/03/24 17:06:47 bardslea Exp $
#
#
#   Utilities for modules that use the graphical user interface
#
#	11 Jun 1999, AMULET Group

li_require windows

namespace eval winutils {
    # make a toplevel window with a menubar containing a window->close command

    proc maketoplevel {shortname longname} {
	global main_menubar
	variable ${shortname}_open 1

	toplevel .$shortname -menu .$shortname.menubar
	wm title .$shortname "$longname"
	wm iconname .$shortname "$longname"

	menu .$shortname.menubar
	.$shortname.menubar add cascade -label "Window" -underline 0 \
	    -menu .$shortname.menubar.window

	menu .$shortname.menubar.window
	.$shortname.menubar.window add command -label "Close" -underline 0 \
	    -command "set [namespace current]::${shortname}_open 0"

	$main_menubar.windows.menu add checkbutton -label "$longname" \
	    -variable [namespace current]::${shortname}_open -underline 1

	trace variable ${shortname}_open w \
	    [namespace code "open_changed $shortname"]
    }

    proc open_changed {shortname name1 name2 op} {
	variable ${shortname}_open

	if {[set ${shortname}_open]} {
	    wm deiconify .$shortname
	} else {
	    wm withdraw .$shortname
	}
    }

    # Make an image, finding the coresponding bitmap somewhere in the
    # load path

    proc make_image {name args} {
	global li_load_path
	foreach dir $li_load_path {
	    if {[file exists "$dir/bitmaps/$name.xbm"]} {
		eval "image create bitmap ${name}_img -file $dir/bitmaps/$name.xbm $args"
		return
	    }
	}
	error "Can't find bitmap $name"
    }
}



